package com.iver.andami.plugins;

import com.iver.andami.plugins.status.IExtensionStatus;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: Extension.java 13656 2007-09-12 16:56:26Z caballero $
 * $Log$
 * Revision 1.6.4.5  2007-09-12 16:56:26  caballero
 * exclusive unsaveddata
 *
 * Revision 1.6.4.4  2007/07/13 09:15:36  cesar
 * Add support for extension status
 *
 * Revision 1.6.4.3  2007/05/31 06:50:43  cesar
 * Add missing comments, translate existing ones to english
 *
 * Revision 1.6.4.2  2006/12/14 16:12:44  jmvivo
 * Modificaciones para dar soporte a que una Extension tenga el control exclusivo del UI (visibilidad/habilitado)
 *
 * Revision 1.6.4.1  2006/11/15 00:08:21  jjdelcerro
 * *** empty log message ***
 *
 * Revision 1.7  2006/09/15 10:39:18  caballero
 * multisplah y postInitialize
 *
 * Revision 1.6  2006/07/31 18:22:13  cesar
 * Rename finalize method to terminate method
 *
 * Revision 1.5  2006/05/02 15:53:06  jorpiell
 * Se ha cambiado la interfaz Extension por dos clases: una interfaz (IExtension) y una clase abstract(Extension). A partir de ahora todas las extensiones deben heredar de Extension
 *
 *
 */
/**
 * Extensions are the way in which plugins extend Andami. Extensions
 * can add controls to user interface (GUI) and execute some code
 * when controls are activated. Every class implementing
 * {@link IExtension} is an extension, but directly implementing
 * that interface is discouraged. The preferred way to create
 * an extension is extending this absctract class.
 *
 * @see IExtension
 *
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public abstract class Extension implements IExtension {

	/*
	 *  (non-Javadoc)
	 * @see com.iver.andami.plugins.Extension#terminate()
	 */
	public void terminate(){

	}
	/*
	 *  (non-Javadoc)
	 * @see com.iver.andami.plugins.Extension#postInitialize()
	 */
	public void postInitialize(){

	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.andami.plugins.IExtension#isEnabled(IExtension extension)
	 */
	public boolean isEnabled(IExtension extension) {
		return extension.isEnabled();
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.andami.plugins.IExtension#isVisible(IExtension extension)
	 */
	public boolean isVisible(IExtension extension) {
		return extension.isVisible();
	}

    /*
	 *  (non-Javadoc)
	 * @see com.iver.andami.plugins.IExtension#getStatus()
	 */
    public IExtensionStatus getStatus() {
    	return null;
    }

	/*
	 *  (non-Javadoc)
	 * @see com.iver.andami.plugins.IExtension#getStatus(IExtension extension)
	 */
	public IExtensionStatus getStatus(IExtension extension) {
		return extension.getStatus();
	}
}

