package es.gva.cit.catalog.protocols;

import java.net.URI;
import java.util.Hashtable;

import org.jzkit.search.provider.iface.SearchException;
import org.jzkit.search.provider.iface.Searchable;
import org.jzkit.search.provider.z3950.Z3950ServiceFactory;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.FileSystemXmlApplicationContext;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: Z3950ConnectionFactory.java 568 2007-07-27 07:23:27Z jpiera $
 * $Log$
 * Revision 1.1.2.2.4.2  2007/07/13 12:00:35  jorpiell
 * Add the posibility to add a new panel
 *
 * Revision 1.1.2.2.4.1  2007/07/10 11:18:04  jorpiell
 * Added the registers
 *
 * Revision 1.1.2.2  2006/11/15 00:08:08  jjdelcerro
 * *** empty log message ***
 *
 * Revision 1.2  2006/10/02 08:29:07  jorpiell
 * Modificados los cambios del Branch 10 al head
 *
 * Revision 1.1.2.1  2006/09/20 12:01:07  jorpiell
 * Se ha cambiado la versin de la jzkit, se ha incorporado la funcionalidad de cargar arcims
 *
 * Revision 1.1  2006/09/20 11:20:17  jorpiell
 * Se ha cambiado la versin de la librera jzkit de la 1 a la 2.0
 *
 *
 */
/**
 * This factory creates Z39.50 connections
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class Z3950ConnectionFactory {
	private static ApplicationContext app_context = null;
	private static boolean appContextInitialized = false;
	private static Z3950ServiceFactory factory = null;
	private static Hashtable connections = new Hashtable();
	
	public static Z3950Connection getConnection(URI uri){
		Z3950Connection connection = (Z3950Connection)connections.get(uri);
		if (connection == null){
			connection = new Z3950Connection(uri);
			connections.put(uri,connection);
		}
		return connection;
	}
	
	public static Searchable getSearchable(URI uri) throws SearchException{
		if (!(appContextInitialized)){
			initAppContext(uri);
			factory = new Z3950ServiceFactory();		
			factory.setApplicationContext(app_context);
			factory.setDefaultRecordSyntax("usmarc");
			factory.setDefaultElementSetName("F");
			factory.getRecordArchetypes().put("Default","usmarc::F");
			factory.getRecordArchetypes().put("FullDisplay","usmarc::F");
			factory.getRecordArchetypes().put("BriefDisplay","usmarc::B");
			factory.getRecordArchetypes().put("Holdings","usmarc::F");
		}
		factory.setHost(uri.getHost());
		factory.setPort(uri.getPort());
		Searchable s = factory.newSearchable();
		s.setApplicationContext(app_context);	
		return s;
	}

	/**
	 * Initialize the application context
	 * @param url 
	 */
	private static void initAppContext(URI uri){
		System.setProperty("javax.xml.parsers.DocumentBuilderFactory","org.apache.xerces.jaxp.DocumentBuilderFactoryImpl");
		try{
			app_context = new FileSystemXmlApplicationContext(new String[]{"config/ApplicationContextAlone.xml"},false);
			((FileSystemXmlApplicationContext)app_context).setClassLoader(Z3950ConnectionFactory.class.getClassLoader());
			((FileSystemXmlApplicationContext)app_context).refresh();
		}catch(Exception e){
			e.printStackTrace();
			app_context = new FileSystemXmlApplicationContext(new String[]{"gvSIG/extensiones/es.gva.cit.gvsig.catalogClient/config/ApplicationContext.xml"},false);
			((FileSystemXmlApplicationContext)app_context).setClassLoader(Z3950ConnectionFactory.class.getClassLoader());
			((FileSystemXmlApplicationContext)app_context).refresh();
		}			
		
		if (app_context == null){
			throw new RuntimeException("Unable to locate TestApplicationContext.xml definition file");
		}
		appContextInitialized = true;
	}
	
}
