package es.gva.cit.gazetteer.wfsg.drivers;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.util.Collection;

import org.gvsig.remoteClient.gml.exceptions.GMLException;
import org.gvsig.remoteClient.utils.Utilities;

import com.iver.utiles.swing.jcomboServer.ServerData;

import es.gva.cit.catalog.drivers.DiscoveryServiceCapabilities;
import es.gva.cit.catalog.protocols.HTTPPostProtocol;
import es.gva.cit.gazetteer.querys.Feature;
import es.gva.cit.gazetteer.querys.GazetteerQuery;
import es.gva.cit.gazetteer.wfs.drivers.WFSServiceDriver;
import es.gva.cit.gazetteer.wfsg.parsers.WfsgCapabilitiesParser;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: WFSGServiceDriver.java 539 2007-07-26 11:21:38Z jpiera $
 * $Log$
 * Revision 1.1.2.2  2007/07/13 12:00:35  jorpiell
 * Add the posibility to add a new panel
 *
 * Revision 1.1.2.1  2007/07/10 11:18:04  jorpiell
 * Added the registers
 *
 *
 */
/**
 * @author Jorge Piera LLodr (jorge.piera@iver.es)
 */
public class WFSGServiceDriver extends WFSServiceDriver {
	//TODO use the download manager instead of this
	private static final String tempDirectoryPath = System.getProperty("java.io.tmpdir")+"/tmp-andami";
	
	/*
	 * (non-Javadoc)
	 * @see es.gva.cit.gazetteer.wfs.drivers.WFSServiceDriver#getCapabilities(java.net.URI)
	 */
	public DiscoveryServiceCapabilities getCapabilities(URI uri) {
		URL url = null;
		try {
			url = uri.toURL();
		} catch (MalformedURLException e) {
			setServerAnswerReady("errorServerNotFound");
			return null;
		}
		Collection nodes = new HTTPPostProtocol().doQuery(url,
				WFSGMessages.getHTTPPOSTCapabilities(), 0);
		
		WfsgCapabilitiesParser parser = new WfsgCapabilitiesParser(this);
		parser.parse(nodes);
		return parser.getCapabilities();			
	}
	
	/*
	 * (non-Javadoc)
	 * @see es.gva.cit.gazetteer.wfs.drivers.WFSServiceDriver#getFeature(java.net.URI, es.gva.cit.gazetteer.querys.Query)
	 */
	 public Feature[] getFeature(URI uri, GazetteerQuery query) throws GMLException {        
		 URL url = null;
			try {
				url = uri.toURL();
			} catch (MalformedURLException e) {
				setServerAnswerReady("errorServerNotFound");
				return null;
			}
			query.setFieldAttribute("geographicIdentifier");			
			try {
				File file = Utilities.downloadFile(url, WFSGMessages.getHTTPPOSTFeature(query,0), "wfsg_getFeature.xml", null);
				return parseOutputFile(file,query.getFieldAttribute());
			} catch (Exception e) {
				throw new GMLException(e);
			} 				 
	 }

	/*
	 * (non-Javadoc)
	 * @see es.gva.cit.gazetteer.drivers.AsbtractGazetteerServiceDriver#isDescribeFeatureTypeNeeded()
	 */
	public boolean isDescribeFeatureTypeNeeded(){
		return false;
	}

	/*
	 * (non-Javadoc)
	 * @see es.gva.cit.gazetteer.drivers.IGazetteerServiceDriver#getServiceName()
	 */
	public String getServiceName() {
		return ServerData.SERVER_SUBTYPE_GAZETTEER_WFSG;
	}	
}
