/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig;

import com.iver.andami.PluginServices;
import com.iver.andami.plugins.Extension;
import com.iver.cit.gvsig.fmap.MapControl;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLayers;
import com.iver.cit.gvsig.fmap.layers.FLyrRaster;
import com.iver.cit.gvsig.fmap.layers.SelectableDataSource;
import com.iver.cit.gvsig.project.documents.view.IProjectView;
import com.iver.cit.gvsig.project.documents.view.gui.View;


/**
 * Extensin para raster.
 * El control es visible si al menos una de las capas cargadas es raster.
 *
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class RasterExtension extends Extension{
	private SelectableDataSource dataSource = null;


	/* (non-Javadoc)
	 * @see com.iver.andami.plugins.Extension#inicializar()
	 */
	public void initialize() {
	}


	/* (non-Javadoc)
	 * @see com.iver.andami.plugins.Extension#execute(java.lang.String)
	 */
	public void execute(String actionCommand) {
		View vista = (View) PluginServices.getMDIManager().getActiveWindow();
		MapControl mapCtrl = vista.getMapControl();
		if ("SELECTIMAGE".equals(actionCommand)) {
			mapCtrl.setTool("selectImage");
		}
	}

	
	/* (non-Javadoc)
	 * @see com.iver.andami.plugins.Extension#isEnabled()
	 */
	public boolean isEnabled() {
		com.iver.andami.ui.mdiManager.IWindow vista = (com.iver.andami.ui.mdiManager.IWindow) PluginServices.getMDIManager().getActiveWindow();

		if (vista == null) 
			return false;

		if (vista instanceof com.iver.cit.gvsig.project.documents.view.gui.View) {
				com.iver.cit.gvsig.project.documents.view.gui.View view = (com.iver.cit.gvsig.project.documents.view.gui.View) vista;
				IProjectView model = view.getModel();
				FLayer[] seleccionadas = model.getMapContext().getLayers().getActives();
				FLayers lyrs = model.getMapContext().getLayers();
				
				//Se visualiza el control si al menos una de las capas cargadas es raster
				for(int i=0;i<lyrs.getLayersCount();i++){
					if(lyrs.getLayer(i) instanceof FLyrRaster && lyrs.getLayer(i).isAvailable())
						return true;
				}				
		}

		return false;
	}

	
	/* (non-Javadoc)
	 * @see com.iver.andami.plugins.Extension#isVisible()
	 */
	public boolean isVisible() {
		com.iver.andami.ui.mdiManager.IWindow vista = (com.iver.andami.ui.mdiManager.IWindow) PluginServices.getMDIManager().getActiveWindow();

		if (vista == null) 
			return false;

		if (vista instanceof com.iver.cit.gvsig.project.documents.view.gui.View) {
				com.iver.cit.gvsig.project.documents.view.gui.View view = (com.iver.cit.gvsig.project.documents.view.gui.View) vista;
				IProjectView model = view.getModel();
				FLayer[] seleccionadas = model.getMapContext().getLayers().getActives();
				FLayers lyrs = model.getMapContext().getLayers();
				
				//Se visualiza el control si al menos una de las capas cargadas es raster
				for(int i=0;i<lyrs.getLayersCount();i++){
					if(lyrs.getLayer(i) instanceof FLyrRaster)
						return true;
				}				
		}

		return false;
	}

}
