/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 */
package com.prodevelop.cit.gvsig.vectorialdb.wizard;

import com.hardcode.driverManager.Driver;
import com.iver.andami.PluginServices;
import com.iver.andami.messages.NotificationManager;
import com.iver.cit.gvsig.fmap.MapControl;
import com.iver.cit.gvsig.fmap.core.ICanReproject;
import com.iver.cit.gvsig.fmap.crs.CRSFactory;
import com.iver.cit.gvsig.fmap.drivers.DBException;
import com.iver.cit.gvsig.fmap.drivers.DBLayerDefinition;
import com.iver.cit.gvsig.fmap.drivers.IConnection;
import com.iver.cit.gvsig.fmap.drivers.IVectorialDatabaseDriver;
import com.iver.cit.gvsig.gui.panels.CRSSelectPanel;
import com.iver.cit.gvsig.project.documents.gui.FOpenDialog;

import javax.swing.JCheckBox;
import javax.swing.JOptionPane;

import org.cresques.cts.IProjection;


/**
 * Utility class that represents a table list item as a selectable check box.
 *
 * @author jldominguez
 *
 */
public class TablesListItem extends JCheckBox {
    private String tableName = "";
    private UserSelectedFieldsPanel selectedFieldsPanel = null;
    private UserTableSettingsPanel tableSettingsPanel = null;
    private IVectorialDatabaseDriver driver = null;
    private IConnection conn = null;
    private MapControl mc;
    private WizardVectorialDB parent = null;
    private boolean activated = false;
	private CRSSelectPanel jPanelProj;

    public TablesListItem(String name, IVectorialDatabaseDriver drv, IConnection _conn,
        MapControl _mc, WizardVectorialDB _parent) {
        tableName = name;
        setText(name);
        driver = drv;
        conn = _conn;
        mc = _mc;
        parent = _parent;
    }

    public void activate() {
        activated = true;
        selectedFieldsPanel.loadValues();
        tableSettingsPanel.loadValues();
    }

    public boolean isActivated() {
        return activated;
    }

    /**
     * Tells whether this item prevents the wizard from being in a valid final state.
     * @return whether this item prevents the wizard from being in a valid final state.
     */
    public boolean disturbsWizardValidity() {
        if (isSelected()) {
            return (!hasValidValues());
        }
        else {
            return false;
        }
    }

    private boolean hasValidValues() {
        return tableSettingsPanel.hasValidValues();
    }

    public String toString() {
        return tableName;
    }

    public String getTableName() {
        return tableName;
    }

    public void setEnabledPanels(boolean b) {
        selectedFieldsPanel.enableControls(b);
        tableSettingsPanel.enableControls(b);
    }

    public UserSelectedFieldsPanel getUserSelectedFieldsPanel()
        throws DBException {
        if (selectedFieldsPanel == null) {
            String[] allf = driver.getAllFields(conn, tableName);
            String[] allt = driver.getAllFieldTypeNames(conn, tableName);
            selectedFieldsPanel = new UserSelectedFieldsPanel(allf, allt, true,
                    parent);
        }

        return selectedFieldsPanel;
    }

    public UserTableSettingsPanel getUserTableSettingsPanel(String espView)
        throws DBException {
        if (tableSettingsPanel == null) {

        	String[] ids = new String[0];
        	try {
        		ids = driver.getIdFieldsCandidates(conn, tableName);
        	} catch (DBException se) {
        		String msg = PluginServices.getText(this, "id_not_available") + " " + tableName
        		+ ":\n" + se.getMessage();
        		String title = PluginServices.getText(this, "id_error");
        		JOptionPane.showMessageDialog(parent, msg, title, JOptionPane.ERROR_MESSAGE);
        		setSelected(false);
        	}

        	String[] geos = new String[0];
        	try {
        		geos = driver.getGeometryFieldsCandidates(conn, tableName);
        	} catch (DBException se) {
        		String msg = PluginServices.getText(this, "geo_field_not_available")
        		+ ":\n" + PluginServices.getText(this, se.getMessage()) + ": " + tableName;
        		String title = PluginServices.getText(this, "geo_field_error");
        		JOptionPane.showMessageDialog(parent, msg, title, JOptionPane.ERROR_MESSAGE);
        		setSelected(false);
        	}

            int ids_size = ids.length;
            FieldComboItem[] ids_ci = new FieldComboItem[ids_size];

            for (int i = 0; i < ids_size; i++)
                ids_ci[i] = new FieldComboItem(ids[i]);

            int geos_size = geos.length;
            FieldComboItem[] geos_ci = new FieldComboItem[geos_size];

            for (int i = 0; i < geos_size; i++)
                geos_ci[i] = new FieldComboItem(geos[i]);

            tableSettingsPanel = new UserTableSettingsPanel(ids_ci, geos_ci,
                    tableName, mc, true, parent, projectionPanel(espView,ids_ci,geos_ci));
        }

        return tableSettingsPanel;
    }
    public CRSSelectPanel projectionPanel(String espView,
			FieldComboItem[] ids_ci, FieldComboItem[] geos_ci) {
		IProjection proj = FOpenDialog.getLastProjection();
		if (driver instanceof ICanReproject) {
			try {
				DBLayerDefinition lyrDef = new DBLayerDefinition();
				lyrDef.setName(tableName);
				String[] tokens = tableName.split("\\u002E", 2);
				String schema = null;
				String tableName;
				if (tokens.length > 1) {
					schema = tokens[0];
					tableName = tokens[1];
				} else {
					tableName = tokens[0];
				}
				lyrDef.setSchema(schema);
				lyrDef.setTableName(tableName);
				String fidField = ids_ci[0].toString();
				String geomField = geos_ci[0].toString();
				lyrDef.setFieldGeometry(geomField);
				lyrDef.setFieldID(fidField);
				String strEPSG = espView;
				lyrDef.setSRID_EPSG(strEPSG);
				((ICanReproject) driver).setDestProjection(strEPSG);

				proj = CRSFactory.getCRS("EPSG:"
						+ ((ICanReproject) driver).getSourceProjection(conn,
								lyrDef));
			} catch (Exception e) {
				NotificationManager.addInfo("Incorrect projection", e);
			}
		}
		return getJPanelProj(proj);
	}
    private CRSSelectPanel getJPanelProj(IProjection proj) {
//		if (jPanelProj == null) {
//			IProjection proj = CRSFactory.getCRS("EPSG:23030");
//			if (PluginServices.getMainFrame() != null) {
//				proj = FOpenDialog.getLastProjection();
//			}
			jPanelProj = CRSSelectPanel.getPanel(proj);

			jPanelProj.setTransPanelActive(true);
			jPanelProj.setBounds(new java.awt.Rectangle(210, 22, 280, 32));
			jPanelProj.setPreferredSize(new java.awt.Dimension(448,35));
			jPanelProj.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
			        if (jPanelProj.isOkPressed()) {
			        	FOpenDialog.setLastProjection(jPanelProj.getCurProj());
			        }
				}
			});

//		}

		return jPanelProj;
	}
//    public IConnection getConnection() {
//    	return conn;
//    }
}
