/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig;

import javax.swing.ImageIcon;

import com.hardcode.gdbms.engine.data.driver.DriverException;
import com.hardcode.gdbms.engine.instruction.FieldNotFoundException;
import com.iver.andami.PluginServices;
import com.iver.andami.messages.NotificationManager;
import com.iver.andami.plugins.Extension;
import com.iver.andami.preferences.IPreference;
import com.iver.andami.preferences.IPreferenceExtension;
import com.iver.andami.ui.mdiManager.IWindow;
import com.iver.cit.gvsig.fmap.MapContext;
import com.iver.cit.gvsig.fmap.layers.Annotation_Layer;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLyrVect;
import com.iver.cit.gvsig.fmap.layers.SelectableDataSource;
import com.iver.cit.gvsig.gui.simpleWizard.SimpleWizard;
import com.iver.cit.gvsig.project.documents.gui.Annotation_ConfigureLabel;
import com.iver.cit.gvsig.project.documents.gui.Annotation_Create;
import com.iver.cit.gvsig.project.documents.gui.Annotation_FieldSelect;
import com.iver.cit.gvsig.project.documents.gui.Annotation_Open;
import com.iver.cit.gvsig.project.documents.view.IProjectView;
import com.iver.cit.gvsig.project.documents.view.legend.preferences.Annotation_Preferences;
import com.iver.cit.gvsig.project.documents.view.toc.AbstractTocContextMenuAction;
import com.iver.cit.gvsig.project.documents.view.toc.gui.FLyrVectEditPropertiesTocMenuEntry2;
import com.iver.utiles.extensionPoints.ExtensionPoint;
import com.iver.utiles.extensionPoints.ExtensionPoints;
import com.iver.utiles.extensionPoints.ExtensionPointsSingleton;

/**
 * Extension to create an annotation layer.
 *
 * @author Vicente Caballero Navarro
 */
public class Annotation_Extension extends Extension implements
		IPreferenceExtension {
	private MapContext map = null;

	private Annotation_Preferences ap = new Annotation_Preferences();

	/**
	 * @see com.iver.andami.plugins.IExtension#initialize()
	 */
	public void initialize() {
		ExtensionPoints extensionPoints = ExtensionPointsSingleton
				.getInstance();

		AbstractTocContextMenuAction ma = (AbstractTocContextMenuAction) ((ExtensionPoint) extensionPoints
				.get("View_TocActions")).get("FLyrVectEditProperties");
		((ExtensionPoint) extensionPoints.get("View_TocActions"))
				.remove("FLyrVectEditProperties");
		((ExtensionPoint) extensionPoints.get("View_TocActions")).put(
				"FLyrVectEditProperties",
				new FLyrVectEditPropertiesTocMenuEntry2(ma));
		AddLayer.addWizard(Annotation_Open.class);
	}

	/**
	 * @see com.iver.andami.plugins.IExtension#execute(java.lang.String)
	 */
	public void execute(String actionCommand) {
		if ("ANNOTATIONLAYER".equals(actionCommand)) {
			ImageIcon Logo = new javax.swing.ImageIcon(this.getClass()
					.getClassLoader()
					.getResource("images/package_graphics.png"));

			SimpleWizard wizard = new SimpleWizard(Logo);

			FLyrVect lv = (FLyrVect) map.getLayers().getActives()[0];
			Annotation_Layer la=null;
			try {
				la = Annotation_Layer.createLayerFromVect(lv);
			} catch (FieldNotFoundException e) {
				NotificationManager.addError(e);
			} catch (com.iver.cit.gvsig.fmap.DriverException e) {
				NotificationManager.addError(e);
			}
			la.setName(lv.getName());

			Annotation_FieldSelect panel1 = new Annotation_FieldSelect(wizard
					.getWizardComponents(), la);
			Annotation_ConfigureLabel panel2 = new Annotation_ConfigureLabel(
					wizard.getWizardComponents(), la);

			wizard.getWizardComponents().addWizardPanel(panel1);
			wizard.getWizardComponents().addWizardPanel(panel2);

			wizard.getWizardComponents()
					.setFinishAction(
							new Annotation_Create(wizard.getWizardComponents(),
									map, la));

			wizard.getWindowInfo().setWidth(540);
			wizard.getWindowInfo().setHeight(380);
			wizard.getWindowInfo().setTitle(
					PluginServices.getText(this, "to_annotation"));

			PluginServices.getMDIManager().addWindow(wizard);

		}
	}

	/**
	 * @see com.iver.andami.plugins.IExtension#isEnabled()
	 */
	public boolean isEnabled() {
		IWindow v = PluginServices.getMDIManager().getActiveWindow();

		if ((v != null)
				&& v instanceof com.iver.cit.gvsig.project.documents.view.gui.View) {
			com.iver.cit.gvsig.project.documents.view.gui.View vista = (com.iver.cit.gvsig.project.documents.view.gui.View) v;
			IProjectView model = vista.getModel();
			map = model.getMapContext();

			FLayer[] layers = map.getLayers().getActives();

			if (layers.length == 1) {
				if (layers[0].isAvailable() && layers[0] instanceof FLyrVect) {
					FLyrVect lv = (FLyrVect) layers[0];
					try {
						SelectableDataSource sds = lv.getSource()
								.getRecordset();

						if (sds.getFieldCount() > 0) {
							return true;
						}
					} catch (DriverException e) {
						return false;
					}
				}
			}
		}

		return false;
	}

	/**
	 * @see com.iver.andami.plugins.IExtension#isVisible()
	 */
	public boolean isVisible() {
		IWindow v = PluginServices.getMDIManager().getActiveWindow();

		if (v == null) {
			return false;
		} else if (v instanceof com.iver.cit.gvsig.project.documents.view.gui.View) {
			return true;
		} else {
			return false;
		}
	}

	public IPreference getPreferencesPage() {
		return ap;
	}
}
