/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2005 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */

package com.iver.cit.gvsig.project.documents.view.legend.preferences;

import java.awt.Color;
import java.awt.Font;
import java.awt.GridBagLayout;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;

import com.iver.andami.PluginServices;
import com.iver.andami.preferences.AbstractPreferencePage;
import com.iver.andami.preferences.StoreException;
import com.iver.cit.gvsig.fmap.layers.Annotation_Mapping;
import com.iver.cit.gvsig.gui.panels.ColorChooserPanel;
import com.iver.cit.gvsig.project.documents.view.gui.FontOptions;
import com.iver.utiles.swing.JComboBox;
/**
 *  Default configuration page.
 *  <b><b>
 *  Here the user can establish what settings wants to use by default annotations.
 *
 *
 * @author Vicente Caballero Navarro
 */
public class Annotation_Preferences extends AbstractPreferencePage {


	protected String id;
	private ImageIcon icon;
	private JTextField txtDefaultText;
	private JComboBox cmbDefaultTypeFont;
	private JComboBox cmbDefaultStyleFont;
	private JTextField txtDefaultRotate;
	private JTextField txtDefaultHeight;
	private ColorChooserPanel jccDefaultColor;
//	private JSlider jsDefaultColorAlpha;
	private static boolean panelStarted = false;

	/**
	 * Creates a new panel containing View preferences settings.
	 *
	 */
	public Annotation_Preferences() {
		super();
		id = this.getClass().getName();
		icon = new ImageIcon(this.getClass().getClassLoader().getResource("images/AnnotationProperties.png"));
	}

	public void initializeValues() {
		if (!panelStarted) getPanel();

		txtDefaultText.setText(Annotation_Mapping.DEFAULTTEXT);
		cmbDefaultTypeFont.setSelectedItem(Annotation_Mapping.DEFAULTTYPEFONT);
		String style=FontOptions.getFontStyles()[Annotation_Mapping.DEFAULTSTYLEFONT];
		cmbDefaultStyleFont.setSelectedItem(style);
		Color color=new Color(Annotation_Mapping.DEFAULTCOLOR);
		jccDefaultColor.setColor(color);
		jccDefaultColor.setAlpha(color.getAlpha());
		txtDefaultHeight.setText(String.valueOf(Annotation_Mapping.DEFAULTHEIGHT));
		txtDefaultRotate.setText(String.valueOf(Annotation_Mapping.DEFAULTROTATE));
	}

	public String getID() {
		return id;
	}

	public String getTitle() {
		return PluginServices.getText(this, "annotation_preferences");
	}

	public JPanel getPanel() {
		if (panelStarted) return this;
		panelStarted = true;

		// just a separator
		addComponent(new JLabel(" "));

		addComponent(new JLabel(PluginServices.getText(this,"change_options_of_annotations")));

		JPanel optionsPanel = new JPanel();
		optionsPanel.setBorder(new TitledBorder(PluginServices.getText(this, "options.annotations")));
		optionsPanel.setLayout(new GridBagLayout());

		addComponent(new JLabel(PluginServices.getText(this,"text")),
				txtDefaultText = new JTextField());

		addComponent(new JLabel(PluginServices.getText(this,"fonttype")),
				cmbDefaultTypeFont = new JComboBox());
		String[] types=FontOptions.getFontTypes();
		for (int i=0;i<types.length;i++){
			cmbDefaultTypeFont.addItem(types[i]);
		}
		cmbDefaultTypeFont.setSelectedItem(FontOptions.ARIAL);
		addComponent(new JLabel(PluginServices.getText(this,"fontstyle")),
				cmbDefaultStyleFont = new JComboBox());
		String[] styles=FontOptions.getFontStyles();
		for (int i=0;i<styles.length;i++){
			cmbDefaultStyleFont.addItem(styles[i]);
		}
		cmbDefaultStyleFont.setSelectedItem(FontOptions.PLAIN);
		addComponent(new JLabel(PluginServices.getText(this,"fontheight")),
				txtDefaultHeight = new JTextField());

		addComponent(new JLabel(PluginServices.getText(this,"fontrotate")),
				txtDefaultRotate = new JTextField());

		addComponent(new JLabel(PluginServices.getText(this,"fontcolor")),
				jccDefaultColor = new ColorChooserPanel());
//				jsDefaultColorAlpha = new JSlider());

		//addComponent(optionsPanel);
		addComponent(new JLabel(" "));

		initializeValues();
		return this;
	}

	public void storeValues() throws StoreException {
		Color fontColor=jccDefaultColor.getColor();
		String text=txtDefaultText.getText();
		String fontType=(String)cmbDefaultTypeFont.getSelectedItem();
		int fontStyle=cmbDefaultStyleFont.getSelectedIndex();
		int fontHeight=Integer.parseInt(txtDefaultHeight.getText());
		int fontRotate=Integer.parseInt(txtDefaultRotate.getText());

		Annotation_Mapping.storeValues(fontColor,text,fontType,fontStyle,fontHeight,fontRotate);
	}


	public void initializeDefaults() {
		Color fontColor=Color.black;
		String text="";
		String fontType=FontOptions.ARIAL;
		int fontStyle=Font.PLAIN;
		int fontHeight=10;
		int fontRotate=0;

		jccDefaultColor.setColor(fontColor);
//		jsDefaultColorAlpha.setValue(fontColor.getAlpha());

		txtDefaultText.setText(text);
		cmbDefaultTypeFont.setSelectedItem(fontType);
		cmbDefaultStyleFont.setSelectedItem(FontOptions.PLAIN);
		txtDefaultHeight.setText(String.valueOf(fontHeight));
		txtDefaultRotate.setText(String.valueOf(fontRotate));

		Annotation_Mapping.DEFAULTCOLOR=fontColor.getRGB();
		Annotation_Mapping.DEFAULTTEXT=text;
		Annotation_Mapping.DEFAULTTYPEFONT=fontType;
		Annotation_Mapping.DEFAULTSTYLEFONT=fontStyle;
		Annotation_Mapping.DEFAULTHEIGHT=fontHeight;
		Annotation_Mapping.DEFAULTROTATE=fontRotate;
	}

	public ImageIcon getIcon() {
		return icon;
	}

	public boolean isValueChanged() {
		return super.hasChanged();
	}

	public void setChangesApplied() {
		setChanged(false);
	}
}
