package org.gvsig.fmap.drivers.gpe.writer;

import java.awt.geom.PathIterator;
import java.util.ArrayList;

import org.cresques.cts.ICoordTrans;
import org.cresques.cts.IProjection;
import org.gvsig.gpe.gml.utils.CoordsContainer;
import org.gvsig.gpe.writers.GPEWriterHandler;
import org.gvsig.remoteClient.gml.schemas.XMLElement;

import com.iver.cit.gvsig.fmap.core.FMultiPoint2D;
import com.iver.cit.gvsig.fmap.core.FPoint2D;
import com.iver.cit.gvsig.fmap.core.FPoint3D;
import com.iver.cit.gvsig.fmap.core.FPolygon2D;
import com.iver.cit.gvsig.fmap.core.FPolygon3D;
import com.iver.cit.gvsig.fmap.core.FPolyline2D;
import com.iver.cit.gvsig.fmap.core.FPolyline3D;
import com.iver.cit.gvsig.fmap.core.FShape;
import com.iver.cit.gvsig.fmap.core.IGeometry;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id$
 * $Log$
 *
 */
/**
 * @author Jorge Piera LLodr (jorge.piera@iver.es)
 */
public class ExportGeometry {
	private GPEWriterHandler writer = null;
	//To know if the geometry is multiple
	private boolean isMultiple = false;
	//To reproject geometries
	private IProjection projOrig = null;
	private IProjection projDest = null;
	private ICoordTrans coordTrans = null;
	private String crs = null;

	public ExportGeometry(GPEWriterHandler writer) {
		super();
		this.writer = writer;
	}

	/**
	 * It writes a geometry
	 * @param geom
	 * The geometry to write
	 * @param crs
	 * The coordinates reference system
	 */
	public void writeGeometry(IGeometry geom){
		crs = null;
		if (projDest != null){
			crs = projDest.getAbrev();
		}
		if (geom instanceof FMultiPoint2D){
			FMultiPoint2D multi = (FMultiPoint2D)geom;
			for (int i=0 ; i<multi.getNumPoints() ; i++){
				reproject(multi.getPoint(i));
			}
			writeMultiPoint(multi, crs);
			return;
		}
		FShape shp = (FShape)geom.getInternalShape();
		reproject(shp);
		if (shp.getShapeType() == FShape.POINT){
			writePoint((FPoint2D)shp, crs);
		}else if ((shp.getShapeType()==FShape.LINE)||
				(shp.getShapeType()==FShape.POLYGON)){
			writeGeneralPath(shp, crs);
		}
	}

	/**
	 * Reproject a geometry
	 * @param shp
	 */
	private void reproject(FShape shp){
		ICoordTrans coordTrans = getCoordTrans();
		if (coordTrans != null){
			try{
				shp.reProject(coordTrans);
			}catch(Exception e){
				//The server is the responsible to reproject
				if (projOrig != null){
					crs = projOrig.getAbrev();
				}
			}
		}
	}

	/**
	 * It writes a geometry based on a General path
	 * @param shp
	 * @param crs
	 */
	private void writeGeneralPath(FShape shp, String crs){		
		PathIterator it = shp.getPathIterator(null);
		int theType;
		double[] theData = new double[6];
		ArrayList geometries = new ArrayList();
		CoordsContainer coordsContainer = null;;
		double[] coordinates = null;
		while (!it.isDone()){
			theType = it.currentSegment(theData);
			switch (theType) {
			case PathIterator.SEG_MOVETO:
				if (coordsContainer != null){
					geometries.add(coordsContainer);
				}
				coordsContainer = new CoordsContainer();				
				coordinates = new double[3];
				coordinates[0] = theData[0];
				coordinates[1] = theData[1];
				coordinates[2] = theData[2];
				coordsContainer.addCoordinates(coordinates);				
				break;
			case PathIterator.SEG_LINETO:
				coordinates = new double[3];
				coordinates[0] = theData[0];
				coordinates[1] = theData[1];
				coordinates[2] = theData[2];
				coordsContainer.addCoordinates(coordinates);
				break;
			case PathIterator.SEG_CLOSE:
				break;
			}
			it.next();
		}
		geometries.add(coordsContainer);
		if (shp.getShapeType()==FShape.LINE){				
			writeLine((FPolyline2D)shp, crs, geometries);
		}else if (shp.getShapeType()==FShape.POLYGON){					
			writePolygon((FPolygon2D)shp, crs, geometries);
		}
	}

	/**
	 * Writes a point in 2D
	 * @param point
	 * The point to write
	 * @param crs
	 * The coordinates reference system
	 */
	private void writePoint(FPoint2D point, String crs){
		double z = 0;
		if (point instanceof FPoint3D){
			z = ((FPoint3D)point).getZs()[0];
		}
		writer.startPoint(null, point.getX(), point.getY(), z, crs);
		writer.endPoint();
	}

	/**
	 * Writes a multipoint in 2D
	 * @param point
	 * The point to write
	 * @param crs
	 * The coordinates reference system
	 */
	private void writeMultiPoint(FMultiPoint2D multi, String crs){
		writer.startMultiPoint(null, crs);
		for (int i=0 ; i<multi.getNumPoints() ; i++){
			FPoint2D point = multi.getPoint(i);
			writePoint(point, crs);
		}
		writer.endMultiPoint();
	}

	/**
	 * Writes a line in 2D
	 * @param line
	 * The line to write
	 * @param crs
	 * The coordinates reference system
	 * @param geometries
	 * The parsed geometries
	 */
	private void writeLine(FPolyline2D line, String crs, ArrayList geometries){
		if ((isMultiple) || (geometries.size() > 1)){
			writer.startMultiLineString(null, crs);
		}
		for (int i=0 ; i<geometries.size() ; i++){
			CoordsContainer coordsContainer = (CoordsContainer)geometries.get(i);
			double[][] coords = coordsContainer.getCoordinates();
			writer.startLineString(null, coords[0], coords[1], coords[2], crs);
			writer.endLineString();
		}
		if ((isMultiple) || (geometries.size() > 1)){
			writer.endMultiLineString();
		}
	}

	/**
	 * Writes a polygon in 2D
	 * @param polygon
	 * The polygon to write
	 * @param crs
	 * The coordinates reference system
	 * @param geometries
	 * The parsed geometries
	 */
	private void writePolygon(FPolygon2D polygon, String crs, ArrayList geometries){
		if ((isMultiple) || (geometries.size() > 1)){
			writer.startMultiPolygon(null, crs);
		}		
		for (int i=0 ; i<geometries.size() ; i++){
			CoordsContainer coordsContainer = (CoordsContainer)geometries.get(i);
			double[][] coords = coordsContainer.getCoordinatesClosed();
			writer.startPolygon(null, coords[0], coords[1], coords[2], crs);
			writer.endPolygon();
		}
		if ((isMultiple) || (geometries.size() > 1)){
			writer.endMultiPolygon();
		}
	}

	/**
	 * @param projOrig the projOrig to set
	 */
	public void setProjOrig(IProjection projOrig) {
		this.projOrig = projOrig;
	}

	/**
	 * @param projDest the projDest to set
	 */
	public void setProjDest(IProjection projDest) {
		this.projDest = projDest;
	}

	/**
	 * @return the coordTrans
	 */
	private ICoordTrans getCoordTrans() {
		if (coordTrans == null){
			if ((projOrig == null) || (projDest == null)){
				return null;
			}
			coordTrans = projOrig.getCT(projDest);
		}
		return coordTrans;
	}

	/**
	 * @param writer the writer to set
	 */
	public void setWriter(GPEWriterHandler writer) {
		this.writer = writer;
	}



	/**
	 * @param geometry the geometry to set
	 */
	public void setGeometry(XMLElement geometry) {
		if (geometry != null){
			if (geometry.getEntityType().getName().toLowerCase().indexOf("multi") > 0){
				isMultiple = true;				
			}else{
				isMultiple = false;	
			}
		}
	}

	/**
	 * @return the isMultiple
	 */
	public boolean isMultiple() {
		return isMultiple;
	}

	/**
	 * @param isMultiple the isMultiple to set
	 */
	public void setMultiple(boolean isMultiple) {
		this.isMultiple = isMultiple;
	}

	/**
	 * @return the projDest
	 */
	public IProjection getProjDest() {
		return projDest;
	}
}
