package org.gvsig.fmap.drivers.gpe.writer;

import java.awt.geom.Rectangle2D;
import java.io.File;
import java.io.IOException;

import javax.swing.JComponent;
import javax.swing.JOptionPane;

import org.gvsig.fmap.drivers.gpe.reader.GPEDriverFactory;
import org.gvsig.fmap.drivers.gpe.reader.GPEVectorialDriver;
import org.gvsig.gpe.GPEParser;
import org.gvsig.gpe.GPERegister;
import org.gvsig.gpe.exceptions.GPEParserCreationException;
import org.gvsig.gpe.kml.writer.GPEKmlWriterHandlerImplementor;
import org.gvsig.gpe.writers.GPEWriterHandler;

import com.hardcode.gdbms.engine.values.Value;
import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.MapContext;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.crs.CRSFactory;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.layers.FBitSet;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLyrVect;
import com.iver.cit.gvsig.fmap.layers.LayerFactory;
import com.iver.cit.gvsig.fmap.layers.ReadableVectorial;
import com.iver.cit.gvsig.fmap.layers.SelectableDataSource;
import com.iver.cit.gvsig.fmap.layers.layerOperations.LayerCollection;
import com.iver.cit.gvsig.project.documents.view.gui.IView;
import com.iver.utiles.swing.threads.AbstractMonitorableTask;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id$
 * $Log$
 *
 */
/**
 * This class writes a gvSIG layer and its children
 * (if the driver supports a layer with children)
 * @author Jorge Piera LLodr (jorge.piera@iver.es)
 */
public class ExportTask extends AbstractMonitorableTask{
	private FLayer rootLayer = null;
	private GPEWriterHandler writer = null;
	private MapContext mapContext = null;
	private ExportGeometry eGeometry = null;	
	
	public ExportTask(FLayer layer, GPEWriterHandler writer, MapContext mapContext){
		this.rootLayer = layer;
		this.writer = writer;
		this.mapContext = mapContext;
		this.eGeometry = new ExportGeometry(writer);
		eGeometry.setProjOrig(mapContext.getProjection());
		eGeometry.setProjDest(layer.getProjection());
		if (writer.getDefaultFormat().equals("KML")){
			eGeometry.setProjDest(CRSFactory.getCRS("EPSG:4326"));			
		}
		setInitialStep(0);
		setDeterminatedProcess(true);
		setStatusMessage(PluginServices.getText(this, "gpe_exporting"));
	}

	public void run() throws Exception {
		writer.initialize();
		exportLayer(rootLayer);
		writer.close();
		if (isFileLoaded()){
			loadExportedFile();
		}
	}
	
	/**
	 * Load the exported file
	 * @throws GPEParserCreationException 
	 * @throws IOException 
	 */
	private void loadExportedFile() throws GPEParserCreationException, IOException{
		File file = writer.getOutputFile();
		GPEParser parser = GPERegister.createParser(file.toURI());
		GPEVectorialDriver driver = GPEDriverFactory.createDriver(parser);
		driver.open(file);				
		FLayer layer = LayerFactory.createLayer(file.getAbsolutePath(),
				driver, driver.getProjection());
		((IView)PluginServices.getMDIManager().getActiveWindow()).
		getMapControl().getMapContext().getLayers().addLayer(layer);
	}
	
	/**
	 * @return true if the exported files have to be loaded
	 */
	private boolean isFileLoaded(){
		int load = JOptionPane.showConfirmDialog(
				(JComponent) PluginServices.getMDIManager().getActiveWindow()
				, PluginServices.getText(this, "insertar_en_la_vista_la_capa_creada"),
				PluginServices.getText(this,"insertar_capa"),
				JOptionPane.YES_NO_OPTION);
		if (load == JOptionPane.YES_OPTION){
			return true;
		}
		return false;
	}

	/**
	 * It writes a layer and its children recursively
	 * @param layer
	 * The layer to write
	 */
	private void exportLayer(FLayer layer){
		String projection  = null;
		if (layer.getProjection() != null){
			projection = layer.getProjection().getAbrev();
		}
		writer.startLayer(null, layer.getName(), null, 
				projection, null);
		//Sets the extent
		Rectangle2D r2d;
		try {
			r2d = layer.getFullExtent();
			double[] x = new double[2];
			x[0] = r2d.getMinX();
			x[1] = r2d.getMaxX();
			double[] y = new double[2];
			y[0] = r2d.getMinY();
			y[1] = r2d.getMaxY();
			double[] z = new double[2];
			z[0] = r2d.getMinY();
			z[1] = r2d.getMaxY();
			writer.startBbox(null, x, y, z, mapContext.getProjection().getAbrev());
			writer.endBbox();
		} catch (Exception e) {
			writer.getErrorHandler().addWarning(new ExtentExportWarning(layer,e));
		} 
		//Export the layer information
		exportLayerInfo(layer);
		//If the layer has children...
		if (layer instanceof LayerCollection){
			LayerCollection layers = (LayerCollection)layer;
			for (int i=0 ; i<layers.getLayersCount() ; i++){
				exportLayer(layers.getLayer(i));
			}
		}
		writer.endLayer();
	}

	/**
	 * It exports the layer information. Geometries if is
	 * a vectorial layer, images if is a raster layer...
	 * @param layer
	 */
	private void exportLayerInfo(FLayer layer){
		try {
			if (layer instanceof FLyrVect){
				exportVectorialLayer((FLyrVect)layer);
			}
		} catch (Exception e) {
			writer.getErrorHandler().addError(e);
		}
	}	

	/**
	 * Export the geometries of a vectorial layer
	 * @param layer
	 * @throws DriverException 
	 * @throws DriverIOException 
	 * @throws ReadDriverException 
	 */
	private void exportVectorialLayer(FLyrVect layer) throws DriverException, DriverIOException {
		System.out.println(layer.getName());
		ReadableVectorial rv = layer.getSource();
		SelectableDataSource sds = layer.getRecordset();
		rv.start();
		//If there is a selection the rows to export have to be 
		//the selected rows
		FBitSet bitSet = sds.getSelection();
		int rowCount;		
		if (bitSet.cardinality() == 0){
			rowCount = rv.getShapeCount();
			for (int i = 0; i < rowCount; i++) {
				exportFeature(sds, rv, i, layer);
			}
		}else{
			rowCount = bitSet.cardinality();
			for (int i = bitSet.nextSetBit(0); i >= 0; i = bitSet
			.nextSetBit(i + 1)) {
				exportFeature(sds, rv, i, layer);
			}
		}	
		rv.stop();
	}

	/**
	 * It writes a feature (geometry + attributes)
	 * @param sds
	 * The selectable datasource to get the attributes
	 * @param rv
	 * The readable vectorial to get the geoemtries
	 * @param index
	 * The feature index
	 * @param layer
	 * Only to personalize the exceptions
	 * @throws ReadDriverException
	 */
	private void exportFeature(SelectableDataSource sds, ReadableVectorial rv, int index, FLayer layer){
		try {
			writer.startFeature(String.valueOf(index), "FEATURE", null);
			//Add the geoemtry
			IGeometry geom = rv.getShape(index);			
			eGeometry.writeGeometry(geom);
			//Add the attributes
			Value[] values = sds.getRow(index);
			for (int i=0 ; i<values.length ; i++){				
				writer.startElement(sds.getFieldName(i), 
						values[i].toString(),
						null);
				writer.endElement();
			}
			writer.endFeature();
		} catch (Exception e) {
			writer.getErrorHandler().addError(new FeatureExportException(layer,index,e));
		}		
	}

	/* (non-Javadoc)
	 * @see com.iver.utiles.swing.threads.IMonitorableTask#finished()
	 */
	public void finished() {		

	}
}
