/*
 * Created on 29-jun-2006
 *
 * gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
*
* $Id: UnitUtils.java 10552 2007-02-28 12:26:01Z azabala $
* $Log$
* Revision 1.3.2.5  2007-02-28 12:26:01  azabala
* solved bug when view has a geodetic projection
*
* Revision 1.3.2.4  2007/01/12 13:25:53  caballero
* unidades de mapa y medida
*
* Revision 1.3.2.3  2006/12/22 10:15:07  caballero
* unidades de mapa y de medida
*
* Revision 1.3.2.2  2006/12/21 18:21:52  azabala
* Bug fixed: change getMapUnits for getDistanceUnits
*
* Revision 1.3.2.1  2006/11/15 00:08:17  jjdelcerro
* *** empty log message ***
*
* Revision 1.4  2006/09/15 10:42:54  caballero
* extensibilidad de documentos
*
* Revision 1.3  2006/08/29 07:56:30  cesar
* Rename the *View* family of classes to *Window* (ie: SingletonView to SingletonWindow, ViewInfo to WindowInfo, etc)
*
* Revision 1.2  2006/08/29 07:21:09  cesar
* Rename com.iver.cit.gvsig.fmap.Fmap class to com.iver.cit.gvsig.fmap.MapContext
*
* Revision 1.1  2006/06/29 17:30:36  azabala
* first version in cvs
*
*
*/
package com.iver.cit.gvsig.geoprocess.core.util;

import org.cresques.cts.IProjection;
import org.cresques.cts.gt2.CSUTM;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.MapContext;
import com.iver.cit.gvsig.project.documents.view.gui.View;

/**
 * Utility methods to manage distances
 * (conversion from user entries to internal units,
 * verifying if projection is planar or we are
 * working with geographic coordinates, etc.)
 * @author azabala
 *
 */
public class UnitUtils {

	public static final double EARTH_RADIUS = 6378137d;

	/**
	 * For computing with geodetic coordinates:
	 * returns the angular measure (in radians)
	 * for a distance over the earth along a
	 * meridiam.
	 * Because this consideration is an approximation,
	 * we consideer the eart like an sphere (not an
	 * ellipsoid)
	 * @param dist distance in meters
	 * @return arc of meridian whose length is the specified
	 * distance
	 */
	public static double toSexaAngularMeasure(double dist){
		/*
		 *  dist = 6378km(terrestrial radius) -> 2PI
		 *  passed distance -> incognite
		 */
		return Math.toDegrees((2 * Math.PI * dist)/EARTH_RADIUS);
	}


	
	/**
	 * Converts a distance entered by user in the GUI in the same distance
	 * in internal units (measure units)
	 * 
	 * */
	public static double getInInternalUnits(double userEntryDistance){
		// VCN he modificado esto a como creo que debera de estar, 
		//as tiene en cuenta para calcular la distancia tanto las unidades
		//en las que se encuentra la cartografa como las unidades de medida 
		//seleccionadas por el usuario.
		IProjection proj =  ((View)PluginServices.
				getMDIManager().
				getActiveWindow()).
				getMapControl().
				getViewPort().
				getProjection();
		
		int distanceUnits = ((View)PluginServices.
				getMDIManager().
				getActiveWindow()).getMapControl().getViewPort().getDistanceUnits();
		
		boolean isProjected = proj.isProjected();
		int mapUnits = -1;
		if(isProjected){
			mapUnits = ((View)PluginServices.
				getMDIManager().
				getActiveWindow()).getMapControl().getViewPort().getMapUnits();
		}else{
			mapUnits = 1;
		}
		
		double distInInternalUnits = (userEntryDistance/MapContext.CHANGEM[mapUnits]) * 
								MapContext.CHANGEM[distanceUnits];
		
		/*
		 * if layer's projections is in geographics
		coords, pass distance to a angular measure
		*/
		
		if( (proj != null) && !(isProjected)){
			distInInternalUnits =
				UnitUtils.toSexaAngularMeasure(distInInternalUnits);
		}
		return distInInternalUnits;
	}






}

