/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.gvsig.georeferencing.gui.dialog;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.beans.PropertyVetoException;
import java.io.File;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JInternalFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JToggleButton;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.DefaultTableModel;

import org.cresques.cts.IProjection;
import org.cresques.px.Extent;
import org.gvsig.georeferencing.GeoOperations;
import org.gvsig.georeferencing.gui.listeners.GeoRasterFrameListener;
import org.gvsig.georeferencing.gui.listeners.GeorefMovePointListener;
import org.gvsig.georeferencing.gui.listeners.GeorefPanListener;
import org.gvsig.georeferencing.gui.listeners.GeorefPointSelectorListener;
import org.gvsig.georeferencing.gui.listeners.ZoomGeorefListener;
import org.gvsig.georeferencing.gui.panels.ConectorPanel;
import org.gvsig.georeferencing.gui.panels.OptionsPanel;
import org.gvsig.georeferencing.gui.panels.ZoomControlPanel;
import org.gvsig.georeferencing.gui.pointsTable.TablePointsPanel;
import org.gvsig.georeferencing.gui.pointsTable.TablePointsPanel.PointTable;
import org.gvsig.georeferencing.gui.selectPoints.SelectPointsPanel;
import org.gvsig.georeferencing.toc.GeoRasterTocMenuEntry;
import org.gvsig.georeferencing.utils.GeoLayerFactory;
import org.gvsig.georeferencing.utils.GeoPointPersistence;
import org.gvsig.georeferencing.utils.GeoUtils;
import org.gvsig.georeferencing.utils.PointManager;
import org.gvsig.georeferencing.utils.StackZoom;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.MapControl;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.drivers.RasterDriver;
import com.iver.cit.gvsig.fmap.layers.FLayers;
import com.iver.cit.gvsig.fmap.layers.FLyrGeoRaster;
import com.iver.cit.gvsig.fmap.layers.FLyrPoints;
import com.iver.cit.gvsig.fmap.layers.FLyrRaster;
import com.iver.cit.gvsig.fmap.layers.IGeoUi;
import com.iver.cit.gvsig.fmap.layers.RasterFileAdapter;
import com.iver.cit.gvsig.fmap.tools.ZoomOutRightButtonListener;
import com.iver.cit.gvsig.fmap.tools.Behavior.Behavior;
import com.iver.cit.gvsig.fmap.tools.Behavior.DraggedBehavior;
import com.iver.cit.gvsig.fmap.tools.Behavior.GeoMoveBehavior;
import com.iver.cit.gvsig.fmap.tools.Behavior.GeoRedimBehavior;
import com.iver.cit.gvsig.fmap.tools.Behavior.MouseMovementBehavior;
import com.iver.cit.gvsig.fmap.tools.Behavior.PointBehavior;
import com.iver.cit.gvsig.gui.wizards.WizardListener;
import com.iver.cit.gvsig.project.Project;
import com.iver.cit.gvsig.project.documents.view.gui.View;
import com.iver.cit.gvsig.project.documents.view.toolListeners.StatusBarListener;

/**
 * Panel que contiene el Wizard de georeferenciacin con los tabs
 * interiores
 *
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class GeoreferencingDialog extends JPanel implements IGeoUi, ActionListener, ComponentListener {

	//**********************Vars**********************************
	static private IProjection 			proj = Project.getDefaultProjection();
	private JPanel 						pBotones = null;  //  @jve:decl-index=0:
	private JButton 					bProcesar = null;  //  @jve:decl-index=0:
	private JButton 					bCancelar = null;  //  @jve:decl-index=0:
	private WizardListener 				wizardListener = new DialogWizardListener();
	private GeoRedimBehavior 			rb = null;
	private GeoMoveBehavior 			mb = null;
	private boolean 					loadTools = false;
	private JInternalFrame 				frame = null;
	/**
	 * Lista de formatos soportados
	 */
	private String[] 					fileFilters = {"csv"};
	/**
	 * Recuerda la ltima ruta seleccionada por el usuario
	 */
	private String 						lastPathLoad = "./";
	private String 						lastPathSave = "./";
	private ConectorPanel				conectorPanel = null;

	private boolean 					enlarge = false;
	private int 						normalWidth = 400;
	private int 						normalHeight = 273;
	private int 						enlargeHeight = 448;
	private int 						tmpWidth = normalWidth;

	private JButton 					bAceptar = null;

	private FLyrGeoRaster 				lyrGeoRaster = null;
	private TablePointsPanel 			tablePointsPanel = null;
	private	PointManager				pointManager = null;
	/**
	 * True si ha de crearse un fichero de georreferenciacin
	 */
	private boolean 					createWorldFile = true;
	/**
	 * True si los errores han de incluirse en el fichero csv
	 */
	private boolean 					errorCSV = true;
	/**
	 * Estado de los botones cuando se ejecuta disableAllControls
	 */
	private boolean[] 					buttonsState = new boolean[7];
	/**
	 * DisableAllControls ha sido ejecutada si est a true esta variabled
	 */
	private boolean 					disableAllControls = false;
	/**
	 * Variable que controla si la capa de georreferenciacin se ha de desactivar
	 * despus del primer punto.
	 */
	private boolean 					offLayer = true;
	/**
	 * Variable que controla si la vista ha de centrarse sobre el punto seleccionado
	 * o no
	 */
	private boolean 					centerPoint = true;
	/**
	 * Variable que controla si la capa de puntos se ha de desactivar
	 * despus del primer punto.
	 */
	private boolean 					offPoints = true;
	/**
	 * Panel que contiene los botones de la barra inferior
	 */
	private DialogButtons				dialogButtons = null;
	//**********************End Vars******************************

	//**********************Classes*******************************
	/**
	 * Filtro para seleccin de ficheros.
	 * @author Nacho Brodin (brodin_ign@gva.es)
	 */
	class SelectFileFilter extends javax.swing.filechooser.FileFilter {

		private JFileChooser chooser = null;
		private String file = null;

		public SelectFileFilter(JFileChooser ch, String file){
			this.chooser = ch;
			this.file = file;
		}

	    public boolean accept(File f) {

	    	return f.isDirectory() || f.getName().toLowerCase().endsWith("."+file);
	    }

	    public String getDescription() {
	    	return file;
	    }

	}

	/**
	 * @author Nacho Brodin (brodin_ign@gva.es)
	 */
	public class DialogWizardListener implements WizardListener {

		/**
		 * @see org.gvsig.georeferencing.gui.wms.WizardListener#error(java.lang.Exception)
		 */
		public void error(Exception e) {
		}

		/**
		 * @see org.gvsig.georeferencing.gui.wms.WizardListener#wizardStateChanged(boolean)
		 */
		public void wizardStateChanged(boolean finishable) {
			getBProcess().setEnabled(finishable);
		}

	}
	//**********************End Classes***************************

	//**********************Methods*******************************
    /**
     * Constructor.
     */
    public GeoreferencingDialog(JInternalFrame f, FLyrGeoRaster lyr) {
    	frame = f;
    	lyrGeoRaster = lyr;

    	frame.addComponentListener(this);
    	initialize();

    	//Creamos el pointManager y lo asignamos a la capa de puntos
    	pointManager = new PointManager(this, lyrGeoRaster.getFLyrPoints());
    	this.lyrGeoRaster.getFLyrPoints().setPointManager(pointManager);

    	//En estos casos entra cuando la capa se crea al abrir un proyecto
    	if(lyrGeoRaster.getPersistence() == null){
    		lyrGeoRaster.setPersistence(new GeoPointPersistence(lyrGeoRaster.getFLyrPoints()));
    		f.addInternalFrameListener(new GeoRasterFrameListener());
    		GeoRasterTocMenuEntry.isOpen = true;
    	}
    	if(lyrGeoRaster.getStackZoom() == null)
    		lyrGeoRaster.setZoom(new StackZoom());

    }

    /**
     * En la inicializacin de la ventana aadimos los tags de est y cargamos
     * ls herramientas para manejar las imagenes a georeferenciar.
     */
    private void initialize() {
    	//Cargamos las herramientas la primera vez que abrimos la ventana
        if(!loadTools){
        	com.iver.cit.gvsig.project.documents.view.gui.View  vista = null;
			try{
				vista = (com.iver.cit.gvsig.project.documents.view.gui.View) PluginServices.getMDIManager().getActiveWindow();
			}catch(ClassCastException exc){
				return;
			}

			MapControl mapCtrl = null;
			if(vista != null)
				mapCtrl = vista.getMapControl();

	        if(mapCtrl != null){
	        	loadTools = true;
				StatusBarListener sbl = new StatusBarListener(mapCtrl);

				ZoomOutRightButtonListener zoil = new ZoomOutRightButtonListener(mapCtrl);
				ZoomGeorefListener zigl = new ZoomGeorefListener(mapCtrl);
				rb = new GeoRedimBehavior(zigl);
				mapCtrl.addMapTool("geoZoom", new Behavior[]{rb,
		        				new PointBehavior(zoil), new MouseMovementBehavior(sbl)});

				GeorefPanListener pl = new GeorefPanListener(mapCtrl);
				mb = new GeoMoveBehavior(pl);
				mapCtrl.addMapTool("geoPan", new Behavior[]{mb, new MouseMovementBehavior(sbl)});

				//Seleccion de un punto sobre la vista
		        GeorefPointSelectorListener psl = new GeorefPointSelectorListener(this);
		        mapCtrl.addMapTool("pointLyrSelection", new Behavior[]{new PointBehavior(psl), new MouseMovementBehavior(sbl)});

		        GeorefMovePointListener mpl = new GeorefMovePointListener(this);
		        mapCtrl.addMapTool("geoMovePoint", new Behavior[]{new DraggedBehavior(mpl), new MouseMovementBehavior(sbl)});
	        }
        }

        this.setLayout(new BorderLayout());
        if(!enlarge){
        	this.setPreferredSize(new java.awt.Dimension(normalWidth, normalHeight));
        	this.setSize(new java.awt.Dimension(normalWidth, normalHeight));
        	this.getConectorPanel().getAdjustGeorefPanel().getZoomLeft().setVisible(false);
        	this.getConectorPanel().getAdjustGeorefPanel().getZoomRight().setVisible(false);
        }else{
        	this.setPreferredSize(new java.awt.Dimension(normalWidth, enlargeHeight));
        	this.setSize(new java.awt.Dimension(normalWidth, enlargeHeight));
        	this.getConectorPanel().getAdjustGeorefPanel().getZoomLeft().setVisible(true);
        	this.getConectorPanel().getAdjustGeorefPanel().getZoomRight().setVisible(true);
        }
        this.setLocation(new java.awt.Point(0,0));

        this.add(this.getPBotones(), BorderLayout.SOUTH);
        this.add(this.getConectorPanel(), BorderLayout.NORTH);

        //Aadimos los listener de los botones de la tabla
        //Los botones de borrado los gestiona SelectPointsPanel
        //Los botones de cargar y exportar los gestiona GeoreferencigDialog
        tablePointsPanel = getConectorPanel().getDataPointsTabPanel().getTablePointsPanel();
        tablePointsPanel.getTableControlerPanel().getBClear().addActionListener(this.getConectorPanel().getDataPointsTabPanel().getSelectPointsPanel());
        tablePointsPanel.getTableControlerPanel().getBDelPoint().addActionListener(this.getConectorPanel().getDataPointsTabPanel().getSelectPointsPanel());

        getDialogButtons().getBExportToCSV().addActionListener(this);
        getDialogButtons().getBImportFromCSV().addActionListener(this);
        getDialogButtons().getBLoad().addActionListener(this);
        getDialogButtons().getBSave().addActionListener(this);

        conectorPanel.getDataPointsTabPanel().getSelectPointsPanel().getDataPointPanel().getCbActive().addActionListener(this);
		getBInsertPointOfTablePoints().addActionListener(conectorPanel.getDataPointsTabPanel().getSelectPointsPanel());
		getBNewPointOfTablePoints().addActionListener(conectorPanel.getDataPointsTabPanel().getSelectPointsPanel());
    }

	/**
	 * Comprueba si existe el fichero pasado por parmetro existe solicitando confirmacin
	 * de sobreescritura. Si el usuario acepta devolver true y si no acepta devuelve false.
	 * Si se acepta la sobreescritura el fichero es borrado.
	 * @param file Cadena con el nombre del fichero a comprobar su existencia.
	 * @return Devuelve true si el fichero existe y va a ser sobreescrito
	 */
	public boolean checkFileExists(String file, boolean delete){
		File f = new File(file);
		if(f.exists()){
			if (messageBoxYesOrNot("file_exists")){
				if(delete)
					f.delete();
				return true;
			}else
				return false;
		}
		return true;
	}

	/**
	 * MessageBox con dos botones (OK y Cancel) que mostrar el mensaje pasado por
	 * parmetro
	 * @param msg Mensaje a mostrar
	 * @return true si el usuario pulsa Ok y false si pulsa Cancel
	 */
	public boolean messageBoxYesOrNot(String msg){
		String string1 = PluginServices.getText(this, "yes");
		String string2 = PluginServices.getText(this, "no");
		Object[] options = {string1, string2};
		int n = JOptionPane.showOptionDialog((Component)PluginServices.getMainFrame(),
					PluginServices.getText(this, msg),
					PluginServices.getText(this, "confirmacion"),
					JOptionPane.YES_NO_OPTION,
					JOptionPane.QUESTION_MESSAGE,
					null,
					options,
					string1);
		if (n == JOptionPane.YES_OPTION)
			return true;
		else
			return false;
	}

	/**
	 * Accin cuando se pulsa el botn de aceptar en el dialogo.
	 * <UL>
	 * <LI>Cambiamos el nombre a la capa georraster</LI>
	 * <LI>A la capa Georraster le asignamos la capa de puntos para poder recuperarla</LI>
	 * <LI>Cerramos la ventana</LI>
	 * </UL>
	 */
	public void actionPerformed(java.awt.event.ActionEvent e) {
		com.iver.cit.gvsig.project.documents.view.gui.View  theView = null;
		try{
			theView = (com.iver.cit.gvsig.project.documents.view.gui.View) PluginServices.getMDIManager().getActiveWindow();
		}catch(ClassCastException exc){
			return;
		}

		try{
			//ACEPTAR
			if(e.getSource() == bAceptar)
				accept(theView);

			//CANCELAR
			if(e.getSource() == bCancelar)
				cancel(theView);

			//CARGAR PUNTOS
			if(e.getSource() == getDialogButtons().getBLoad())
				loadPoints(theView);

			//SALVAR PUNTOS
			if(e.getSource() == getDialogButtons().getBSave())
				savePoints();

			//EXPORTAR A CSV
			if(e.getSource() == getDialogButtons().getBExportToCSV())
				exportToCSV(theView);

			//IMPORTAR DESDE CSV
			if(e.getSource() == getDialogButtons().getBImportFromCSV())
				importFromCSV(theView);

			//DESACTIVAR PANELES
			JCheckBox cb = this.conectorPanel.getDataPointsTabPanel().getSelectPointsPanel().getDataPointPanel().getCbActive();
			if(e.getSource() == cb)
				deactivatePanel(theView, cb);

			//PROCESAR
			if(e.getSource() == this.getBProcess())
				process(theView);
		}catch(Exception exc){
			exc.printStackTrace();
			return;
		}
	}

	/**
	 * Cierra el dialogo
	 */
	public void close(){
		try{
			frame.setClosed(true);
		}catch(PropertyVetoException exc){}
	}

	/**
	 * Funcin que se ejecuta al pulsar el botn procesar. Este procesamiento calcula a partir de
	 * los puntos de control la georreferenciacin de la imagen y la asigna como extent temporal de
	 * la capa FLyrGeoRaster. Cuando termina actualiza la visualizacin.
	 * @param theView
	 */
	private void process(View theView)throws Exception{
		if(getLyrPoints() == null)
			throw new Exception("LyrPoints not loaded.");

		if(this.getLyrPoints().getCountPoints() > 0){

			//Salvamos los centros
			Point2D[] oldCenter = new Point2D.Double[lyrGeoRaster.getFLyrPoints().getCountPoints()];
			for(int i=0; i<lyrGeoRaster.getFLyrPoints().getCountPoints(); i++)
				oldCenter[i] = lyrGeoRaster.img2World(lyrGeoRaster.getFLyrPoints().getPoint(i).pixelPoint);

			//Calculamos las coordenadas de georreferenciacin
			GeoOperations go = new GeoOperations(getLyrPoints());
			
			double[] begin = go.transformPoint(0, 0, go.getAffine());
			double[] end = go.transformPoint((int)lyrGeoRaster.getImageWidth(), (int)lyrGeoRaster.getImageHeight(), go.getAffine());
			Extent ext = new Extent(begin[0], begin[1], end[0], end[1]);

			//Asignamos el extent temporal a la capa de georreferenciacin
			ViewPort vp = theView.getMapControl().getMapContext().getViewPort();
			((FLyrGeoRaster)theView.getMapControl().getMapContext().getLayers().getLayer(lyrGeoRaster.getName())).setAssignExtent(ext);

			//Si la capa tiene puntos de control hacemos un update de estos
			//a su posicin para que se actualicen los controles

			for(int i=0; i<lyrGeoRaster.getFLyrPoints().getCountPoints(); i++){
				Point2D center = lyrGeoRaster.img2World(lyrGeoRaster.getFLyrPoints().getPoint(i).pixelPoint);
				lyrGeoRaster.getFLyrPoints().setMiniExtent(	i,
															center,
															GeoUtils.shiftExtent(oldCenter[i], center, getLyrPoints().getMiniExtent(i, false)),
															false);
				if(i == getSelectedPoint())
					updateData(i + 1, lyrGeoRaster.getFLyrPoints().getPoint(i).pixelPoint, null, null);
			}
			theView.getMapControl().getMapContext().invalidate();
		}
	}

	/**
	 * Funcin que se ejecuta al pulsar el botn aceptar. Este procesamiento calcula a partir de
	 * los puntos de control la georreferenciacin de la imagen, crear el fichero .rmf de georreferenciacin
	 * y descarga la capa del TOC. A continuacin vuelve a cargar la capa como FLyrRaster dando
	 * por finalizada la georreferenciacin.
	 * @param theView
	 */
	private void accept(View theView)throws Exception{
		if(getLyrPoints() == null)
			throw new Exception("LyrPoints not loaded.");

		if(this.getLyrPoints().getCountPoints() > 0){
			savePoints();

			//Creamos el fichero de georreferenciacin
			GeoOperations go = new GeoOperations(getLyrPoints());
			go.setCreateWorldFile(this.createWorldFile);
			go.createGeorefFile( (int)getLyrGeoRaster().getImageWidth(),
								(int)getLyrGeoRaster().getImageHeight(),
								getLyrGeoRaster().getSource().getFiles()[0].getName());

			FLayers lyrs = this.getLyrGeoRaster().getParentLayer();

			for(int i = 0; i < lyrs.getLayersCount();i++){
				if(this.getLyrGeoRaster().getName().equals(lyrs.getLayer(i).getName())){

					//Eliminamos la capa
					lyrs.removeLayer(i);
					File fich = new File(getLyrGeoRaster().getSource().getFiles()[0].getName());
					try{
						//Creamos la nueva capa GeoRaster y la aadimos al TOC.
						FLyrRaster lyrRaster = GeoLayerFactory.createLayer(getLyrGeoRaster().getName().substring(1, getLyrGeoRaster().getName().length()),
																			(RasterDriver)getLyrGeoRaster().getSource().getDriver(),
																			fich,
																			getLyrGeoRaster().getProjection());
						lyrs.addLayer(lyrRaster);
						theView.getMapControl().getMapContext().invalidate();
					}catch(DriverException exc){

					}
					break;
				}
			}
		}
		//Cerramos la ventana
		try{
			frame.setClosed(true);
		}catch(PropertyVetoException exc){}

	}

	/**
	 * Funcin que se ejecuta al pulsar el botn cancelar
	 * @param theView
	 */
	private void cancel(View theView)throws Exception{
		if(getLyrPoints() == null)
			throw new Exception("LyrPoints not loaded.");

		//Cerramos la ventana
		//(se ejecuta el evento internalFrameClosing de GeoRasterFrameListener)
		try{
			frame.setClosed(true);
			theView.getMapControl().getMapContext().invalidate();
		}catch(PropertyVetoException exc){}
	}

	/**
	 * Funcin que se ejecuta al pulsar el botn de cargar puntos
	 * @param theView
	 */
	private void loadPoints(View theView)throws Exception{
		if(getLyrPoints() == null)
			throw new Exception("LyrPoints not loaded.");

		if(messageBoxYesOrNot("cargar_puntos")){
			String name = lyrGeoRaster.getName().substring(lyrGeoRaster.getName().indexOf("*") + 1, lyrGeoRaster.getName().lastIndexOf("."));
			String path = ((RasterFileAdapter)lyrGeoRaster.getSource()).getFile().getAbsolutePath();
			String nameRmf = path.substring(0, path.lastIndexOf(File.separator) + 1) + name + ".rmf";

			//Antes de cargar puntos plegamos el cuadro
		 	getSelectPointsPanel().setEnlarge(true);

			getLyrPoints().saveState();
			getLyrPoints().XML2PointList(nameRmf);

			//Esto es por si el panel ha sido estirado cuando cargamos los puntos
			frame.pack();
			frame.show();

			//Por problemas con repaint e hilos concurrentes tenemos que poner esto explicitamente
			//ya que es posible que se haga un paint de las miniimagenes antes de que cleanCanvas vuelva a valer false
			this.getZoomControlLeft().getCanvas().setCleanCanvas(false);
			this.getZoomControlRight().getCanvas().setCleanCanvas(false);

			if(getLyrPoints().getCountPoints() > 0)
				setEnableSaveButtons(true);
		}
	}

	/**
	 * Funcin usada para salvar puntos
	 * @param theView
	 */
	public void savePoints()throws Exception{
		if(getLyrPoints() == null)
			throw new Exception("LyrPoints not loaded.");
		String name = lyrGeoRaster.getName().substring(lyrGeoRaster.getName().indexOf("*") + 1, lyrGeoRaster.getName().lastIndexOf("."));
		String path = ((RasterFileAdapter)lyrGeoRaster.getSource()).getFile().getAbsolutePath();
		String nameRmf = path.substring(0, path.lastIndexOf(File.separator) + 1) + name + ".rmf";

		File f = new File(nameRmf);
		if(f.exists()){
			if (messageBoxYesOrNot(PluginServices.getText(this, "gcps_exist"))){
				if(getLyrPoints() != null)
			 		getLyrPoints().PointList2XML( path.substring(0, path.lastIndexOf(File.separator) + 1) + name + ".grf");
			}
		}else
			getLyrPoints().PointList2XML( path.substring(0, path.lastIndexOf(File.separator) + 1) + name + ".grf");
	}

	/**
	 * Funcin que se ejecuta al pulsar el botn de export a ascii
	 * @param theView
	 */
	private void exportToCSV(View theView)throws Exception{
		if(getLyrPoints() == null)
			throw new Exception("LyrPoints not loaded.");

		JFileChooser chooser = new JFileChooser(lastPathSave);
		chooser.setDialogTitle(PluginServices.getText(this, "seleccionar_fichero"));

		FileFilter f = null;
		for(int i=0; i<this.fileFilters.length;i++){
			f = new SelectFileFilter(chooser, this.fileFilters[i]);
			chooser.addChoosableFileFilter(f);
		}
		int returnVal = chooser.showOpenDialog(this);
		if(returnVal == JFileChooser.APPROVE_OPTION){
		 	String fName = chooser.getSelectedFile().toString();
		 	if(!fName.endsWith(".csv"))
		 		fName = fName + ".csv";
		 	lastPathSave = chooser.getCurrentDirectory().getAbsolutePath();
		 	if(checkFileExists(fName, true)){
		 		if(getLyrPoints() != null)
		 			getLyrPoints().PointList2CSV(fName);
		 	}
		}
	}

	/**
	 * Funcin que se ejecuta al pulsar el botn de importar desde CSV
	 * @param theView
	 */
	private void importFromCSV(View theView)throws Exception{
		if(getLyrPoints() == null)
			throw new Exception("LyrPoints not loaded.");

		JFileChooser chooser = new JFileChooser(lastPathSave);
		chooser.setDialogTitle(PluginServices.getText(this, "seleccionar_fichero"));

		FileFilter f = null;
		for(int i=0; i<this.fileFilters.length;i++){
			f = new SelectFileFilter(chooser, this.fileFilters[i]);
			chooser.addChoosableFileFilter(f);
		}
		int returnVal = chooser.showOpenDialog(this);
		if(returnVal == JFileChooser.APPROVE_OPTION){
		 	String fName = chooser.getSelectedFile().toString();
		 	lastPathSave = chooser.getCurrentDirectory().getAbsolutePath();
		 	getLyrPoints().PointListFromCSV(fName);

			//Antes de cargar puntos plegamos el cuadro
		 	getSelectPointsPanel().setEnlarge(true);

			//Por problemas con repaint e hilos concurrentes tenemos que poner esto explicitamente
			//ya que es posible que se haga un paint de las miniimagenes antes de que cleanCanvas vuelva a valer false
			this.getZoomControlLeft().getCanvas().setCleanCanvas(false);
			this.getZoomControlRight().getCanvas().setCleanCanvas(false);

		 	//Si se han cargado puntos desde el CSV se activan los controles
			if(getLyrPoints().getCountPoints() > 0)
				setEnableSaveButtons(true);
		}
	}

	/**
	 * Funcin que se ejecuta al pulsar el botn desactivar panel
	 * @param theView
	 */
	public void deactivatePanel(View theView, JCheckBox cb)throws Exception{
		if(getLyrPoints() == null)
			throw new Exception("LyrPoints not loaded.");

		//Sincronizamos con el checkBox de la tabla
		tablePointsPanel.getJTable().getTableModel().setValueAt(new Boolean(cb.isSelected()),
																			tablePointsPanel.getJTable().getTable().getSelectedRow(),
																			0);

		if(cb.isSelected()){
			this.setEnabled(true);
			getLyrPoints().setPointActive(getSelectedPoint(), true);
		}else{
			this.setEnabled(false);
			getLyrPoints().setPointActive(getSelectedPoint(), false);
		}

		//Al activar y desactivar tenemos que recalcular errores
		this.getPointManager().setErrors(getSelectedPoint());

		theView.getMapControl().getMapContext().invalidate();
		getZoomControlLeft().draw();
		getZoomControlRight().draw();
	}

	/**
	 * Recalcula el extent de la mini-imagen que corresponde a los puntos en coordenadas
	 * pixel.Para esto calcula el nuevo centro a partir de la capa y con la distancia entre
	 * el nuevo centro y el viejo desplaza el extent completo de la mini-imagen.
	 * @param theView
	 */
	private void calcNewMiniExtent(View theView){

		ViewPort viewPort = theView.getMapControl().getMapContext().getViewPort();
		for(int j = 0; j < getLyrPoints().getCountPoints(); j++){
			//Calculamos el nuevo centro
			Point2D oldCenter = getLyrPoints().getPoint(j).leftCenterPoint;
			Point2D newCenter = new Point2D.Double();
			newCenter.setLocation(	lyrGeoRaster.img2World(getLyrPoints().getPoint(j).pixelPoint));

			double diffX = oldCenter.getX() - newCenter.getX();
			double diffY = oldCenter.getY() - newCenter.getY();

			getLyrPoints().getPoint(j).leftCenterPoint = newCenter;

			ViewPort vpOld = getLyrPoints().getPoint(j).leftViewPort;
			Rectangle2D newExtent = new Rectangle2D.Double(	vpOld.getExtent().getMinX() - diffX,
															vpOld.getExtent().getMinY() - diffY,
															vpOld.getExtent().getWidth(),
															vpOld.getExtent().getHeight() );
			ViewPort vpNew = new ViewPort(vpOld.getProjection());
			vpNew.setExtent(newExtent);
			vpNew.setImageSize(vpOld.getImageSize());
			vpNew.refreshExtent();
			getLyrPoints().getPoint(j).leftViewPort = vpNew;
		}
	}

	/**
	 * This method initializes jButton
	 *
	 * @return javax.swing.JButton
	 */
	public JButton getBAccept() {
		if (bAceptar == null) {
			bAceptar = new JButton();
			bAceptar.setText(PluginServices.getText(this,"aceptar"));
			bAceptar.setToolTipText(PluginServices.getText(this,"accept_tip"));
			bAceptar.addActionListener(this);
		}
		return bAceptar;
	}

	/**
	 * This method initializes bProcesar
	 *
	 * @return javax.swing.JButton
	 */
	public JButton getBProcess() {
		if (bProcesar == null) {
			bProcesar = new JButton();
			bProcesar.setText(PluginServices.getText(this,"procesar"));
			bProcesar.setToolTipText(PluginServices.getText(this,"process_tip"));
			bProcesar.setEnabled(true);
			bProcesar.addActionListener(this);
			
			//TODO: Descomentar cuando se pueda procesar al vuelo la imagen
			bProcesar.setVisible(false);
		}
		return bProcesar;
	}

	/**
	 * El botn cancelar restaura el extent con el que se carg la imagen a georreferenciar
	 * y cierra la ventana.
	 *
	 * @return javax.swing.JButton
	 */
	public JButton getBCancel() {
		if (bCancelar == null) {
			bCancelar = new JButton();
			bCancelar.setText(PluginServices.getText(this,"cancelar"));
			bCancelar.setToolTipText(PluginServices.getText(this,"cancel_tip"));
			bCancelar.addActionListener(this);
		}
		return bCancelar;
	}

	/**
	 * Asigna a las textbox el valor de pixel pasado
	 * @param x	valor x
	 * @param y	valor y
	 */
	public void setPixelText(double x, double y){
		getSelectPointsPanel().getDataPointPanel().getTX().setText(String.valueOf(x));
		getSelectPointsPanel().getDataPointPanel().getTY().setText(String.valueOf(y));
	}

	/**
	 * Asigna a las textbox el valor de coordenadas pasado
	 * @param x	valor x
	 * @param y	valor y
	 */
	public void setMapCoordText(double x, double y){
		getSelectPointsPanel().getDataPointPanel().getLongitud().setText(GeoUtils.tailDecimals(String.valueOf(x), 5));
		getSelectPointsPanel().getDataPointPanel().getLatitud().setText(GeoUtils.tailDecimals(String.valueOf(y), 5));
	}

	/**
	 * Esta funcin resetea los controles del panel de info de un punto.
	 */
	public void resetControls(boolean active){
		getConectorPanel().getDataPointsTabPanel().getTablePointsPanel().setSaveToDisk(active);
		getConectorPanel().getDataPointsTabPanel().getSelectPointsPanel().resetControls(active);
		getConectorPanel().getAdjustGeorefPanel().getZoomLeft().clear();
		getConectorPanel().getAdjustGeorefPanel().getZoomRight().clear();
		setEnabled(active);
	}

	/* (non-Javadoc)
	 * @see java.awt.event.ComponentListener#componentHidden(java.awt.event.ComponentEvent)
	 */
	public void componentHidden(ComponentEvent arg0) {

	}
	/* (non-Javadoc)
	 * @see java.awt.event.ComponentListener#componentMoved(java.awt.event.ComponentEvent)
	 */
	public void componentMoved(ComponentEvent arg0) {

	}
	/* (non-Javadoc)
	 * @see java.awt.event.ComponentListener#componentResized(java.awt.event.ComponentEvent)
	 */
	public void componentResized(ComponentEvent ev) {

		if(frame.getWidth() <= this.normalWidth + 10)
			frame.setSize(this.normalWidth, frame.getHeight());

		if(this.getConectorPanel().getDataPointsTabPanel().getTbPoints().getSelectedIndex() == 1){
			tmpWidth = frame.getWidth();
			this.newFrameSize(frame.getWidth(), frame.getHeight() - 10);
		}
	}
	/* (non-Javadoc)
	 * @see java.awt.event.ComponentListener#componentShown(java.awt.event.ComponentEvent)
	 */
	public void componentShown(ComponentEvent arg0) {

	}

    /**
     *Inicializa el tamao del dialogo
     */
    public void resetSize(){
    	if(!enlarge){
    		frame.setSize(this.normalWidth + 12, this.normalHeight + 10);
    	}else{
    		frame.setSize(this.normalWidth + 12, this.enlargeHeight + 10);
    	}
    	newFrameSize((int)Math.round(this.normalWidth / 0.98), this.normalHeight);
    }

	/**
	 * Calculo del nuevo tamao a partir de un frame redimensionado
	 * @param w Ancho del frame
	 * @param h Alto del frame
	 */
	public void newFrameSize(int w, int h){
		int newWidth = (int)Math.round(w * 0.98);

        this.setSize(new java.awt.Dimension(newWidth, normalHeight ));
        this.setPreferredSize(new java.awt.Dimension(newWidth, normalHeight ));
        this.getConectorPanel().newFrameSize(newWidth, h);
	}

	/**
	 * Elimina un punto de la tabla de puntos
	 * @param pos Posicin del punto a eliminar
	 */
	public void removeTableValueAt(int pos) throws ArrayIndexOutOfBoundsException{
		((DefaultTableModel)getTable().getTable().getModel()).removeRow(pos);
	}

	/**
	 * A partir de nuevas coordenadas actualiza la vista, minimagen, capa de puntos el
	 * dialogo y la tabla.
	 *
	 */
	public void updateData(int nPunto, Point2D pixel, Point2D map, View view){
		getPointManager().updateData(nPunto, pixel, map, this, view);
	}

	/**
	 * Esta funcin deshabilita todos los controles y guarda sus valores
	 * de habilitado o deshabilitado para que cuando se ejecute restoreControlsValue
	 * se vuelvan a quedar como estaba
	 */
	public void disableAllControls(){
		if(!disableAllControls){
			disableAllControls = true;

			//Salvamos los estados
			buttonsState[0] = getBProcess().isEnabled();
			buttonsState[1] = getBAccept().isEnabled();
			buttonsState[2] = getBCancel().isEnabled();
			buttonsState[3] = getDialogButtons().getBExportToCSV().isEnabled();
			buttonsState[4] = getDialogButtons().getBImportFromCSV().isEnabled();
			buttonsState[5] = getDialogButtons().getBLoad().isEnabled();
			buttonsState[6] = getDialogButtons().getBSave().isEnabled();

			//Desactivamos controles
			getBProcess().setEnabled(false);
			getBAccept().setEnabled(false);
			getBCancel().setEnabled(false);

			getDialogButtons().getBExportToCSV().setEnabled(false);
			getDialogButtons().getBImportFromCSV().setEnabled(false);
			getDialogButtons().getBLoad().setEnabled(false);
			getDialogButtons().getBSave().setEnabled(false);
		}
	}

	/**
	 * Esta funcin deja los controles como estaban al ejecutar la funcin
	 * disableAllControls
	 */
	public void restoreControlsValue(){
		if(disableAllControls){
			disableAllControls = false;
			getBProcess().setEnabled(buttonsState[0]);
			getBAccept().setEnabled(buttonsState[1]);
			getBCancel().setEnabled(buttonsState[2]);
			getDialogButtons().getBExportToCSV().setEnabled(buttonsState[3]);
			getDialogButtons().getBImportFromCSV().setEnabled(buttonsState[4]);
			getDialogButtons().getBLoad().setEnabled(buttonsState[5]);
			getDialogButtons().getBSave().setEnabled(buttonsState[6]);
		}
	}
	//**********************End Methods***************************

	//**********************Setters & Getters*********************
	/**
	 * Obtiene la tabla de puntos
	 */
	public PointTable getTable(){
		return this.getConectorPanel().getDataPointsTabPanel().getTablePointsPanel().getJTable();
	}

	/**
	 * Asigna un valor a una posicin de la tabla
	 * @param value Valor a asignar
	 * @param row	Fila en la que se asigna
	 * @param col	Columna en la que se asigna
	 */
	public void setTableValueAt(String value, int row, int col)throws ArrayIndexOutOfBoundsException{
		((DefaultTableModel)getTable().getTable().getModel()).setValueAt(value, row, col);
	}

	/**
	 * Asigna un valor del checkBox para una fila de la tabla
	 * @param row Fila en la que se asigna
	 */
	public void setCheckValueAt(Boolean value, int row)throws ArrayIndexOutOfBoundsException{
		((DefaultTableModel)getTable().getTable().getModel()).setValueAt(value, row, 0);
	}

	/**
	 * Obtiene el nmero de filas de la tabla
	 * @return Nmero de filas de la tabla
	 */
	public int getTableRowCount(){
		return getTable().getTable().getRowCount();
	}

	/**
	 * Aade una nueva fila a la tabla de puntos
	 */
	public void addTableNew(){
		getTable().getTableModel().addNew();
	}
	/**
	 * @return Returns the tmpWidth.
	 */
	public int getTmpWidth() {
		return tmpWidth;
	}
	/**
	 * @param tmpWidth The tmpWidth to set.
	 */
	public void setTmpWidth(int tmpWidth) {
		this.tmpWidth = tmpWidth;
	}
	/**
	 * @return Returns the enlargeHeight.
	 */
	public int getEnlargeHeight() {
		return enlargeHeight;
	}
	/**
	 * @return Returns the normalHeight.
	 */
	public int getNormalHeight() {
		return normalHeight;
	}

	/**
	 * @return Returns the normalWidth.
	 */
	public int getNormalWidth() {
		return normalWidth;
	}

	/**
	 * Asigna el valor del campo RMS
	 * @param rms Cadena que representa el RMS
	 */
	public void setRMS(String rms){
		this.getConectorPanel().getDataPointsTabPanel().getSelectPointsPanel().getErrorPointPanel().getTRMS().setText(rms);
	}

	/**
	 * Asigna el valor del campo de residuo en X
	 * @param rms Cadena que representa el RMS
	 */
	public void setResX(String resX){
		this.getConectorPanel().getDataPointsTabPanel().getSelectPointsPanel().getErrorPointPanel().getTResX().setText(resX);
	}

	/**
	 * Asigna el valor del campo de residuo en Y
	 * @param rms Cadena que representa el RMS
	 */
	public void setResY(String resY){
		this.getConectorPanel().getDataPointsTabPanel().getSelectPointsPanel().getErrorPointPanel().getTResY().setText(resY);
	}

	/**
	 * Asigna el valor del campo RMS
	 * @param rms Cadena que representa el RMS
	 */
	public void setTotal(String total){
		this.getConectorPanel().getDataPointsTabPanel().getSelectPointsPanel().getErrorPointPanel().getTTotal().setText(total);
	}

	/**
	 * @return Returns the lyrGeoRaster.
	 */
	public FLyrGeoRaster getLyrGeoRaster() {
		return lyrGeoRaster;
	}

	/**
	 * @return Returns the lyrPoints.
	 */
	public FLyrPoints getLyrPoints() {
		try{
			return lyrGeoRaster.getFLyrPoints();
		}catch(ClassCastException exc){
			return null;
		}
	}

	/**
	 * Selecciona un tab del cuadro
	 * @param index
	 */
	public void setSelectedTab(int index){
		try{
			this.getConectorPanel().getDataPointsTabPanel().getTbPoints().setSelectedIndex(index);
		}catch(IllegalArgumentException ex){
			//Si peta al seleccionar un tab, pues nada.
		}
	}

	/**
	 * @param extended The extended to set.
	 */
	public void setEnlarge(boolean enlarge) {
		this.enlarge = enlarge;
		if(!enlarge){
			frame.setSize(new Dimension(this.normalWidth + 12, this.normalHeight + 10));
			frame.setPreferredSize(new Dimension(this.normalWidth + 12, this.normalHeight + 10));
			this.setPreferredSize(new Dimension(this.normalWidth, this.normalHeight));
			this.setSize(new Dimension(this.normalWidth, this.normalHeight));
			this.getConectorPanel().setCanvasVisible(false);
		}else{
			frame.setSize(new Dimension(this.normalWidth + 12, this.enlargeHeight + 10));
			frame.setPreferredSize(new Dimension(this.normalWidth + 12, this.enlargeHeight + 10));
			this.setPreferredSize(new Dimension(this.normalWidth, this.enlargeHeight));
			this.setSize(new Dimension(this.normalWidth, this.enlargeHeight));
			this.getConectorPanel().setCanvasVisible(true);
		}
		frame.pack();
	}

	/**
	 * @return Returns the extended.
	 */
	public boolean isEnlarge() {
		return enlarge;
	}

	/**
     * Obtiene el panel conector
     * @return
     */
    public ConectorPanel getConectorPanel() {
	    if (conectorPanel == null) {
	    	conectorPanel = new ConectorPanel(this);
	    }

	    return conectorPanel;
    }

	/**
	 * This method initializes jPanel
	 *
	 * @return javax.swing.JPanel
	 */
	private JPanel getPBotones() {
		if (pBotones == null) {
			FlowLayout flowLayout1 = new FlowLayout();
			pBotones = new JPanel();
			pBotones.setLayout(flowLayout1);
			flowLayout1.setHgap(2);
			flowLayout1.setAlignment(java.awt.FlowLayout.RIGHT);
			pBotones.add(getDialogButtons(), null);
			pBotones.add(getBProcess(), null);
			pBotones.add(getBAccept(), null);
			pBotones.add(getBCancel(), null);
		}
		return pBotones;
	}

	public static IProjection getLastProjection() {
		return proj;
	}
	public static void setLastProjection(IProjection proj) {
		GeoreferencingDialog.proj = proj;
	}

	/**
	 * @return Returns the GeoRedimBehavior.
	 */
	public GeoRedimBehavior getGeoRedimBehavior() {
		return rb;
	}

	/**
	 * @param rb The GeoRedimBehavior to set.
	 */
	public void setGeoRedimBehavior(GeoRedimBehavior rb) {
		this.rb = rb;
	}

	/**
	 * @return Returns the GeoMoveBehavior
	 */
	public GeoMoveBehavior getGeoMoveBehavior() {
		return mb;
	}

	/**
	 * @param mb The GeoMoveBehavior to set.
	 */
	public void setGeoMoveBehavior(GeoMoveBehavior mb) {
		this.mb = mb;
	}


	/**
	 * Obtiene el punto de la lista que ha sido seleccionado
	 * @return
	 */
	public int getSelectedPoint(){
		return getConectorPanel().getDataPointsTabPanel().
			getSelectPointsPanel().getTableControlerPanel().getSelectedIndex();
	}

	/**
	 * Obtiene el control de la mini imagen que tiene los botones a la izquierda
	 * @return ZoomControlPanel
	 */
	public ZoomControlPanel getZoomControlLeft(){
		return this.getConectorPanel().getAdjustGeorefPanel().getZoomLeft();
	}

	/**
	 * Obtiene el control de la mini imagen que tiene los botones a la derecha
	 * @return ZoomControlPanel
	 */
	public ZoomControlPanel getZoomControlRight(){
		return this.getConectorPanel().getAdjustGeorefPanel().getZoomRight();
	}

	/**
	 * Obtiene el panel de seleccin de puntos
	 * @return Panel de seleccin de puntos
	 */
	public SelectPointsPanel getSelectPointsPanel(){
		return this.getConectorPanel().getDataPointsTabPanel().getSelectPointsPanel();
	}

	/**
	 * Obtiene el panel de opciones
	 * @return OptionPanel
	 */
	public OptionsPanel getOptionsPanel(){
		return getConectorPanel().getAdjustGeorefPanel().getOptionsPanel();
	}

    /**
     * Asigna el frame
     * @param f
     */
    public void setFrame(JInternalFrame f){
    	frame = f;
    	frame.addComponentListener(this);
    }

    /**
     * Obtiene el frame
     * @return frame
     */
    public JInternalFrame getFrame(){
    	return frame;
    }

    /**
     * Obtiene el gestor de puntos
     * @return PointManager
     */
    public PointManager getPointManager() {
		return pointManager;
	}

    /**
     * Activa o desactiva este panel y todos los que lo componen
     * @param enabled variable booleana para la activacin y/o desactivacin
     */
    public void setEnabled(boolean enabled){
    	conectorPanel.getDataPointsTabPanel().getSelectPointsPanel().setEnabled(enabled);
    	conectorPanel.getAdjustGeorefPanel().setEnabled(enabled);
    }

    /**
	 * Mtodo para notificar si se desea crear o no el fichero de coordenadas .tfw, .wld .jpgw ,...
	 * @param createWorldFile
	 */
    public void setCreateWorldFile(boolean createWorldFile) {
		this.createWorldFile = createWorldFile;
	}

    /**
     *
     */
    public ViewPort getLeftInitViewport(ViewPort vp, Point2D center, ViewPort initVp, double zoom){
    	return getZoomControlLeft().getCanvas().initViewPort(vp, center, initVp, zoom);
    }

    /**
	 * Consulta si se desea incluir los errores cuando se exporta a fichero .csv
	 * @param errorCVS true si se incluyen los errores dentro del fichero y false si no se incluyen
	 */
    public boolean isErrorCSV() {
		return errorCSV;
	}

    /**
	 * Mtodo para notificar si se desea incluir los errores cuando se exporta a fichero .csv
	 * @param errorCVS true si se incluyen los errores dentro del fichero y false si no se incluyen
	 */
	public void setErrorCSV(boolean errorCSV) {
		this.errorCSV = errorCSV;
	}

	/**
	 * Obtiene los errores en una matriz de Nx3 en la que cada fila es un punto y la
	 * columna 0 es el error en X, la columna 1 es el error en Y y la columna 2 es el error RMS.
	 * @return matriz de errores
	 */
	public double[][] getErrors(){
		return pointManager.getErrors();
	}

	/**
	 * Pregunta si la capa a georreferenciar se desactiva cuando se introduce el
	 * primer punto y devuelve true si lo hace.
	 * @return true o false
	 */
	public boolean isOffLayer() {
		return offLayer;
	}

	/**
	 * Asigna el flag que dice si la capa a georreferenciar se desactiva cuando se introduce el
	 * primer punto
	 * @param offLayer true para desactivar la capa
	 */
	public void setOffLayer(boolean offLayer) {
		this.offLayer = offLayer;
	}

	/**
	 * Consulta si la vista ha de centrarse sobre el punto seleccionado o no.
	 * @return true o false
	 */
	public boolean isCenterPoint() {
		return centerPoint;
	}

	/**
	 * Asigna el flag que dice si vista ha centrarse sobre el punto seleccionado o no
	 * @param centerPoint true para centrar la vista y false no.
	 */
	public void setCenterPoint(boolean centerPoint) {
		this.centerPoint = centerPoint;
	}

	/**
	 * Pregunta si la capa de puntos se desactiva cuando se introduce el
	 * primer punto y devuelve true si lo hace.
	 * @return true o false
	 */
	public boolean isOffPoints() {
		return offPoints;
	}

	/**
	 * Asigna el flag que dice si la capa de puntos se desactiva cuando se introduce el
	 * primer punto
	 * @param offPoints true para desactivar la capa
	 */
	public void setOffPoints(boolean offPoints) {
		this.offPoints = offPoints;
	}

	/**
	 * Obtiene el botn de activar/desactivar la insercin de punto del panel del control de
	 * tabla
	 * @return JToggleButton
	 */
	public JToggleButton getBInsertPointOfTablePoints(){
		return tablePointsPanel.getPTablePanelButtons().getBInsertPoint();
	}

	/**
	 * Obtiene el botn de activar/desactivar la insercin de punto del panel del control de
	 * tabla
	 * @return JToggleButton
	 */
	public JButton getBNewPointOfTablePoints(){
		return tablePointsPanel.getTableControlerPanel().getBNew();
	}

	/**
	 * This method initializes jPanel
	 *
	 * @return javax.swing.JPanel
	 */
	public DialogButtons getDialogButtons() {
		if (dialogButtons == null) {
			dialogButtons = new DialogButtons();
		}
		return dialogButtons;
	}

	/**
	 * Activa/desactiva los botones de la barra correspondientes al salvado
	 * de datos a disco.
	 * @param active true activa los controles y false los desactiva.
	 */
	public void setEnableSaveButtons(boolean active){
		getDialogButtons().getBSave().setEnabled(active);
		getDialogButtons().getBExportToCSV().setEnabled(active);
	}
	//**********************End Setters & Getters*****************

 }  //  @jve:decl-index=0:visual-constraint="10,10"
 //  @jve:visual-info  decl-index=0 visual-constraint="10,10"
//  @jve:visual-info  decl-index=0 visual-constraint="10,10"
