/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 */
package es.prodevelop.cit.gvsig.fmap.drivers.jdbc.oracle;

import com.iver.cit.gvsig.fmap.core.FCircle2D;
import com.iver.cit.gvsig.fmap.core.FGeometryCollection;
import com.iver.cit.gvsig.fmap.core.FMultiPoint2D;
import com.iver.cit.gvsig.fmap.core.FMultipoint3D;
import com.iver.cit.gvsig.fmap.core.FPoint2D;
import com.iver.cit.gvsig.fmap.core.FPoint3D;
import com.iver.cit.gvsig.fmap.core.FPolygon2D;
import com.iver.cit.gvsig.fmap.core.FPolygon3D;
import com.iver.cit.gvsig.fmap.core.FPolyline3D;
import com.iver.cit.gvsig.fmap.core.FShape;
import com.iver.cit.gvsig.fmap.core.FShape3D;
import com.iver.cit.gvsig.fmap.core.GeneralPathX;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.core.ShapeFactory;
import com.iver.cit.gvsig.fmap.drivers.ConnectionJDBC;
import com.iver.cit.gvsig.fmap.drivers.IConnection;

import com.vividsolutions.jts.algorithm.CGAlgorithms;
import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.CoordinateArrays;
import com.vividsolutions.jts.geom.Envelope;
import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.geom.LineString;
import com.vividsolutions.jts.geom.LinearRing;
import com.vividsolutions.jts.geom.impl.CoordinateArraySequence;

import oracle.sql.ARRAY;
import oracle.sql.Datum;
import oracle.sql.NUMBER;
import oracle.sql.STRUCT;
import oracle.sql.StructDescriptor;

import org.apache.log4j.Logger;

import java.awt.geom.PathIterator;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;

import java.lang.reflect.Array;

import java.net.URL;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;

import java.util.ArrayList;
import java.util.Random;


/**
 * Utility class with static methods.
 *
 * @author jldominguez
 *
 */
public class OracleSpatialUtils {
    private static Logger logger = Logger.getLogger(OracleSpatialUtils.class.getName());
    private static double FLATNESS = 0.8;
    private static GeometryFactory geomFactory = new GeometryFactory();
    private static final double IRRELEVANT_DISTANCE = 0.00000001;
    private static Random rnd = new Random();
    private static DecimalFormat df = new DecimalFormat();
    private static DecimalFormatSymbols dfs = new DecimalFormatSymbols();
    public static final int ORACLE_GTYPE_UNKNOWN = 0;
    public static final int ORACLE_GTYPE_POINT = 1;
    public static final int ORACLE_GTYPE_LINE = 2;
    public static final int ORACLE_GTYPE_POLYGON = 3;
    public static final int ORACLE_GTYPE_COLLECTION = 4;
    public static final int ORACLE_GTYPE_MULTIPOINT = 5;
    public static final int ORACLE_GTYPE_MULTILINE = 6;
    public static final int ORACLE_GTYPE_MULTIPOLYGON = 7;

    /**
     * COnstructs a geometry from a file that contains a vertex per line:
     *
     * x1 y1 z1
     * x2 y2 z2
     * ...
     *
     * @param filepath vertices text file path
     * @param polygon whether it is a polygon or not
     * @return the created geometry
     */
    public static IGeometry readGeometry3D(URL filepath, boolean polygon) {
        GeneralPathX resp = new GeneralPathX();
        File file = new File(filepath.getFile());
        ArrayList z = new ArrayList();

        try {
            FileReader fr = new FileReader(file);
            BufferedReader br = new BufferedReader(fr);
            double[] coords = new double[3];

            boolean move = true;

            String line = br.readLine();

            while (line != null) {
                coords = parseLine(line);

                if (line.length() == 0) {
                    move = true;
                }
                else {
                    if (move) {
                        resp.moveTo(coords[0], coords[1]);
                        z.add(new Double(coords[2]));
                    }
                    else {
                        resp.lineTo(coords[0], coords[1]);
                        z.add(new Double(coords[2]));
                    }

                    move = false;
                }

                line = br.readLine();
            }
        }
        catch (Exception ex) {
        	logger.error("Error while creating GeneralPathX: " +
                ex.getMessage());

            return null;
        }

        double[] zz = new double[z.size()];

        for (int i = 0; i < z.size(); i++) {
            zz[i] = ((Double) z.get(i)).doubleValue();
        }

        if (polygon) {
            return ShapeFactory.createPolygon3D(resp, zz);
        }
        else {
            return ShapeFactory.createPolyline3D(resp, zz);
        }
    }

    private static double[] parseLine(String line) {
        String[] sep = line.split(" ");
        double[] resp = new double[3];

        for (int i = 0; i < 3; i++)
            resp[i] = 0.0;

        try {
            resp[0] = Double.parseDouble(sep[0]);
        }
        catch (Exception ex) {
        }

        if (sep.length > 1) {
            try {
                resp[1] = Double.parseDouble(sep[1]);
            }
            catch (Exception ex) {
            }

            if (sep.length > 2) {
                try {
                    resp[2] = Double.parseDouble(sep[2]);
                }
                catch (Exception ex) {
                }
            }
        }

        return resp;
    }

    /**
     * Utility method to convert a gvSIG FShape into a oracle struct
     *
     * @param fshp the FShape object
     * @param c the connection
     * @param srid the SRS (oarcle code)
     * @param agu_b whether to check holes validity
     * @param hasSrid whether the SRS is non-NULL
     * @return a oracle struct representing the geometry
     *
     * @throws SQLException
     */
    public static STRUCT fShapeToSTRUCT(Object fshp, IConnection c, int srid,
        boolean agu_b, boolean hasSrid) throws SQLException {
        boolean three = false;

        if (fshp instanceof FShape3D) {
            three = true;
        }

        STRUCT resp = null;

        if (fshp instanceof FMultiPoint2D) {
            resp = multiPoint2DToStruct((FMultiPoint2D) fshp, c, srid, hasSrid);

            return resp;
        }

        if (!(fshp instanceof FShape)) {
            logger.error("Unknown geometry: " + fshp.toString());

            return null;
        }

        if (fshp instanceof FPoint2D) { // point 2/3d

            // resp = pointToWKT((FPoint2D) fshp, three);
            Coordinate p = getSingleCoordinate((FPoint2D) fshp);
            resp = getMultiPointAsStruct(p, srid, three, c, hasSrid);
        }
        else {
            if (fshp instanceof FPolygon2D) { // polygon 2/3d

                if (fshp instanceof FCircle2D) {
                    resp = getCircleAsStruct((FCircle2D) fshp, srid, c, hasSrid);
                }
                else {
                    // also FEllipse2D
                    resp = getMultiPolygonAsStruct((FShape) fshp, srid, three,
                            c, agu_b, hasSrid);

                    // ArrayList polys = getPolygonsEasily(fshp);
                    // resp = getMultiPolygonAsStruct(polys, srid, three, c);
                }
            }
            else { // line 2/3d

                ArrayList _lines = getLineStrings((FShape) fshp);
                resp = getMultiLineAsStruct(_lines, srid, three, c, hasSrid);
            }
        }

        return resp;
    }

    private static STRUCT multiPoint2DToStruct(FMultiPoint2D mp2d,
        IConnection c, int srid, boolean hasSrid) throws SQLException {
        int np = mp2d.getNumPoints();
        boolean threed = (mp2d instanceof FMultipoint3D);
        int gtype = 2005;
        int dim = 2;
        FMultipoint3D mp3d = null;

        if (threed) {
            gtype = 3005;
            dim = 3;
            mp3d = (FMultipoint3D) mp2d;
        }

        NUMBER[] indices = new NUMBER[3];
        indices[0] = new NUMBER(1);
        indices[1] = new NUMBER(1);
        indices[2] = new NUMBER(np);

        NUMBER[] ords = new NUMBER[dim * np];

        for (int i = 0; i < np; i++) {
            ords[dim * i] = new NUMBER(mp2d.getPoint(i).getX());
            ords[(dim * i) + 1] = new NUMBER(mp2d.getPoint(i).getY());

            if (threed) {
                ords[(dim * i) + 2] = new NUMBER(mp3d.getZs()[i]);
            }
        }

        STRUCT resp;
        StructDescriptor dsc = StructDescriptor.createDescriptor("MDSYS.SDO_GEOMETRY",
        		((ConnectionJDBC)c).getConnection());
        Object[] obj = new Object[5];
        obj[0] = new NUMBER(gtype);

        if (hasSrid) {
            obj[1] = new NUMBER(srid);
        }
        else { // , boolean hasSrid
            obj[1] = null;
        }

        obj[2] = null;
        obj[3] = indices;
        obj[4] = ords;
        resp = new STRUCT(dsc, ((ConnectionJDBC)c).getConnection(), obj);

        return resp;
    }

    private static STRUCT getCircleAsStruct(FCircle2D fcirc, int srid,
        IConnection _conn, boolean hasSrid) throws SQLException {
        int geotype = 2003;
        NUMBER[] indices = new NUMBER[3];
        indices[0] = new NUMBER(1);
        indices[1] = new NUMBER(1003);
        indices[2] = new NUMBER(4);

        NUMBER[] ords = new NUMBER[6];
        Coordinate[] three_points = getThreePointsOfCircumference(fcirc.getCenter(),
                fcirc.getRadio());

        for (int i = 0; i < three_points.length; i++) {
            ords[i * 2] = new NUMBER(three_points[i].x);
            ords[(i * 2) + 1] = new NUMBER(three_points[i].y);
        }

        STRUCT resp;
        StructDescriptor dsc = StructDescriptor.createDescriptor("MDSYS.SDO_GEOMETRY",
        		((ConnectionJDBC)_conn).getConnection());
        Object[] obj = new Object[5];
        obj[0] = new NUMBER(geotype);

        if (hasSrid) {
            obj[1] = new NUMBER(srid);
        }
        else {
            obj[1] = null;
        }

        obj[2] = null;
        obj[3] = indices;
        obj[4] = ords;
        resp = new STRUCT(dsc, ((ConnectionJDBC)_conn).getConnection(), obj);

        return resp;
    }

    private static Coordinate[] getThreePointsOfCircumference(Point2D cntr,
        double radius) {
        Coordinate[] resp = new Coordinate[3];
        double x;
        double y;
        double alpha = 0;

        for (int i = 0; i < 3; i++) {
            alpha = (i * 120.0 * Math.PI) / 180.0;
            x = cntr.getX() + (radius * Math.cos(alpha));
            y = cntr.getY() + (radius * Math.sin(alpha));
            resp[i] = new Coordinate(x, y);
        }

        return resp;
    }

    private static Coordinate getSingleCoordinate(FPoint2D p2d) {
        // TODO Auto-generated method stub
        Coordinate resp = new Coordinate();
        resp.x = p2d.getX();
        resp.y = p2d.getY();

        if (p2d instanceof FPoint3D) {
            resp.z = ((FPoint3D) p2d).getZs()[0];
        }

        return resp;
    }

    private static ArrayList ensureSensibleLineString(ArrayList cc) {
        if (cc.size() == 2) {
            if (sameCoordinate((Coordinate) cc.get(0),
                        (Coordinate) cc.get(cc.size() - 1))) {
                ArrayList resp = new ArrayList();
                resp.add(cc.get(0));

                Coordinate newc = new Coordinate((Coordinate) cc.get(0));
                newc.x = newc.x + IRRELEVANT_DISTANCE;
                resp.add(newc);

                return resp;
            }
        }

        return cc;
    }

    private static boolean sameCoordinate(Coordinate c1, Coordinate c2) {
        if (c1.x != c2.x) {
            return false;
        }

        if (c1.y != c2.y) {
            return false;
        }

        return true;
    }

    private static ArrayList getClosedRelevantPolygon(ArrayList cc) {
        if (cc.size() == 2) {
            return getMinClosedCoords((Coordinate) cc.get(0));
        }

        if (cc.size() == 3) {
            if (sameCoordinate((Coordinate) cc.get(0), (Coordinate) cc.get(1))) {
                return getMinClosedCoords((Coordinate) cc.get(0));
            }

            if (sameCoordinate((Coordinate) cc.get(0), (Coordinate) cc.get(2))) {
                return getMinClosedCoords((Coordinate) cc.get(0));
            }

            if (sameCoordinate((Coordinate) cc.get(1), (Coordinate) cc.get(2))) {
                return getMinClosedCoords((Coordinate) cc.get(1));
            }

            cc.add(cc.get(0));

            return cc;
        }

        if (!sameCoordinate((Coordinate) cc.get(0),
                    (Coordinate) cc.get(cc.size() - 1))) {
            cc.add(cc.get(0));
        }

        return cc;
    }

    private static ArrayList getMinClosedCoords(Coordinate c) {
        ArrayList resp = new ArrayList();
        resp.add(c);

        Coordinate nc = new Coordinate(c);
        nc.x = nc.x + IRRELEVANT_DISTANCE;
        resp.add(nc);

        Coordinate nc2 = new Coordinate(nc);
        nc2.y = nc2.y + IRRELEVANT_DISTANCE;
        resp.add(nc2);

        resp.add(new Coordinate(c));

        return resp;
    }

    private static LinearRing getMinLinearRing(Coordinate c) {
        Coordinate[] p = new Coordinate[4];
        p[0] = c;

        Coordinate nc = new Coordinate(c);
        nc.x = nc.x + IRRELEVANT_DISTANCE;

        Coordinate nc2 = new Coordinate(nc);
        nc2.y = nc2.y - IRRELEVANT_DISTANCE;
        p[1] = nc;
        p[2] = nc2;
        p[3] = new Coordinate(c);

        CoordinateArraySequence cs = new CoordinateArraySequence(p);
        LinearRing ls = new LinearRing(cs, geomFactory);

        return ls;
    }

    private static double[] getMinLinearRingZ() {
        double[] resp = new double[4];

        for (int i = 0; i < 4; i++)
            resp[i] = 0.0;

        return resp;
    }

    private static boolean pointInList(Coordinate testPoint,
        Coordinate[] pointList) {
        int t;
        int numpoints;
        Coordinate p;

        numpoints = Array.getLength(pointList);

        for (t = 0; t < numpoints; t++) {
            p = pointList[t];

            if ((testPoint.x == p.x) && (testPoint.y == p.y) &&
                    ((testPoint.z == p.z) || (!(testPoint.z == testPoint.z))) //nan test; x!=x iff x is nan
            ) {
                return true;
            }
        }

        return false;
    }

    private static ArrayList getPolygonsEasily(FShape mpolygon) {
        boolean threed = false;

        if (mpolygon instanceof FPolygon3D) {
            threed = true;
        }

        int start_ind = 0;
        int end_ind = 0;
        int ind = 0;
        int new_size;
        ArrayList arrayCoords = null;
        ArrayList resp = new ArrayList();
        Coordinate[] points = null;
        int theType = -99;
        double[] theData = new double[6];
        Coordinate onlyCoord = null;
        int numParts = 0;

        PathIterator theIterator = mpolygon.getPathIterator(null, FLATNESS);

        while (!theIterator.isDone()) {
            //while not done
            theType = theIterator.currentSegment(theData);

            if (onlyCoord == null) {
                onlyCoord = new Coordinate();
                onlyCoord.x = theData[0];
                onlyCoord.y = theData[1];
            }

            switch (theType) {
            case PathIterator.SEG_MOVETO:

                if (arrayCoords == null) {
                    arrayCoords = new ArrayList();
                }
                else {
                    end_ind = ind - 1;

                    arrayCoords = getClosedRelevantPolygon(arrayCoords);
                    new_size = arrayCoords.size();

                    if (arrayCoords != null) {
                        points = CoordinateArrays.toCoordinateArray(arrayCoords);

                        try {
                            LinearRing aux = geomFactory.createLinearRing(points);
                            double[] z = null;

                            if (threed) {
                                z = getZ((FPolygon3D) mpolygon, start_ind,
                                        end_ind, new_size);
                            }

                            LineString3D ring = new LineString3D(aux, z);

                            if (CGAlgorithms.isCCW(points)) {
                                resp.add(ring);
                            }
                            else {
                                resp.add(ring.createReverse());
                            }
                        }
                        catch (Exception e) {
                            logger.error("Topology exception: " +
                                e.getMessage());

                            return null;
                        }
                    }

                    arrayCoords = new ArrayList();

                    start_ind = ind;
                }

                numParts++;

                arrayCoords.add(new Coordinate(theData[0], theData[1]));
                ind++;

                break;

            case PathIterator.SEG_LINETO:
                arrayCoords.add(new Coordinate(theData[0], theData[1]));
                ind++;

                break;

            case PathIterator.SEG_QUADTO:
                logger.debug("SEG_QUADTO Not supported here");
                arrayCoords.add(new Coordinate(theData[0], theData[1]));
                arrayCoords.add(new Coordinate(theData[2], theData[3]));
                ind++;
                ind++;

                break;

            case PathIterator.SEG_CUBICTO:
            	logger.debug("SEG_CUBICTO Not supported here");
                arrayCoords.add(new Coordinate(theData[0], theData[1]));
                arrayCoords.add(new Coordinate(theData[2], theData[3]));
                arrayCoords.add(new Coordinate(theData[4], theData[5]));
                ind++;
                ind++;
                ind++;

                break;

            case PathIterator.SEG_CLOSE:

                // Coordinate firstCoord = (Coordinate) arrayCoords.get(0);
                // arrayCoords.add(new Coordinate(firstCoord.x, firstCoord.y));
                break;
            } //end switch

            theIterator.next();
        } //end while loop

        end_ind = ind - 1;

        // null shape:
        if (arrayCoords == null) {
            arrayCoords = new ArrayList();

            Coordinate _c = new Coordinate(0, 0, 0);
            arrayCoords.add(new Coordinate(_c));
            arrayCoords.add(new Coordinate(_c));
        }

        // --------------------------------------------
        arrayCoords = getClosedRelevantPolygon(arrayCoords);
        new_size = arrayCoords.size();

        if (arrayCoords != null) {
            points = CoordinateArrays.toCoordinateArray(arrayCoords);

            try {
                LinearRing aux = geomFactory.createLinearRing(points);
                double[] z = null;

                if (threed) {
                    z = getZ((FPolygon3D) mpolygon, start_ind, end_ind, new_size);
                }

                LineString3D ring = new LineString3D(aux, z);

                if (CGAlgorithms.isCCW(points)) {
                    resp.add(ring);
                }
                else {
                    resp.add(ring.createReverse());
                }
            }
            catch (Exception e) {
            	logger.error("Topology exception: " + e.getMessage());

                return null;
            }
        }

        if (resp.size() == 0) {
            resp.add(new LineString3D(getMinLinearRing(onlyCoord),
                    getMinLinearRingZ()));
        }

        return resp;
    }

    /**
     * Utility method to reverse an array of doubles.
     *
     * @param _z an array of doubles to be reversed.
     *
     * @return the reversed array of doubles
     */
    public static double[] reverseArray(double[] _z) {
        int size = _z.length;
        double[] resp = new double[size];

        for (int i = 0; i < size; i++) {
            resp[i] = _z[size - 1 - i];
        }

        return resp;
    }

    /**
     * Utility method to reverse an array of coordinates
     *
     * @param _z an array of coordinaes to be reversed.
     *
     * @return the reversed array of coordinates
     */
    public static Coordinate[] reverseCoordinateArray(Coordinate[] _z) {
        int size = _z.length;
        Coordinate[] resp = new Coordinate[size];

        for (int i = 0; i < size; i++) {
            resp[i] = _z[size - 1 - i];
        }

        return resp;
    }

    private static double[] getZ(FShape3D p3d, int _str, int _end, int size) {
        double[] resp = new double[size];
        double[] allz = p3d.getZs();

        for (int i = _str; ((i <= _end) && ((i - _str) < size)); i++) {
            resp[i - _str] = allz[i];
        }

        if ((_end - _str + 1) < size) {
            double repe = allz[_end];

            for (int i = (_end - _str + 1); i < size; i++) {
                resp[i] = repe;
            }
        }

        return resp;
    }

    private static ArrayList getLineStrings(FShape mlines) {
        boolean threed = false;

        if (mlines instanceof FPolyline3D) {
            threed = true;
        }

        int start_ind = 0;
        int end_ind = 0;
        int ind = 0;
        int new_size = 0;

        LineString3D lin;

        ArrayList arrayLines = new ArrayList();
        PathIterator theIterator = mlines.getPathIterator(null, FLATNESS);
        int theType = -99;
        double[] theData = new double[6];
        ArrayList arrayCoords = null;
        int numParts = 0;

        while (!theIterator.isDone()) {
            //while not done
            theType = theIterator.currentSegment(theData);

            switch (theType) {
            case PathIterator.SEG_MOVETO:

                // System.out.println("SEG_MOVETO");
                if (arrayCoords == null) {
                    arrayCoords = new ArrayList();
                }
                else {
                    end_ind = ind - 1;
                    arrayCoords = ensureSensibleLineString(arrayCoords);
                    new_size = arrayCoords.size();

                    LineString aux = geomFactory.createLineString(CoordinateArrays.toCoordinateArray(
                                arrayCoords));
                    double[] z = null;

                    if (threed) {
                        z = getZ((FPolyline3D) mlines, start_ind, end_ind,
                                new_size);
                    }

                    lin = new LineString3D(aux, z);
                    arrayLines.add(lin);
                    arrayCoords = new ArrayList();

                    start_ind = ind;
                }

                numParts++;
                arrayCoords.add(new Coordinate(theData[0], theData[1]));

                break;

            case PathIterator.SEG_LINETO:
                arrayCoords.add(new Coordinate(theData[0], theData[1]));

                break;

            case PathIterator.SEG_QUADTO:
            	logger.debug("Not supported here");

                break;

            case PathIterator.SEG_CUBICTO:
            	logger.debug("Not supported here");

                break;

            case PathIterator.SEG_CLOSE:

                Coordinate firstCoord = (Coordinate) arrayCoords.get(0);
                arrayCoords.add(new Coordinate(firstCoord.x, firstCoord.y));

                break;
            } //end switch

            theIterator.next();
            ind++;
        } //end while loop

        arrayCoords = ensureSensibleLineString(arrayCoords);
        new_size = arrayCoords.size();

        LineString aux = geomFactory.createLineString(CoordinateArrays.toCoordinateArray(
                    arrayCoords));
        double[] z = null;

        if (threed) {
            z = getZ((FPolyline3D) mlines, start_ind, end_ind, new_size);
        }

        lin = new LineString3D(aux, z);
        arrayLines.add(lin);

        return arrayLines;
    }

    private static String lineStringToWKT(LineString3D ls, boolean threed) {
        String resp = "(";
        Coordinate[] cc = ls.getLs().getCoordinates();
        double[] z = ls.getZc();
        int size = cc.length;

        if (threed) {
            for (int i = 0; i < size; i++) {
                resp = resp + cc[i].x + " " + cc[i].y + " " + z[i] + ", ";
            }

            resp = resp.substring(0, resp.length() - 2);
            resp = resp + ")";
        }
        else {
            for (int i = 0; i < size; i++) {
                resp = resp + cc[i].x + " " + cc[i].y + ", ";
            }

            resp = resp.substring(0, resp.length() - 2);
            resp = resp + ")";
        }

        return resp;
    }

    private static String multiLineStringToWKT(ArrayList ml, boolean threed) {
        String resp = "MULTILINESTRING(";

        for (int i = 0; i < ml.size(); i++) {
            LineString3D ls = (LineString3D) ml.get(i);
            resp = resp + lineStringToWKT(ls, threed) + ", ";
        }

        resp = resp.substring(0, resp.length() - 2) + ")";

        return resp;
    }

    private static String polygonsToWKT(ArrayList pols, boolean threed) {
        String resp = "MULTIPOLYGON(";
        LineString3D ls = null;

        for (int i = 0; i < pols.size(); i++) {
            ls = (LineString3D) pols.get(i);
            resp = resp + "(" + lineStringToWKT(ls, threed) + "), ";
        }

        resp = resp.substring(0, resp.length() - 2) + ")";

        return resp;
    }

    private static String shellAndHolesToWKT(LineString3D shell,
        ArrayList holes, boolean threed) {
        String resp = "(";
        resp = resp + lineStringToWKT(shell, threed);

        if (holes.size() > 0) {
            for (int i = 0; i < holes.size(); i++) {
                LineString3D ls = (LineString3D) holes.get(i);
                resp = resp + ", " + lineStringToWKT(ls, threed);
            }
        }

        resp = resp + ")";

        return resp;
    }

    private static String multiPolygonToWKT(ArrayList shells, ArrayList hFs,
        boolean threed) {
        String resp = "MULTIPOLYGON(";
        LineString3D ls = null;
        ArrayList holes;

        for (int i = 0; i < shells.size(); i++) {
            ls = (LineString3D) shells.get(i);
            holes = (ArrayList) hFs.get(i);
            resp = resp + shellAndHolesToWKT(ls, holes, threed) + ", ";
        }

        resp = resp.substring(0, resp.length() - 2) + ")";

        return resp;
    }

    private static String pointToWKT(FPoint2D point, boolean threed) {
        String resp = "POINT(" + point.getX() + " " + point.getY();

        if ((threed) && (point instanceof FPoint3D)) {
            resp = resp + " " + ((FPoint3D) point).getZs()[0];
        }

        resp = resp + ")";

        return resp;
    }

    private static int twoDIndexToDimsIndex(int n, int d) {
        return ((d * (n - 1)) / 2) + 1;
    }

    private static ARRAY setSubelementsToDim(ARRAY old, int d)
        throws SQLException {
        Datum[] infos = (Datum[]) old.getOracleArray();

        for (int i = 3; i < infos.length; i = i + 3) {
            int oldind = infos[i].intValue();
            oldind = twoDIndexToDimsIndex(oldind, d);
            infos[i] = new NUMBER(oldind);

            //
            oldind = infos[i + 1].intValue();
            infos[i + 1] = new NUMBER(infos[1].intValue());
        }

        ARRAY resp = new ARRAY(old.getDescriptor(), old.getOracleConnection(),
                infos);

        return resp;
    }

    private static boolean isPointInsideLineString(Coordinate p, LineString ls) {
        Envelope env = ls.getEnvelopeInternal();

        if (!env.contains(p)) {
            return false;
        }

        return CGAlgorithms.isPointInRing(p, ls.getCoordinates());
    }

    private static boolean lineString3DIsContainedBy(LineString3D contained,
        LineString3D container) {
        int samples = 10;
        LineString _in = contained.getLs();
        LineString _out = container.getLs();
        Coordinate[] inc = _in.getCoordinates();
        Coordinate aux;
        int size = inc.length;

        if (size <= 10) {
            for (int i = 0; i < size; i++) {
                aux = inc[i];

                if (!isPointInsideLineString(aux, _out)) {
                    return false;
                }
            }

            return true;
        }
        else {
            for (int i = 0; i < samples; i++) {
                aux = inc[rnd.nextInt(size)];

                if (!isPointInsideLineString(aux, _out)) {
                    return false;
                }
            }

            return true;
        }
    }

    private static STRUCT getMultiPolygonAsStruct(ArrayList pols, int srid,
        boolean threed, IConnection _conn, boolean agu_bien, boolean hasSrid)
        throws SQLException {
        int size = pols.size();
        int geotype = 2007;
        int dim = 2;
        int acum = 0;

        if (threed) {
            geotype = 3007;
            dim = 3;
        }

        NUMBER[] indices = new NUMBER[3 * size];

        for (int i = 0; i < size; i++) {
            indices[3 * i] = new NUMBER(acum + 1);
            indices[(3 * i) + 1] = new NUMBER(1003);
            indices[(3 * i) + 2] = new NUMBER(1);
            acum = acum +
                (dim * ((LineString3D) pols.get(i)).getLs().getNumPoints());
        }

        int _ind = 0;
        NUMBER[] ords = new NUMBER[acum];

        for (int i = 0; i < size; i++) {
            LineString3D ls = (LineString3D) pols.get(i);
            int num_p = ls.getLs().getNumPoints();

            for (int j = 0; j < num_p; j++) {
                ords[_ind] = new NUMBER(ls.getLs().getCoordinateN(j).x);
                ords[_ind + 1] = new NUMBER(ls.getLs().getCoordinateN(j).y);

                if (threed) {
                    ords[_ind + 2] = new NUMBER(ls.getZc()[j]);
                }

                _ind = _ind + dim;
            }
        }

        STRUCT resp;
        StructDescriptor dsc = StructDescriptor.createDescriptor("MDSYS.SDO_GEOMETRY",
        		((ConnectionJDBC)_conn).getConnection());
        Object[] obj = new Object[5];
        obj[0] = new NUMBER(geotype);

        if (hasSrid) {
            obj[1] = new NUMBER(srid);
        }
        else {
            obj[1] = null;
        }

        obj[2] = null;
        obj[3] = indices;
        obj[4] = ords;
        resp = new STRUCT(dsc, ((ConnectionJDBC)_conn).getConnection(), obj);

        return resp;
    }

    private static STRUCT getMultiLineAsStruct(ArrayList lines, int srid,
        boolean threed, IConnection _conn, boolean hasSrid)
        throws SQLException {
        /*
        if (lines.size() == 1) {
                return getOneLineStringAsStruct((LineString3D) lines.get(0), srid, threed, _conn);
        }
        */
        int size = lines.size();
        int geotype = 2006;
        int dim = 2;
        int acum = 0;

        if (threed) {
            geotype = 3006;
            dim = 3;
        }

        NUMBER[] indices = new NUMBER[3 * size];

        for (int i = 0; i < size; i++) {
            indices[3 * i] = new NUMBER(acum + 1);
            indices[(3 * i) + 1] = new NUMBER(2);
            indices[(3 * i) + 2] = new NUMBER(1);
            acum = acum +
                (dim * ((LineString3D) lines.get(i)).getLs().getNumPoints());
        }

        int _ind = 0;
        NUMBER[] ords = new NUMBER[acum];

        for (int i = 0; i < size; i++) {
            LineString3D ls = (LineString3D) lines.get(i);
            int num_p = ls.getLs().getNumPoints();

            for (int j = 0; j < num_p; j++) {
                ords[_ind] = new NUMBER(ls.getLs().getCoordinateN(j).x);
                ords[_ind + 1] = new NUMBER(ls.getLs().getCoordinateN(j).y);

                if (threed) {
                    ords[_ind + 2] = new NUMBER(ls.getZc()[j]);
                }

                _ind = _ind + dim;
            }
        }

        STRUCT resp;
        StructDescriptor dsc = StructDescriptor.createDescriptor("MDSYS.SDO_GEOMETRY",
        		((ConnectionJDBC)_conn).getConnection());
        Object[] obj = new Object[5];
        obj[0] = new NUMBER(geotype);

        if (hasSrid) {
            obj[1] = new NUMBER(srid);
        }
        else {
            obj[1] = null;
        }

        obj[2] = null;
        obj[3] = indices;
        obj[4] = ords;
        resp = new STRUCT(dsc,((ConnectionJDBC)_conn).getConnection(), obj);

        return resp;
    }

    private static STRUCT getMultiPointAsStruct(Coordinate pnt, int srid,
        boolean threed, IConnection _conn, boolean hasSrid)
        throws SQLException {
        int geotype = 2001;
        int dim = 2;

        if (threed) {
            geotype = 3001;
            dim = 3;
        }

        Object[] ords = new Object[3];
        ords[0] = new NUMBER(pnt.x);
        ords[1] = new NUMBER(pnt.y);
        ords[2] = (dim == 3) ? new NUMBER(pnt.z) : null; // ole ole y ole

        StructDescriptor ord_dsc = StructDescriptor.createDescriptor("MDSYS.SDO_POINT_TYPE",
        		((ConnectionJDBC)_conn).getConnection());
        STRUCT ords_st = new STRUCT(ord_dsc, ((ConnectionJDBC)_conn).getConnection(), ords);

        STRUCT resp;

        StructDescriptor dsc = StructDescriptor.createDescriptor("MDSYS.SDO_GEOMETRY",
        		((ConnectionJDBC)_conn).getConnection());
        Object[] obj = new Object[5];

        obj[0] = new NUMBER(geotype);

        if (hasSrid) {
            obj[1] = new NUMBER(srid);
        }
        else {
            obj[1] = null;
        }

        obj[2] = ords_st;
        obj[3] = null;
        obj[4] = null;
        resp = new STRUCT(dsc, ((ConnectionJDBC)_conn).getConnection(), obj);

        return resp;
    }

    /**
     * Utility method to compute a circle's center and radius from three given points.
     *
     * @param points three points of a circumference
     * @return a 2-item array with the circumference's center (Point2D) and radius (Double)
     */
    public static Object[] getCenterAndRadiousOfCirc(Point2D[] points) {
        Object[] resp = new Object[2];
        resp[0] = new Point2D.Double(0, 0);
        resp[1] = new Double(0);

        double m11;
        double m12;
        double m13;
        double m14;

        if (points.length != 3) {
            logger.error("Needs 3 points (found " + points.length +
                ") - circle cannot be computed.");

            // not a circle
            return resp;
        }

        double[][] a = new double[3][3];

        for (int i = 0; i < 3; i++) { // find minor 11
            a[i][0] = points[i].getX();
            a[i][1] = points[i].getY();
            a[i][2] = 1;
        }

        m11 = determinant(a, 3);

        for (int i = 0; i < 3; i++) { // find minor 12
            a[i][0] = (points[i].getX() * points[i].getX()) +
                (points[i].getY() * points[i].getY());
            a[i][1] = points[i].getY();
            a[i][2] = 1;
        }

        m12 = determinant(a, 3);

        for (int i = 0; i < 3; i++) // find minor 13
         {
            a[i][0] = (points[i].getX() * points[i].getX()) +
                (points[i].getY() * points[i].getY());
            a[i][1] = points[i].getX();
            a[i][2] = 1;
        }

        m13 = determinant(a, 3);

        for (int i = 0; i < 3; i++) { // find minor 14
            a[i][0] = (points[i].getX() * points[i].getX()) +
                (points[i].getY() * points[i].getY());
            a[i][1] = points[i].getX();
            a[i][2] = points[i].getY();
        }

        m14 = determinant(a, 3);

        Double resp_radius = new Double(0);
        Point2D resp_center = new Point2D.Double(0, 0);

        if (m11 == 0) {
            logger.error("Three points aligned - circle cannot be computed."); // not a circle
        }
        else {
            double x = (0.5 * m12) / m11;
            double y = (-0.5 * m13) / m11;
            resp_center.setLocation(x, y);
            resp_radius = new Double(Math.sqrt((x * x) + (y * y) + (m14 / m11)));
            resp[0] = resp_center;
            resp[1] = resp_radius;
        }

        return resp;
    }

    /**
     * Utility method to compute a matrix determinant
     * @param a the matrix
     * @param n matrix size
     * @return the matrix's determinant
     */
    public static double determinant(double[][] a, int n) {
        double resp = 0;
        double[][] m = new double[3][3];

        if (n == 2) { // terminate recursion
            resp = (a[0][0] * a[1][1]) - (a[1][0] * a[0][1]);
        }
        else {
            resp = 0;

            for (int j1 = 0; j1 < n; j1++) { // do each column

                for (int i = 1; i < n; i++) { // create minor

                    int j2 = 0;

                    for (int j = 0; j < n; j++) {
                        if (j == j1) {
                            continue;
                        }

                        m[i - 1][j2] = a[i][j];
                        j2++;
                    }
                }

                // sum (+/-)cofactor * minor
                resp = resp +
                    (Math.pow(-1.0, j1) * a[0][j1] * determinant(m, n - 1));
            }
        }

        return resp;
    }

    private static int getSmallestContainerExcept(LineString3D ls,
        ArrayList list, int self) {
        int resp = -1;
        ArrayList provList = new ArrayList();

        int size = list.size();

        for (int i = 0; i < self; i++) {
            if (lineString3DIsContainedBy(ls, (LineString3D) list.get(i))) {
                provList.add(new Integer(i));
            }
        }

        for (int i = (self + 1); i < size; i++) {
            if (lineString3DIsContainedBy(ls, (LineString3D) list.get(i))) {
                provList.add(new Integer(i));
            }
        }

        if (provList.size() == 0) {
            // logger.debug("LineString is not contained by any other ls.");
        }
        else {
            if (provList.size() == 1) {
                resp = ((Integer) provList.get(0)).intValue();
            }
            else {
                if (provList.size() == 2) {
                    int ind_1 = ((Integer) provList.get(0)).intValue();
                    int ind_2 = ((Integer) provList.get(1)).intValue();
                    LineString3D ls1 = (LineString3D) list.get(ind_1);
                    LineString3D ls2 = (LineString3D) list.get(ind_2);

                    if (lineString3DIsContainedBy(ls1, ls2)) {
                        resp = ind_1;
                    }
                    else {
                        resp = ind_2;
                    }
                }
                else {
                    // not so deep, sorry!
                    // it's going to be a shell: resp = -1;
                }
            }
        }

        return resp;
    }

    private static int[] getIndicesOfShells(int[] containings) {
        ArrayList resp = new ArrayList();

        for (int i = 0; i < containings.length; i++) {
            if (containings[i] == -1) {
                resp.add(new Integer(i));
            }
        }

        int size = resp.size();
        int[] _resp = new int[size];

        for (int i = 0; i < size; i++) {
            _resp[i] = ((Integer) resp.get(i)).intValue();
        }

        return _resp;
    }

    private static int[] getIndicesOfHoles(int[] containings, int[] shells) {
        ArrayList resp = new ArrayList();

        for (int i = 0; i < containings.length; i++) {
            int cont_by = containings[i];

            if ((cont_by != -1) && (isOneOf(cont_by, shells))) {
                resp.add(new Integer(i));
            }
        }

        int size = resp.size();
        int[] _resp = new int[size];

        for (int i = 0; i < size; i++) {
            _resp[i] = ((Integer) resp.get(i)).intValue();
        }

        return _resp;
    }

    private static int[] getFinalContainings(int[] containings, int[] holes) {
        ArrayList resp = new ArrayList();

        for (int i = 0; i < containings.length; i++) {
            int cont_by = containings[i];

            if (isOneOf(cont_by, holes)) {
                resp.add(new Integer(-1));
            }
            else {
                resp.add(new Integer(cont_by));
            }
        }

        int size = resp.size();
        int[] _resp = new int[size];

        for (int i = 0; i < size; i++) {
            _resp[i] = ((Integer) resp.get(i)).intValue();
        }

        return _resp;
    }

    private static ArrayList getHolesOf(int ind, int[] final_contn,
        ArrayList all) {
        ArrayList resp_ind = new ArrayList();

        for (int i = 0; i < final_contn.length; i++) {
            if (final_contn[i] == ind) {
                resp_ind.add(new Integer(i));
            }
        }

        ArrayList resp = new ArrayList();

        for (int i = 0; i < resp_ind.size(); i++) {
            Integer aux = (Integer) resp_ind.get(i);
            resp.add(all.get(aux.intValue()));
        }

        return resp;
    }

    private static ArrayList getShellsIn(int[] final_contn, ArrayList all) {
        ArrayList resp_ind = new ArrayList();

        for (int i = 0; i < final_contn.length; i++) {
            if (final_contn[i] == -1) {
                resp_ind.add(new Integer(i));
            }
        }

        ArrayList resp = new ArrayList();

        for (int i = 0; i < resp_ind.size(); i++) {
            Integer aux = (Integer) resp_ind.get(i);
            resp.add(all.get(aux.intValue()));
        }

        return resp;
    }

    /**
     * This method tries to guess who is a shell and who is a hole from a set of
     * linestrings.
     *
     * @param all_ls a set of linestrings to be checked.
     *
     * @return a 2-item array. the first is an arraylist of linestrings thought to be shells.
     * the second is an array of arraylists containing the holes of each shell found in the
     * first item
     *
     */
    public static Object[] getHolesForShells(ArrayList all_ls) {
        int no_of_ls = all_ls.size();
        int[] containedby = new int[no_of_ls];
        int[] shells;
        int[] holes;
        int[] final_cont;

        for (int i = 0; i < no_of_ls; i++) {
            LineString3D ls_aux = (LineString3D) all_ls.get(i);
            containedby[i] = getSmallestContainerExcept(ls_aux, all_ls, i);
        }

        shells = getIndicesOfShells(containedby);
        holes = getIndicesOfHoles(containedby, shells);
        final_cont = getFinalContainings(containedby, holes);

        // true shells:
        shells = getIndicesOfShells(final_cont);

        ArrayList resp_shells = new ArrayList();
        ArrayList resp_holes_for_shells = new ArrayList();
        ArrayList aux_holes;

        for (int i = 0; i < shells.length; i++) {
            resp_shells.add(all_ls.get(shells[i]));
            aux_holes = getHolesOf(i, final_cont, all_ls);
            resp_holes_for_shells.add(aux_holes);
        }

        Object[] _resp = new Object[2];
        _resp[0] = resp_shells;
        _resp[1] = resp_holes_for_shells;

        return _resp;
    }

    private static int getTotalSize(ArrayList listOfLists) {
        int resp = 0;

        for (int i = 0; i < listOfLists.size(); i++) {
            resp = resp + ((ArrayList) listOfLists.get(i)).size();
        }

        return resp;
    }

    // private static STRUCT // private static ArrayList getPolygonsEasily(FShape mpolygon) {
    private static STRUCT getMultiPolygonAsStruct(FShape mpol, int srid,
        boolean threed, IConnection _conn, boolean agu_bien, boolean hasSrid)
        throws SQLException {
        ArrayList all_ls = getPolygonsEasily(mpol);
        Object[] hs = getHolesForShells(all_ls);
        ArrayList sh = (ArrayList) hs[0];
        ArrayList _ho = (ArrayList) hs[1];
        ArrayList ho = reverseHoles(_ho);

        return getMultiPolygonAsStruct(sh, ho, srid, threed, _conn, agu_bien,
            hasSrid);
    }

    private static ArrayList reverseHoles(ArrayList hh) {
        ArrayList resp = new ArrayList();

        for (int i = 0; i < hh.size(); i++) {
            ArrayList item = (ArrayList) hh.get(i);
            ArrayList newitem = new ArrayList();

            for (int j = 0; j < item.size(); j++) {
                LineString3D ls = (LineString3D) item.get(j);
                newitem.add(ls.createReverse());
            }

            resp.add(newitem);
        }

        return resp;
    }

    private static STRUCT getMultiPolygonAsStruct(ArrayList shells,
        ArrayList holes, int srid, boolean threed, IConnection _conn,
        boolean explicito, boolean hasSrid) throws SQLException {
        int t = 1003;

        if (explicito) {
            t = 2003;
        }

        int size = shells.size() + getTotalSize(holes);
        int geotype = 2003;
        if (size > 1) geotype = 2007;

        int dim = 2;

        if (threed) {
            geotype = geotype + 1000;
            dim = 3;
        }

        NUMBER[] indices = new NUMBER[3 * size];

        int acum = 0;
        int start_ind = 0;

        for (int i = 0; i < shells.size(); i++) {
            indices[start_ind] = new NUMBER(acum + 1);
            indices[start_ind + 1] = new NUMBER(1003);
            indices[start_ind + 2] = new NUMBER(1);
            start_ind = start_ind + 3;
            acum = acum +
                (dim * ((LineString3D) shells.get(i)).getLs().getNumPoints());

            ArrayList item_holes = (ArrayList) holes.get(i);

            for (int j = 0; j < item_holes.size(); j++) {
                indices[start_ind] = new NUMBER(acum + 1);
                indices[start_ind + 1] = new NUMBER(t); // 1003
                indices[start_ind + 2] = new NUMBER(1);
                start_ind = start_ind + 3;
                acum = acum +
                    (dim * ((LineString3D) item_holes.get(j)).getLs()
                            .getNumPoints());
            }
        }

        int _ind = 0;
        NUMBER[] ords = new NUMBER[acum];

        for (int i = 0; i < shells.size(); i++) {
            // --------------------------------
            LineString3D ls = (LineString3D) shells.get(i);
            int num_p = ls.getLs().getNumPoints();

            for (int j = 0; j < num_p; j++) {
                ords[_ind] = new NUMBER(ls.getLs().getCoordinateN(j).x);
                ords[_ind + 1] = new NUMBER(ls.getLs().getCoordinateN(j).y);

                if (threed) {
                    ords[_ind + 2] = new NUMBER(ls.getZc()[j]);
                }

                _ind = _ind + dim;
            }

            // -------------------------------
            ArrayList item_holes = (ArrayList) holes.get(i);

            for (int j = 0; j < item_holes.size(); j++) {
                ls = (LineString3D) item_holes.get(j);
                num_p = ls.getLs().getNumPoints();

                for (int k = 0; k < num_p; k++) {
                    ords[_ind] = new NUMBER(ls.getLs().getCoordinateN(k).x);
                    ords[_ind + 1] = new NUMBER(ls.getLs().getCoordinateN(k).y);

                    if (threed) {
                        ords[_ind + 2] = new NUMBER(ls.getZc()[k]);
                    }

                    _ind = _ind + dim;
                }
            }
        }

        STRUCT resp;
        StructDescriptor dsc = StructDescriptor.createDescriptor("MDSYS.SDO_GEOMETRY",
        		((ConnectionJDBC)_conn).getConnection());
        Object[] obj = new Object[5];
        obj[0] = new NUMBER(geotype);

        if (hasSrid) {
            obj[1] = new NUMBER(srid);
        }
        else {
            obj[1] = null;
        }

        obj[2] = null;
        obj[3] = indices;
        obj[4] = ords;

        // String ind_str = printArray(indices);
        // String ord_str = printArray(ords);
        resp = new STRUCT(dsc, ((ConnectionJDBC)_conn).getConnection(), obj);

        return resp;
    }

    public static String printArray(NUMBER[] array) {
        String resp = "[ ";

        for (int i = 0; i < array.length; i++) {
            resp = resp + " " + array[i].doubleValue() + " , ";
        }

        resp = resp.substring(0, resp.length() - 2) + "]";

        return resp;
    }

    private static boolean isOneOf(int ind, int[] list) {
        for (int i = 0; i < list.length; i++) {
            if (list[i] == ind) {
                return true;
            }
        }

        return false;
    }

    /**
     * This method appends the geometries from a geometry collection in one STRUCT.
     *
     * @param co the geometry collection
     * @param _forced_type a type that has to be used as the struct's main type
     * @param _conn the connection
     * @param _o_srid the geometry's SRS (oracle code)
     * @param withSrid whether the SRS is non-NULL
     * @param agu_bien whether to check holes' validity
     * @param _isGeoCS whether the SRS is geodetic
     * @return the STRUCT with the appended geometries
     */
    public static STRUCT appendGeometriesInStruct(FGeometryCollection co,
        int _forced_type, IConnection _conn, String _o_srid, boolean withSrid,
        boolean agu_bien, boolean _isGeoCS) {
        IGeometry[] geoms = co.getGeometries();
        int size = geoms.length;
        STRUCT[] sts = new STRUCT[size];

        for (int i = 0; i < size; i++) {
            sts[i] = OracleSpatialDriver.iGeometryToSTRUCT(geoms[i],
                    _forced_type, _conn, _o_srid, withSrid, agu_bien, _isGeoCS);
        }

        if (size == 1) {
            return sts[0];
        }

        STRUCT aux = sts[0];

        for (int i = 1; i < size; i++) {
            aux = appendStructs(aux, sts[i], _conn);
        }

        return aux;
    }

    private static STRUCT appendStructs(STRUCT st1, STRUCT st2, IConnection _conn) {
        try {
            ARRAY _ords = (ARRAY) st1.getOracleAttributes()[4];
            int length_of_head_ords = _ords.getOracleArray().length;

            NUMBER gtype = new NUMBER(4 +
                    (((NUMBER) st1.getOracleAttributes()[0]).intValue() / 1000));
            NUMBER srid = (NUMBER) st1.getOracleAttributes()[1];
            NUMBER middle = (NUMBER) st1.getOracleAttributes()[2];

            ARRAY info1 = (ARRAY) st1.getOracleAttributes()[3];
            ARRAY info2 = (ARRAY) st2.getOracleAttributes()[3];
            ARRAY ords1 = (ARRAY) st1.getOracleAttributes()[4];
            ARRAY ords2 = (ARRAY) st2.getOracleAttributes()[4];

            Datum[] info = appendDatumArrays(info1.getOracleArray(),
                    info2.getOracleArray(), length_of_head_ords);

            Datum[] ords = appendDatumArrays(ords1.getOracleArray(),
                    ords2.getOracleArray(), 0);

            StructDescriptor dsc = st1.getDescriptor();

            Object[] atts = new Object[5];
            atts[0] = gtype;
            atts[1] = srid;
            atts[2] = middle;
            atts[3] = info;
            atts[4] = ords;

            STRUCT resp = new STRUCT(dsc, ((ConnectionJDBC)_conn).getConnection(), atts);

            return resp;
        }
        catch (SQLException sqle) {
            logger.error("While appending structs: " + sqle.getMessage(), sqle);
        }

        return null;
    }

    private static Datum[] appendDatumArrays(Datum[] head, Datum[] tail,
        int offset) {
        int head_l = head.length;
        int tail_l = tail.length;
        Datum[] resp = new Datum[head_l + tail_l];

        for (int i = 0; i < head_l; i++)
            resp[i] = head[i];

        if (offset == 0) {
            for (int i = 0; i < tail_l; i++)
                resp[head_l + i] = tail[i];
        }
        else {
            try {
                for (int i = 0; i < tail_l; i++) {
                    if ((i % 3) == 0) {
                        resp[head_l + i] = new NUMBER(tail[i].intValue() +
                                offset);
                    }
                    else {
                        resp[head_l + i] = tail[i];
                    }
                }
            }
            catch (SQLException se) {
                logger.error("Unexpected error: " + se.getMessage());
            }
        }

        return resp;
    }

    /**
     * Utility method to get an ineteger as a formatted string.
     *
     * @param n the integer
     * @return the formatted string
     */
    public static String getFormattedInteger(int n) {
        df.setGroupingUsed(true);
        df.setGroupingSize(3);
        dfs.setGroupingSeparator('.');
        df.setDecimalFormatSymbols(dfs);

        return df.format(n);
    }

    /**
     * Tells whether these arrays belong to a rectangle polygon.
     *
     * @param info the struct's element info array
     * @param ords the struct's coordinate array
     * @return true if it is a rectangle polygon. false otherwise.
     */
    public static boolean polStructIsRectStruct(ARRAY info, ARRAY ords) {
        try {
            int[] infos = info.getIntArray();

            return ((infos[2] == 3) && (infos.length == 3));
        }
        catch (SQLException se) {
            logger.error("While ckecking rectangle: " + se.getMessage(), se);
        }

        return false;
    }

    /**
     * Utility method to deal with oracle info arrays.
     */
    public static ARRAY getDevelopedInfoArray(ARRAY info) {
        ARRAY _resp = null;

        try {
            Datum[] resp = new Datum[3];
            Datum[] in = info.getOracleArray();
            resp[0] = in[0];
            resp[1] = in[1];
            resp[2] = new NUMBER(1);
            _resp = new ARRAY(info.getDescriptor(),
                    info.getInternalConnection(), resp);
        }
        catch (SQLException se) {
            logger.error("While creating ARRAY: " + se.getMessage(), se);
        }

        return _resp;
    }

    /**
     * Utility method to deal with oracle coordinate arrays.
     */
    public static ARRAY getDevelopedOrdsArray(ARRAY ords) {
        ARRAY _resp = null;

        try {
            Datum[] resp = new Datum[10];
            Datum[] corners = ords.getOracleArray();

            // x
            resp[0] = corners[0];
            resp[2] = corners[2];
            resp[4] = corners[2];
            resp[6] = corners[0];
            resp[8] = corners[0];

            // y
            resp[1] = corners[1];
            resp[3] = corners[1];
            resp[5] = corners[3];
            resp[7] = corners[3];
            resp[9] = corners[1];
            _resp = new ARRAY(ords.getDescriptor(),
                    ords.getInternalConnection(), resp);
        }
        catch (SQLException se) {
            logger.error("While creating ARRAY: " + se.getMessage(), se);
        }

        return _resp;
    }

    /**
     * utility method to convert a STRUCT into a GeneralPathX
     * @param aux the struct's datum array
     * @return the GeneralPathX instance created
     */
    public static GeneralPathX structToGPX(Datum[] aux) {
        GeneralPathX resp = new GeneralPathX();
        ARRAY infoARRAY = null;
        ARRAY ordsARRAY = null;
        Datum[] info_array = null;
        Datum[] ords_array = null;
        int info_array_size = 0;
        int[] start_ind;
        int[] end_ind;
        int dims = 0;
        boolean next_must_do_first = true;

        try {
            infoARRAY = (ARRAY) aux[3];
            ordsARRAY = (ARRAY) aux[4];

            if (polStructIsRectStruct(infoARRAY, ordsARRAY)) {
                infoARRAY = getDevelopedInfoArray(infoARRAY);
                ordsARRAY = getDevelopedOrdsArray(ordsARRAY);
            }

            dims = ((NUMBER) aux[0]).intValue() / 1000;

            if (dims == 0) {
                dims = 2;
            }

            info_array = (Datum[]) infoARRAY.getOracleArray();
            ords_array = (Datum[]) ordsARRAY.getOracleArray();
            info_array_size = info_array.length / 3;

            if (info_array_size  > 1) {
            	info_array_size++;
            	info_array_size--;
            } else {
            	info_array_size++;
            	info_array_size--;
            }

            int last_index = ords_array.length - dims + 1;

            // set indices:
            start_ind = new int[info_array_size];
            end_ind = new int[info_array_size];

            for (int i = 0; i < info_array_size; i++)
                start_ind[i] = ((NUMBER) info_array[3 * i]).intValue();

            for (int i = 0; i < (info_array_size - 1); i++)
                end_ind[i] = start_ind[i + 1] - 1;

            end_ind[info_array_size - 1] = last_index;

            int lineType = PathIterator.SEG_LINETO;

            if (end_ind[0] == 0) { // collection of paths

                for (int i = 1; i < info_array_size; i++) {
                    lineType = getLineToType(info_array, i);

                    // -----------------------
                    if (end_ind[i] == (start_ind[i] - 1))
                    	lineType = PathIterator.SEG_MOVETO;
                    // -----------------------

                    next_must_do_first = addOrdsToGPX(resp, start_ind[i] - 1,
                            end_ind[i] - 1, ords_array, dims, lineType,
                            (i == 1) || (lineType == PathIterator.SEG_MOVETO),
                            next_must_do_first);
                }
            }
            else {
                // standard case, do the moveto always
                for (int i = 0; i < info_array_size; i++) {
                    lineType = getLineToType(info_array, i);
                    addOrdsToGPX(resp, start_ind[i] - 1, end_ind[i] - 1,
                        ords_array, dims, lineType, true, true);
                }
            }

            // boolean do_the_moves = true;
        }
        catch (SQLException se) {
            logger.error("While creating GPX: " + se.getMessage(), se);
        }

        return resp;
    }

    private static int getLineToType(Datum[] infos, int i) {
        int resp = PathIterator.SEG_LINETO;

        try {
            if (((NUMBER) infos[(3 * i) + 2]).intValue() == 2) {
                resp = PathIterator.SEG_QUADTO;
            }
        }
        catch (SQLException e) {
            logger.error("While getting line-to type: " + e.getMessage() +
                " (returned SEG_LINETO)");
        }

        return resp;
    }

    private static boolean addOrdsToGPX(GeneralPathX gpx, int zero_based_start,
        int zero_based_include_end, Datum[] ords, int d, int ltype,
        boolean do_the_move, boolean must_do_first) {
        int length = ords.length;
        boolean return_following_must_do_first = true;

        double x = ((NUMBER) ords[zero_based_start]).doubleValue();
        double y = ((NUMBER) ords[zero_based_start + 1]).doubleValue();

        if (must_do_first) {
            if (do_the_move) {
                gpx.moveTo(x, y);
            }
            else {
                gpx.lineTo(x, y);
            }
        }

        int ind = 1;

        int size = ((zero_based_include_end - zero_based_start) / d) + 1;
        int indx;
        int indx2;

        if (ltype == PathIterator.SEG_QUADTO) { // (interpretation = 2)

            double x2;
            double y2;

            while (ind < size) {
                indx = zero_based_start + (ind * d);
                x = ((NUMBER) ords[indx]).doubleValue();
                y = ((NUMBER) ords[indx + 1]).doubleValue();

                indx2 = zero_based_start + ((ind + 1) * d);

                if (indx >= length) {
                    indx2 = zero_based_start;
                }

                x2 = ((NUMBER) ords[indx2]).doubleValue();
                y2 = ((NUMBER) ords[indx2 + 1]).doubleValue();
                gpx.quadTo(x, y, x2, y2);
                ind++;
                ind++;
            }

            return_following_must_do_first = false;
        }
        else { // PathIterator.SEG_LINETO (interpretation = 1)

            while (ind < size) {
                indx = zero_based_start + (ind * d);
                x = ((NUMBER) ords[indx]).doubleValue();
                y = ((NUMBER) ords[indx + 1]).doubleValue();
                gpx.lineTo(x, y);
                ind++;
            }
        }

        return return_following_must_do_first;
    }

    /**
     * Utility method. Gets FShape type from oracle geometry type.
     * @param otype
     * @return FShape type
     */
    public static int oracleGTypeToFShapeType(int otype) {
        switch (otype) {
        case ORACLE_GTYPE_UNKNOWN:
            return FShape.NULL;

        case ORACLE_GTYPE_POINT:
        case ORACLE_GTYPE_MULTIPOINT:
            return FShape.POINT;

        case ORACLE_GTYPE_LINE:
        case ORACLE_GTYPE_MULTILINE:
            return FShape.LINE;

        case ORACLE_GTYPE_POLYGON:
        case ORACLE_GTYPE_MULTIPOLYGON:
            return FShape.POLYGON;

        case ORACLE_GTYPE_COLLECTION:
            return FShape.MULTI;
        }

        logger.warn("Unknown oracle geometry type: " + otype);

        return FShape.NULL;
    }

    /**
     * Utility method to get struct's type.
     * @param the_data the struct's datum array
     * @return the struct type
     */
    public static int getStructType(Datum[] the_data) {
        int resp = -1;

        try {
            resp = ((NUMBER) the_data[0]).intValue() % 1000;
        }
        catch (SQLException se) {
            logger.error("Error: " + se.getMessage(), se);
        }

        return resp;
    }

    /**
     * Utility method to get struct's SRID.
     * @param the_data the struct's datum array
     * @return the struct0's SRID
     */
    public static int getStructSRID(Datum[] the_data) {
        int resp = -1;

        try {
            resp = ((NUMBER) the_data[1]).intValue();
        }
        catch (SQLException se) {
            logger.error("Error: " + se.getMessage(), se);
        }

        return resp;
    }

    /**
     * Utility method to find out if  a struct is a circle.
     *
     * @param the_data the struct's datum array
     * @return whether it is a circle
     */
    public static boolean isCircle(Datum[] the_data) {
        int[] info = null;

        try {
            info = ((ARRAY) the_data[3]).getIntArray();
        }
        catch (SQLException se) {
            logger.error("While cheking circle: " + se.getMessage(), se);

            return false;
        }

        if (info == null) {
            return false;
        }

        boolean resp = ((info.length == 3) && (info[2] == 4));

        return resp;
    }

    /**
     * Gets the struct's dimension size.
     * @param st the struct
     * @return the structs dimension
     */
    public static int getStructDimensions(STRUCT st) {
        int resp = -1;

        try {
            resp = ((NUMBER) st.getOracleAttributes()[0]).intValue() / 1000;
        }
        catch (SQLException se) {
            logger.error("Error: " + se.getMessage(), se);
        }

        if (resp < 2) {
            resp = 2;
        }

        return resp;
    }

    /**
     * Gets a struct's coordinates array.
     * @param the_data the struct's datum array
     * @return the coordinates array
     */
    public static double[] getOrds(Datum[] the_data) {
        double[] resp = null;

        try {
            ARRAY aux = (ARRAY) the_data[4];

            if (aux == null) {
                return null;
            }

            resp = aux.getDoubleArray();
        }
        catch (SQLException se) {
            logger.error("While getting ordinates: " + se.getMessage(), se);
        }

        return resp;
    }

    /**
     * Utility method to create a struct with the given data.
     * @param type struct type
     * @param srid coordinate system
     * @param info element info array
     * @param ords coordinates array
     * @param conn connection
     * @return the created struct
     */
    public static STRUCT createStruct(NUMBER type, NUMBER srid, Datum[] info,
        Datum[] ords, Connection conn) {
        try {
            StructDescriptor dsc = StructDescriptor.createDescriptor("MDSYS.SDO_GEOMETRY",
                    conn);
            Object[] obj = new Object[5];
            obj[0] = type;
            obj[1] = srid;
            obj[2] = null;
            obj[3] = info;
            obj[4] = ords;

            return new STRUCT(dsc, conn, obj);
        }
        catch (SQLException se) {
            logger.error("While creating STRUCT: " + se.getMessage(), se);
        }

        return null;
    }

    public static String getDimInfoAsString(ARRAY dim_info) {
    	String resp = "DIMENSIONS: ";

        if (dim_info == null) {
            return "NULL" + "\n";
        }
        else {
        	try {
				Datum[] da = dim_info.getOracleArray();
				int size = da.length;
				resp = resp + size + "\n";
				for (int i = 0; i < size; i++) {
					STRUCT dim_itemx = (STRUCT) da[i];
					Object[] dim_desc = dim_itemx.getAttributes();
					resp = resp + "DIMENSION " + i + ": " + ", NAME: "
							+ dim_desc[0].toString() + ", MIN: "
							+ dim_desc[1].toString() + ", MAX: "
							+ dim_desc[2].toString() + ", TOL: "
							+ dim_desc[3].toString();
					if (i != (size -1)) {
						resp = resp + "\n";
					}
				}
			} catch (Exception ex) {
				return "ERROR: " + ex.getMessage() + "\n";
			}
        }
        return resp;
    }

    public static STRUCT reprojectGeometry(IConnection conn, STRUCT fromStruct, String toSrid) {

    	String qry = "SELECT SDO_CS.TRANSFORM( ?, " + toSrid + ") FROM DUAL";
    	STRUCT resp = null;

    	try {
			PreparedStatement _st = ((ConnectionJDBC)conn).getConnection().prepareStatement(qry);
			_st.setObject(1, fromStruct);
			ResultSet _rs = _st.executeQuery();

			if (_rs.next()) {
				resp = (STRUCT) _rs.getObject(1);
			} else {
				logger.error("While executing reprojection: empty resultset (?)");
				return fromStruct;
			}
		} catch (Exception ex) {
			logger.error("While reprojecting: " + ex.getMessage());
			return fromStruct;
		}

        if (resp == null) {
        	return fromStruct;
        } else {
        	return resp;
        }
    }

}
