/*
 * Created on 17-feb-2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.rasterTools.saveRaster.util;

import org.cresques.io.GeoRasterWriter;

/**
 * Clase que representa a las propiedades que se seleccionan de cada driver.
 * Inicialmente se resetea el array driversProps a null. Cada elemento de este
 * array representa una extensin de fichero soportada. Cuando se modifica
 * las propiedades para una determinada extensin de un driver estas se asignan
 * a su entrada del vector driversExtensionProps y desde aqu son leidas para
 * ser restauradas en caso de volver el cuadro de dialogo de propiedades.
 *  
 * @author Nacho Brodin (brodin_ign@gva.es)
 *
 */
public class DriverProperties{
		
	private String[][]				driversExtensionProps = null;
	private String[]				driversExtension = null;
		
	public DriverProperties(){
			
		driversExtensionProps = new String[GeoRasterWriter.getNDrivers()][];
		for(int i=0;i<GeoRasterWriter.getNDrivers();i++)
			driversExtensionProps[i] = null;
		driversExtension = GeoRasterWriter.getDriversExtensions();
			
	}
		
	/**
	 * Asigna un vector de propiedades de driver de escritura.
	 * @param ext	extensin del driver.
	 * @param props	propiedades.
	 */
	public void setProperties(String ext, String[] props){
		for(int i=0;i<driversExtension.length;i++){
			if(ext.equals(driversExtension[i])){
				driversExtensionProps[i] = props;
			}
		}
	}
		
	/**
	 * Obtiene un vector de propiedades de driver de escritura.
	 * @param ext	extensin del driver.
	 * @return	propiedades.
	 */
	public String[] getProperties(String ext){
		for(int i=0;i<driversExtension.length;i++){
			if(ext.equals(driversExtension[i])){
				return driversExtensionProps[i];
			}
		}
		return null;
	}
	
	/**
	 * Para una extensin de fichero determinada obtiene el valor de una propiedad si existe este.
	 * La extensin solicitadad deber estar registrada
	 * @param extension Extensin del fichero.
	 * @param property Propiedad a recuperar.
	 * @return Valor de la propiedad o null en caso de que la extensin de fichero no exista o la propiedad
	 * no se encuentre.
	 */
	public String getValue(String extension, String property) {
		String[] props = getProperties(extension);
		if(props != null){
			for (int i = 0; i < props.length; i++) {
				if(props[i].startsWith(property + "="))
					return props[i].substring(props[i].lastIndexOf("=") + 1, props[i].length());
			}
		}
		return null;
	}
		
}