package com.iver.cit.gvsig.gui.panels;

import java.lang.reflect.InvocationTargetException;
import java.util.Map;
import java.util.prefs.Preferences;

import javax.swing.JOptionPane;
import javax.swing.JTabbedPane;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.gvsig.remoteClient.wfs.WFSStatus;
import org.gvsig.remoteClient.wms.ICancellable;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.drivers.wfs.FMapWFSDriver;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLyrWFS;
import com.iver.cit.gvsig.fmap.layers.WFSLayerNode;
import com.iver.cit.gvsig.gui.WizardPanel;
import com.iver.cit.gvsig.gui.dialogs.WFSPropsDialog;
import com.iver.cit.gvsig.gui.wizards.WFSWizard;
import com.iver.cit.gvsig.gui.wizards.WFSWizardData;
import com.iver.cit.gvsig.gui.wizards.WizardListener;
import com.iver.cit.gvsig.gui.wizards.WizardListenerSupport;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: WFSParamsPanel.java 18381 2008-01-30 12:00:25Z jpiera $
 * $Log$
 * Revision 1.10.2.8  2007-06-26 09:33:07  jorpiell
 * Refreshing the information tab always a change happens
 *
 * Revision 1.10.2.7  2007/01/08 09:52:31  ppiqueras
 * Corridos bugs + nueva funcionalidad.
 *
 * Revision 1.28  2006/12/29 09:27:02  ppiqueras
 * Corregidos varios bugs:
 *
 * - Se ejecuta 2 veces "Aplicar" o "Aceptar" en los filtros WFS.
 * - Cuando se carga por primera vez una capa WFS, si se pone un
 * filtro incorrecto, se ignora el filtro (cargando toda la información posible de la capa, y borrando el texto de filtrado).
 *
 * Revision 1.27  2006/12/26 09:19:40  ppiqueras
 * Cambiado "atttibutes" en todas las aparaciones en atributos, métodos, clases, paquetes o comentarios por "fields". (Sólo a aquellas que afectan a clases dentro del proyecto extWFS2).
 *
 * Revision 1.26  2006/12/20 14:22:06  ppiqueras
 * Añadido comentario
 *
 * Revision 1.25  2006/12/15 13:59:36  ppiqueras
 * -Permite que se almacenen y/o recojan todos los campos y valores conocidos de la capa actual.
 *
 * -Además, control frentre a consultas de filtro erróneas, (esto a medias aún).
 *
 * - Algún comentario más.
 *
 * Revision 1.24  2006/12/12 10:24:45  ppiqueras
 * Nueva funcionalidad: Pulsando doble 'click' sobre una capa de un servidor, se carga (igual que antes), pero además se avanza a la siguiente pestaña sin tener que pulsar el botón 'Siguiente'.
 *
 * Revision 1.23  2006/12/11 11:02:24  ppiqueras
 * Corregido bug -> que se mantenga la frase de filtrado
 *
 * Revision 1.22  2006/12/04 08:59:47  ppiqueras
 * Algunos bugs corregidos. A cambio hay 2 bugs relacionados que todavía no han sido corregidos (ver PHPCollab) (los tiene asignados Jorge).
 *
 * Revision 1.21  2006/11/28 08:05:13  jorpiell
 * Se escribe la query en la pestaa del filtro
 *
 * Revision 1.20  2006/11/14 13:45:49  ppiqueras
 * Añadida pequeña funcionalidad:
 * Cuando se pulsa el botón "Aplicar", (al seleccionar un nuevo filtro), si el árbol de campos posee alguno seleccionado, actualiza los valores (con los nuevos), asociados a éste campo seleccionado.
 *
 * Revision 1.19  2006/10/27 12:10:02  ppiqueras
 * Nueva funcionalidad
 *
 * Revision 1.16  2006/10/23 07:37:04  jorpiell
 * Ya funciona el filterEncoding
 *
 * Revision 1.14  2006/10/13 13:05:32  ppiqueras
 * Permite el refrescado de datos del panel de filtrado sobre capa WFS.
 *
 * Revision 1.13  2006/10/10 12:55:06  jorpiell
 * Se ha aadido el soporte de features complejas
 *
 * Revision 1.12  2006/10/02 09:17:48  jorpiell
 * Aadido el setCRS a la capa
 *
 * Revision 1.11  2006/09/29 13:02:38  ppiqueras
 * Filtro para WFS. De momento sólo interfaz gráfica.
 *
 * Revision 1.10  2006/07/21 11:50:31  jaume
 * improved appearance
 *
 * Revision 1.9  2006/06/21 12:35:45  jorpiell
 * Se ha aadido la ventana de propiedades. Esto implica aadir listeners por todos los paneles. Adems no se muestra la geomatra en la lista de atributos y se muestran nicamnete los que se van a descargar
 *
 * Revision 1.8  2006/06/15 07:50:58  jorpiell
 * Aadida la funcionalidad de reproyectar y hechos algunos cambios en la interfaz
 *
 * Revision 1.7  2006/05/25 16:22:47  jorpiell
 * Se limpia el panel cada vez que se conecta con un servidor distinto
 *
 * Revision 1.6  2006/05/25 16:01:43  jorpiell
 * Se ha aadido la funcionalidad para eliminar el namespace de los tipos de atributos
 *
 * Revision 1.5  2006/05/25 10:31:06  jorpiell
 * Como ha cambiado la forma de mostrar las capas (una tabla, en lugar de una lista), los paneles han tenido que ser modificados
 *
 * Revision 1.4  2006/05/23 08:09:39  jorpiell
 * Se ha cambiado la forma en la que se leian los valores seleccionados en los paneles y se ha cambiado el comportamiento de los botones
 *
 * Revision 1.3  2006/05/19 12:57:08  jorpiell
 * Modificados algunos paneles
 *
 * Revision 1.2  2006/04/20 16:38:24  jorpiell
 * Ahora mismo ya se puede hacer un getCapabilities y un getDescribeType de la capa seleccionada para ver los atributos a dibujar. Queda implementar el panel de opciones y hacer el getFeature().
 *
 * Revision 1.1  2006/04/19 12:50:16  jorpiell
 * Primer commit de la aplicacin. Se puede hacer un getCapabilities y ver el mensaje de vienvenida del servidor
 *
 *
 */
/**
 * A JPanel with a JTabbedPane with JPanels for WFS
 * 
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 * @author Pablo Piqueras Bartolom (p_queras@hotmail.com)
 */
public class WFSParamsPanel extends WizardPanel{
	private JTabbedPane jTabbedPane = null;  //  @jve:decl-index=0:visual-constraint="37,30"
	private WFSSelectFeaturePanel featurePanel = null;
	private WFSSelectAttributesPanel fieldsPanel = null;
	private WFSOptionsPanel optionsPanel = null;
	private WFSFormatPanel formatPanel = null;
	private WFSFilterPanel filterPanel = null;
	private WFSAreaPanel areaPanel = null;
	private WFSWizardData data;
	private WizardListenerSupport listenerSupport;
	public	static Preferences fPrefs = Preferences.userRoot().node( "gvsig.wcs-wizard" );
	private WFSInfoPanel infoPanel;
	private int indFields;
	private int indFilter;
	private int indArea;	
	private RefreshDataThread refreshDataThread = null;

	/**
	 * This method initializes jTabbedPane
	 *
	 * @return javax.swing.JTabbedPane
	 */

	public WFSParamsPanel(){
		super();
		initialize();
	}
	/**
	 * Returns the tab amount that the WFSParamsPanel currently have
	 *
	 * Devuelve el nmero de solapas que tiene actualmente el WFSParamsPanel
	 *
	 * @return int
	 */
	public int getNumTabs(){
		return getJTabbedPane().getTabCount();
	}

	private void initialize() {
		this.setLayout(null);
		this.setVisible(false);
		this.setBounds(0, 0, 510, 427);
		this.add(getJTabbedPane(), null);
	}

	/**
	 * Sets the focus to the tab next to the current one.
	 *
	 * Enfoca a la solapa siguiente a la actualmente enfocada del TabbedPane
	 *
	 */
	public void avanzaTab(){
		int currentPage = currentPage();
		int nPages = getNumTabs();
		if (nPages -1 > currentPage){
			getJTabbedPane().setSelectedIndex(nextPageEnabled());
		}
	}

	/**
	 * Sets the focus to the tab previous to the current one.
	 *
	 * Enfoca a la solapa anterior a la actualmente enfocada del TabbedPane
	 *
	 */
	public void retrocedeTab(){
		this.getJTabbedPane().setSelectedIndex(previousEnabledPage());
	}

	/**
	 * Returns the current tab index.
	 *
	 * Devuelve el nmero de solapa en que se encuentra
	 *
	 * @return
	 */
	public int getIndiceSolapaActual(){
		return this.getJTabbedPane().getSelectedIndex();
	}

	public JTabbedPane getJTabbedPane() {
		if (jTabbedPane == null) {
			jTabbedPane = new JTabbedPane();
			jTabbedPane.setBounds(4, 4, 502, 415);
			jTabbedPane.addTab(PluginServices.getText(this, "info"), null, getInfoPanel());
			jTabbedPane.addTab(PluginServices.getText(this, "feature"), null, getFeaturesPanel(), null);
			jTabbedPane.addTab(PluginServices.getText(this, "attributes"),null, getAttributesPanel(), null);
			indFields = jTabbedPane.getTabCount() - 1;
			jTabbedPane.setEnabledAt(indFields, false);
			jTabbedPane.addTab(PluginServices.getText(this, "options"),null, getOptionsPanel(), null);
			jTabbedPane.addTab(PluginServices.getText(this, "filter"),null, getFilterPanel(), null);
			indFilter = jTabbedPane.getTabCount() - 1;
			jTabbedPane.setEnabledAt(indFilter, false);
			indArea = jTabbedPane.getTabCount() - 1;
			jTabbedPane.setEnabledAt(indArea, false);
			jTabbedPane.addChangeListener(new ChangeListener() {
				// This method is called whenever the selected tab changes
				public void stateChanged(ChangeEvent evt) {
					JTabbedPane pane = (JTabbedPane)evt.getSource();
					// Get current tab
					int sel = pane.getSelectedIndex();
					if (sel == 0){
						refreshInfo();
					}
				}     


			});
		}
		return jTabbedPane;
	}

	private WFSInfoPanel getInfoPanel() {
		if (infoPanel==null) {
			infoPanel = new WFSInfoPanel();			
		}
		return infoPanel;
	}

	/**
	 *
	 * @return the selected list
	 */
	private Object getSelectedFeature() {
		return featurePanel.getSelectedFeature();
	}


	/**
	 * This method initializes panelFeatures
	 *
	 * @return javax.swing.JPanel
	 */
	private WFSSelectFeaturePanel getFeaturesPanel() {
		if (featurePanel == null) {
			featurePanel = new WFSSelectFeaturePanel(this);
			featurePanel.setLayout(null);
		}
		return featurePanel;
	}

	/**
	 * This method initializes fieldsPanel
	 *
	 * @return javax.swing.JPanel
	 */
	private WFSSelectAttributesPanel getAttributesPanel(){
		if (fieldsPanel == null) {
			fieldsPanel = new WFSSelectAttributesPanel(this);
			fieldsPanel.setLayout(null);
		}
		return fieldsPanel;
	}



	/**
	 * This method initializes fieldsPanel
	 *
	 * @return javax.swing.JPanel
	 */
	private WFSOptionsPanel getOptionsPanel(){
		if (optionsPanel == null) {
			optionsPanel = new WFSOptionsPanel(this);
			optionsPanel.setLayout(null);
		}
		return optionsPanel;
	}


	/**
	 * This method initializes filterPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private WFSFilterPanel getFilterPanel(){
		if (filterPanel == null) {
			filterPanel = new WFSFilterPanel(this);
			filterPanel.setVisible(true);			
		}
		return filterPanel;
	}

	/**
	 * This method initializes areaPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private WFSAreaPanel getAreaPanel(){
		if (areaPanel == null) {
			areaPanel = new WFSAreaPanel(this);
			areaPanel.setVisible(true);			
		}
		return areaPanel;
	}

	/**
	 * This method initializes formatPanel
	 *
	 * @return javax.swing.JPanel
	 */
	private WFSFormatPanel getFormatPanel(){
		if (formatPanel == null) {
			formatPanel = new WFSFormatPanel();
			formatPanel.setLayout(null);
		}
		return formatPanel;
	}

	/**
	 * Verifies that the selected parameters are enough to request the coverage
	 * to the server.
	 *
	 * Comprueba que los parmetros seleccionados son suficientes para pedir la
	 * cobertura al servidor.
	 *
	 * @return boolean
	 */
	public boolean isCorretlyConfigured() {
		if (featurePanel.getSelectedFeature()!= null){
			return true;
		}else{
			return false;
		}
	}

	/**
	 * Cleans up the wizard's components but the server's layers list.
	 *
	 * Limpia todos los componentes del wizard excepto la lista de capas del
	 * servidor.
	 */
	public void cleanupWizard() {
		jTabbedPane.setEnabledAt(indFields,false);
//		jTabbedPane.setEnabledAt(indFormat,false);
	}

	/**
	 * Refreshes the wizard components data each time a feature
	 * is selected.
	 *
	 * Actualiza los datos de los componentes del wizard cada vez
	 * que se selecciona una feature diferente.
	 *
	 * @param feature
	 * 
	 * Selected feature
	 */
	public void refreshData(WFSLayerNode feature){
		if (refreshDataThread != null){
			refreshDataThread.setCanceled(true);
		}
		refreshDataThread = new RefreshDataThread(feature);	
		refreshDataThread.start();
	}
	
	/**
	 * Refresh all the panels
	 * @param feature
	 */
	private void refreshAllPanels(WFSLayerNode feature){
		jTabbedPane.setEnabledAt(indFields,true);
		jTabbedPane.setEnabledAt(indFilter,true);
		jTabbedPane.setEnabledAt(indArea,true);
		getFeaturesPanel().refresh(feature);
		getAttributesPanel().refresh(feature);
		getFilterPanel().refresh(feature);
		getOptionsPanel().refresh(feature);			
	}

	/**
	 * Refresh the info panel
	 */
	public void refreshInfo() {
		WFSLayerNode selectedNode = featurePanel.getSelectedFeature();
		if (selectedNode != null){
			selectedNode.setSelectedFields(fieldsPanel.getSelectedFields());
		}
		data.setUserName(optionsPanel.getUserName());
		data.setBuffer(optionsPanel.getBuffer());
		data.setTimeOut(optionsPanel.getTimeout());
		infoPanel.refresh(data,selectedNode);
	}

	/**
	 * Sets the object that holds the wizard data.
	 *
	 * Establece el objeto que contiene los datos del wizard.
	 *
	 * @param data
	 */
	public void setWizardData(WFSWizardData data) {
		this.data = data;
		//WFSLayerNode selectedNode = (WFSLayerNode)featurePanel.getSelectedFeature();
		featurePanel.setWizardData(data);
		data.setBuffer(optionsPanel.getBuffer());
		data.setTimeOut(optionsPanel.getTimeout());
		infoPanel.refresh(data,null);
	}

	/**
	 * Sets the wizard's data source
	 *
	 * Establece el origen de los datos del wizard
	 * @param dataSource
	 */
	public void setDataSource(WFSWizardData dataSource) {
		this.data = dataSource;

	}


	/**
	 * Returns the next enabled tab's index.
	 *
	 * Devuelve el ndicie de la siguiente pestaa habilitada del wizard o -1 si no hay
	 * ninguna.
	 *
	 * @return The index or -1 if there is no one.
	 */
	public int nextPageEnabled() {
		int currentPage = currentPage();
		int nPages = getNumTabs();
		if (currentPage == nPages)
			return -1;
		for (int i=currentPage+1; i<nPages; i++){
			if (getJTabbedPane().isEnabledAt(i)){
				return i;
			}
		}
		return -1;
	}

	/**
	 * Returns the index of the current tab.
	 *
	 * Devuelve el ndice de la pgina actual del wizard.
	 *
	 * @return
	 */
	public int currentPage() {
		return getJTabbedPane().getSelectedIndex();
	}

	/**
	 * Returns the index of the previous enabled tab.
	 *
	 * Devuelve el ndice de la anterior pgina habilitada del wizard o -1 si no hay
	 * ninguna.
	 *
	 * @return The index, or -1 if there is no one.
	 */
	public int previousEnabledPage() {
		int currentPage = currentPage();
		int j=0;
		if (currentPage == 0)
			j= -1;
		for (int i = currentPage-1; i>-1; i--){
			if (getJTabbedPane().isEnabledAt(i)){
				j= i;
				break;
			}
		}
		return j;
	}

	public FMapWFSDriver getDriver() {
		return data.getDriver();
	}

	public void setListenerSupport(WizardListenerSupport support) {
		listenerSupport = support;
	}


	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.gui.WizardPanel#initWizard()
	 */
	public void initWizard() {

	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.gui.WizardPanel#execute()
	 */
	public void execute() {

	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.gui.ILayerPanel#getLayer()
	 */
	public FLayer getLayer() {
		FLyrWFS layer = new FLyrWFS();
		layer.setName(getFeaturesPanel().getLayerText());
		layer.setLayerName(getFeaturesPanel().getLayerName());
		layer.setWfsLayerNode(getLayerNode());
		layer.setFields(getAttributesPanel().getSelectedFields());
		layer.setUserName(getOptionsPanel().getUserName());
		layer.setPassword(getOptionsPanel().getPassword());
		layer.setNumfeatures(getOptionsPanel().getBuffer());
		layer.setTimeout(getOptionsPanel().getTimeout());
		layer.setSrs(getOptionsPanel().getSRS());

		// Set the filter subconsultation that layer is
		layer.setFieldsQuery(getLayerNode().getFilter());

		layer.setVisualFilterQueyr(getFilterPanel().getFilterExpressionFromInterface());
		layer.setWfsDriver(data.getDriver());
		return layer;
	}

	private WFSLayerNode getLayerNode(){
		WFSLayerNode layerNode = getFeaturesPanel().getWFSLayerNode();
		layerNode.setSelectedFields(getAttributesPanel().getSelectedFields());

		// If the query is incorrect -> no query (no filter)
		String query = getFilterPanel().getQuery();
		if (query == null) {
			layerNode.setFilter("");

			// Removes filter expression from the JTextArea
			getFilterPanel().removeFilterExpression();

			// Notify to user that no filter will be applied
			JOptionPane.showMessageDialog(null, PluginServices.getText(null, "no_filter_will_be_applied_in_the_load_of_the_layer"), PluginServices.getText(null, "warning"), JOptionPane.WARNING_MESSAGE);
		}
		else
			layerNode.setFilter(query);

		return layerNode;
	}


	public void setSelectedFeature(WFSLayerNode layerNode){
		getFeaturesPanel().setSelectedFeature(layerNode);
		getAttributesPanel().setSelectedFields(layerNode);
//		getFilterPanel().setFilterExpressionIntoInterface(layerNode.getFilter());
	}	


	/**
	 * @return Returns the data.
	 */
	public WFSWizardData getData() {
		return data;
	}
	public void setLayerName(String name) {
		getFeaturesPanel().setLayerName(name);

	}

	public void addWizardListener(WizardListener listener) {
		listenerSupport.addWizardListener(listener);
	}

	public void isApplicable(boolean applicable){
		listenerSupport.callStateChanged(applicable);
	}


	public void setStatus(WFSStatus status) {
		getOptionsPanel().setStatus(status);
		getFilterPanel().setFilterExpressionIntoInterface(status.getFilterVisualText());
	}

	/**
	 * 
	 *
	 */
	public void updateWFSFilterFieldValues()  {
		filterPanel.updateFieldValues();
	}

	/**
	 * @see WFSFilterPanel#setWFSFilterPanelIsAsTabForWFSLayersLoad(boolean)
	 */
	public void setWFSFilterPanelIsAsTabForWFSLayersLoad(boolean b) {
		this.getFilterPanel().setWFSFilterPanelIsAsTabForWFSLayersLoad(b);
	}

	/**
	 * @see WFSFilterPanel#getWFSFilterPanelIsAsTabForWFSLayersLoad()
	 */
	public boolean getWFSFilterPanelIsAsTabForWFSLayersLoad() {
		return this.getFilterPanel().getWFSFilterPanelIsAsTabForWFSLayersLoad();
	}

	/**
	 * @see WFSSelectAttributesPanel#getFieldsSelectedOfSameLayerHasChanged()
	 */
	public boolean getFieldsSelectedOfSameLayerHasChanged() {
		return this.getAttributesPanel().getFieldsSelectedOfSameLayerHasChanged();
	}

	/**
	 * @see WFSSelectAttributesPanel#resetFieldsSelectedOfSameLayerHasChanged()
	 */
	public void resetFieldsSelectedOfSameLayerHasChanged() {
		this.getAttributesPanel().resetFieldsSelectedOfSameLayerHasChanged();
	}

	/**
	 * @see WFSWizard#doClickOnNextButton()
	 */
	public void doClickOnNextButton() {
		Object obj = this.getParent();

		// When we start to connect to a server -> the interface is the dialog WFSWizard
		if (obj instanceof WFSWizard)
			((WFSWizard)this.getParent()).doClickOnNextButton();
		else {
			// When we are modifying a loaded layer -> the interface is the dialog WFSPropsDialog
			if (obj instanceof WFSPropsDialog)
				this.advanceCurrentTab();
		}
	}

	/**
	 * Selects the next tab
	 */
	public void advanceCurrentTab() {
		int currentTabIndex = this.getJTabbedPane().getSelectedIndex();

		// Ensure we can select next tab
		if ((currentTabIndex != -1) && (currentTabIndex < (this.getJTabbedPane().getTabCount() -1)))
			this.getJTabbedPane().setSelectedIndex(currentTabIndex + 1);
	}

	/**
	 * @see WFSFilterPanel#getAllFieldsAndValuesKnownOfCurrentLayer(Map)
	 */
	public Map getAllFieldsAndValuesKnownOfCurrentLayer() {
		return this.getFilterPanel().getAllFieldsAndValuesKnownOfCurrentLayer();
	}

	/**
	 * @see WFSFilterPanel#setAllFieldsAndValuesKnownOfCurrentLayer(Map)
	 */
	public void setAllFieldsAndValuesKnownOfCurrentLayer(Map allFieldsAndValuesKnownOfCurrentLayer) {
		this.getFilterPanel().setAllFieldsAndValuesKnownOfCurrentLayer(allFieldsAndValuesKnownOfCurrentLayer);
	}

	/**
	 * @see WFSFilterPanel#getQuery()
	 */
	public String getFilterQuery() {
		return this.getFilterPanel().getQuery();
	}

	/**
	 * It refresh all the WFS wizard in a separate thread
	 * @author Jorge Piera Llodr (jorge.piera@iver.es)
	 */
	private class RefreshDataThread extends Thread implements ICancellable{
		private WFSLayerNode feature = null;		
		private boolean isCanceled = false;

		public RefreshDataThread(WFSLayerNode feature) {
			super();
			this.feature = feature;
		}

		/*
		 * (non-Javadoc)
		 * @see java.lang.Thread#run()
		 */
		public void run() {
			if (feature != null){
				//Make the other panels empty
				WFSLayerNode auxNode = new WFSLayerNode();
				fieldsPanel.refresh(auxNode);
				optionsPanel.refresh(auxNode);
//				filterPanel.resetFieldsAndValuesData();
				//Retrieve the new layer information
				feature = (WFSLayerNode)data.getFeatureInfo(feature.getName(), this);
				if (!isCanceled){
					//Update the panels
					refreshAllPanels(feature);
				}
			}
			if (!isCanceled){
				listenerSupport.callStateChanged(isCorretlyConfigured());
			}
		}

		/*
		 * (non-Javadoc)
		 * @see org.gvsig.remoteClient.wms.ICancellable#getID()
		 */
		public Object getID() {
			return getClass().getName();
		}

		/*
		 * (non-Javadoc)
		 * @see org.gvsig.remoteClient.wms.ICancellable#isCanceled()
		 */
		public boolean isCanceled() {
			return isCanceled;
		}

		/**
		 * @param isCanceled the isCanceled to set
		 */
		public void setCanceled(boolean isCanceled) {
			this.isCanceled = isCanceled;
		}
	}
}
