package com.iver.cit.gvsig.gui.panels;

import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.event.ListSelectionListener;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.layers.WFSLayerNode;
import com.iver.cit.gvsig.gui.wizards.WFSWizardData;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: WFSSelectFeaturePanel.java 18381 2008-01-30 12:00:25Z jpiera $
 * $Log$
 * Revision 1.7.2.3  2006-12-04 09:39:34  ppiqueras
 * 1 Bug corregido
 *
 * Revision 1.9  2006/12/04 08:59:47  ppiqueras
 * Algunos bugs corregidos. A cambio hay 2 bugs relacionados que todavía no han sido corregidos (ver PHPCollab) (los tiene asignados Jorge).
 *
 * Revision 1.8  2006/10/27 06:44:56  jorpiell
 * Se han cambiado algunas etiquetas de texto que salan recortadas
 *
 * Revision 1.7  2006/07/21 11:50:31  jaume
 * improved appearance
 *
 * Revision 1.6  2006/06/21 12:35:45  jorpiell
 * Se ha aadido la ventana de propiedades. Esto implica aadir listeners por todos los paneles. Adems no se muestra la geomatra en la lista de atributos y se muestran nicamnete los que se van a descargar
 *
 * Revision 1.5  2006/05/25 10:31:06  jorpiell
 * Como ha cambiado la forma de mostrar las capas (una tabla, en lugar de una lista), los paneles han tenido que ser modificados
 *
 * Revision 1.4  2006/05/23 08:09:39  jorpiell
 * Se ha cambiado la forma en la que se leian los valores seleccionados en los paneles y se ha cambiado el comportamiento de los botones
 *
 * Revision 1.3  2006/05/19 12:57:08  jorpiell
 * Modificados algunos paneles
 *
 * Revision 1.2  2006/04/20 16:38:24  jorpiell
 * Ahora mismo ya se puede hacer un getCapabilities y un getDescribeType de la capa seleccionada para ver los atributos a dibujar. Queda implementar el panel de opciones y hacer el getFeature().
 *
 *
 */
/**
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class WFSSelectFeaturePanel extends JPanel {
	private JPanel coveragesListPanel = null;
	private JScrollPane jScrollPane = null;
	private LayerTable lstFeatures = null;
	private JTextField txtName = null;
	private JCheckBox chkExtendedNames = null;
	private WFSWizardData data = null;
	private WFSParamsPanel parent = null;
	private JPanel layerNamePanel = null;
	private WFSLayerNode currentLayerNode = null;
	

	/**
	 * This method initializes
	 *
	 */
	public WFSSelectFeaturePanel(WFSParamsPanel parent) {
		super();
		this.parent = parent;
		initialize();
	}

	/**
	 * This method initializes this
	 *
	 */
	private void initialize() {
		this.setLayout(null);
		this.setBounds(10, 5, 481, 427);
		this.add(getFeaturesListPanel(), null);
		this.add(getChkExtendedNames(), null);
		this.add(getLayerNamePanel(), null);

	}

	/**
	 * This method initializes coveragesListPanel
	 *
	 * @return javax.swing.JPanel
	 */
	private JPanel getFeaturesListPanel() {
		if (coveragesListPanel == null) {
			coveragesListPanel = new JPanel();
			coveragesListPanel.setLayout(null);
			coveragesListPanel.setLocation(new java.awt.Point(7,57));
			coveragesListPanel.setSize(new java.awt.Dimension(482,302));
			coveragesListPanel.add(getJScrollPane(), null);
			coveragesListPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(
					null, PluginServices.getText(this, "select_features"),
					javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
					javax.swing.border.TitledBorder.DEFAULT_POSITION, null, null));
		}
		return coveragesListPanel;
	}

	/**
	 * This method initializes jScrollPane
	 *
	 * @return javax.swing.JScrollPane
	 */
	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane();
			jScrollPane.setBounds(6, 17, 469, 280);
			jScrollPane.setViewportView(getLstFeatures());
		}
		return jScrollPane;
	}

	/**
	 * This method initializes jList1
	 *
	 * @return javax.swing.JList
	 */
	public LayerTable getLstFeatures() {
		if (lstFeatures == null) {
			lstFeatures = new LayerTable();
			lstFeatures.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
			lstFeatures.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
				public void valueChanged(javax.swing.event.ListSelectionEvent e) {
					if ((currentLayerNode==null) || 
							(currentLayerNode != (WFSLayerNode)getLstFeatures().getSelectedValue())){
						currentLayerNode = (WFSLayerNode)getLstFeatures().getSelectedValue();
						refreshData();
						// Indicate that now we are loading a new layer
						parent.setWFSFilterPanelIsAsTabForWFSLayersLoad(true);
					}					
				}
			});
		}
		return lstFeatures;
	}
	
	/**
	 * This method initializes jTextField
	 *
	 * @return javax.swing.JTextField
	 */
	public JTextField getTxtName() {
		if (txtName == null) {
			txtName = new JTextField();
			txtName.setBounds(6, 19, 469, 20);
			txtName.setText(PluginServices.getText(this, "WFSLayer"));
			txtName.addKeyListener(new java.awt.event.KeyAdapter() {
				public void keyTyped(java.awt.event.KeyEvent e) {
					parent.isApplicable(true);
				}
			});
		}
		return txtName;
	}
	/**
	 * This method initializes chkExtendedNames
	 *
	 * @return javax.swing.JCheckBox
	 */
	private JCheckBox getChkExtendedNames() {
		if (chkExtendedNames == null) {
			chkExtendedNames = new JCheckBox();
			chkExtendedNames.setText(PluginServices.getText(this, "show_layer_names"));
			chkExtendedNames.setBounds(10, 368, 380, 20);
			chkExtendedNames.addItemListener(new java.awt.event.ItemListener() {
				public void itemStateChanged(java.awt.event.ItemEvent e) {
					boolean b = chkExtendedNames.isSelected();
					getLstFeatures().setShowLayerNames(b);
					getLstFeatures().repaint();
				}
			});
			chkExtendedNames.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
//					fPrefs.putBoolean("show_layer_names", chkExtendedNames.isSelected());
				}
			});
//			chkExtendedNames.setSelected(fPrefs.getBoolean("show_layer_names", false));

		}
		return chkExtendedNames;
	}

	/**
	 *
	 * @return The selected Feature
	 */
	public WFSLayerNode getSelectedFeature(){
		return (WFSLayerNode)getLstFeatures().getSelectedValue();
	}

	public void setSelectedFeature(WFSLayerNode layerNode){
		int index = -1;
		for (int i=0; i<getLstFeatures().getRowCount(); i++){
			WFSLayerNode node = (WFSLayerNode)getLstFeatures().getValueAt(i);
			if (node != null && layerNode.getName().equals(node.getName())){
				index = i;
			}
		}
		if (index != -1){
			getLstFeatures().changeSelection(index,0,false,false);
		}
	}

	/**
	 * Sets the object that holds the wizard data.
	 *
	 * Establece el objeto que contiene los datos del wizard.
	 *
	 * @param data
	 */
	public void setWizardData(WFSWizardData data) {
		this.data = data;
		refreshInfo();
	}

	/**
	 * Refresh the features list
	 *
	 */
	public void refreshInfo(){
		getLstFeatures().addFeatures(data.getFeatures());
	}

	/**
	 * Refreshes the wizard components data each time a feature
	 * is selected. The describeFeatureType operation must be
	 * sent
	 */
	public void refreshData(){
		WFSLayerNode lyr = (WFSLayerNode)getLstFeatures().getSelectedValue();
		try{
			getTxtName().setText(lyr.getTitle());
		}catch(NullPointerException e){
			getTxtName().setText(PluginServices.getText(this,"default_name"));
		}
		parent.refreshData(lyr);
	}
	
	/**
	 * Refresh the 
	 * @param feature
	 */
	public void refresh(WFSLayerNode feature){
		this.repaint();		
	}

	/**
	 * This method initializes layerNamePanel
	 *
	 * @return javax.swing.JPanel
	 */
	private JPanel getLayerNamePanel() {
		if (layerNamePanel == null) {
			layerNamePanel = new JPanel();
			layerNamePanel.setLayout(null);
			layerNamePanel.setBorder(javax.swing.BorderFactory.createTitledBorder(
					null, PluginServices.getText(this, "layer_name"),
					javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
					javax.swing.border.TitledBorder.DEFAULT_POSITION, null, null));
			layerNamePanel.setBounds(new java.awt.Rectangle(7, 4, 482, 49));
			layerNamePanel.add(getTxtName(), null);

		}
		return layerNamePanel;
	}

	/**
	 *
	 * @return the layer name
	 */
	public String getLayerName(){
		return ((WFSLayerNode)getLstFeatures().getSelectedValue()).getName();
	}

	public WFSLayerNode getWFSLayerNode(){
		WFSLayerNode layerNode = (WFSLayerNode)getLstFeatures().getSelectedValue();
		return layerNode;
	}

	/**
	 *
	 * @return the test that will be showed it the gvSIG
	 * toc
	 */
	public String getLayerText(){
		return getTxtName().getText();
	}

	public void setLayerName(String name) {
		getTxtName().setText(name);
	}
}  //  @jve:decl-index=0:visual-constraint="10,16"
