/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.filter;


/**
 * @author Luis W. Sevilla (sevilla_lui@gva.es)
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public interface IRasterFilter {
    /**
     * Acciones a realizar antes de la ejecucin del filtro
     *
     */
    public void pre();

    /**
     * Ejecucin.
     *
     */
    public void execute();

    /**
     * Procesa la posicin x,y de la imagen
     * @param x posicin X
     * @param y posicin Y
     */
    public void process(int x, int y);

    /**
     * Acciones a realizar despus de la ejecucin del filtro
     *
     */
    public void post();

    /**
     * Parmetros obligatorios:
     *         inRaster (IRaster)
     * parmetros obligatorios (si se da el caso)
     *         previousFilter (IRasterFilter)
     *
     * @param name
     * @param value
     */
    public void addParam(String name, Object value);

    /**
     * Devuelve los resultados despues de la ejecucin del filtro.
     * @param name
     * @return
     */
    public Object getResult(String name);

    /**
     * Obtiene el tipo de datos de entrada al filtro
     * @return Tipo de dato
     */
    public int getInRasterDataType();

    /**
     * Obtiene el tipo de datos de salida del filtro
     * @return Tipo de dato
     */
    public int getOutRasterDataType();
}
