/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */

package org.cresques.filter.bands;

import java.awt.Image;
import java.awt.image.BufferedImage;

import org.cresques.io.data.RasterBuf;
import org.cresques.io.datastruct.Palette;
import org.cresques.util.Utilities;

/**
 * Filtro de paleta a aplicar sobre una imagen con tipo de dato image. Para cada elemento
 * del RasterBuf se realiza la conversin de dato a travs de la clase palette que se pasa
 * como parmetro al filtro en la inicializacin.
 * 
 * @author Nacho Brodin (brodin_ign@gva.es)
 *
 */
public class PaletteImageFilter extends PaletteFilter {

	/**
	 * Constructor
	 *
	 */
	public PaletteImageFilter(){
		super();
	}
	
	
	public void pre(){
		this.image = (Image) params.get("raster");
		height = image.getHeight(null);
        width = image.getWidth(null);
		super.pre();
	}
	
	public void process(int col, int line) {
		int px = ((BufferedImage) image).getRGB(col, line);
		int alpha = px & 0xff000000;
		px = (px & 0x00ff0000) >> 16;
		if(interpolate)
			px = palette.getInterpolateRGB2(px);
		else
			px = palette.getRGB(px);
		px = (alpha & (px & 0xff000000)) | (px & 0x00ffffff);
		((BufferedImage) image).setRGB(col, line, px);
	}

	 /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#processSuperSampling(int, int)
     */
	public void processSuperSampling(int col, int line) {
		int px = ((BufferedImage) image).getRGB(col, line);
		int alpha = px & 0xff000000;
		px = (px & 0x00ff0000) >> 16;
		if(interpolate)
			px = palette.getInterpolateRGB2(px);
		else
			px = palette.getRGB(px);
		px = (alpha & (px & 0xff000000)) | (px & 0x00ffffff);
        for(int j = col; j < image.getWidth(null) && j < (col + stepX[contX + 1]); j++)
			for(int i = line; i < image.getHeight(null) && i < (line + stepY[contY + 1]); i++){
				px = (alpha & (px & 0xff000000)) | (px & 0x00ffffff);
				((BufferedImage) image).setRGB(j, i, px);			
			}
	}
	
	public void processLine(int y) {
		// TODO Auto-generated method stub

	}

	public int getInRasterDataType() {
		return RasterBuf.TYPE_IMAGE;
	}

	public int getOutRasterDataType() {
		return RasterBuf.TYPE_IMAGE;
	}

	public Object getResult(String name) {
		if (name.equals("raster")) {
            return (Object) this.image;
        } else {
            return null;
        }
	}

}
