/* gvSIG. Sistema de Informaci�n Geogr�fica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */

package org.cresques.filter.bands;

import java.util.ArrayList;

import org.cresques.filter.IStackManager;
import org.cresques.filter.RasterFilter;
import org.cresques.filter.RasterFilterStack;
import org.cresques.filter.RasterFilterStackManager;
import org.cresques.io.GeoRasterFile;
import org.cresques.io.data.RasterBuf;
import org.cresques.io.datastruct.Palette;

/**
 * Gestor de la pila de filtros para el filtro de brillo.
 * @author Miguel �ngel Querol Carratal�  <querol_mig@gva.es>
 *
 */

public class PaletteStackManager implements IStackManager{
	
	public final static int				palette = 13;
	
	protected RasterFilterStack 		filterStack = null;
	private RasterFilterStackManager	filterStackManager = null;
	
	
	public PaletteStackManager(RasterFilterStackManager filterStackManager){
		this.filterStackManager = filterStackManager;
		this.filterStack = filterStackManager.getFilterStack();
		filterStackManager.addTypeFilter("palette", PaletteStackManager.palette, 5);		
	}
	
	
	/**
	 * Obtiene el valor entero que corresponde al tipo de filtro.
	 * @param rasterFilter
	 * @return
	 */
	public int getType(RasterFilter rasterFilter){
		if(rasterFilter instanceof PaletteFilter)
			return PaletteStackManager.palette;
							
		return -1;
	}
	
	/**
	 * A�ade un filtro de brillo a la pila de filtros.
	 * @param incrBrillo
	 */
	public void addPaletteFilter(Palette palette, boolean interpolate){
		//Si ya hay un filtro de paleta en la pila lo sobreescribimos
		
		if(filterStack.isActive(((Integer) filterStackManager.getTypeFilters().get("palette")).intValue()))
			filterStack.removeFilter(((Integer) filterStackManager.getTypeFilters().get("palette")).intValue());
	
		RasterFilter filtro = null;
		
		switch(filterStack.getDataTypeInFilter(((Integer)this.filterStackManager.getTypeFilters().get("palette")).intValue())){
		case RasterBuf.TYPE_IMAGE:
			filtro = new PaletteImageFilter();
			break;
		case RasterBuf.TYPE_SHORT:
			filtro = new PaletteShortFilter();
			break;
		case RasterBuf.TYPE_USHORT:
			filtro = new PaletteShortFilter();
			break;
		case RasterBuf.TYPE_INT:
			filtro = new PaletteIntFilter();
			break;
		case RasterBuf.TYPE_FLOAT:
			filtro = new PaletteFloatFilter();
			break;
		case RasterBuf.TYPE_DOUBLE:
			filtro = new PaletteDoubleFilter();
			break;
		}
		
		//Cuando el filtro esta creado, tomamos los valores y lo aadimos a la pila
		
		if(filtro != null){
			filtro.addParam("palette", palette);
			filtro.addParam("interpolate", new Boolean (interpolate));
			filterStack.addFilter(((Integer) filterStackManager.getTypeFilters().get("palette")).intValue(), filtro);
			filterStackManager.controlTypes();
		}
	}
	
		
	
	public void createStackFromStrings(ArrayList f, Integer pos) {
		// TODO Auto-generated method stub
		
	}

	/**
     * Crea una pila de filtros a partir de un Array de Strings. Cada elemento del array debe
     * tener la forma elemento=valor.
     * @param filters
     */
	public int createStackFromStrings(ArrayList filters, String fil, GeoRasterFile[] grfList, int filteri) {
		if(fil.startsWith("filter.palette.active")){
			
			boolean exec = true;
			if((RasterFilterStackManager.getValue(fil).equals("false")))
				exec = false;
			
			Palette palette = new Palette();	
			int lengthList = 0;
			boolean interpolate = false;
			
			filters.remove(0);
			for(int prop = 0; prop < filters.size() ; prop++){	
				String elem = (String) filters.get(prop);
				if(elem.startsWith("filter.palette.type")){
					palette.setType(Integer.parseInt(RasterFilterStackManager.getValue(elem)));
					filters.remove(prop);
					prop--;
				}
								
				if(elem.startsWith("filter.palette.values")){
					String[] listString = RasterFilterStackManager.getValue(elem).split(" ");
					int[] listInt = new int[listString.length];
					for(int i = 0; i < listString.length; i++)
						listInt[i] = Integer.parseInt(listString[i]);
					palette.setPalette(listInt);
					filters.remove(prop);
					prop--;
				}
				
				if(elem.startsWith("filter.palette.range")){
					String[] listString = RasterFilterStackManager.getValue(elem).split(" ");
					lengthList = listString.length;
					if(palette.getType() == RasterBuf.TYPE_BYTE || palette.getType() == RasterBuf.TYPE_SHORT || palette.getType() == RasterBuf.TYPE_INT){
						int[] listInt = new int[listString.length];
						for(int i = 0; i < listString.length; i++)
							listInt[i] = Integer.parseInt(listString[i]);	
						palette.setIntRange(listInt);
					}else{
						double[] listDouble = new double[listString.length];
						for(int i = 0; i < listString.length; i++)
							listDouble[i] = Double.parseDouble(listString[i]);	
						palette.setDoubleRange(listDouble);
					}
					filters.remove(prop);
					prop--;
				}
				
				if(elem.startsWith("filter.palette.names")){
					String[] listString = RasterFilterStackManager.getValue(elem).split(" ");
					if(listString.length == 0){
						listString = new String[lengthList];
						for(int i = 0; i < listString.length; i++)
							listString[i] = "";
					}
					palette.setNameClass(listString);
					filters.remove(prop);
					prop--;
				}
				
				if(elem.startsWith("filter.palette.interpolate")){
					interpolate = Boolean.valueOf(RasterFilterStackManager.getValue(elem)).booleanValue();
					filters.remove(prop);
					prop--;
				}
			}	
			this.addPaletteFilter(palette,interpolate);	
			
			PaletteFilter pf = (PaletteFilter)filterStack.getByType(PaletteStackManager.palette);
			pf.setExec(exec);
		}
		//return filteri;
		return -1;
	}

	public ArrayList getStringsFromStack() {
		// TODO Auto-generated method stub
		return null;
	}
	
	/**
	 * Obtiene un Array de Strings a partir de una pila de filtros. Cada elemento del array
	 * tendr� la forma de elemento=valor.
	 */
	
	public ArrayList getStringsFromStack(ArrayList filterList, RasterFilter rf) {
		if(rf instanceof PaletteFilter){
			PaletteFilter paletteFilter = (PaletteFilter)rf;
			//Palette palette = ((PaletteFilter)rf).getPalette();
			Palette palette = (Palette)((PaletteFilter)rf).getParam("palette");
			if (palette !=null){
				if (rf.isExec())
					filterList.add("filter.palette.active=true");
				else
					filterList.add("filter.palette.active=false");
				filterList.add("filter.palette.type="+palette.getType());
				filterList.add("filter.palette.interpolate="+paletteFilter.interpolate);
							
				String range = "";
				if(palette.getType() == RasterBuf.TYPE_BYTE || palette.getType() == RasterBuf.TYPE_SHORT || palette.getType() == RasterBuf.TYPE_INT){
					for(int i = 0; i < palette.getIntRange().length; i++)
						range += palette.getIntRange()[i]+" ";
				}else{
					for(int i = 0; i < palette.getDoubleRange().length; i++)
						range += palette.getDoubleRange()[i]+" ";
				}
				filterList.add("filter.palette.range="+range);
				
				String values = "";
				for(int i = 0; i < palette.getPalette().length; i++)
					values += palette.getPalette()[i]+" ";
				filterList.add("filter.palette.values="+values);
				
				String names = "";
				for(int i = 0; i < palette.getNameClass().length; i++)
					names += palette.getNameClass()[i]+" ";
				filterList.add("filter.palette.names="+names);
			}
		}
		return filterList;
	}

}
