package org.cresques.filter.correction;

import java.util.Arrays;

import org.cresques.io.data.RasterBuf;

public class MedianDoubleFilter extends MedianFilter {

	public void process(int col, int line) {
		double[] pxOut = new double[4];
		double[] px = new double[4];
		
		ladoVentana = Math.abs(ladoVentana);     // El lado de la ventana debe
		if(ladoVentana % 2 == 0) ladoVentana++;  // ser positivo e impar.
	    int tamVentana = ladoVentana*ladoVentana;
	    int mitadLado = (ladoVentana-1)>>1;
	    double [] ventanaR = new double[tamVentana];
	    double [] ventanaG = new double[tamVentana];
	    double [] ventanaB = new double[tamVentana];
		int k;
		
		raster.getElemDouble(line,col,px); 
		
		if((col-mitadLado >= 0) && (line-mitadLado >= 0) &&(col+mitadLado < width)&&(line+mitadLado < height)) 
		{
				// Obtener el vector con la ventanas de muestras (una por componente banda)
				k=0;
				for (int i=-mitadLado;i<=mitadLado;i++)
					for(int j=-mitadLado;j<=mitadLado;j++)
					{
						raster.getElemDouble(line+j,col+i,px); 
						ventanaR[k] =px[0];
						ventanaG[k] =px[1];
						ventanaB[k] =px[2];
						k++;
					}
				
				// Ordenar los valores de las ventanas
			    	// se supone que usa quickSort.
				Arrays.sort(ventanaR);
				Arrays.sort(ventanaG);
				Arrays.sort(ventanaB);
				
				pxOut[0]=ventanaR[(tamVentana-1)/2];
				pxOut[1]=ventanaG[(tamVentana-1)/2];
				pxOut[2]=ventanaB[(tamVentana-1)/2];
			
			//Extraer los elementos centrales y asignarselos al pixel (x,y)
				
				rasterResult.setElemDouble(line, col, pxOut);
		}
		else rasterResult.setElemDouble(line, col, px);

	}

	public void processSuperSampling(int col, int line) {
		double[] px = new double[4];
		double[] pxOut = new double[4];
		ladoVentana = Math.abs(ladoVentana);     // El lado de la ventana debe
		if(ladoVentana % 2 == 0) ladoVentana++;  // ser positivo e impar.
	    int tamVentana = ladoVentana*ladoVentana;
	    int mitadLado = (ladoVentana-1)>>1;
	    int indiceX,indiceY,origenX,origenY;
	    
	    double [] ventanaR = new double[tamVentana];
		double [] ventanaG = new double[tamVentana];
		double [] ventanaB = new double[tamVentana];
		
		raster.getElemDouble(line,col,px);
		
		if ((contX+1-mitadLado>=0) && (contY+1-mitadLado>=0)&& (contX+1+mitadLado<stepX.length) && (contY+1+mitadLado<stepY.length)){
			
			// Calcular el alcance del kernel cuyo centro es (x,y)
			int offsetX=0;
			int offsetY=0;
			for (int i=0;i<mitadLado;i++){
				offsetX=offsetX+stepX[contX+i+1];
				offsetY=offsetY+stepY[contY+i+1];
			}
			if((col+offsetX<width)&&(line+offsetY<height)){
//				 Obtener el vector con las ventanas de muestras (una por banda) **************************
				// Me situo en la esquina superior izquierda del "kernel"
				origenX=col;
				origenY=line;
				for (int i=0;i<mitadLado;i++){
					origenX=origenX-stepX[contX-i];
					origenY=origenY-stepY[contY-i];	
				}
				//Recorro el kernel seg�n los step
				int k=0;
				indiceX=origenX;
				for (int i=-mitadLado;i<=mitadLado;i++){
					indiceY=origenY;
					for(int j=-mitadLado;j<=mitadLado;j++){		
						raster.getElemDouble(indiceY,indiceX,px);
						ventanaR[k] =px[0];
						ventanaG[k] =px [1];
						ventanaB[k] =px[2];
						indiceY=indiceY+stepY[contY+j+1];
						k++;
					}
					indiceX=indiceX+stepX[contX+i+1];
				}
				//**************************************************************************************************
				
				// Ordenar los valores de las ventanas
			    	// se supone que usa quickSort.
				Arrays.sort(ventanaR);
				Arrays.sort(ventanaG);
				Arrays.sort(ventanaB);
				
				pxOut[0]=ventanaR[(tamVentana-1)/2];
				pxOut[1]=ventanaG[(tamVentana-1)/2];
				pxOut[2]=ventanaB[(tamVentana-1)/2];
			
				//Extraer los elementos centrales y asignarselos al pixel (x,y)
				for(int i = col; i < width && i < (col + stepX[contX + 1]); i++)
					for(int j = line; j < height && j < (line + stepY[contY + 1]); j++)
						rasterResult.setElemDouble(j, i,pxOut);
				return;
			}
		}
		
		// Si la ventana se sale de la imagen dejar el pixeles original
		for(int i = col; i < width && i < (col + stepX[contX + 1]); i++)
			for(int j = line; j < height && j < (line + stepY[contY + 1]); j++)
				rasterResult.setElemDouble(j, i,px);			
	}

	public void processLine(int y) {
		// TODO Auto-generated method stub

	}

	public int getInRasterDataType() {
		return RasterBuf.TYPE_DOUBLE;
	}

	public int getOutRasterDataType() {
		return RasterBuf.TYPE_DOUBLE;
	}

	public Object getResult(String name) {
		if (name.equals("raster")) {
            return (Object) this.rasterResult;
        } else {
            return null;
        }
	}

	public void pre() {
		exec = true;
		this.raster = (RasterBuf) params.get("raster");
		height = raster.getHeight();
        width = raster.getWidth();
		this.ladoVentana = ((Integer) params.get("ladoVentana")).intValue();
		this.rasterResult = new RasterBuf(raster.getDataType(),width,height,raster.getBandCount(),true);
		
		super.pre();
	}

}
