/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.filter.enhancement;

import java.awt.Image;
import java.awt.image.BufferedImage;

import org.cresques.io.data.RasterBuf;
import org.cresques.io.datastruct.Statistic;


/**
 * Realzado Lineal (Amplitude Rescaling) para objetos Image.
 * @author Luis W. Sevilla (sevilla_lui@gva.es)
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class LinearEnhancementImageFilter extends LinearEnhancementFilter {
    /**
     * Constructor
     */
    public LinearEnhancementImageFilter() {
        super();
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#pre()
     */
    public void pre() {
        //Obtencin de parmetros
        this.image = (Image) params.get("raster");
        this.stats = (Statistic) params.get("stats");
        this.removeExtrema = ((Boolean) params.get("remove")).booleanValue();
        this.filename = (String) params.get("filename");
        height = image.getHeight(null);
        width = image.getWidth(null);
        super.pre();
    }

    /**
     * Calculo del realce
     * @param pt pixel
     * @return pixel con el calculo de realce aplicado
     */
    private int[] calcLinearEnhancement(int pt){
        int[] px4 = {
                ((pt & 0xff000000) >> 24) & 0xff,
                ((pt & 0xff0000) >> 16) & 0xff,
                ((pt & 0xff00) >> 8) & 0xff, pt & 0xff
            };

		for (int i = 0; i < 3; i++) {
		    if (px4[i + 1] > maxBandValue[i]) {
		        px4[i + 1] = (int)maxBandValue[i];
		    } else if (px4[i + 1] < minBandValue[i]) {
		        px4[i + 1] = (int)minBandValue[i];
		    }
		
		    px4[i + 1] = (((int) ((((double) px4[i + 1]) * scale[i]) +
		                 offset[i])) & 0xff);
		}
		return px4;
    }
    
    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#process(int, int)
     */
    public void process(int col, int line) {
        int pt = ((BufferedImage) image).getRGB(col, line);
        int[] px4 = calcLinearEnhancement(pt);
        ((BufferedImage) image).setRGB(col, line,	(((px4[0] << 24) & 0xff000000) |
                                       			((px4[1] << 16) & 0xff0000) |
                                       			((px4[2] << 8) & 0xff00) |
                                       			(px4[3] & 0xff)));
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#processSuperSampling(int, int)
     */
    public void processSuperSampling(int col, int line) {
    	int pt = ((BufferedImage) image).getRGB(col, line);
        int[] px4 = calcLinearEnhancement(pt);
        for(int j = col; j < image.getWidth(null) && j < (col + stepX[contX + 1]); j++)
			for(int i = line; i < image.getHeight(null) && i < (line + stepY[contY + 1]); i++)
				 ((BufferedImage) image).setRGB(j, i, 	(((px4[0] << 24) & 0xff000000) |
                         								((px4[1] << 16) & 0xff0000) |
                         								((px4[2] << 8) & 0xff00) |
                         								(px4[3] & 0xff)));
	}
    
    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#getResult(java.lang.String)
     */
    public Object getResult(String name) {
        if (name.equals("raster")) {
            return (Object) this.image;
        } else {
            return null;
        }
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#getInRasterDataType()
     */
    public int getInRasterDataType() {
        return RasterBuf.TYPE_IMAGE;
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#getOutRasterDataType()
     */
    public int getOutRasterDataType() {
        return RasterBuf.TYPE_IMAGE;
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#post()
     */
    public void post() {
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.RasterFilter#processLine(int)
     */
    public void processLine(int y) {
    }
}
