/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.filter.enhancement;

import java.awt.image.BufferedImage;

import org.cresques.io.data.RasterBuf;
import org.cresques.io.datastruct.Statistic;


/**
 * Realzado Lineal (Amplitude Rescaling) para imagenes de 16 bits.
 * @author Luis W. Sevilla (sevilla_lui@gva.es)
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class LinearEnhancementShortFilter extends LinearEnhancementFilter {
	private short[] 		px = new short[4];
	private int 			bandsCnt = 3;
	
    /**
     * Constructor
     */
    public LinearEnhancementShortFilter() {
        super();
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#pre()
     */
    public void pre() {
        //Obtencin de parmetros
        this.raster = (RasterBuf) params.get("raster");
        this.stats = (Statistic) params.get("stats");
        this.removeExtrema = ((Boolean) params.get("remove")).booleanValue();
        this.filename = (String) params.get("filename");
        height = raster.getHeight();
        width = raster.getWidth();
        super.pre();
    }

    /**
     * Calculo del realce
     * @param pt pixel
     * @return pixel con el calculo de realce aplicado
     */
    private short[] calcLinearEnhancement(short[] px){
    	for (int i = 0; i < bandsCnt; i++) {
            if (px[i] > maxBandValue[i]) {
                px[i] = (short)maxBandValue[i];
            } else if (px[i] < minBandValue[i]) {
                px[i] = (short)minBandValue[i];
            }

            px[i] = (short)(((short) ((((double) px[i]) * scale[i]) + offset[i])) & 0xff);
        }
    	return px;
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#process(int, int)
     */
    public void process(int col, int line) {
        raster.getElemShort(line, col, px);
        px = calcLinearEnhancement(px);
        raster.setElemShort(line, col, px);
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#processSuperSampling(int, int)
     */
    public void processSuperSampling(int col, int line) {
    	raster.getElemShort(line, col, px);
        px = calcLinearEnhancement(px);
		for(int j = col; j < raster.getWidth() && j < (col + stepX[contX + 1]); j++)
			for(int i = line; i < raster.getHeight() && i < (line + stepY[contY + 1]); i++)
				 raster.setElemShort(i, j, px);
	}
    
    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#getResult(java.lang.String)
     */
    public Object getResult(String name) {
        if (name.equals("raster")) {
            return (Object) this.raster;
        } else {
            return null;
        }
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#getInRasterDataType()
     */
    public int getInRasterDataType() {
        return RasterBuf.TYPE_SHORT;
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#getOutRasterDataType()
     */
    public int getOutRasterDataType() {
        return RasterBuf.TYPE_SHORT;
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#post()
     */
    public void post() {
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.RasterFilter#processLine(int)
     */
    public void processLine(int y) {
    }
}
