/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */

package org.cresques.filter.statistics;

import java.awt.Image;
import java.awt.image.BufferedImage;
import java.util.Vector;

import org.cresques.io.data.RasterBuf;
/**
 * Filtro para crear histogramas de brillo para cada una de las bandas de la imagen.
 * Toma como entrada la imagen y da como salida un histograma para cada una de las
 * bandas representado mediante una matriz.
 * @author Miguel ngel Querol Carratal  <querol_mig@gva.es>
 *
 */

public class HistogramImageFilter extends HistogramFilter {

	
	/**
	 * Matriz donde se van a almacenar los histogramas.
	 */
	int histoRed[] = new int[256];
	int histoGreen[] =  new int[256];
	int histoBlue[] = new int[256];
	
	
	/**
	 * Constructor
	 *
	 */
	
	
	public HistogramImageFilter(){
		super();
	}
	
	public void pre(){
		this.exec=true;
		this.image = (Image) params.get("raster");
		this.width = image.getWidth(null);
		this.height = image.getHeight(null);
		
		for(int i = 0 ; i <= 255 ; i++){
			histoRed[i]= 0; 
			histoGreen[i] = 0;
			histoBlue[i] = 0;
		}
		
		super.pre();
	}
	
	
	public void post() {
	}

	public void process(int col, int line) {
		int px = ((BufferedImage) image).getRGB(col, line);
		int px3[] = {(px & 0xff0000) >> 16, (px & 0xff00) >> 8, (px & 0xff)};
		
		this.histoRed[px3[0]]+=1;  		//Histograma para la banda R
		this.histoGreen[px3[1]]+=1;  	//Histograma para la banda G
		this.histoBlue[px3[2]]+=1;  	//Histograma para la banda B
		pixel++;						//Cuenta de los pixels de la imagen
	}
	
	/* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#processSuperSampling(int, int)
     */
    public void processSuperSampling(int col, int line) {
    	process(col, line);
	}

	/**
	 * Metodo para obtener el valor medio de brillo para una banda de color.
	 * @param vector
	 * @return
	 */
	public int calculaMedia(int []vector){
		int media = 0;
		int suma = 0;
		
		for (int i = 1 ; i <= 255 ; i++){
			suma += vector[i]*i;
		}
		media = suma/pixel;
		
		return media;
	}
	
	public int getPixel(){
		return this.pixel;
	}
	
	public void processLine(int y) {
		// TODO Auto-generated method stub

	}

	public int getInRasterDataType() {
		return RasterBuf.TYPE_IMAGE;
	}

	public int getOutRasterDataType() {
		return RasterBuf.TYPE_IMAGE;
	}

	public Object getResult(String name) {
		if (name.equals("raster")) {
            return (Object) this.image;
        } else {
            return null;
        }
	}
	
	/**
	 * Metodos para acceder al los valores medios de brillo para cada banda
	 * @return Media de brillo para la banda de color correspondiente.
	 */
	
	public int getMediaRed(){
		return calculaMedia(this.histoRed);
	}

	public int getMediaGreen(){
		return calculaMedia(this.histoGreen);
	}
	
	public int getMediaBlue(){
		return calculaMedia(this.histoBlue);
	}
	
	
	/**
	 * Metodos para acceder desde fuera a los histogramas calculados por el filtro
	 * @return Histogramas de cada una de las bandas de color.
	 */
	public Object getRedHistogram(){
		return (Object) this.histoRed;
	}
	
	public Object getGreenHistogram(){
		return (Object) this.histoGreen;
	}
	
	public Object getBlueHistogram(){
		return (Object) this.histoBlue;
	}
	
	
}
