/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.geo;

import org.cresques.cts.ICoordTrans;
import org.cresques.cts.IProjection;


/**
 * <p>Interface that must implement all classes which allow execute a re-projection.</p>
 *
 * @author "Luis W. Sevilla" <sevilla_lui@gva.es>*
 */
public interface Projected {
	/**
	 * Returns the current projection.
	 * 
	 * @return reference to an object which implements the interface <code>IProjection</code>
	 * 
	 * @see #reProject(ICoordTrans)
	 */
    public IProjection getProjection();

    /**
     * Changes the current projection.
     * 
     * @param ct transformation coordinates
     * 
     * @see #getProjection()
     */
    public void reProject(ICoordTrans ct);
}
