/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.io;

import org.cresques.cts.ICoordTrans;
import org.cresques.cts.IProjection;

import org.cresques.px.IObjList;
import org.cresques.px.gml.Feature;
import org.cresques.px.gml.FeatureCollection;
import org.cresques.px.gml.Geometry;
import org.cresques.px.gml.LineString;
import org.cresques.px.gml.MultiGeometry;
import org.cresques.px.gml.MultiPolygon;
import org.cresques.px.gml.Point;
import org.cresques.px.gml.Polygon;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;

import java.util.Hashtable;


/**
 * Cargador de ficheros .gml de JUMP de vivid solutions.
 * Actualmente lee un subconjunto de datos. Datos de prueba generados por JUMP.
 * @author "Luis W. Sevilla" <sevilla_lui@gva.es>
 */
public class GmlFile extends GeoFile {
    final static int NOWHERE = -1;
    final static int INCOORDINATES = 0;
    final static int INPROPERTY = 1;
    final static String LT = "<";
    final static String GT = ">";
    int where = NOWHERE;
    Hashtable tipos = new Hashtable();
    FeatureCollection collection = null;
    Feature activeFeature = null;
    Geometry activeGeometry = null;
    Geometry activeMember = null;
    String buf = null;
    String propName = null;
    long lineNr = 0;
    BufferedReader fi;
    long l = 0;
    int count = 0;

    /**
     * Constructor de la clase Gml
     *
     */
    public GmlFile(IProjection proj, String name) {
        super(proj, name);
    }

    /**
     * Carga un .gml
     * @param name nombre del fichero
     */
    public GeoFile load() {
        System.out.println("Cargando " + name + " ...");

        try {
            if (FileFolder.isUrl(name)) {
                ZipFileFolder zFolder = new ZipFileFolder(name);
                InputStream is = zFolder.getInputStream(name);

                return load(new InputStreamReader(is));
            } else {
                return load(new FileReader(name));
            }
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException ie) {
            System.err.println("ERROR." + l + "lineas leidas");
            ie.printStackTrace();
        }

        return this;
    }

    public GeoFile load(Reader fr) {
        try {
            fi = new BufferedReader(fr);

            while ((buf = fi.readLine()) != null) {
                //System.out.println(buf);
                l++;
                preParseLine(buf.trim());
            }

            fi.close();
            System.out.println("'" + name + "' cargado. (" + l + " lneas).");
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException ie) {
            System.err.println("ERROR." + l + "lineas leidas");
            ie.printStackTrace();
        }

        this.lineNr = l;
        extent.add(collection.getExtent());

        return this;
    }

    /**
     * Analiza una lnea del fichero .gml
     * @param l linea que analiza
     */
    void preParseLine(String l) {
        String line1;

        //System.err.println("pre0: '"+l+"'");
        //if (count++>500) System.exit(1);
        if (l.indexOf(LT) > 0) {
            line1 = l.substring(0, l.indexOf(LT)).trim();
            l = l.substring(l.indexOf(LT)).trim();

            //System.err.println("pre1: '"+line1+"', '"+l+"'");
            preParseLine(line1);
            preParseLine(l);

            return;
        }

        if ((l.indexOf(GT) >= 0) && (l.indexOf(GT) != (l.length() - 1))) {
            line1 = l.substring(0, l.indexOf(GT) + 1).trim();
            l = l.substring(l.indexOf(GT) + 1).trim();

            //System.err.println("pre2: '"+line1+"', '"+l+"'");
            preParseLine(line1);
            preParseLine(l);

            return;
        }

        parseToken(l);
    }

    /**
     * Analiza un token
     * @param l
     */
    void parseToken(String l) {
        String token = null;

        //System.err.println(l);
        if ((l.length() > 0) && (l.substring(0, 1).compareTo(LT) == 0)) {
            token = l.substring(1, l.indexOf(GT));

            if (token.compareTo("gml:coordinates") == 0) {
                where = INCOORDINATES;
            } else if (token.compareTo("/gml:coordinates") == 0) {
                where = NOWHERE;
            } else if (token.compareTo("gml:LinearRing") == 0) {
            } else if (token.compareTo("/gml:LinearRing") == 0) {
            } else if (token.compareTo("gml:Polygon") == 0) {
                activeMember = new Polygon();

                if (activeGeometry == null) {
                    activeGeometry = activeMember;
                }
            } else if (token.compareTo("/gml:Polygon") == 0) {
                if (activeGeometry.getClass() == MultiPolygon.class) {
                    ((MultiPolygon) activeGeometry).add(activeMember);
                }

                activeMember = null;
            } else if (token.compareTo("gml:LineString") == 0) {
                activeMember = new LineString();

                if (activeGeometry == null) {
                    activeGeometry = activeMember;
                }
            } else if (token.compareTo("/gml:LineString") == 0) {
                if (activeGeometry.getClass() == MultiGeometry.class) {
                    ((MultiGeometry) activeGeometry).add(activeMember);
                }

                activeMember = null;
            } else if (token.compareTo("gml:outerBoundaryIs") == 0) {
            } else if (token.compareTo("/gml:outerBoundaryIs") == 0) {
            } else if (token.compareTo("featureCollection") == 0) {
                collection = new FeatureCollection(proj);
            } else if (token.compareTo("/featureCollection") == 0) {
            } else if (token.compareTo("feature") == 0) {
                System.err.println("feature");
                activeFeature = new Feature();
            } else if (token.compareTo("/feature") == 0) {
                collection.add(activeFeature);
                activeFeature = null;
            } else if (token.compareTo("geometry") == 0) {
            } else if (token.compareTo("/geometry") == 0) {
                activeFeature.setGeometry(activeGeometry);
                activeGeometry = null;
                activeMember = null;
            } else if (token.compareTo("gml:Point") == 0) {
                activeMember = new Point();

                if (activeGeometry == null) {
                    activeGeometry = activeMember;
                }
            } else if (token.compareTo("/gml:Point") == 0) {
                activeMember = null;
            } else if (token.compareTo("gml:MultiPolygon") == 0) {
                activeGeometry = new MultiPolygon();
            } else if (token.compareTo("/gml:MultiPolygon") == 0) {
            } else if (token.compareTo("gml:MultiPoint") == 0) {
                //activeGeometry = new MultiPoint();
            } else if (token.compareTo("/gml:MultiPoint") == 0) {
            } else if (token.compareTo("gml:polygonMember") == 0) {
            } else if (token.compareTo("/gml:polygonMember") == 0) {
            } else if (token.compareTo("gml:innerBoundaryIs") == 0) {
                ((Polygon) activeMember).setInnerBoundary();
            } else if (token.compareTo("/gml:innerBoundaryIs") == 0) {
            } else if (token.startsWith("property")) {
                where = INPROPERTY;
                propName = l.substring(l.indexOf("\"") + 1);
                propName = propName.substring(0, propName.indexOf("\""));
            } else if (token.startsWith("/property")) {
                where = NOWHERE;

                Integer tipo = new Integer(0);

                if (tipos.containsKey(token)) {
                    tipo = (Integer) tipos.get(token);
                }

                tipos.put(token, new Integer(tipo.intValue() + 1));
            }
        } else {
            switch (where) {
            case INPROPERTY:
                activeFeature.setProp(propName, l);

                break;

            case INCOORDINATES:

                String[] txt = l.split(",");
                double[] pt = new double[3];

                for (int i = 0; i < txt.length; i++)
                    pt[i] = Double.parseDouble(txt[i]);

                activeMember.add(proj.createPoint(pt[0], pt[1]));

                break;
            }
        }
    }

    /**
     * Obtiene la lista de features.
     */
    public IObjList getObjects() {
        return collection;
    }

    /**
     * Saca las estadisticas del fichero por stdout.
     */
    public void pintaStats() {
        /*                print "GML file '%s': %d lineas" % (self.name, self.lineNr)
                        print "GML Tokens:"
                        kk = self.tipos.keys()
                        kk.sort()
                        for k in kk:
                                print k, self.tipos[k]
                        print "\nTotal Coordenadas %d\n" % geo.Polygon.numCoords
                        #print "\nExtent: min=(%.2f,%df), max=(%.2f,%.2f)" % (self.minX, self.minY, self.maxX, self.maxY)
                        print "\nExtent: ", (self.minX, self.minY, self.maxX, self.maxY)*/
        String txt = "GML file: '" + name + "': " + lineNr + " lneas\n";
        txt += ("\nGML file: Total Coordenadas: " + Polygon.pointNr + "\n");
        txt += ("GML file: Total features: " + collection.size() + "\n");
        txt += ("GML file: " + collection.getExtent() + "\n");
        txt += ("GML file: " + ((Feature) collection.get(0)).getExtent());
        System.out.println(txt);
    }

    public void reProject(ICoordTrans rp) {
        collection.reProject(rp);
        setProjection(rp.getPDest());
    }

    /* (non-Javadoc)
     * @see org.cresques.io.GeoFile#close()
     */
    public void close() {
        // TODO Auto-generated method stub
    }
}
