/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.cresques.io.data;



/**
 * Interfaz que debe implemetar la clase que instancie un TailRasterProcess.
 * 
 * @author Nacho Brodin (nachobrodin@gmail.com)
 */
public interface ICutRaster{
	
	/**
	 * Mtodo en el que deben escribirse las acciones a realizar cuando un proceso
	 * de recortado finaliza. Esto es necesario ya que el proceso es un hilo de 
	 * ejecucin que al finalizar debe comunicar que los raster creados ya estn
	 * listos para ser tratados. Las acciones comunes a implemetar son , el
	 * cerrado de la ventana que llama y la carga de estas capas que ya estn listas.
	 */
	public void cutFinalize(String[] fileNames);
	
	/**
	 * Mtodo que es llamado al pulsar el botn de seleccionar herramienta. El que implemente
	 * este interfaz debe definir una accin para dicho botn. En el caso de gvSIG deber
	 * realizar las acciones para seleccionar la tool cutRaster del MapTool.
	 */
	public void selectToolButton();
	
	/**
	 * Asigna el valor inicial del recorte en pixeles
	 * @param w Ancho del recorte
	 * @param h Alto del recorte
	 */
	public void setInitValuesToSize(double w, double h);
}
