/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.ui.filter;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.cresques.ui.raster.IResize;
import org.cresques.ui.raster.RGBInputPanel;


/**
 * Dialogo para asignar la transparencia por pixel y global al raster.
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class RasterTransparencyPanel extends JPanel implements ActionListener, FocusListener, ChangeListener, IResize{
    final private static long serialVersionUID = -3370601314380922368L;

    /**
     * Variables para la asignacin de tamao de los componentes del panel.
     */
    private int						wComp = 445, hComp = 239;
    private int						wPixel = wComp - 15, hPixel = hComp - 59;
    private int						wList = wPixel - 20, hList = hPixel - 60;
    private int						wGeneral = wPixel, hGeneral = 55;
    private int						wSlider = wGeneral - 160;
    
    
    /**
     * Nombre del panel
     */
    private String 						nom = "Transparencia";
    FilterRasterDialogPanel 			parent = null;

    /**
     * Nmero de bandas del raster
     */
    public int 							nBands = 3;
    private JPanel 						jPanel1 = null;
    private JCheckBox 					cbTransparencia = null;

    /**
     * Variable pblica para la traduccin
     */
    public JPanel 						pGeneralTrans = null;
    private JCheckBox 					cbOpacidad = null;
    private JSlider 					slOpacidad = null;
    private JTextField 					tOpacidad = null;
	private TransparencyByPixelPanel 	pTranspByPixel = null;
	private JPanel 						pTransparencyByPixel = null;
		
    /**
     * Constructor. Asigna la variable panel que contiene este
     * @param parent FilterRasterDialogPanel
     */
    public RasterTransparencyPanel(FilterRasterDialogPanel parent) {
        this.parent = parent;
        initialize();
    }

    /**
     * This method initializes this
     *
     * @return void
     */
    private void initialize() {
        GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
        gridBagConstraints4.insets = new java.awt.Insets(0,0,0,0);
        gridBagConstraints4.gridx = 0;
        gridBagConstraints4.gridy = 1;
        gridBagConstraints4.gridwidth = 1;
        GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 0;
        this.setLayout(new GridBagLayout());
        this.setBounds(0, 0, wComp, hComp);
        this.setPreferredSize(new java.awt.Dimension(wComp, hComp));
        this.add(getPTransparencyByPixel(), gridBagConstraints1);
        this.add(getOpacityPanel(), gridBagConstraints4);
        initControls();
        //this.setComponentSize(wComp, hComp);
    }

    /**
     * Obtiene el nombre del panel
     * @return Cadena con el nombre del panel
     */
    public String getName(){
    	return this.nom;
    }
    
    /**
     * Asigna el nmero de bandas de la imagen
     * @param nBands
     */
    public void setBands(int nBands) {
        this.nBands = nBands;
    }

    /**
     * Inicializa controles a sus valores por defecto
     */
    public void initControls() {
        this.getOpacityText().setText("100");
        this.setActiveOpacityControl(false);
        this.setActiveTransparencyControl(false);
    }

    /**
     * This method initializes jPanel1
     *
     * @return javax.swing.JPanel
     */
    private JPanel getJPanel1() {
        if (jPanel1 == null) {
            jPanel1 = new JPanel();
            jPanel1.setLayout(new BorderLayout());
            jPanel1.setPreferredSize(new java.awt.Dimension(415,25));
            jPanel1.setBorder(javax.swing.BorderFactory.createEmptyBorder(0,0,0,0));
            jPanel1.add(getTransparencyCheck(), java.awt.BorderLayout.NORTH);
        }

        return jPanel1;
    }

    /**
     * This method initializes jCheckBox
     *
     * @return javax.swing.JCheckBox
     */
    public JCheckBox getTransparencyCheck() {
        if (cbTransparencia == null) {
            cbTransparencia = new JCheckBox();
            cbTransparencia.setText("Activar");
            cbTransparencia.addActionListener(this);
        }

        return cbTransparencia;
    }

    /**
     * This method initializes jPanel4
     *
     * @return javax.swing.JPanel
     */
    public JPanel getOpacityPanel() {
        if (pGeneralTrans == null) {
        	pGeneralTrans = new JPanel();
        	pGeneralTrans.setBorder(javax.swing.BorderFactory.createTitledBorder(null, "Opacidad", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, null, null));
        	pGeneralTrans.setPreferredSize(new java.awt.Dimension(wGeneral,hGeneral));
        	pGeneralTrans.add(getOpacityCheck(), null);
        	pGeneralTrans.add(getOpacitySlider(), null);
        	pGeneralTrans.add(getOpacityText(), null);
        }

        return pGeneralTrans;
    }

    /**
     * This method initializes jCheckBox1
     *
     * @return javax.swing.JCheckBox
     */
    public JCheckBox getOpacityCheck() {
        if (cbOpacidad == null) {
            cbOpacidad = new JCheckBox();
            cbOpacidad.setText("Activar");
            cbOpacidad.addActionListener(this);
            cbOpacidad.addFocusListener(this);
        }

        return cbOpacidad;
    }

    /**
     * This method initializes jSlider
     *
     * @return javax.swing.JSlider
     */
    public JSlider getOpacitySlider() {
        if (slOpacidad == null) {
            slOpacidad = new JSlider();
            slOpacidad.setPreferredSize(new java.awt.Dimension(wSlider, 16));
            slOpacidad.addChangeListener(this);
        }

        return slOpacidad;
    }

    /**
     * This method initializes jTextField3
     *
     * @return javax.swing.JTextField
     */
    public JTextField getOpacityText() {
        if (tOpacidad == null) {
            tOpacidad = new JTextField();
            tOpacidad.setPreferredSize(new java.awt.Dimension(30, 19));
            tOpacidad.addActionListener(this);
        }

        return tOpacidad;
    }

    /**
     * Activa/Desactiva los controles de opacidad
     * @param active
     */
    public void setActiveOpacityControl(boolean active) {
        this.getOpacityCheck().setSelected(active);
        this.getOpacitySlider().setEnabled(active);
        this.getOpacityText().setEnabled(active);
    }

    /**
     * Activa/Desactiva los controles de transparencia
     * @param active
     */
    public void setActiveTransparencyControl(boolean active) {
        this.getTransparencyCheck().setSelected(active);
        RGBInputPanel rgbPanel = this.getPTranspByPixel().getPRGBInput();
        rgbPanel.getTRed().setEnabled(active);
        
        if (parent.nbands == 2) {
        	rgbPanel.getTGreen().setEnabled(active);
        	rgbPanel.getTBlue().setEnabled(false);
        }

        if (parent.nbands == 3) {
        	rgbPanel.getTGreen().setEnabled(active);
        	rgbPanel.getTBlue().setEnabled(active);
        }
    }

    /**
     * Asigna el valor de opacidad a los controles de la ventana
     * para que cuando esta se abra tenga los valores seleccionados en la
     * imagen.
     * @param alpha
     */
    public void setOpacity(int alpha) {
        int opacityPercent = (int) ((alpha * 100) / 255);
        this.getOpacityText().setText(String.valueOf(opacityPercent));
        this.getOpacitySlider().setValue(opacityPercent);
        this.setActiveOpacityControl(true);

        if (opacityPercent == 100) {
            this.setActiveOpacityControl(false);
        }
    }

	/**
	 * This method initializes jPanel2	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	public TransparencyByPixelPanel getPTranspByPixel() {
		if (pTranspByPixel == null) {
			pTranspByPixel = new TransparencyByPixelPanel();
			pTranspByPixel.setPreferredSize(new java.awt.Dimension(wList,hList));
			pTranspByPixel.setBorder(javax.swing.BorderFactory.createEmptyBorder(0,0,0,0));
			pTranspByPixel.setControlEnabled(false);
		}
		return pTranspByPixel;
	}

	/**
	 * This method initializes jPanel2	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	public JPanel getPTransparencyByPixel() {
		if (pTransparencyByPixel == null) {
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.gridx = 0;
			gridBagConstraints.gridy = 1;
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.anchor = GridBagConstraints.WEST;
			gridBagConstraints2.insets = new java.awt.Insets(0,0,0,0);
			gridBagConstraints2.gridheight = 1;
			gridBagConstraints2.gridwidth = 1;
			gridBagConstraints2.gridx = 0;
			gridBagConstraints2.gridy = 0;
			gridBagConstraints2.ipady = 0;
			gridBagConstraints2.fill = GridBagConstraints.NONE;
			pTransparencyByPixel = new JPanel();
			pTransparencyByPixel.setLayout(new GridBagLayout());
			pTransparencyByPixel.setBorder(javax.swing.BorderFactory.createTitledBorder(null, "Transparencia por pixel", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, null, null));
			pTransparencyByPixel.setPreferredSize(new java.awt.Dimension(wPixel,hPixel));
			pTransparencyByPixel.add(getJPanel1(), gridBagConstraints2);
			pTransparencyByPixel.add(getPTranspByPixel(), gridBagConstraints);
		}
		return pTransparencyByPixel;
	}

	/**
	 * Ajusta el tamao de todos los componentes a partir del tamao
	 * introducido para el panel.
	 * @param w
	 * @param h
	 */
	public void setComponentSize(int w, int h){
		wComp = w; hComp = h;
		wPixel = wComp - 15; hPixel = hComp - 59;
		wList = wPixel - 20; hList = hPixel - 60;
		wGeneral = wComp - 15; hGeneral = 55; 
		wSlider = wGeneral - 160;
		    
		this.setPreferredSize(new java.awt.Dimension(wComp, hComp));
		pTransparencyByPixel.setPreferredSize(new java.awt.Dimension(wPixel,hPixel));
		pGeneralTrans.setPreferredSize(new java.awt.Dimension(wGeneral,hGeneral));
		pTranspByPixel.setPreferredSize(new java.awt.Dimension(wList,hList));
		pTranspByPixel.setComponentSize(wList, hList);
		slOpacidad.setPreferredSize(new java.awt.Dimension(wSlider, 16));
        
		
	}

	/**
     * Eventos sobre TextField y CheckBox. Controla eventos de checkbox de opacidad, transparencia,
     * recorte de colas y los textfield de opacidad, valores de transparencia por banda y
     * porcentaje de recorte.
     */
	public void actionPerformed(ActionEvent e) {
		//Evento sobre el checkbox de opacidad
		if (e.getSource().equals(getOpacityCheck())) {
	           //Check de opacidad activado -> Activar controles de opacidad
	           if (getOpacityCheck().isSelected()) {
	               getOpacitySlider().setEnabled(true);
	               getOpacityText().setEnabled(true);
	           } else {
	               getOpacitySlider().setEnabled(false);
	               getOpacityText().setEnabled(false);
	           }
		}
		 
		//Evento sobre el checkbox de transparencia
	    if (e.getSource().equals(getTransparencyCheck())) {
	           //Check de opacidad activado -> Activar controles de opacidad
	           if (getTransparencyCheck().isSelected()) {
	        	   getPTranspByPixel().setControlEnabled(true);
	        	   getPTranspByPixel().getPRGBInput().setActiveBands(nBands);
	           }else
	        	   getPTranspByPixel().setControlEnabled(false);
	    }
	    
	    //Evento sobre el textfield de opacidad
        if (e.getSource().equals(getOpacityText()))
            checkOpacityText();
	}
	
    public void focusGained(FocusEvent e) {
        //pTrans.updateTextBox();
    }

    public void focusLost(FocusEvent e) {
        checkOpacityText();
    }
    
    /*
    *
    */
   public void stateChanged(ChangeEvent e) {
       //Ponemos el valor del texto de la opacidad de pendiendo de la posicin del Slider
       if (e.getSource().equals(getOpacitySlider())) {
           getOpacityText().setText(
           		String.valueOf(getOpacitySlider().getValue()));
       }        
   }
   
	/**
     * Controla que si el formato introducido en el textfield
     * de opacidad es numerico se actualiza la posicin del
     * slider.
     */
    private void checkOpacityText() {
        String op = getOpacityText().getText();
        int value = 0;

        try {
            if (!op.equals("")) {
                value = Integer.parseInt(op);
            }

            getOpacitySlider().setValue(value);
        } catch (NumberFormatException exc) {
            System.err.println("Formato no numerico");
            getOpacityText().setText("100");
            getOpacitySlider().setValue(100);
        }
    }
    
} //  @jve:decl-index=0:visual-constraint="36,15"
