/*
 * Created on 22-ago-2006
 *
 */
/* gvSIG. Sistema de Informaci�n Geogr�fica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.cresques.ui.raster;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.geom.Rectangle2D;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.cresques.io.data.ICutRaster;
import org.cresques.ui.raster.listener.CutRasterListener;
import org.gvsig.gui.beans.buttonBar.ButtonBarContainer;
import org.gvsig.gui.beans.coordDataInput.CoordDataInputContainer;
import org.gvsig.gui.beans.table.TableContainer;
import org.gvsig.i18n.Messages;
import java.awt.Dimension;

public class CutRasterPanel extends BaseComponent{

	private int						wPanel = 570, hPanel = 600;
	private int						wComp = wPanel - 6, hComp = hPanel - 40;
	private int						wTable = wComp - 10;
	private int						hTable = hComp - 370 ;

	private JPanel 					pTailImage = null;
	private CoordDataInputContainer pCoordPixel = null;
	private CoordDataInputContainer pCoordReal = null;
	private ButtonBarContainer 		pResize = null;
	private JButton 				jButton = null;

	private ResolutionPanel 		pResolution = null;

	private JPanel 					pSelection = null;
	private TableContainer 			tSelection = null;
	private String[] 				columnNames = {Messages.getText("bandas"), Messages.getText("nombre")};
	private int[] 					columnWidths = {75, 305};
	private boolean 				contract = true;

	private JPanel 					pCheck = null;
	private JCheckBox 				jCheckBox = null;
	private JLabel 					jLabel = null;
	private JPanel 					pButtons = null;
	private JPanel 					pAccept = null;
	private JButton 				bSave = null;
	private JButton 				bAccept = null;
	private JButton 				bCancel = null;
	private JPanel 					pBorder = null;
	private CutRasterListener 		tailImageListener = null;
	/**
	 * Valores reales para el ancho, alto y tamao de celda. Esto es necesario porque
	 * en la caja de texto se guardan con decimales recortados y cuando se pide el valor
	 * se devuelven completos.
	 */
	private double 					realWidth, realHeight, realCellSize;
	/**
	 * Relacin entre el ancho y alto en pixeles de la imagen
	 */
	private double					relWidthHeight = 0;
	/**
	 * Clase que instancia a esta y que implemente ITailRaster.
	 */
	private ICutRaster				parent = null;

	/**
	 * Extensin de la capa.
	 */
	private Rectangle2D 			extent = null;

	private int 					widthPx = 0;
	private int 					heightPx = 0;


	/**
	 * Valores reales para el tamao de la ventana. Esto es necesario porque
	 * en la caja de texto se guardan con decimales recortados y cuando se pide el valor
	 * se devuelven completos.
	 */
	private double					wcMinX, wcMinY, wcMaxX, wcMaxY;
	private double					pxMinX, pxMinY, pxMaxX, pxMaxY;



	/**
	 * This is the default constructor
	 */
	public CutRasterPanel(Rectangle2D extent, int widthPx, int heightPx, ICutRaster parent) {
		super();
		this.parent = parent;
		this.extent = extent;
		this.widthPx = widthPx;
		this.heightPx = heightPx;
		initialize();
		tailImageListener = new CutRasterListener(this, extent, widthPx, heightPx, parent);
		this.selectLayerFullExtend();
	}

	/**
	 * This method initializes this
	 *
	 * @return void
	 */
	private void initialize() {
		FlowLayout flowLayout = new FlowLayout();
		flowLayout.setHgap(0);
		flowLayout.setVgap(0);
		this.setLayout(flowLayout);
		this.add(getPTailImage(), null);
		this.setSize(wPanel, hPanel);
		if(contract){
			this.getPResolution().setVisible(false);
			this.getPSelection().setVisible(false);
			this.getPCheck().setVisible(false);
			this.setSize(wPanel, hPanel - hTable - 120 - 30);
			this.getPTailImage().setPreferredSize(new java.awt.Dimension(wPanel, hPanel - hTable - 120 - 30));
			this.getPBorder().setSize(wComp, hComp - hTable - 120 - 30);
			this.getPBorder().setPreferredSize(new java.awt.Dimension(wComp, hComp - hTable - 120 - 30));
		}
	}

	/**
	 * This method initializes jPanel
	 *
	 * @return javax.swing.JPanel
	 */
	private JPanel getPTailImage() {
		if (pTailImage == null) {
			GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
			gridBagConstraints5.gridx = 0;
			gridBagConstraints5.gridy = 0;
			GridBagConstraints gridBagConstraints8 = new GridBagConstraints();
			gridBagConstraints8.gridx = 0;
			gridBagConstraints8.gridy = 6;
			GridBagConstraints gridBagConstraints7 = new GridBagConstraints();
			gridBagConstraints7.gridx = 0;
			gridBagConstraints7.gridy = 6;
			pTailImage = new JPanel();
			pTailImage.setLayout(new GridBagLayout());
			pTailImage.setPreferredSize(new java.awt.Dimension(wPanel,hPanel));
			pTailImage.add(getPButtons(), gridBagConstraints8);
			pTailImage.add(getPBorder(), gridBagConstraints5);
		}
		return pTailImage;
	}

	/**
	 * This method initializes jPanel
	 *
	 * @return javax.swing.JPanel
	 */
	public CoordDataInputContainer getPCoordPixel() {
		if (pCoordPixel == null) {
			pCoordPixel = new CoordDataInputContainer();
			pCoordPixel.setParameters(Messages.getText("coordenadas_pixel"),
										Messages.getText("sup_izq"),
										"X" ,"Y",
										Messages.getText("inf_der"),
										"X", "Y");
			pCoordPixel.setDecimalValues(true);
			pCoordPixel.setPreferredSize(new java.awt.Dimension(550,70));
		}
		return pCoordPixel;
	}

	/**
	 * This method initializes jPanel1
	 *
	 * @return javax.swing.JPanel
	 */
	public CoordDataInputContainer getPCoordReal() {
		if (pCoordReal == null) {
			pCoordReal = new CoordDataInputContainer();
			pCoordReal.setParameters(Messages.getText("coordenadas_reales"),
										Messages.getText("sup_izq"),
										"X" ,"Y",
										Messages.getText("inf_der"),
										"X", "Y");
			pCoordReal.setDecimalValues(true);
			pCoordReal.setPreferredSize(new java.awt.Dimension(550,70));
		}
		return pCoordReal;
	}

	/**
	 * This method initializes jPanel
	 *
	 * @return javax.swing.JPanel
	 */
	public ButtonBarContainer getPResize() {
		if (pResize == null) {
			pResize = new ButtonBarContainer();
			pResize.setPreferredSize(new java.awt.Dimension(550,35));
			pResize.addButton("fullExtent.png", Messages.getText("fullExtent"), 0);
			pResize.addButton("selectTool.png", Messages.getText("select_tool"), 1);
			pResize.addButton("down.png", Messages.getText("expandir"), 2);
			pResize.setButtonAlignment("right");
			pResize.setComponentBorder(true);

		}
		return pResize;
	}


	/**
	 * This method initializes jPanel
	 *
	 * @return javax.swing.JPanel
	 */
	public ResolutionPanel getPResolution() {
		if (pResolution == null) {
			pResolution = new ResolutionPanel();
			pResolution.setPreferredSize(new java.awt.Dimension(550,140));
			pResolution.setComponentSize(wTable,140);
		}
		return pResolution;
	}

	/**
	 * This method initializes jPanel1
	 *
	 * @return javax.swing.JPanel
	 */
	private JPanel getPSelection() {
		if (pSelection == null) {
			pSelection = new JPanel();
			pSelection.setLayout(new FlowLayout());
			pSelection.setPreferredSize(new java.awt.Dimension(550,160));
			pSelection.setBorder(javax.swing.BorderFactory.createTitledBorder(null, Messages.getText("seleccion_bandas"), javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Dialog", java.awt.Font.PLAIN, 10), null));
			pSelection.add(getTSelection(), null);

		}
		return pSelection;
	}

	/**
	 * Obtiene la tabla de seleccin de bandas
	 * @return Tabla de seleccin de bandas
	 */
	public TableContainer getTSelection(){
		if (tSelection == null) {
			tSelection = new TableContainer(wTable - 20, hTable - 40, columnNames, columnWidths);
			tSelection.setModel("CheckBoxModel");
			tSelection.setControlVisible(false);
			tSelection.initialize();
		}

		return tSelection;
	}

	/**
	 * This method initializes jPanel
	 *
	 * @return javax.swing.JPanel
	 */
	private JPanel getPButtons() {
		if (pButtons == null) {
			pButtons = new JPanel();
			pButtons.setLayout(new BorderLayout());
			pButtons.setPreferredSize(new java.awt.Dimension(550,30));
			pButtons.add(getPAccept(), java.awt.BorderLayout.EAST);
		}
		return pButtons;
	}

	/**
	 * This method initializes jPanel
	 *
	 * @return javax.swing.JPanel
	 */
	private JPanel getPAccept() {
		if (pAccept == null) {
			FlowLayout flowLayout1 = new FlowLayout();
			flowLayout1.setAlignment(java.awt.FlowLayout.RIGHT);
			flowLayout1.setVgap(5);
			flowLayout1.setHgap(2);
			pAccept = new JPanel();
			pAccept.setLayout(flowLayout1);
			pAccept.setPreferredSize(new java.awt.Dimension(550,30));
			pAccept.add(getBSave(), null);
			//pAccept.add(getBAccept(), null);
			pAccept.add(getBCancel(), null);
		}
		return pAccept;
	}

	/**
	 * This method initializes getBAccept
	 *
	 * @return javax.swing.JButton
	 */
	public JButton getBSave() {
		if (bSave == null) {
			bSave = new JButton();
			bSave.setPreferredSize(new java.awt.Dimension(85,25));
			bSave.setText(Messages.getText("salvar"));
			//bSave.setEnabled(false);
			bSave.setEnabled(true);
		}
		return bSave;
	}

	/**
	 * This method initializes getBAccept
	 *
	 * @return javax.swing.JButton
	 */
	public JButton getBAccept() {
		if (bAccept == null) {
			bAccept = new JButton();
			bAccept.setPreferredSize(new java.awt.Dimension(85,25));
			bAccept.setText(Messages.getText("aplicar"));
		}
		return bAccept;
	}

	/**
	 * This method initializes getBCancel
	 *
	 * @return javax.swing.JButton
	 */
	public JButton getBCancel() {
		if (bCancel == null) {
			bCancel = new JButton();
			bCancel.setPreferredSize(new java.awt.Dimension(85,25));
			bCancel.setText(Messages.getText("cerrar"));
		}
		return bCancel;
	}

	/**
	 * This method initializes jPanel
	 *
	 * @return javax.swing.JPanel
	 */
	private JPanel getPBorder() {
		if (pBorder == null) {
			GridBagConstraints gridBagConstraints31 = new GridBagConstraints();
			gridBagConstraints31.gridx = 0;
			gridBagConstraints31.gridy = 5;
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
			gridBagConstraints3.gridx = 0;
			gridBagConstraints3.gridy = 4;
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.gridx = 0;
			gridBagConstraints2.gridy = 3;
			GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
			gridBagConstraints4.gridx = 0;
			gridBagConstraints4.gridy = 2;
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.gridx = 0;
			gridBagConstraints1.gridy = 1;
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.gridx = 0;
			gridBagConstraints.gridy = 0;
			pBorder = new JPanel();
			pBorder.setLayout(new GridBagLayout());
			pBorder.setPreferredSize(new java.awt.Dimension(550,400));
			pBorder.setBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED));
			pBorder.add(getPCoordPixel(), gridBagConstraints);
			pBorder.add(getPCoordReal(), gridBagConstraints1);
			pBorder.add(getPResize(), gridBagConstraints4);
			pBorder.add(getPResolution(), gridBagConstraints2);
			pBorder.add(getPSelection(), gridBagConstraints3);
			pBorder.add(getPCheck(), gridBagConstraints31);
		}
		return pBorder;
	}

	/**
	 * This method initializes jPanel
	 *
	 * @return javax.swing.JPanel
	 */
	private JPanel getPCheck() {
		if (pCheck == null) {
			jLabel = new JLabel();
			jLabel.setText(Messages.getText("crear_1_capa_por_banda"));
			jLabel.setPreferredSize(new java.awt.Dimension(500,16));
			pCheck = new JPanel();
			pCheck.setLayout(new BorderLayout());
			pCheck.setPreferredSize(new Dimension(500 - 10, 30));
			pCheck.add(getCbOneLyrPerBand(), java.awt.BorderLayout.WEST);
			pCheck.add(jLabel, java.awt.BorderLayout.CENTER);
		}
		return pCheck;
	}

	/**
	 * This method initializes jCheckBox1
	 *
	 * @return javax.swing.JCheckBox
	 */
	public JCheckBox getCbOneLyrPerBand() {
		if (jCheckBox == null) {
			jCheckBox = new JCheckBox();
		}
		return jCheckBox;
	}

	/***************************Metodos***************************/


	public void setComponentSize(int w, int h){
		/*wPanel = w; hPanel = h;
		wComp = wPanel - 6; hComp = hPanel - 40;
		hTable = hComp - 412;
		wTable = wComp - 10;

		if (contract){
			this.setSize(wPanel, hPanel - hTable - 120 - 30);
			pTailImage.setPreferredSize(new java.awt.Dimension(wPanel, hPanel - hTable - 120 - 30));
			this.getPBorder().setSize(wComp, hComp - hTable - 120 - 30);
			this.getPBorder().setPreferredSize(new java.awt.Dimension(wComp, hComp - hTable - 120 - 30));
		}
		if (!contract){
			this.setSize(wPanel, hPanel);
			pTailImage.setPreferredSize(new java.awt.Dimension(wPanel, hPanel));
			this.getPBorder().setSize(wComp, hComp);
			this.getPBorder().setPreferredSize(new java.awt.Dimension(wComp, hComp));

		}

		pCoordPixel.setComponentSize(wTable, 90);
		pCoordReal.setComponentSize(wTable, 90);
		pResize.setPreferredSize(new java.awt.Dimension(wTable - 4,32));
		pResolution.setPreferredSize(new java.awt.Dimension(wTable,120));
		pResolution.setComponentSize(wTable,120);
		pSelection.setPreferredSize(new java.awt.Dimension(wTable,hTable));
		pSelection.setBorder(javax.swing.BorderFactory.createTitledBorder(null, Messages.getText("seleccion_bandas"), javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Dialog", java.awt.Font.PLAIN, 10), null));
		tSelection.setComponentSize(wTable - 20, hTable - 40);
		pCheck.setPreferredSize(new java.awt.Dimension(wTable,30));
		pButtons.setPreferredSize(new java.awt.Dimension(wTable,30));*/


	}

	/**
	 * Devuelve los valores de las coordenadas pxel
	 * introducidas por el usuario.
	 * @return
	 */
	public String[] getCoordPixel(){
		String[] s = {String.valueOf(pxMinX), String.valueOf(pxMinY), String.valueOf(pxMaxX), String.valueOf(pxMaxY)};
		return s;
	}

	/**
	 * Asigna los valores de las coordenadas pxel.
	 * @return
	 */
	private void setCoordPixel(String[] values){
		this.pCoordPixel.setValues(values);
	}

	/**
	 * Devuelve los valores de las coordenadas reales
	 * introducidas por el usuario.
	 * @return
	 */
	public String[] getCoordReal(){
		String[] s = {String.valueOf(wcMinX), String.valueOf(wcMaxY), String.valueOf(wcMaxX), String.valueOf(wcMinY)};
		return s;
	}

	/**
	 * Asigna los valores de las coordenadas reales.
	 * @return
	 */
	private void setCoordReal(String[] values){
		this.pCoordReal.setValues(values);
	}

	/**
	 * Asigna las coordenadas del mundo real a partir de nmeros en coma flotante.
	 * @param minx coordenada mnima de X
	 * @param miny coordenada mnima de Y
	 * @param maxx coordenada mxima de X
	 * @param maxy coordenada mxima de Y
	 * @param dec Nmero de decimales a mostrar en la caja de texto
	 */
	public void setCoorRealFromDouble(double minx, double miny, double maxx, double maxy, int dec){
		wcMinX = minx;
		wcMinY = miny;
		wcMaxX = maxx;
		wcMaxY = maxy;
   		setCoordReal(getCoord(minx, miny, maxx, maxy, dec));
	}

	/**
	 * Asigna las coordenadas pixel a partir de nmeros en coma flotante.
	 * @param minx coordenada mnima de X
	 * @param miny coordenada mnima de Y
	 * @param maxx coordenada mxima de X
	 * @param maxy coordenada mxima de Y
	 * @param dec Nmero de decimales a mostrar en la caja de texto
	 */
	public void setCoorPixelFromDouble(double minx, double miny, double maxx, double maxy, int dec){
		pxMinX = minx;
		pxMinY = miny;
		pxMaxX = maxx;
		pxMaxY = maxy;
   		setCoordPixel(getCoord(minx, miny, maxx, maxy, dec));
	}

	/**
	 * Obtiene un texto con las coordenadas a partir de nmeros en coma flotante.
	 * @param minx coordenada mnima de X
	 * @param miny coordenada mnima de Y
	 * @param maxx coordenada mxima de X
	 * @param maxy coordenada mxima de Y
	 * @param dec Nmero de decimales a mostrar en la caja de texto
	 */
	private String[] getCoord(double minx, double miny, double maxx, double maxy, int dec){
		String[] coordPx = new String[4];
		int indexPoint = String.valueOf(minx).indexOf('.');
   		try{
   			coordPx[0] = String.valueOf(minx).substring(0, indexPoint + dec);
   		}catch(StringIndexOutOfBoundsException ex){
   			coordPx[0] = String.valueOf(minx);
   		}
   		indexPoint = String.valueOf(miny).indexOf('.');
   		try{
   			coordPx[1] = String.valueOf(miny).substring(0, indexPoint + dec);
   		}catch(StringIndexOutOfBoundsException ex){
   			coordPx[1] = String.valueOf(miny);
   		}
   		indexPoint = String.valueOf(maxx).indexOf('.');
   		try{
   			coordPx[2] = String.valueOf(maxx).substring(0, indexPoint + dec);
   		}catch(StringIndexOutOfBoundsException ex){
   			coordPx[2] = String.valueOf(maxx);
   		}
   		indexPoint = String.valueOf(maxy).indexOf('.');
   		try{
   			coordPx[3] = String.valueOf(maxy).substring(0, indexPoint + dec);
   		}catch(StringIndexOutOfBoundsException ex){
   			coordPx[3] = String.valueOf(maxy);
   		}
   		return coordPx;
	}

	/**
	 * Asigna el valor del campo "tamao de celda" a partir del double que lo representa
	 * y con el nmero de decimales que se especifica en el parmetro dec
	 * @param cellSize Tamao de celda en double
	 * @param dec Nmero de decimales
	 */
	public void setCellSizeText(double cellSize, int dec){
		realCellSize = cellSize;
		pResolution.setCellSizeText(cellSize, dec);
	}

	/**
	 * Asigna el valor del campo "Ancho" a partir del double que lo representa
	 * y con el nmero de decimales que se especifica en el parmetro dec
	 * @param width Ancho
	 * @param dec Nmero de decimales
	 */
	public void setWidthText(double width, int dec){
		realWidth = width;
		pResolution.setWidthText(width, dec);
	}

	/**
	 * Asigna el valor del campo "Alto" a partir del double que lo representa
	 * y con el nmero de decimales que se especifica en el parmetro dec
	 * @param height Alto
	 * @param dec Nmero de decimales
	 */
	public void setHeightText(double height, int dec){
		realHeight = height;
		pResolution.setHeightText(height, dec);
	}

	/**
	 * Asigna la relacin entre el ancho y alto de la imagen
	 * @param rel
	 */
	public void setRelWidthHeight(double rel){
		relWidthHeight = rel;
	}

	/**
	 * Obtiene la relacin entre el ancho y alto de la imagen
	 * @return
	 */
	public double getRelWidthHeight(){
		return relWidthHeight;
	}

	/**
	 * Devuelve el valor del campo "tamao de celda"
	 * @return
	 */
	public double getCellSizeText(){
		return realCellSize;
	}

	/**
	 * Devuelve el valor del campo "tamao de celda"
	 * @return
	 */
	public double getWidthText(){
		return realWidth;
	}

	/**
	 * Devuelve el valor del campo "tamao de celda"
	 * @return
	 */
	public double getHeightText(){
		return realHeight;
	}

	/**
	 * Expande o contrae el panel inferior segn el estado en el que se encuentre.
	 *
	 */
	public void contractExpandPanel(){
		boolean aux = contract;
		if (aux){
			this.setSize(wPanel, hPanel);
			pTailImage.setPreferredSize(new java.awt.Dimension(wPanel, hPanel));
			this.getPBorder().setSize(wComp, hComp);
			this.getPBorder().setPreferredSize(new java.awt.Dimension(wComp, hComp));
			this.getPResolution().setVisible(true);
			this.getPSelection().setVisible(true);
			this.getPCheck().setVisible(true);
		}
		if (!aux){
			this.setSize(wPanel, hPanel - hTable - 120 - 30);
			pTailImage.setPreferredSize(new java.awt.Dimension(wPanel, hPanel - hTable - 120 - 30));
			this.getPBorder().setSize(wComp, hComp - hTable - 120 - 30);
			this.getPBorder().setPreferredSize(new java.awt.Dimension(wComp, hComp - hTable - 120 - 30));
			this.getPResolution().setVisible(false);
			this.getPSelection().setVisible(false);
			this.getPCheck().setVisible(false);
		}
		this.validate();
		if (aux)	contract = false;
		else		contract = true;

		if (contract == true){
			pResize.getButton(2).setToolTipText("expandir");
			pResize.getButton(2).setIcon(new ImageIcon(getClass().getResource("images/down.png")));
		}
		if (contract == false){
			pResize.getButton(2).setToolTipText("contraer");
			pResize.getButton(2).setIcon(new ImageIcon(getClass().getResource("images/up.png")));
		}

	}

	/**
	 * Obtiene la coordenada de mxima X en pixel.
	 * @return Coordenada de mxima X en pixel
	 */
	public double getPxMaxX() {
		return pxMaxX;
	}

	/**
	 * Obtiene la coordenada de mxima Y en pixel.
	 * @return Coordenada de mxima Y en pixel
	 */
	public double getPxMaxY() {
		return pxMaxY;
	}

	/**
	 * Obtiene la coordenada de mnima X en pixel.
	 * @return Coordenada de mnima X en pixel
	 */
	public double getPxMinX() {
		return pxMinX;
	}

	/**
	 * Obtiene la coordenada de mnima Y en pixel.
	 * @return Coordenada de mnima Y en pixel
	 */
	public double getPxMinY() {
		return pxMinY;
	}

	/**
	 * Obtiene la coordenada de mxima X real.
	 * @return Coordenada de mxima X real.
	 */
	public double getWcMaxX() {
		return wcMaxX;
	}

	/**
	 * Obtiene la coordenada de mxima Y real.
	 * @return Coordenada de mxima Y real
	 */
	public double getWcMaxY() {
		return wcMaxY;
	}

	/**
	 * Obtiene la coordenada de mnima X real.
	 * @return Coordenada de mnima X real
	 */
	public double getWcMinX() {
		return wcMinX;
	}

	/**
	 * Obtiene la coordenada de mnima Y real.
	 * @return Coordenada de mnima Y real
	 */
	public double getWcMinY() {
		return wcMinY;
	}

	public void selectLayerFullExtend(){
		this.setCoorRealFromDouble(extent.getMinX(), extent.getMaxY(), extent.getMaxX(), extent.getMinY(), 6);
   		Rectangle2D r = org.cresques.util.Utilities.getPxRectFromMapRect(extent, widthPx, heightPx, extent);
   		this.setCoorPixelFromDouble(r.getMinX(), r.getMinY(), r.getMaxX(), r.getMaxY(), 3);
   		this.setWidthText(r.getWidth(), 0);
   		this.setHeightText(r.getHeight(), 0);
   		this.setCellSizeText((extent.getWidth() / r.getWidth()), 4);
   		this.setRelWidthHeight((double)(r.getWidth() / r.getHeight()));
   		this.parent.setInitValuesToSize(r.getWidth(), r.getHeight());
	}
}
