/*
 * Created on 09-ene-2007
 *
 * gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */

package com.iver.cit.jdwglib.dwg.readers;

import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

import com.iver.cit.jdwglib.dwg.DwgFile;

/**
 * @author alzabord
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class DwgFileVR2004Reader extends DwgFileV15Reader {
	
	protected DwgFile dwgFile;
	protected ByteBuffer bb;
	static byte[] magicNumber = null;
	
	private String version = "";
	private byte[] unknowSixBytes;
	private byte unknowByte;
	private int previewAddress;
	private byte dwgVer;
	private byte maintReleaseVer;
	private short codepage;
	private byte[] threeBytes0;
	private int securityType;
	private int unknowLong;
	private int dwgPropertyAddr;
	private int vbaProjectAddr;
	private int oneHundredTwentyEight;
	private byte[] unknowTwoBytes;
	private byte[] eightyTwoBytes0;

	private byte[] encryptedData;
	private byte[] decryptedData;
	private String fileID="";
	
	private int rootTreeNodeGap;
	private int lowermostLeftTreeNodeGap;
	private int lowermostRightTreeNodeGap;
	private int unknown;
	private int lastSectionId; //Identificador de la ultima seccion
	private int lastSectionAddress; //Direccion de la ultima seccion
	private int SecondHeaderAddress; //Direccion de la segunda cabecera ???
	private int gapAmount; //Numero de huecos ?
	private int sectionAmount; //Numero de secciones
	private int sectionMapId; //Identificador de la seccion de mapa
	private int sectionMapAddress; //Direccion la seccion de mapa
	private int sectionInfoId; //Identificador de la seccion de informacion
	private int sectionInfoIndex = 0; //Indice en el array de secciones de la seccion de informacion
	private int sectionArraySize;//???
	private int gapArraySize;//???
	private int crc;
	private byte[] generatedSequence;
	
	private Section[] sections;
	private Gap[] gaps;
	
	static final int TYPE_SECTION_MAP = 1;
	static final int TYPE_SECTION_INFO = 2;
	

	
	/**
	 * Reads the DWG version 2004 format
	 * 
	 * @param dwgFile
	 *            Represents the DWG file that we want to read
	 * @throws IOException
	 *             When DWG file path is wrong
	 */
	public void read(DwgFile dwgFile, ByteBuffer bb) throws IOException {
		this.dwgFile = dwgFile;
		this.bb = bb;

		readDwgR2004FileHeader(bb);
		System.out.println("Posicion = "+bb.position());
		bb.position(0);
		readDwgR2004SectionMap(bb);
		
		readInfoSection(bb);
		
		/*
		readDwgSectionOffsets();
		try {
			readDwgObjectOffsets();
			//readDwgClasses(bb);
		} catch (Exception e) {
			//		    System.out.println("Error leyendo offsets y classes. Posible
			// corrupci�n en" +
			//		    		"el DWG file ...");
		}
		readDwgObjects();
		//checkSuperentities();
		 */
	}
	
	protected void readInfoSection(ByteBuffer bb){
		int numDescriptions;
		int dos;
		int entero29696;
		int cero;
		int unknow;
		
		int sizeOfSection;
		int unknow2;
		int numSections;
		int bloquesDeSeccionALeer;
		int maxDecrompessedSize;
		int unknow3;
		int compressed;
		int sectionType;
		int encrypted;
		String sectionName="";
		
		int sectionNumber;
		int dataSize;
		int startOffset;
		int unknow4;
		
		char c;
		int pos;
		/*
		 * OJO: Utilizamos un array con un elemento mas de los necesarios porque en el archivo DWG
		 * se hace referencia a un vector "1 indexed" por lo que siempre que queramos acceder
		 * a este array tendremos que comenzar con el indice 1.  
		 */
		/* TODO:
		 * Tal vez habra que cambiar el modo de guardarnos el arrays de secciones porque no se cargan consecutivamente.
		 * Pensar si sera mejor un map.
		 */
		
//		for (int i=1; i<=sections.length; i++){
//			if(sections[i].getNumber() == sectionInfoId){
//				sectionInfoIndex = i;
//				break;
//			}
//		}
		if(sectionInfoIndex == 0){
			System.out.println(" ****** Posible error buscando la Section Info en el array de secciones.");
		}

		pos=sections[sectionInfoIndex].getAdress();
		
		
		bb.position(pos);

		ByteBuffer dData = readSystemSection(bb);
		dData.order(ByteOrder.LITTLE_ENDIAN);
		dData.position(0);

		numDescriptions = dData.getInt();
		dos = dData.getInt();
		entero29696 = dData.getInt();
		cero = dData.getInt();
		unknow = dData.getInt();
		System.out.println("numDescriptions = 0x"+Integer.toHexString(numDescriptions)+"("+numDescriptions+")");
		System.out.println("dos = 0x"+Integer.toHexString(dos)+"("+dos+")");
		System.out.println("entero29696 = 0x"+Integer.toHexString(entero29696)+"("+entero29696+")");
		System.out.println("cero = 0x"+Integer.toHexString(cero)+"("+cero+")");
		System.out.println("unknow = 0x"+Integer.toHexString(unknow)+"("+unknow+")");
		
		for (int i=0; i<numDescriptions; i++){
			sizeOfSection = dData.getInt();
			unknow2 = dData.getInt();
			numSections = dData.getInt();
			maxDecrompessedSize = dData.getInt();
			unknow3 = dData.getInt();
			compressed = dData.getInt();
			sectionType = dData.getInt();
			encrypted = dData.getInt();
			sectionName="";
			for (int j=0; j<64; j++){
				c=(char)dData.get();
				sectionName = sectionName + c;
			}
			System.out.println("\tsizeOfSection = 0x"+Integer.toHexString(sizeOfSection)+"("+sizeOfSection+")");
			System.out.println("\tunknow2 = 0x"+Integer.toHexString(unknow2)+"("+unknow2+")");
			System.out.println("\tnumSections = 0x"+Integer.toHexString(numSections)+"("+numSections+")");
			System.out.println("\tmaxDecrompessedSize = 0x"+Integer.toHexString(maxDecrompessedSize)+"("+maxDecrompessedSize+")");
			System.out.println("\tunknow3 = 0x"+Integer.toHexString(unknow3)+"("+unknow3+")");
			System.out.println("\tcompressed = 0x"+Integer.toHexString(compressed)+"("+compressed+")");
			System.out.println("\tsectionType = 0x"+Integer.toHexString(sectionType)+"("+sectionType+")");
			System.out.println("\tencrypted = 0x"+Integer.toHexString(encrypted)+"("+encrypted+")");
			System.out.println("\tsectionName = "+sectionName);
			System.out.println();

//			if(numSections == 0 ){//&& (sizeOfSection != 0 || sectionType == 0x06)){
//				bloquesDeSeccionALeer = 1;
//			}else{
//				bloquesDeSeccionALeer = numSections;
//			}
			bloquesDeSeccionALeer = numSections;
			
			for(int k=0; k<bloquesDeSeccionALeer; k++){
				sectionNumber = dData.getInt();
				dataSize = dData.getInt();
				startOffset = dData.getInt();
				unknow4 = dData.getInt();

				System.out.println("\t\tsectionNumber = 0x"+Integer.toHexString(sectionNumber)+"("+sectionNumber+")");
				System.out.println("\t\tdataSize = 0x"+Integer.toHexString(dataSize)+"("+dataSize+")");
				System.out.println("\t\tstartOffset = 0x"+Integer.toHexString(startOffset)+"("+startOffset+")");
				System.out.println("\t\tunknow4 = 0x"+Integer.toHexString(unknow4)+"("+unknow4+")");
				System.out.println();

			}
		}
	}

	protected void readDwgR2004FileHeader(ByteBuffer bb) {
		bb.order(ByteOrder.LITTLE_ENDIAN);
		for(int i=0; i<6; i++){
			version = version + (char)bb.get(); 
		}
		if(!version.equalsIgnoreCase("AC1018")){
			System.out.println("Error en la version: "+version);
		}

		/*Fuera de especificaciones:
		 * En las especificaciones pone que aqu van 6 bytes con 0x00 pero
		 * parece que van 5 con 0x00 y 1 con 0x4C
		*/ 
		unknowSixBytes = new byte[6];
		int pos = 6;
		for (int i=0; i<5; i++){
			unknowSixBytes[i] = bb.get();
			if (unknowSixBytes[i] != 0){
				System.out.println("(unknowSixBytes) En la posicion 0x" + 
						Integer.toHexString(pos+i) + 
						" debera haber un 0, pero hay un 0x" + 
						Integer.toHexString(eightyTwoBytes0[i]));
			}
		}
		pos = 0xB;
		unknowSixBytes[5] = bb.get();
		if (unknowSixBytes[5] != 0x4C){
			System.out.println("(unknowSixBytes) En la posicion 0x" + 
					Integer.toHexString(pos) + 
					" debera haber un 0x4C, pero hay un 0x" + 
					Integer.toHexString(unknowSixBytes[5]));
		}
		
		
		unknowByte = bb.get();
		previewAddress = bb.getInt();
		dwgVer = bb.get();
		maintReleaseVer = bb.get();
		codepage = bb.getShort();
		/*FIXME:
		 * Los cargo solamente para ir comprobando que me encuentro bien posicionado en el fichero
		 * cuando termine la depuracion se puede sustituir por un salto en los bytes
		 */
		/*Fuera de especificaciones:
		 * En las especificaciones pone que aqu van 3 bytes con 0x00 pero
		 * parece que van 1 con 0x00, 1 con 0x19 y 1 con 0x4C
		*/ 
		threeBytes0 = new byte[3];
		pos=0x15;
		bb.get(threeBytes0);
		if (threeBytes0[0] != 0x0){
			System.out.println("En la posicion 0x" + 
					Integer.toHexString(pos) + 
					" debera haber un 0x0, pero hay un 0x" + 
					Integer.toHexString(threeBytes0[0]));
		}
		pos=0x16;
		if (threeBytes0[1] != 0x19){
			System.out.println("En la posicion 0x" + 
					Integer.toHexString(pos) + 
					" debera haber un 0x19, pero hay un 0x" + 
					Integer.toHexString(threeBytes0[1]));
		}
		pos=0x17;
		if (threeBytes0[2] != 0x4C){
			System.out.println("En la posicion 0x" + 
					Integer.toHexString(pos) + 
					" debera haber un 0x4C, pero hay un 0x" + 
					Integer.toHexString(threeBytes0[2]));
		}
//		for(int i=0; i<3; i++){
//			System.out.println("threeBytes0["+i+"] = 0x"+Integer.toHexString(threeBytes0[i]));
//		}
		securityType = bb.getInt();
		unknowLong = bb.getInt();
		dwgPropertyAddr = bb.getInt();
		vbaProjectAddr = bb.getInt();
		oneHundredTwentyEight = bb.getInt();
		if (oneHundredTwentyEight != 128){
			System.out.println("En la posicion 0x28 debera haber un 128(0x80) , pero hay un 0x"+Integer.toHexString(oneHundredTwentyEight));
		}
		pos=0x2C;
		/*FIXME:
		 * Los cargo solamente para ir comprobando que me encuentro bien posicionado en el fichero
		 * cuando termine la depuracion se puede sustituir por un salto en los bytes
		 */
		unknowTwoBytes = new byte[2];
		for (int i=0; i<2; i++){
			unknowTwoBytes[i] = bb.get();
		}
		pos=0x2E;
		eightyTwoBytes0 = new byte[82];
		for (int i=0; i<eightyTwoBytes0.length; i++){
			eightyTwoBytes0[i] = bb.get();
			if (eightyTwoBytes0[i] != 0){
				System.out.println("(eightyTwoBytes0) En la posicion 0x" + 
						Integer.toHexString(pos+i) + 
						" debera haber un 0, pero hay un 0x" + 
						Integer.toHexString(eightyTwoBytes0[i]));
			}
		}

		int len = 0x6C;
		encryptedData = new byte[len];
		decryptedData = new byte[len];
		if(magicNumber==null){
			DwgFileVR2004Reader.setMagicNumber();	
		}
		
		for (int i=0; i<len; i++){
			encryptedData[i] = bb.get();
			decryptedData[i] = (byte) (encryptedData[i] ^ magicNumber[i]);
		}
		ByteBuffer decryptedDataBB = ByteBuffer.allocate(len);
		decryptedDataBB.put(decryptedData);
		decryptedDataBB.position(0);
		decryptedDataBB.order(ByteOrder.LITTLE_ENDIAN);
		for(int i=0; i<12; i++){
			fileID = fileID + (char)decryptedDataBB.get();
		}
		int entero = decryptedDataBB.getInt();
		if(entero!=0){
			System.out.println("En la posicion 0x0C del decryptedDataDD debera haber un 0x00 , pero hay un 0x"+Integer.toHexString(entero));
		}
		entero = decryptedDataBB.getInt();
		if(entero!=0x6C){
			System.out.println("En la posicion 0x10 del decryptedDataDD debera haber un 0x6C , pero hay un 0x"+Integer.toHexString(entero));
		}
		entero = decryptedDataBB.getInt();
		if(entero!=0x04){
			System.out.println("En la posicion 0x14 del decryptedDataDD debera haber un 0x04 , pero hay un 0x"+Integer.toHexString(entero));
		}
		rootTreeNodeGap = decryptedDataBB.getInt();
		System.out.println("rootTreeNodeGap = "+rootTreeNodeGap);
		lowermostLeftTreeNodeGap = decryptedDataBB.getInt();
		System.out.println("lowermostLeftTreeNodeGap = "+lowermostLeftTreeNodeGap);
		lowermostRightTreeNodeGap = decryptedDataBB.getInt();
		System.out.println("lowermostRightTreeNodeGap = "+lowermostRightTreeNodeGap);
		unknown = decryptedDataBB.getInt();
		System.out.println("unknown = "+unknown);
		lastSectionId = decryptedDataBB.getInt(); //Identificador de la ultima seccion ???
		/*
		 * En las pruebas realizadas el lastSectionId coincide con el sectionMapId sin embargo
		 * lasSectionAddress no coincide con sectionMapAdress.
		 * 
		 */
		System.out.println("lastSectionId = "+lastSectionId);
		lastSectionAddress = decryptedDataBB.getInt(); //Direccion de la ultima seccion ???
		System.out.println("lastSectionAddress = "+lastSectionAddress);
		entero = decryptedDataBB.getInt();
		if(entero!=0){
			System.out.println("En la posicion 0x30 del decryptedDataDD debera haber un 0x00 , pero hay un 0x"+Integer.toHexString(entero));
		}
		SecondHeaderAddress = decryptedDataBB.getInt(); //Direccion de la segunda cabecera ?
		System.out.println("SecondHeaderAddress = "+SecondHeaderAddress);
		entero = decryptedDataBB.getInt();
		if(entero!=0){
			System.out.println("En la posicion 0x38 del decryptedDataDD debera haber un 0x00 , pero hay un 0x"+Integer.toHexString(entero));
		}
		gapAmount = decryptedDataBB.getInt(); //Numero de huecos entre secciones
		System.out.println("gapAmount = "+gapAmount);
		sectionAmount = decryptedDataBB.getInt(); //Numero de secciones
		System.out.println("sectionAmount = "+sectionAmount);
		entero = decryptedDataBB.getInt();
		if(entero!=0x20){
			System.out.println("En la posicion 0x44 del decryptedDataDD debera haber un 0x20 , pero hay un 0x"+Integer.toHexString(entero));
		}
		entero = decryptedDataBB.getInt();
		if(entero!=0x80){
			System.out.println("En la posicion 0x48 del decryptedDataDD debera haber un 0x80 , pero hay un 0x"+Integer.toHexString(entero));
		}
		entero = decryptedDataBB.getInt();
		if(entero!=0x40){
			System.out.println("En la posicion 0x4C del decryptedDataDD debera haber un 0x40 , pero hay un 0x"+Integer.toHexString(entero));
		}
		sectionMapId = decryptedDataBB.getInt(); //Identificador de la seccion de mapa
		System.out.println("sectionMapId = "+sectionMapId);
		sectionMapAddress = decryptedDataBB.getInt()+0x100; //Direccion la seccion de mapa
		System.out.println("sectionMapAddress = "+sectionMapAddress);
		entero = decryptedDataBB.getInt();
		if(entero!=0){
			System.out.println("En la posicion 0x54 del decryptedDataDD debera haber un 0x00 , pero hay un 0x"+Integer.toHexString(entero));
		}
		sectionInfoId = decryptedDataBB.getInt(); //Identificador de la seccion de informacion
		System.out.println("sectionInfoId = "+sectionInfoId);
		sectionArraySize = decryptedDataBB.getInt();//???
		System.out.println("sectionArraySize = "+sectionArraySize);
		gapArraySize = decryptedDataBB.getInt();//???
		System.out.println("gapArraySize = "+gapArraySize);
		crc = decryptedDataBB.getInt();
		System.out.println("crc = "+crc);
		len = 0x14;
		generatedSequence = new byte[len];
		bb.get(generatedSequence);
		System.out.println("Generated Sequence:");
		for(int i=0; i<len; i++){
			System.out.print(Integer.toHexString(generatedSequence[i] & 0xff)+" ");
		}
		System.out.println();
		

	}
	
	protected void readDwgR2004SectionMap(ByteBuffer bb){
		bb.order(ByteOrder.LITTLE_ENDIAN);
		bb.position(sectionMapAddress);
		ByteBuffer dData = readSystemSection(bb);
		dData.order(ByteOrder.LITTLE_ENDIAN);
		dData.position(0);
		
		int sectionNumber;
		int gapNumber=0;
		sections = new Section[sectionAmount+1];
		gaps = new Gap[gapAmount];
		/*
		 * De momento en los archivos de prueba siempre nos hemos encontrado que
		 * los numero de seccion van correlativos empezando por el 1, pero
		 * podra esto no ser siempre as?. Si no fuera as cmo se calcularan
		 * las direcciones de las distintas secciones? sumando el tamao de una 
		 * a su direccion para calcular la direccion de la siguiente encontrada
		 * o de la siguiente respecto al numero de seccion?
		 * 
		 */
		
		sections[0] = new Section();
		sections[0].setNumber(0);
		sections[0].setSize(0);
		sections[0].setAdress(0x100);
		
		for (int i=1; i<=sectionAmount; i++){
			sections[i] = new Section();
			sectionNumber = dData.getInt();
			while(sectionNumber<0){
				if(gapNumber >= gapAmount){
					System.out.println("ERROR: Se han encontrado mas gaps que los que hay definidos");
				}
				gaps[gapNumber] = new Gap();
				gaps[gapNumber].setSize(dData.getInt());
				gaps[gapNumber].setParent(dData.getInt());
				gaps[gapNumber].setLeft(dData.getInt());
				gaps[gapNumber].setRight(dData.getInt());
				gaps[gapNumber].setZero(dData.getInt());
				System.out.println("**** gapNumber = "+gapNumber+
						" Size = "+gaps[gapNumber].getSize()+
						" Parent = "+gaps[gapNumber].getParent()+
						" Left = "+gaps[gapNumber].getLeft()+
						" Right = "+gaps[gapNumber].getRight()+
						" Zero = "+gaps[gapNumber].getZero());
				gapNumber++;
				sectionNumber = dData.getInt();
			}
			if(sectionNumber == sectionInfoId){
				sectionInfoIndex = i;
			}
			sections[i].setNumber(sectionNumber);
			sections[i].setSize(dData.getInt());
			if(i==1){
				sections[i].setAdress(0x100);
			} else {
				sections[i].setAdress(sections[i-1].getAdress()+sections[i-1].getSize());
			}
			System.out.println("**** i= "+(i)+
					" sectionNumber = "+sections[i].getNumber()+
					" \tsectionAdress = "+sections[i].getAdress()+" (0x"+Integer.toHexString(sections[i].getAdress())+")"+
					" \tsectionSize = "+sections[i].getSize()+" (0x"+Integer.toHexString(sections[i].getSize())+") ");
		}
	}

	protected ByteBuffer readSystemSection(ByteBuffer bb) {
		System.out.println ("readSystemSection: bb.position = " + bb.position()+
				" bb.order = "+bb.order());
				
		int sectionType = bb.getInt();
		int decompressedSize = bb.getInt();
		int compressedSize = bb.getInt();
		int compressionType = bb.getInt();
		int checksum = bb.getInt();
		System.out.println ("readSystemSection:\n\tsectionType = 0x"+Integer.toHexString(sectionType)+" (" + sectionType +") "+
				"\n\tdecompressedSize = 0x" +Integer.toHexString(decompressedSize)+" (" + decompressedSize +") "+
				"\n\tcompressedSize = 0x" +Integer.toHexString(compressedSize)+" (" + compressedSize +") "+
				"\n\tcompressionType = 0x" +Integer.toHexString(compressionType)+" (" + compressionType +") "+
				"\n\tchecksum = 0x" +Integer.toHexString(checksum)+" (" + checksum +") ");
		
		return readCompressedSection(bb, compressedSize, decompressedSize, checksum);
		
	}
	protected ByteBuffer readCompressedSection(ByteBuffer bb, int compressedSize, int decompressedSize, int checksum) {
		int pos = bb.position();
		System.out.println ("readCompressedSection: bb.position = " + pos);
		long calcSize=0;
		ByteBuffer decompressedData;
		decompressedData = ByteBuffer.allocate(decompressedSize);
		decompressedData.order(ByteOrder.LITTLE_ENDIAN);
		decompressedData.position(0);
		int opcodes=0;
		byte opcode1 = bb.get();
		byte opcode2;
		int literalLength=0;
		int lls=0;
		int compressedBytes = 0;
		int compOffset = 0;
		int litCount = 0;
		int[] twoBytesOffset;
		byte b;
		try{
		if ( ((opcode1&0xFF) >= 0x00) && ((opcode1&0xFF) <= 0x0F)){
			literalLength = readLiteralLength(bb, opcode1);
			lls++;
			if (literalLength != 0){
				for(int i=0; i<literalLength; i++){
					b = bb.get();
					decompressedData.put(b);
					System.out.print ("0x"+Integer.toHexString(b & 0xFF)+" ");
				}
				System.out.println();
				opcode1 = bb.get();
			}
			calcSize = calcSize + literalLength;

		}else{
			/*TODO: Sustituir esto por un error cuando se compruebe que esto
			 * funciona como se supone que debe funcionar.
			 */
			System.out.println("** " +lls + " ** Posible error Leyendo un literal Length. opcode1 = " + opcode1 +
					" Parece un \"set of compression opcodes\"");
		}
		
		while(opcode1 != 0x11){
			System.out.println("pos =0x"+(bb.position()-1));
			System.out.println("opcode1 =0x"+Integer.toHexString(opcode1&0xFF));
			if ( (opcode1&0xFF) >= 0x00 && (opcode1&0xFF) <= 0x0F){
				/*TODO: Sustituir esto por un error cuando se compruebe que esto
				 * funciona como se supone que debe funcionar.
				 */
				System.out.println(" ** Posible error Leyendo un \"set of compression opcodes\". opcode1 = " + opcode1 +
						" Parece un Literal Lenght");
				continue;
			}else{
				compressedBytes = 0;
				compOffset = 0;
				litCount = 0;
				opcodes++;
				if ((opcode1&0xFF) == 0x10) {
					compressedBytes = readLongCompressionOffset(bb) + 9;
					
					//TODO: Ver de juntar la siguientes lineas de codigo en un metodo porque se repiten unas cuantes veces 
					/* 
					 * bytes[0] es el offset
					 * bytes[1] es el literalCount
					 */
					twoBytesOffset = readTwoByteOffset(bb);
					compOffset = twoBytesOffset[0] + 0x3FFF;
					litCount = twoBytesOffset[1]; // esto puede tomar valores de 0 a 3.
					opcode2 = bb.get();
					if(litCount==0){
						litCount = readLiteralLength(bb,opcode2);
						if (litCount != 0){
							opcode2 = bb.get();
						}
					}
				} else if ((opcode1&0xFF)>=0x12 && (opcode1&0xFF)<=0x1f){
					compressedBytes = (opcode1 & 0x0F) + 2;
					
					twoBytesOffset = readTwoByteOffset(bb);
					compOffset = twoBytesOffset[0] + 0x3FFF;
					litCount = twoBytesOffset[1];
					opcode2 = bb.get();
					if(litCount==0){
						litCount = readLiteralLength(bb,opcode2);
						if (litCount != 0){
							opcode2 = bb.get();
						}
					}
				} else if ((opcode1&0xFF)==0x20){			
					// Segun kermit un usuario del foro de openDesign hay que aadirle 0x21
					compressedBytes = readLongCompressionOffset(bb)+0x21;
					
					twoBytesOffset = readTwoByteOffset(bb);
					compOffset = twoBytesOffset[0];
					litCount = twoBytesOffset[1];
					opcode2 = bb.get();
					if(litCount==0){
						litCount = readLiteralLength(bb,opcode2);
						if (litCount != 0){
							opcode2 = bb.get();
						}
					}
				} else if ((opcode1&0xFF)>=0x21 & (opcode1&0xFF)<=0x3f){
					compressedBytes = (opcode1 & 0xFF) - 0x1E;
					
					twoBytesOffset = readTwoByteOffset(bb);
					compOffset = twoBytesOffset[0];
					litCount = twoBytesOffset[1];
					opcode2 = bb.get();
					if(litCount==0){
						litCount = readLiteralLength(bb,opcode2);
						if (litCount != 0){
							opcode2 = bb.get();
						}
					}
				} else if ((opcode1&0xFF)>=0x40 & (opcode1&0xFF)<=0xFF){
					compressedBytes = ((opcode1 & 0xF0) >> 4) - 1;
					opcode2 = bb.get();
					compOffset = ((opcode2&0xFF)<<2)|((opcode1&0x0C)>>2);
					if ((opcode1&0x03) == 0x00){
						litCount = 0;
						System.out.println("$$$$$$$$$$ litCount==0");
						byte opcode3 = bb.get();
						litCount = readLiteralLength(bb,opcode3);
						if (litCount == 0){
							opcode2 = opcode3;
						}else {
							opcode2 = bb.get();
						}
					} else {
						litCount = opcode1&0x03;
						opcode2 = bb.get();
					}
				} else {
					System.out.println("HA APARECIDO UN opcode1 con valor " + Integer.toHexString(opcode1 & 0xFF));
					opcode2 = bb.get();
				}
//				compressedBytes = compressedBytes;
//				compOffset = compOffset;
//				litCount = litCount;
				if(compressedBytes < 0){
					System.out.println("***** compressedBytes < 0");
				}
				if(compOffset < 0){
					System.out.println("***** compOffset < 0");
				}
				if(litCount < 0){
					System.out.println("***** litCount < 0");
				}
				System.out.println ("readCompressedSection:" +
						"\n\tcompressedBytes = " + compressedBytes +
						"\n\tcompOffset = " + compOffset +
						"\n\tlitCount = " + litCount + 
						"\n\tposicion del siguiente opcode1 = " + (bb.position()-1) + 
						"\n\tposicion en decompressedData = " + (decompressedData.position())+
						"\n\topcode actual = "+opcode1+" (0x"+Integer.toHexString(opcode1)+")");

				if(compressedBytes > compOffset){
					System.out.println("Posible error en la lectura de una seccion comprimida:" +
							"No parece logico que compressedBytes > compOffset");
				}
				if(compOffset >= decompressedData.position()){
						System.out.println("Posible error en la lectura de una seccion comprimida:" +
								" el compOffset se sale. compOffset=0x"+Integer.toHexString(compOffset)+
								") posicion en datos descomprimidos =0x"+
								Integer.toHexString(decompressedData.position())+")");
				}
				/* Meter en decompressedData los compressedBytes apuntados por el compOffset.
				 * Hay que ver exactamente a que hace referencia esto.
				 * Bueno, parece que hace referencia a datos anteriormente introducidos en el stream de salida (decompressedData).
				 * Con las pruebas realizadas esto es correcto. 
				 */
				for(int i=0; i<compressedBytes; i++){
					b=decompressedData.get(decompressedData.position()-compOffset-1);
					System.out.print ("0x"+Integer.toHexString(b)+" ");
					decompressedData.put(b);
				}
				System.out.println ();
		
				//De nuevo ms pruebas, hay que ver la imaginacion que hay que echarle a esto.
				//Parece que esto esta bien.
				for(int i=0; i<litCount; i++){
					decompressedData.put(opcode2);
					System.out.print ("0x"+Integer.toHexString(opcode2 & 0xFF)+" ");
					opcode2 = bb.get();
				}
				System.out.println ();
				
				opcode1 = opcode2;
				calcSize = calcSize + compressedBytes + litCount;
			}
		}
		/* 
		 * TODO: Eliminar todo esto hasta el return cuando se termine de depurar esta parte del codigo.
		 * Slo es una traza.
		 */
		System.out.println("Tamanyo de la seccion descomprimida resultante = "+calcSize);
		decompressedData.position(0);
		System.out.println ("Datos descomprimidos:");
		String cadena;
		System.out.print(0+"\t");
		for(int i=0; i<decompressedSize; i++){
			cadena = Integer.toHexString(decompressedData.get()&0xFF);
			if(cadena.length()==1) cadena = "0"+cadena;
			System.out.print(cadena);
			if (((i+1)%16)==0){
				System.out.println();
				System.out.print((i+1)+"\t");
			}else {
				System.out.print("  ");
			}
		}
		System.out.println();
		FileOutputStream f = new FileOutputStream("/home/paco/decompressedData");
		decompressedData.position(0);
		for(int i=0; i<decompressedSize; i++){
			f.write(decompressedData.get());
		}
		f.close();
		}catch(Exception e){
			e.printStackTrace();
			decompressedData.position(0);
			System.out.println ("Datos descomprimidos:hasta el momento");
			String cadena;
			System.out.print(0+"\t");
			for(int i=0; i<decompressedSize; i++){
				cadena = Integer.toHexString(decompressedData.get()&0xFF);
				if(cadena.length()==1) cadena = "0"+cadena;
				System.out.print(cadena);
				if (((i+1)%16)==0){
					System.out.println();
					System.out.print((i+1)+"\t");
				}else {
					System.out.print("  ");
				}
			}
			System.out.println ();
		}
		
		return decompressedData;
	}

	private int readLiteralLength(ByteBuffer bb, byte b) {
		int total = 0; 
		if (0x01 <= b && b <= 0x0F) {
			total = (b&0xFF)+3;
			return total;
		} else if (b == 0x00) {
			total = 0x0F;
			b = bb.get();
			while (b==0x00){
				total += 0xFF;
				b = bb.get();
			}
			total += (b&0xFF) + 3;
			return total;
		} else if ((b & 0xF0) != 0){
			System.out.println("+++++ Se ha encontrado un Literal Lenght cuyo primer byte != 0 (__ & 0xF0). => LL=0. byte = 0x"+Integer.toHexString(b));
			//No entendemos muy bien el significado de este byte a 0xF0.
			//Devolvemos 0 y no leemos ningun byte para que sea el siguiente opcode.
			//Este codigo es el que motiva que no se lea el byte en este metodo
			//sino antes, y se le pase como parametro.
			//Esto parece correcto.
			return 0;
		}
		System.out.println("OJO! Se ha encontrado un Literal Lenght que empieza con "+Integer.toHexString(b & 0xFF));
		return total;
	}
		
	private int[] readTwoByteOffset(ByteBuffer bb) {
		byte firstByte = bb.get();
		int[] values = new int[2];
		int offset = (((firstByte & 0xFF) >> 2) | ((bb.get() & 0xFF) << 6 ));
		int litCount = firstByte & 0x03;
		values[0] = offset;
		values[1] = litCount;
		return values; 
	}

	private int readLongCompressionOffset(ByteBuffer bb) {
		int value = 0;
		byte b = bb.get();
		while (b==0x00){
			value = value + 0xFF;
			b = bb.get();
		}
		value = value + (b & 0xFF);
		return value; 
	}

	static void setMagicNumber(){
		int size = 0x6C;
		magicNumber = new byte[size];
		int randseed = 1;
		for (int i=0; i<size; i++){
			randseed *= 0x343fd;
			randseed += 0x269ec3;
			magicNumber[i] = (byte)(randseed >> 0x10);
		}
	}
	
	protected void readDwgSectionOffsets(ByteBuffer bb) {
		bb.position(12);
		bb.order(ByteOrder.LITTLE_ENDIAN);
		short codePage = bb.getShort();
		int count = bb.getInt();
		for (int i=0; i<count; i++) {
			byte rec = bb.get();
			int seek = bb.getInt();
			int size = bb.getInt();
			if (rec==0) {
				dwgFile.addDwgSectionOffset("HEADERS", seek, size);
			} else if (rec==1) {
				dwgFile.addDwgSectionOffset("CLASSES", seek, size);
			} else if (rec==2) {
				dwgFile.addDwgSectionOffset("OBJECTS", seek, size);
			} else if (rec==3) {
				dwgFile.addDwgSectionOffset("UNKNOWN", seek, size);
			} else if (rec==4) {
				dwgFile.addDwgSectionOffset("R14DATA", seek, size);
			} else if (rec==5) {
				dwgFile.addDwgSectionOffset("R14REC5", seek, size);
			} else {
				System.out.println("ERROR: Codigo de numero de registro no soportado: " + rec);
			}
		}
	}
}

class Section {
	private int number;
	private int size;
	private int adress;
	
	public int getNumber(){
		return number;
	}
	public void setNumber(int number){
		this.number = number;
	}
	public int getSize(){
		return size;
	}
	public void setSize(int size){
		this.size = size;
	}
	public int getAdress(){
		return adress;
	}
	public void setAdress(int adress){
		this.adress = adress;
	}
}

class Gap {
	private int size;
	private int parent;
	private int left;
	private int right;
	private int zero;
	
	public int getParent(){
		return parent;
	}
	public void setParent(int parent){
		this.parent = parent;
	}
	public int getLeft(){
		return left;
	}
	public void setLeft(int left){
		this.left = left;
	}
	public int getRight(){
		return right;
	}
	public void setRight(int right){
		this.right = right;
	}
	public int getZero(){
		return zero;
	}
	public void setZero(int zero){
		this.zero = zero;
	}
	public int getSize(){ 
		return size;
	}
	public void setSize(int size){
		this.size = size;
	}
}
