/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap;

import com.iver.utiles.ExceptionDescription;
import com.iver.utiles.IDescriptableException;

/**
 * Excepcin que indica un problema con los drivers. El problema concreto viene
 * dado por la excepcin hija
 */
public class DriverException extends Exception implements IDescriptableException{
	
	/*
	 * azabala.
	 * El problema que tenemos es que una DriverException puede ser debida
	 * a multiples causas, y para poder mostrar al usuario informacin de cara
	 * a que subsane estas causas (sobretodo considerando el estado UnAvalaible
	 * de un FLayer) seria necesario hacer un analisis exhaustivo de estas causas.
	 * 
	 * Por este motivo aado un ExceptionDescription a DriverException
	 * (si finalmente DriverIOException tb lo requiere, habra que meter herencias)
	 * que S que tiene esta inforamacin.
	 * 
	 */
	/**
	 * It has information about the cause of the DriverException
	 * (and an error message to the user)
	 */
	private ExceptionDescription exceptionType;
	
	
	
	/**
	 * Crea un nuevo DriverException
	 *
	 * @param message
	 */
	public DriverException(String message) {
		super(message);
	}
	
	public DriverException(String message, ExceptionDescription type){
		super(message);
		this.exceptionType = type;
	}
	
	/**
	 * New constructor to specify ExceptionDescription
	 * @param message
	 * @param cause
	 * @param exceptionType
	 */
	public DriverException(String message, 
							Throwable cause, 
						ExceptionDescription exceptionType){
		super(message, cause);
		this.exceptionType = exceptionType;
	}
	
	/**
	 * Returns the ExceptionDescription associated to this DriverException
	 * @return
	 */
	public ExceptionDescription getExceptionType(){
		if(exceptionType != null)
			return exceptionType;
		else return null;
//		else 
//			return ExceptionDescription.getType("GENERIC");
	}
	

	/**
	 * Crea un nuevo DriverException
	 *
	 * @param message
	 * @param cause
	 */
	public DriverException(String message, Throwable cause) {
		super(message, cause);
	}

	/**
	 * Crea un nuevo DriverException
	 *
	 * @param cause
	 */
	public DriverException(Throwable cause) {
		super(cause);
	}
	
	public DriverException(Throwable cause, ExceptionDescription type){
		super(cause);
		this.exceptionType = type;
	}
}
