/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.util.Date;
import java.util.HashMap;
import java.util.Set;

import javax.swing.JComponent;
import javax.swing.Timer;

import org.cresques.cts.IProjection;

import com.iver.cit.gvsig.fmap.crs.CRSFactory;
import com.iver.cit.gvsig.fmap.edition.commands.CommandListener;
import com.iver.cit.gvsig.fmap.layers.LayerCollectionEvent;
import com.iver.cit.gvsig.fmap.layers.LayerEvent;
import com.iver.cit.gvsig.fmap.tools.BehaviorException;
import com.iver.cit.gvsig.fmap.tools.CompoundBehavior;
import com.iver.cit.gvsig.fmap.tools.Behavior.Behavior;
import com.iver.utiles.exceptionHandling.ExceptionHandlingSupport;
import com.iver.utiles.exceptionHandling.ExceptionListener;
import com.iver.utiles.swing.threads.Cancellable;


/**
 * MapControl.
 *
 * @author Fernando Gonzlez Corts
 */
public class MapControl extends JComponent implements ComponentListener,
			CommandListener {
	/** Cuando la vista est actualizada. */
	public static final int ACTUALIZADO = 0;

	/** Cuando la vista est desactualizada. */
	public static final int DESACTUALIZADO = 1;
    public static final int ONLY_GRAPHICS = 2;
    // public static final int FAST_PAINT = 3;
	//private static Logger logger = Logger.getLogger(MapControl.class.getName());
	private MapContext mapContext = null;
    //private boolean drawerAlive = false;
	private HashMap namesMapTools = new HashMap();
	private Behavior currentMapTool = null;
	private int status = DESACTUALIZADO;
	private BufferedImage image = null;
	private String currentTool;
	private CancelDraw canceldraw;
	//private boolean isCancelled = true;
	private Timer timer;
	protected ViewPort vp;
	//private Drawer drawer;
    private Drawer2 drawer2;
    // private boolean firstDraw = true;
	protected MapToolListener mapToolListener = new MapToolListener();
	private MapContextListener mapContextListener = new MapContextListener();
	private ExceptionHandlingSupport exceptionHandlingSupport = new ExceptionHandlingSupport();

	private String prevTool;

	/**
     * We need this to avoid not wanted refresh. REMEMBER TO SET TO TRUE!!
     */
    // private boolean paintEnabled = false;

	/**
	 * Crea un nuevo NewMapControl.
	 */
	public MapControl() {
		this.setName("MapControl");
		setDoubleBuffered(false);
		setOpaque(true);
		status = DESACTUALIZADO;

		//Clase usada para cancelar el dibujado
		canceldraw = new CancelDraw();

		//Modelo de datos y ventana del mismo
		// TODO: Cuando creamos un mapControl, deberamos asignar
		// la projeccin por defecto con la que vayamos a trabajar.
		// 23030 es el cdigo EPSG del UTM30 elipsoide ED50
		vp = new ViewPort(CRSFactory.getCRS("EPSG:23030"));
		setMapContext(new MapContext(vp));

		//eventos
		this.addComponentListener(this);
		this.addMouseListener(mapToolListener);
		this.addMouseMotionListener(mapToolListener);
		this.addMouseWheelListener(mapToolListener);

        this.drawer2 = new Drawer2();
		//Timer para mostrar el redibujado mientras se dibuja
		timer = new Timer(360,
				new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						MapControl.this.repaint();
					}
				});
	}

	/**
	 * Inserta el modelo.
	 *
	 * @param model FMap.
	 */
	public void setMapContext(MapContext model) {
		if (mapContext != null) {
			mapContext.removeAtomicEventListener(mapContextListener);
		}

		mapContext = model;

		if (mapContext.getViewPort() == null) {
			mapContext.setViewPort(vp);
		} else {
			vp = mapContext.getViewPort();

			// vp.setImageSize(new Dimension(getWidth(), getHeight()));
			//System.err.println("Viewport en setMapContext:" + vp);
		}

		mapContext.addAtomicEventListener(mapContextListener);

		status = DESACTUALIZADO;
	}

	/**
	 * Devuelve la proyeccin.
	 *
	 * @return Proyeccin.
	 */
	public IProjection getProjection() {
		return getMapContext().getProjection();
	}

	/**
	 * Inserta una proyeccin.
	 *
	 * @param proj Proyeccin.
	 */
	public void setProjection(IProjection proj) {
		getMapContext().setProjection(proj);
	}

	/**
	 * Devuelve el modelo.
	 *
	 * @return FMap.
	 */
	public MapContext getMapContext() {
		return mapContext;
	}

	/**
	 * Registra una herramienta (tool).
	 *
	 * @param name Nombre de la herramienta.
	 * @param tool Herramienta.
	 */
	public void addMapTool(String name, Behavior tool) {
		namesMapTools.put(name, tool);
		tool.setMapControl(this);
	}

	public void addMapTool(String name, Behavior[] tools){
		CompoundBehavior tool = new CompoundBehavior(tools);
		addMapTool(name, tool);
	}

	/**
	 * Devuelve una herramienta (tool) registrada.
	 *
	 * @param name Nombre de la herramienta.
	 * @return tool Herramienta.
	 */
	public Behavior getMapTool(String name) {
		return (Behavior)namesMapTools.get(name);
	}

	/**
	 * Devuelve los nombres de herramientas (tool) registradas.
	 *
	 * @return Set (String) nombres de las herramientas.
	 */
	public Set getMapToolsKeySet() {
		return namesMapTools.keySet();
	}


	/**
	 * Returns true if this map control contains a tool named with the value
	 * passed in the toolName parameter. If you have added two tools with the
	 * same name, the last tool will overwrite the previous ones.
	 * @param toolName
	 * @return true if the map control contains a tool with the same name than
	 * 		toolName. False, otherwise.
	 */
	public boolean hasTool(String toolName) {
		return namesMapTools.containsKey(toolName);
	}
	/**
	 * DOCUMENT ME!
	 *
	 * @param toolName DOCUMENT ME!
	 */
	public void setTool(String toolName) {
		prevTool=getCurrentTool();
		Behavior mapTool = (Behavior) namesMapTools.get(toolName);
		currentMapTool = mapTool;
		currentTool = toolName;
		this.setCursor(mapTool.getCursor());
	}
	public Behavior getCurrentMapTool(){
		return currentMapTool;
	}
	/**
	 * Returns the name of the current selected tool on this MapControl
	 *
	 * @return A tool name.
	 */
	public String getCurrentTool() {
		return currentTool;
	}


	/**
	 * Cancela el dibujado. Se espera a que la cancelacin surta efecto
	 */
	public void cancelDrawing() {
		/* if (drawer != null) {
			if (!drawer.isAlive()) {
				return;
			}
		}
		*/
		canceldraw.setCanceled(true);

		/* while (!isCancelled) {
			if (!drawer.isAlive()) {
			    // Si hemos llegado aqu con un thread vivo, seguramente
			    // no estamos actualizados.

				break;
			}

		}
		canceldraw.setCancel(false);
		isCancelled = false;
        drawerAlive = false; */
	}

    private boolean adaptToImageSize()
    {
        if ((image == null) || (vp.getImageWidth() != this.getWidth()) || (vp.getImageHeight() != this.getHeight()))
        {
            image = new BufferedImage(this.getWidth(), this.getHeight(),
                    BufferedImage.TYPE_INT_ARGB);
            // ESTILO MAC
//        	image = GraphicsEnvironment.getLocalGraphicsEnvironment()
//        			.getDefaultScreenDevice().getDefaultConfiguration()
//        			.createCompatibleImage(this.getWidth(), this.getHeight());
            vp.setImageSize(new Dimension(getWidth(), getHeight()));
            getMapContext().getViewPort().refreshExtent();


            Graphics gTemp = image.createGraphics();
            Color theBackColor = vp.getBackColor();
            if (theBackColor == null)
                gTemp.setColor(Color.WHITE);
            else
                gTemp.setColor(theBackColor);

            gTemp.fillRect(0,0,getWidth(), getHeight());
            gTemp.dispose();
            status = DESACTUALIZADO;
            // g.drawImage(image,0,0,null);
            return true;
        }
        return false;
    }

	/**
	 * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
	 */
	protected void paintComponent(Graphics g) {
        adaptToImageSize();
      	if (status == ACTUALIZADO) {
			/*
			 * Si hay un behaviour y la imagen es distinta de null se delega el dibujado
			 * en dicho behaviour
			 */
            if (image != null)
            {
                if (currentMapTool != null)
                    currentMapTool.paintComponent(g);
                else
                    g.drawImage(image,0,0,null);
			}
		} else if ((status == DESACTUALIZADO)
                || (status == ONLY_GRAPHICS)) {

                g.drawImage(image,0,0,null);

                drawer2.put(new PaintingRequest());
                timer.start();
       }
	}

	/**
	 * Devuelve la imagen de la vista.
	 *
	 * @return imagen.
	 */
	public BufferedImage getImage() {
		return image;
	}

	/**
	 * Marca el mapa para que en el prximo redibujado se acceda a la
	 * cartografa para reobtener la imagen
	 * @param doClear Solo debera ser true cuando se llama desde pan
	 */
	public void drawMap(boolean doClear) {
		cancelDrawing();
		status = DESACTUALIZADO;
        getMapContext().getLayers().setDirty(true);
		if (doClear)
        {
            // image = null; // Se usa para el PAN
            if (image != null)
            {
                Graphics2D g = image.createGraphics();
                Color theBackColor = vp.getBackColor();
                if (theBackColor == null)
                    g.setColor(Color.WHITE);
                else
                    g.setColor(theBackColor);
                g.fillRect(0, 0, vp.getImageWidth(), vp.getImageHeight());
                g.dispose();
            }
        }
		repaint();
	}

	public void rePaintDirtyLayers()
	{
		cancelDrawing();
        status = DESACTUALIZADO;
        repaint();
	}

    public void drawGraphics() {
        status = ONLY_GRAPHICS;
        repaint();
    }

	/**
	 * @see java.awt.event.ComponentListener#componentHidden(java.awt.event.ComponentEvent)
	 */
	public void componentHidden(ComponentEvent e) {
	}

	/**
	 * @see java.awt.event.ComponentListener#componentMoved(java.awt.event.ComponentEvent)
	 */
	public void componentMoved(ComponentEvent e) {
	}

	/**
	 * @see java.awt.event.ComponentListener#componentResized(java.awt.event.ComponentEvent)
	 */
	public void componentResized(ComponentEvent e) {
		/* image = new BufferedImage(this.getWidth(), this.getHeight(),
				BufferedImage.TYPE_INT_ARGB);
		Graphics gTemp = image.createGraphics();
		gTemp.setColor(vp.getBackColor());
		gTemp.fillRect(0,0,getWidth(), getHeight());
        System.out.println("MapControl resized");
	    // image = null;
	    vp.setImageSize(new Dimension(getWidth(), getHeight()));
		getMapContext().getViewPort().setScale(); */
		// drawMap(true);
	}

	/**
	 * @see java.awt.event.ComponentListener#componentShown(java.awt.event.ComponentEvent)
	 */
	public void componentShown(ComponentEvent e) {
	}

	/**
	 * Aade un listener de tipo ExceptionListener.
	 *
	 * @param o ExceptionListener.
	 */
	public void addExceptionListener(ExceptionListener o) {
		exceptionHandlingSupport.addExceptionListener(o);
	}

	/**
	 * Borra la ExceptioListener que se pasa como parmetro.
	 *
	 * @param o ExceptionListener.
	 *
	 * @return True si se borra correctamente.
	 */
	public boolean removeExceptionListener(ExceptionListener o) {
		return exceptionHandlingSupport.removeExceptionListener(o);
	}

	/**
	 * Lanza una Excepcin.
	 *
	 * @param t Excepcin.
	 */
	protected void throwException(Throwable t) {
		exceptionHandlingSupport.throwException(t);
	}


    private class PaintingRequest
    {

        public PaintingRequest()
        {
        }

        public void paint()
        {
            try
            {
            	canceldraw.setCanceled(false);
                /* if (image == null)
                {
                    image = new BufferedImage(vp.getImageWidth(), vp.getImageHeight(),
                            BufferedImage.TYPE_INT_ARGB);
                    Graphics gTemp = image.createGraphics();
                    Color theBackColor = vp.getBackColor();
                    if (theBackColor == null)
                        gTemp.setColor(Color.WHITE);
                    else
                        gTemp.setColor(theBackColor);

                    gTemp.fillRect(0,0,getWidth(), getHeight());
                    gTemp.dispose();
                    // g.drawImage(image,0,0,null);
                    System.out.println("Imagen con null en DESACTUALIZADO. Width = " + this.getWidth());
                } */
                Graphics2D g = image.createGraphics();

                ViewPort viewPort = mapContext.getViewPort();

                if (status == DESACTUALIZADO)
                {
                	Graphics2D gTemp = image.createGraphics();
                    Color theBackColor = viewPort.getBackColor();
                    if (theBackColor == null)
                        gTemp.setColor(Color.WHITE);
                    else
                        gTemp.setColor(theBackColor);
                    gTemp.fillRect(0, 0, viewPort.getImageWidth(), viewPort.getImageHeight());
                    status = ACTUALIZADO;
                    // ESTILO MAC
//                    BufferedImage imgMac = new BufferedImage(vp.getImageWidth(), vp.getImageHeight(),
//                            BufferedImage.TYPE_INT_ARGB);
//
//                    mapContext.draw(imgMac, g, canceldraw, mapContext.getScaleView());
//                    g.drawImage(imgMac, 0, 0, null);
                    // FIN ESTILO MAC
                    // SIN MAC:
                    mapContext.draw(image, g, canceldraw, mapContext.getScaleView());

                }
                else if (status == ONLY_GRAPHICS)
                {
                    status = ACTUALIZADO;
                    mapContext.drawGraphics(image, g, canceldraw,mapContext.getScaleView());

                }


                // status = FAST_PAINT;
              //  drawerAlive = false;
                timer.stop();
                repaint();


            } catch (Throwable e) {
                timer.stop();
              //  isCancelled = true;
                e.printStackTrace();
                throwException(e);
            } finally {
            }

        }

    }

    /**
     * @author fjp
     *
     * Basasdo en el patrn WorkerThread
     *
     */
    public class Drawer2
    {
        // Una mini cola de 2. No acumulamos peticiones de dibujado
        // dibujamos solo lo ltimo que nos han pedido.
        private PaintingRequest paintingRequest;
        private PaintingRequest waitingRequest;

        private boolean waiting;
        private boolean shutdown;

        public void setShutdown(boolean isShutdown)
        {
            shutdown = isShutdown;
        }

        public Drawer2()
        {
            paintingRequest = null;
            waitingRequest = null;
            waiting = false;
            shutdown = false;
            new Thread(new Worker()).start();
        }

        public void put(PaintingRequest newPaintRequest)
        {
            waitingRequest = newPaintRequest;
            if (waiting)
            {
                synchronized (this) {
                    notifyAll();
                }
            }
        }

        public PaintingRequest take()
        {
            if (waitingRequest == null)
            {
                synchronized (this) {
                    waiting = true;
                    try {
                        wait();
                    }
                    catch (InterruptedException ie)
                    {
                        waiting = false;
                    }
                }
            }
            paintingRequest = waitingRequest;
            waitingRequest = null;
            return paintingRequest;
        }


        private class Worker implements Runnable
        {
            public void run()
            {
                while (!shutdown)
                {
                    PaintingRequest p = take();
                    System.out.println("Pintando");
                    if (image != null){
                    	cancelDrawing();
                        p.paint();
                    } else
                        status = DESACTUALIZADO;
                }
            }
        }
    }

	/**
	 * Clase utilizada para dibujar las capas.
	 *
	 * @author Vicente Caballero Navarro
	 */
	public class Drawer extends Thread {
		//private Graphics g;
		private BufferedImage image = null;
		private CancelDraw cancel;
		//private boolean threadCancel = false;

		/**
		 * Crea un nuevo Drawer.
		 *
		 */
		public Drawer(BufferedImage image, CancelDraw cancel)
        {
			this.image = image;
			this.cancel = cancel;
         //   drawerAlive = true;
		}

		/**
		 * @see java.lang.Runnable#run()
		 */
		public void run() {
			try {
				// synchronized (Drawer.class) {
				    Graphics2D g = image.createGraphics();

				    ViewPort viewPort = mapContext.getViewPort();
                    if (status == DESACTUALIZADO)
                    {
    				    Color theBackColor = viewPort.getBackColor();
    				    if (theBackColor == null)
    				        g.setColor(Color.WHITE);
    				    else
    				        g.setColor(theBackColor);
    					g.fillRect(0, 0, viewPort.getImageWidth(), viewPort.getImageHeight());
                        status = ACTUALIZADO;
                        mapContext.draw(image, g, cancel,mapContext.getScaleView());
                    }
                    else if (status == ONLY_GRAPHICS)
                    {
                        status = ACTUALIZADO;
                        mapContext.drawGraphics(image, g, cancel,mapContext.getScaleView());
                    }

					timer.stop();
                    // status = FAST_PAINT;
                  //  drawerAlive = false;
					repaint();



				// }
			} catch (Throwable e) {
			    timer.stop();
				//isCancelled = true;
                e.printStackTrace();
				throwException(e);
			} finally {
			}
		}
	}

	/**
	 * Clase para cancelar el dibujado.
	 *
	 * @author Fernando Gonzlez Corts
	 */
	public class CancelDraw implements Cancellable {
		private boolean cancel = false;

		/**
		 * Crea un nuevo CancelDraw.
		 */
		public CancelDraw() {
		}

		/**
		 * Insertar si se debe cancelar el dibujado.
		 *
		 * @param b true si se debe cancelar el dibujado.
		 */
		public void setCanceled(boolean b) {
			cancel = b;
		}

		/**
		 * @see com.iver.utiles.swing.threads.Cancellable#isCanceled()
		 */
		public boolean isCanceled() {
			return cancel;
		}
	}


	/**
	 * Listener del MapTool.
	 *
	 * @author Fernando Gonzlez Corts
	 */
	public class MapToolListener implements MouseListener, MouseWheelListener,
		MouseMotionListener {

		long t1;
		Point2D pReal;
		/**
		 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
		 */
		public void mouseClicked(MouseEvent e) {
			try {
				currentMapTool.mouseClicked(e);
			} catch (BehaviorException t) {
				throwException(t);
			}
		}

		/**
		 * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
		 */
		public void mouseEntered(MouseEvent e) {
			try {
				currentMapTool.mouseEntered(e);
			} catch (BehaviorException t) {
				throwException(t);
			}
		}

		/**
		 * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
		 */
		public void mouseExited(MouseEvent e) {
			try {
				currentMapTool.mouseExited(e);
			} catch (BehaviorException t) {
				throwException(t);
			}
		}

		/**
		 * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
		 */
		public void mousePressed(MouseEvent e) {
			try {
				currentMapTool.mousePressed(e);
			} catch (BehaviorException t) {
				throwException(t);
			}
		}

		/**
		 * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
		 */
		public void mouseReleased(MouseEvent e) {
			try {
				currentMapTool.mouseReleased(e);
			} catch (BehaviorException t) {
				throwException(t);
			}
		}

		/**
		 * @see java.awt.event.MouseWheelListener#mouseWheelMoved(java.awt.event.MouseWheelEvent)
		 */
		public void mouseWheelMoved(MouseWheelEvent e) {
			try {
				currentMapTool.mouseWheelMoved(e);

				// Si el tool actual no ha consumido el evento
				// entendemos que quiere el comportamiento por defecto.
				if (!e.isConsumed())
				{
					// Para usar el primer punto sobre el que queremos centrar
					// el mapa, dejamos pasar un segundo para considerar el siguiente
					// punto como vlido.
					if (t1 == 0)
					{
						t1= System.currentTimeMillis();
						pReal = vp.toMapPoint(e.getPoint());
					}
					else
					{
						long t2 = System.currentTimeMillis();
						if ((t2-t1) > 1000)
							t1=0;
					}
					cancelDrawing();
					ViewPort vp = getViewPort();


					/* Point2D pReal = new Point2D.Double(vp.getAdjustedExtent().getCenterX(),
							vp.getAdjustedExtent().getCenterY()); */
					int amount = e.getWheelRotation();
					double nuevoX;
					double nuevoY;
					double factor;

					if (amount < 0) // nos acercamos
					{
						factor = 0.9;
					} else // nos alejamos
					{
						factor = 1.2;
					}
					Rectangle2D.Double r = new Rectangle2D.Double();
					if (vp.getExtent() != null) {
						nuevoX = pReal.getX()
								- ((vp.getExtent().getWidth() * factor) / 2.0);
						nuevoY = pReal.getY()
								- ((vp.getExtent().getHeight() * factor) / 2.0);
						r.x = nuevoX;
						r.y = nuevoY;
						r.width = vp.getExtent().getWidth() * factor;
						r.height = vp.getExtent().getHeight() * factor;

						vp.setExtent(r);
					}


				}
			} catch (BehaviorException t) {
				throwException(t);
			}
		}

		/**
		 * @see java.awt.event.MouseMotionListener#mouseDragged(java.awt.event.MouseEvent)
		 */
		public void mouseDragged(MouseEvent e) {
			try {
				currentMapTool.mouseDragged(e);
			} catch (BehaviorException t) {
				throwException(t);
			}
		}

		/**
		 * @see java.awt.event.MouseMotionListener#mouseMoved(java.awt.event.MouseEvent)
		 */
		public void mouseMoved(MouseEvent e) {
			try {
				currentMapTool.mouseMoved(e);
			} catch (BehaviorException t) {
				throwException(t);
			}
		}
	}

	/**
	 * Listener sobre el MapContext.
	 *
	 * @author Fernando Gonzlez Corts
	 */
	public class MapContextListener implements AtomicEventListener {
		/**
		 * @see com.iver.cit.gvsig.fmap.AtomicEventListener#atomicEvent(com.iver.cit.gvsig.fmap.AtomicEvent)
		 */
		public void atomicEvent(AtomicEvent e) {
			boolean redraw = false;
			LayerEvent[] layerEvents = e.getLayerEvents();

			for (int i = 0; i < layerEvents.length; i++) {
				if (layerEvents[i].getProperty().equals("visible")) {
					redraw = true;
				}
			}

			if (e.getColorEvents().length > 0) {
				redraw = true;
			}

			if (e.getExtentEvents().length > 0) {
				redraw = true;
			}

			if (e.getProjectionEvents().length > 0) {
				//redraw = true;
			}


			LayerCollectionEvent[] aux = e.getLayerCollectionEvents();
			if (aux.length > 0) {
				for (int i=0; i < aux.length; i++)
				{
					if (aux[i].getAffectedLayer().getFLayerStatus().isDriverLoaded())
					{
						redraw = true;
						break;
					}
				}

			}

			if (e.getLegendEvents().length > 0) {
				redraw = true;
			}

			if (e.getSelectionEvents().length > 0) {
				redraw = true;
			}

			if (redraw) {
                System.out.println("MapContextListener redraw");
				MapControl.this.drawMap(false);
			}
		}
	}
	public ViewPort getViewPort() {
		return vp;
	}

	/**
	 * Returns a HashMap with the tools that have been registered
	 * in the Mapcontrol.
	 */
	public HashMap getNamesMapTools() {
		return namesMapTools;
	}

	public void commandRepaint() {
		drawMap(false);
	}

	public void commandRefresh() {
		// TODO Auto-generated method stub
	}

	public void setPrevTool() {
		setTool(prevTool);
	}

	public void zoomIn() {
		getMapContext().clearAllCachingImageDrawnLayers();
		Behavior mapTool = (Behavior) namesMapTools.get("zoomIn");
		ViewPort vp=getViewPort();
		Rectangle2D r=getViewPort().getAdjustedExtent();
		Point2D pCenter=vp.fromMapPoint(r.getCenterX(),r.getCenterY());
		MouseEvent e=new MouseEvent((Component)this,MouseEvent.MOUSE_RELEASED,MouseEvent.ACTION_EVENT_MASK,MouseEvent.BUTTON1,(int)pCenter.getX(),(int)pCenter.getY(),1,true,MouseEvent.BUTTON1);
		try {
			mapTool.mousePressed(e);
			mapTool.mouseReleased(e);
		} catch (BehaviorException t) {
			throwException(t);
		}
	}
	public void zoomOut() {
		getMapContext().clearAllCachingImageDrawnLayers();
		Behavior mapTool = (Behavior) namesMapTools.get("zoomOut");
		ViewPort vp=getViewPort();
		Rectangle2D r=getViewPort().getAdjustedExtent();
		Point2D pCenter=vp.fromMapPoint(r.getCenterX(),r.getCenterY());
		MouseEvent e=new MouseEvent((Component)this,MouseEvent.MOUSE_RELEASED,MouseEvent.ACTION_EVENT_MASK,MouseEvent.BUTTON1,(int)pCenter.getX(),(int)pCenter.getY(),1,true,MouseEvent.BUTTON1);
		try {
			mapTool.mousePressed(e);
			mapTool.mouseReleased(e);
		} catch (BehaviorException t) {
			throwException(t);
		}
	}
	public CancelDraw getCanceldraw() {
		return canceldraw;
	}

}
