/*
 * Created on 20-dic-2004
 */
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.layers;

import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.util.ArrayList;

import org.cresques.cts.ICoordTrans;
import org.cresques.filter.RasterFilterStack;
import org.cresques.io.GeoRasterFile;
import org.cresques.io.data.Grid;
import org.cresques.io.data.RasterBuf;
import org.cresques.px.Extent;

import com.hardcode.driverManager.Driver;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.core.ISymbol;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.drivers.GeorreferencedRasterDriver;
import com.iver.cit.gvsig.fmap.drivers.RasterDriver;
import com.iver.utiles.swing.threads.Cancellable;

/**
 * @author Luis W. Sevilla (sevilla_lui@gva.es)
 */
public class RasterAdapter {
	private boolean driverInitialized = false;
	private RasterDriver driver;
	
	/**
	 * Set data buffer to create a memory raster driver
	 * @param buf data buffer
	 */
	public void setBuffer(RasterBuf buf){
		((RasterDriver) driver).setBuffer(buf);
	}
	
	/**
	 * Set extent to create a memory raster driver
	 * @param ext Extent
	 */
	public void setExtent(Extent ext){
		((RasterDriver) driver).setExtent(ext);
	}
	
    /**
     * Establece el driver sobre el que acta el adaptador 
     */
    public void setDriver(Driver driver) {
    	this.driver = (RasterDriver) driver;
    }

	/**
     * Obtiene una referencia al objeto que implementa la interfaz vectorial con
     *  el fin de que las Strategy puedan optimizar en funcin del driver.
     */
    public Driver getDriver(){
    	return driver;
    }
    
	/**
	 * Obtiene una imagen de tamao especificado, de la zona (en
	 * coordenadas de usuario) que se le pide.
	 * @param size	Tamao (en pxeles de imgen).
	 * @param userSize Zona especificada (en coordenadas de usuario).
	 * @param rp Reproyeccin.
	 * @return La imagen.
	 */
	public Image getImage(Dimension size, Rectangle2D userSize, ICoordTrans rp) {
		return ((RasterDriver) driver).getImage(size, userSize, rp);
	}
	
	/**
	 * Obtiene el valor de transparencia
	 * @return entero que representa el valor de transparencia
	 */
	public int getTransparency() {
		return ((RasterDriver) driver).getTransparency();
	}
	
	/**
	 * Asigna el valor de transparencia
	 * @param trans	entero que representa el valor de transparencia
	 */
	public void setTransparency(int trans) {
		((RasterDriver) driver).setTransparency(trans);
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.LayerOperations#draw(java.awt.image.BufferedImage, java.awt.Graphics2D, ISymbol)
	 */
	public void draw(BufferedImage image, Graphics2D g, ViewPort vp, Cancellable cancel) throws DriverIOException {
		((RasterDriver) driver).draw(image,g, vp);
	}
	
	/**
	 * Obtiene la lista de atributos de raster
	 * @return	lista de atributos. Cada elmento de la lista es un array Object
	 * con dos elementos. El primero el nombre del atributo y el segundo el valor
	 * del mismo.
	 */
	public ArrayList getAttributes() {
		return ((RasterDriver) driver).getAttributes();
	}
	
	/**
	 * Devuelve el colorBand activo en la banda especificada.
	 * @param flag banda.
	 * @return color de banda activo
	 */
	public int getBand(int flag){
		return ((RasterDriver) driver).getBand(flag);
	}
	
	/**
	 * Devuelve la posicin del fichero para la banda especificada.
	 * @param flag banda.
	 * @return posicin del fichero
	 */
	public int getPosFile(int flag){
		return ((RasterDriver) driver).getPosFile(flag);
	}
		
	/**
	 * Activa o desactiva la transparencia
	 * @param t	true activa la transparencia y false la desactiva
	 */
	public void setTransparency(boolean t){
		((RasterDriver) driver).setTransparency(t);
	}
	
	/**
	 * Asocia un colorBand al rojo, verde o azul.
	 * @param flag cual (o cuales) de las bandas.
	 * @param nBand	que colorBand
	 */
	public void setBand(int flag, int nBand){
		((RasterDriver) driver).setBand(flag, nBand);
	}
	
	/**
	 * Obtiene el tipo de dato de la imagen
	 * @return entero que representa el tipo de dato. La clase RasterBuf 
	 * tiene accesible de forma esttica todos los tipos de datos permitidos
	 */
	public int getDataType(){
		return ((RasterDriver) driver).getRasterDataType();
	}
	
	/**
	 * Obtiene la pila de filtros 
	 * @return pila de filtros
	 */
	public RasterFilterStack getFilterStack(){
		return ((RasterDriver) driver).getFilterStack();
	}
	
	/**
	 * Asigna la pila de filtros 
	 * @param pila de filtros
	 */
	public void setFilterStack(RasterFilterStack  stack){
		((RasterDriver) driver).setFilterStack(stack);
	}
	
	/**
	 * 
	 * @return
	 */
	public GeoRasterFile[] getFiles(){
		return ((RasterDriver) driver).getFiles();
	}
	
	/**
	 * Obtiene el nmero de bandas de la imagen
	 * @return nmero de bandas
	 */
	public int getNumBands(){
		return ((RasterDriver)driver).getNumBands();
	}
	
	/**
	 * Obtiene el extent de la vista
	 * @return extent
	 */
	public Rectangle2D getFullExtent(){
		return ((RasterDriver)driver).getFullExtent();
	}
	
	/**
	 * Asigna un fichero al PxRaster
	 * @param fileName Nombre del fichero
	 */
	public void addFile(String fileName){
		((RasterDriver)driver).addFile(fileName);
	}
	
	/**
	 * Elimina un fichero al PxRaster
	 * @param fileName Nombre del fichero
	 */
	public void delFile(String fileName){
		((RasterDriver)driver).delFile(fileName);
	}
	
	/**
	 * Obtiene el valor del pixel del Image en la posicin x,y
	 * @param x Posicin x
	 * @param y Posicin y
	 * @return valor de pixel
	 */
	public int[] getPixel(double wcx, double wcy){
		return ((RasterDriver)driver).getPixel(wcx, wcy);
	}
	
	/**
	 * Obtiene el flag que dice si la imagen est o no georreferenciada
	 * @return true si est georreferenciada y false si no lo est.
	 */
	public boolean isGeoreferenced() {
		return ((GeorreferencedRasterDriver)driver).isGeoreferenced();
	}
	
	/**
	 * Obtiene el grid asociado
	 * @return grid
	 */
	public Grid getGrid(){
		return ((RasterDriver)driver).getGrid();
	}
}
