/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.layers;

import java.io.IOException;

import org.apache.log4j.Logger;
import org.xml.sax.SAXException;

import com.hardcode.driverManager.Driver;
import com.hardcode.driverManager.DriverLoadException;
import com.hardcode.gdbms.engine.data.DataSource;
import com.hardcode.gdbms.engine.data.DataSourceFactory;
import com.hardcode.gdbms.engine.data.IDataSourceListener;
import com.hardcode.gdbms.engine.data.NoSuchTableException;
import com.hardcode.gdbms.engine.data.SourceInfo;
import com.hardcode.gdbms.engine.data.driver.DriverException;
import com.hardcode.gdbms.engine.data.edition.DataWare;
import com.hardcode.gdbms.engine.data.persistence.Memento;
import com.hardcode.gdbms.engine.data.persistence.MementoContentHandler;
import com.hardcode.gdbms.engine.data.persistence.MementoException;
import com.hardcode.gdbms.engine.instruction.EvaluationException;
import com.hardcode.gdbms.engine.instruction.FieldNotFoundException;
import com.hardcode.gdbms.engine.instruction.SemanticException;
import com.hardcode.gdbms.engine.values.Value;
import com.hardcode.gdbms.engine.values.ValueCollection;
import com.hardcode.gdbms.parser.ParseException;
import com.iver.cit.gvsig.fmap.drivers.FieldDescription;
import com.iver.cit.gvsig.fmap.layers.layerOperations.Selectable;
import com.iver.utiles.NumberUtilities;
import com.iver.utiles.XMLEntity;


/**
 * DataSource seleccionable.
 *
 * @author Fernando Gonzlez Corts
 */
public class SelectableDataSource implements DataSource,Selectable {
	private static Logger logger = Logger.getLogger(SelectableDataSource.class.getName());
	private SelectionSupport selectionSupport = new SelectionSupport();
	private DataSource dataSource;

	private int[] mapping = null;

	/**
	 * Crea un nuevo SelectableDataSource.
	 *
	 * @param name
	 * @param ds
	 * @throws DriverException
	 */
	public SelectableDataSource(DataSource ds) throws DriverException {
		dataSource = ds;
		// CHEMA: No deberiamos abrir el dataSource porque si
		// Lo tiene que hacer quien lo va a usar
		//dataSource.start();

		// Creamos el mapping de campos externos que no muestran el PK.
		mapExternalFields();


	}

	/**
	 * Maps real fields or "external" fields. We don't want to see virtual fields.
	 * @throws DriverException
	 */
	public void mapExternalFields() throws DriverException {
		//CHEMA: Abrimos y cerramos el dataSource para preparar
		//		el mapping
		this.dataSource.start();
		int numExternalFields = 0;
		for (int i=0; i < dataSource.getFieldCount(); i++)
		{
			if (!dataSource.isVirtualField(i))
				numExternalFields++;

		}

		mapping = new int[numExternalFields];
		int j=0;
		for (int i=0; i < dataSource.getFieldCount(); i++)
		{
			if (!dataSource.isVirtualField(i))
				mapping[j++] = i;

		}
		this.dataSource.stop();
	}

	public static SelectableDataSource createSelectableDataSource(XMLEntity xml) throws NoSuchTableException, ParseException, DriverLoadException, DriverException, SemanticException, IOException, XMLException{

		SelectionSupport ss = new SelectionSupport();
		ss.setXMLEntity(xml.getChild(0));
		XMLEntity xmlDS = xml.getChild(1);
		GDBMSParser parser = new GDBMSParser(xmlDS);
		MementoContentHandler gdbmsHandler = new MementoContentHandler();
		parser.setContentHandler(gdbmsHandler);
		try {
			parser.parse();
		} catch (SAXException e) {
			throw new XMLException(e);
		}
		SelectableDataSource sds;
        try {
//        	 CHEMA: AUTOMATIC DATA SOURCE
            //sds = new SelectableDataSource(gdbmsHandler.getDataSource(LayerFactory.getDataSourceFactory(), DataSourceFactory.MANUAL_OPENING));
            sds = new SelectableDataSource(gdbmsHandler.getDataSource(LayerFactory.getDataSourceFactory(), DataSourceFactory.AUTOMATIC_OPENING));
        } catch (EvaluationException e1) {
            throw new XMLException(e1);
        }
        sds.selectionSupport=ss;
		return sds;
	}

	public void setDataSourceFactory(DataSourceFactory dsf) {
		dataSource.setDataSourceFactory(dsf);
	}
	public void setSourceInfo(SourceInfo sourceInfo) {
		dataSource.setSourceInfo(sourceInfo);
	}
	/**
	 * Aade el soporte para la seleccin.
	 *
	 * @param selectionSupport
	 */
	public void setSelectionSupport(SelectionSupport selectionSupport) {
		this.selectionSupport = selectionSupport;
	}

	/**
	 * Devuelve el nmero de campos.
	 *
	 * @return Nmero de campos.
	 *
	 * @throws DriverException
	 */
	public int getFieldCount() throws DriverException {
		// return dataSource.getFieldCount()-numVirtual;
//		if (mapping.length != dataSource.getFieldCount())
//		{
//			mapExternalFields();
//			RuntimeException e = new RuntimeException("Recalculamos los campos de recordset!!");
//			e.printStackTrace();
//		}
		return mapping.length;
	}

	/**
	 * Return index field searching by its name
	 *
	 * @param arg0 field name.
	 *
	 * @return field index. -1 if not found
	 *
	 * @throws DriverException
	 * @throws FieldNotFoundException
	 */
	public int getFieldIndexByName(String arg0)
		throws DriverException {
		int internal = dataSource.getFieldIndexByName(arg0);
		for (int i=0; i < mapping.length; i++)
		{
			if (mapping[i] == internal)
				return i;
		}
		//OJO Parche para rodear poblema de gdbms + FileDriver
		// Cuando en le fichero existe un campo de nombre 'PK'
		if (arg0.equalsIgnoreCase("pk")){
			for (int i=0; i < mapping.length; i++)
			{
				if (dataSource.getFieldName(mapping[i]).equalsIgnoreCase(arg0)){
					return i;
				}
			}

		}
		return -1;
	}

	/**
	 * Devuelve el nombre del campo a partir del ndice.
	 *
	 * @param arg0 ndice.
	 *
	 * @return nombre del campo.
	 *
	 * @throws DriverException
	 */
	public String getFieldName(int arg0) throws DriverException {
	    // return dataSource.getFieldName(arg0);
		return dataSource.getFieldName(mapping[arg0]);
	}

	/**
	 * Devuelve el valor a partir del nmro de fila y columna.
	 *
	 * @param arg0 nmero de registro.
	 * @param arg1 nmero de campo.
	 *
	 * @return Valor.
	 *
	 * @throws DriverException
	 */
	public Value getFieldValue(long arg0, int arg1) throws DriverException {
		return dataSource.getFieldValue(arg0, mapping[arg1]);
		// return dataSource.getFieldValue(arg0, arg1);
	}

	/**
	 * Devuelve el nombre del DataSource.
	 *
	 * @return Nombre.
	 */
	public String getName() {
		return dataSource.getName();
	}

	/**
	 * Devuelve el nmero de filas en total.
	 *
	 * @return nmero de filas.
	 *
	 * @throws DriverException
	 */
	public long getRowCount() throws DriverException {
		return dataSource.getRowCount();
	}

	/**
	 * Inicializa el dataSource.
	 *
	 * @throws DriverException
	 */
	public void start() throws DriverException {
		// logger.debug("dataSource.start()");
		dataSource.start();
	}

	/**
	 * Finaliza el DataSource.
	 *
	 * @throws DriverException
	 */
	public void stop() throws DriverException {
		// logger.debug("dataSource.stop()");
		dataSource.stop();
	}

	/**
	 * A partir del XMLEntity se rellenan los atributos del DataSource.
	 *
	 * @param child
	 */
	public void setXMLEntity03(XMLEntity child) {
		selectionSupport.setXMLEntity(child.getChild(0));
	}

	/**
	 * Cuando ocurre un evento de cambio en la seleccin, ste puede ser uno de
	 * una gran cantidad de eventos. Con el fin de no propagar todos estos
	 * eventos, se realiza la propagacin de manera manual al final de la
	 * "rfaga" de eventos
	 */
	public void fireSelectionEvents() {
		selectionSupport.fireSelectionEvents();
	}

	/**
	 * Aade un nuevo Listener al SelectionSupport.
	 *
	 * @param listener SelectionListener.
	 */
	public void addSelectionListener(SelectionListener listener) {
		selectionSupport.addSelectionListener(listener);
	}

	/**
	 * Borra un Listener al SelectionSupport.
	 *
	 * @param listener Listener a borrar.
	 */
	public void removeSelectionListener(SelectionListener listener) {
		selectionSupport.removeSelectionListener(listener);
	}

	/**
	 * Borra la seleccin.
	 */
	public void clearSelection() {
		selectionSupport.clearSelection();
	}

	/**
	 * Develve un FBitSet con los ndices de los elementos seleccionados.
	 *
	 * @return FBitset con los elementos seleccionados.
	 */
	public FBitSet getSelection() {
		return selectionSupport.getSelection();
	}

	/**
	 * Devuelve el SelectionSupport.
	 *
	 * @return SelectinSuport.
	 */
	public SelectionSupport getSelectionSupport() {
		return selectionSupport;
	}

	/**
	 * Devuelve true si el elemento est seleccionado.
	 *
	 * @param recordIndex ndice del registro.
	 *
	 * @return True si el registro est seleccionado.
	 */
	public boolean isSelected(int recordIndex) {
		return selectionSupport.isSelected(recordIndex);
	}

	/**
	 * Inserta una nueva seleccin.
	 *
	 * @param selection FBitSet.
	 */
	public void setSelection(FBitSet selection) {
		selectionSupport.setSelection(selection);
	}

	private void putMemento(XMLEntity xml) throws XMLException {
		try {
			GDBMSHandler handler = new GDBMSHandler();
			Memento m = getMemento();
			m.setContentHandler(handler);
			m.getXML();
			XMLEntity child = handler.getXMLEntity();

			xml.addChild(child);
		} catch (MementoException e) {
			throw new XMLException(e);
		} catch (SAXException e) {
			throw new XMLException(e);
		}
	}

	/**
	 * Devuelve el XMLEntity con la informacin necesaria para reproducir el
	 * DataSource.
	 *
	 * @return XMLEntity.
	 * @throws XMLException
	 */
	public XMLEntity getXMLEntity() throws XMLException {
		XMLEntity xml = new XMLEntity();
		xml.putProperty("className",this.getClass().getName());
		xml.addChild(selectionSupport.getXMLEntity());
		putMemento(xml);

		return xml;
	}

	/**
	 * @see com.hardcode.gdbms.engine.data.DataSource#getWhereFilter()
	 */
	public long[] getWhereFilter() throws IOException {
		return dataSource.getWhereFilter();
	}

	/**
	 * @see com.hardcode.gdbms.engine.data.ReadDriver#getFieldType(int)
	 */
	public int getFieldType(int i) throws DriverException {
		// return dataSource.getFieldType(i);
		return dataSource.getFieldType(mapping[i]);
	}

	/**
	 * @see com.hardcode.gdbms.engine.data.DataSource#getDataSourceFactory()
	 */
	public DataSourceFactory getDataSourceFactory() {
		return dataSource.getDataSourceFactory();
	}

	/**
	 * @see com.hardcode.gdbms.engine.data.DataSource#getAsString()
	 */
	public String getAsString() throws DriverException {
		return dataSource.getAsString();
	}

	/**
	 * @throws DriverException
	 * @see com.hardcode.gdbms.engine.data.DataSource#remove()
	 */
	public void remove() throws DriverException {
		dataSource.remove();
	}

	/**
	 * @see com.hardcode.gdbms.engine.data.DataSource#getMemento()
	 */
	public Memento getMemento() throws MementoException {
		return dataSource.getMemento();
	}

	/**
	 * @see com.hardcode.gdbms.engine.data.DataSource#getSourceInfo()
	 */
	public SourceInfo getSourceInfo() {
		return dataSource.getSourceInfo();
	}

    /**
     * @see com.hardcode.gdbms.engine.data.DataSource#getPrimaryKeys()
     */
    public int[] getPrimaryKeys() throws DriverException {
    	return dataSource.getPrimaryKeys();
    }

    /**
     * @see com.hardcode.gdbms.engine.data.DataSource#getPKValue(long)
     */
    public ValueCollection getPKValue(long rowIndex) throws DriverException {
        return dataSource.getPKValue(rowIndex);
    }

    /**
     * @see com.hardcode.gdbms.engine.data.DataSource#getPKName(int)
     */
    public String getPKName(int fieldId) throws DriverException {
        return dataSource.getPKName(fieldId);
    }

    /**
     * @see com.hardcode.gdbms.engine.data.DataSource#getPKType(int)
     */
    public int getPKType(int i) throws DriverException {
        return dataSource.getPKType(i);
    }

    /**
     * @throws DriverException
     * @see com.hardcode.gdbms.engine.data.DataSource#getPKCardinality()
     */
    public int getPKCardinality() throws DriverException {
        return dataSource.getPKCardinality();
    }

    /**
     * @see com.hardcode.gdbms.engine.data.DataSource#getRow(long)
     */
    public Value[] getRow(long rowIndex) throws DriverException {
    	Value[] withoutVirtuals = new Value[mapping.length];
    	Value[] internal = dataSource.getRow(rowIndex);
    	for (int i=0; i < mapping.length; i++)
    	{
    		withoutVirtuals[i] = internal[mapping[i]];

    	}
        return withoutVirtuals;
    }

    /**
     * @see com.hardcode.gdbms.engine.data.DataSource#getFieldNames()
     */
    public String[] getFieldNames() throws DriverException {
    	String[] fieldNames = new String[getFieldCount()];
		int j=0;
		for (int i=0; i < dataSource.getFieldCount(); i++)
		{
			if (!dataSource.isVirtualField(i))
				fieldNames[j++] = dataSource.getFieldName(i);

		}
        // return dataSource.getFieldNames();
    	return fieldNames;
    }

    /**
     * @see com.hardcode.gdbms.engine.data.DataSource#getPKNames()
     */
    public String[] getPKNames() throws DriverException {
        return dataSource.getPKNames();
    }

	public void removeLinksSelectionListener() {
		selectionSupport.removeLinkSelectionListener();
	}

    /**
     * @throws DriverException
     * @see com.hardcode.gdbms.engine.data.DataSource#getDataWare(int)
     */
    public DataWare getDataWare(int arg0) throws DriverException {
        return dataSource.getDataWare(arg0);
    }

	public int getFieldWidth(int i) throws DriverException {
		return dataSource.getFieldWidth(mapping[i]);
		// return dataSource.getFieldWidth(i);
	}

	public boolean isVirtualField(int fieldId) throws DriverException {
		return dataSource.isVirtualField(fieldId);
	}

	/**
	 * Useful to writers, to know the field definitions.
	 * NOTE: Maximun precision: 6 decimals. (We may need to change this)
	 * @return Description of non virtual fields
	 * @throws DriverException
	 */
	public FieldDescription[] getFieldsDescription() throws DriverException
	{
		int numFields = getFieldCount();
		FieldDescription[] fieldsDescrip = new FieldDescription[numFields];
		for (int i = 0; i < numFields; i++) {
			fieldsDescrip[i] = new FieldDescription();
			int type = getFieldType(i);
			fieldsDescrip[i].setFieldType(type);
			fieldsDescrip[i].setFieldName(getFieldName(i));
			fieldsDescrip[i].setFieldLength(getFieldWidth(i));
			if (NumberUtilities.isNumeric(type))
			{
				if (!NumberUtilities.isNumericInteger(type))
					// TODO: If there is a lost in precision, this should be changed.
					fieldsDescrip[i].setFieldDecimalCount(6);
			}
			else
				fieldsDescrip[i].setFieldDecimalCount(0);
			// TODO: DEFAULTVALUE?
			// fieldsDescrip[i].setDefaultValue(get)
		}
		return fieldsDescrip;
	}

	public Driver getDriver() {
		return this.dataSource.getDriver();
	}

	public void reload() throws DriverException, IOException {
		dataSource.reload();
		mapExternalFields();

	}

	public void addDataSourceListener(IDataSourceListener listener) {
		dataSource.addDataSourceListener(listener);

	}

	public void removeDataSourceListener(IDataSourceListener listener) {
		dataSource.removeDataSourceListener(listener);

	}
}
