/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.rendering;

/**
 * <p>The interface <code>LegendListener</code> defines methods for an object that listens events from legends.</p>
 */
public interface LegendListener {
	/**
	 * <p>Called when changes the value in the field of labeling of the legend.</p>
	 * <p>If any of the listeners returns <code>false</code>, no change takes effect.</p>
	 *
	 * @param e the label legend event
	 *
	 * @return <code>true</code> if has been no problem; <code>false</code> otherwise
	 */
	boolean labelFieldChange(LabelLegendEvent e);

	/**
	 * <p>Called when changes the default symbol of the legend.</p>
	 * <p>If any of the listeners returns <code>false</code>, no change takes effect.</p>
	 * 
	 * @param e the symbol legend event
	 *
	 * @return <code>true</code> if has been no problem; <code>false</code> otherwise
	 */
	boolean defaultSymbolChange(SymbolLegendEvent e);

	/**
	 * <p>Called when changes one symbol of the legend's symbols.</p>
	 * <p>The legend must be of "unique values" or "intervals" type.</p>
	 * <p>If any of the listeners returns <code>false</code>, no change takes effect.</p>
	 *
	 * @param e the symbol legend event
	 *
	 * @return <code>true</code> if has been no problem; <code>false</code> otherwise
	 */
	boolean classifiedSymbolChange(SymbolLegendEvent e);

	/**
	 * <p>Called when changes any interval value of a "legend by intervals".</p>
	 * <p>If any of the listeners returns <code>false</code>, no change takes effect.</p>
	 *
	 * @param e the interval legend event
	 *
	 * @return <code>true</code> if has been no problem; <code>false</code> otherwise
	 */
	boolean intervalChange(IntervalLegendEvent e);

	/**
	 * <p>Called when changes any value of a "legend of unique values".</p>
	 * <p>If any of the listeners returns <code>false</code>, no change takes effect.</p>
	 *
	 * @param e the value legend event
	 *
	 * @return <code>true</code> if has been no problem; <code>false</code> otherwise
	 */
	boolean valueChange(ValueLegendEvent e);
}
