/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.rendering;

import java.util.ArrayList;

import com.hardcode.gdbms.engine.data.DataSource;
import com.iver.cit.gvsig.fmap.core.FShape;
import com.iver.cit.gvsig.fmap.core.IFeature;
import com.iver.cit.gvsig.fmap.core.ISLDCompatible;
import com.iver.cit.gvsig.fmap.core.ISymbol;
import com.iver.cit.gvsig.fmap.core.SLDTags;
import com.iver.cit.gvsig.fmap.core.v02.FConstant;
import com.iver.cit.gvsig.fmap.core.v02.FSymbol;
import com.iver.cit.gvsig.fmap.layers.XMLException;
import com.iver.utiles.XMLEntity;


/**
 * Leyenda se smbolo nico.
 *
 * @author Vicente Caballero Navarro
 */
public class SingleSymbolLegend implements VectorialLegend, Legend{
	private ISymbol defaultSymbol;
	private String labelFieldName = null;
	private String labelFieldHeight = null;
	private String labelFieldRotation = null;
	private int shapeType = FShape.POLYGON; // Por defecto, tipo polgono
	// private boolean bWithHeightText;
	/**
	 * Crea un nuevo SingleSymbolLegend.
	 *
	 * @param style Smbolo.
	 */
	public SingleSymbolLegend(FSymbol style) {
		defaultSymbol = style;
	}

	/**
	 * Crea un nuevo SingleSymbolLegend.
	 */
	public SingleSymbolLegend() {
		defaultSymbol = LegendFactory.DEFAULT_POLYGON_SYMBOL;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.rendering.VectorialLegend#setDefaultSymbol(ISymbol)
	 */
	public void setDefaultSymbol(ISymbol s) {
		setShapeType(s.getSymbolType());
		defaultSymbol = s;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.rendering.VectorialLegend#getLabelField()
	 */
	public String getLabelField() {
		return labelFieldName;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.rendering.VectorialLegend#setLabelField(int)
	 */
	public void setLabelField(String fieldName) {
		labelFieldName = fieldName;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.rendering.VectorialLegend#getSymbol(int)
	 */
	public ISymbol getSymbol(int recordIndex) {
		return defaultSymbol;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.rendering.Legend#getDefaultSymbol()
	 */
	public ISymbol getDefaultSymbol() {
		return defaultSymbol;
	}

	/**
	 * creates the SLD String that defines this legend type.
	 */
	public String getSLDString(String layerName)
	{		
//		StyledLayerDescriptor sld = new StyledLayerDescriptor();
//		StyleFactory styleFactory = StyleFactory.createStyleFactory();
//		Style style = styleFactory.createStyle();
//		style.setName("default");		
//	    try{	    	
//			FeatureTypeStyle featStyle = styleFactory.createFeatureTypeStyle();			
//			Symbolizer[] theSymbolizers = new Symbolizer[1];			
//			theSymbolizers[0] = SDLUtils.toGeotoolsSymbol(this.defaultSymbol);						
//			Rule myRule = styleFactory.createRule();
//			myRule.setSymbolizers(theSymbolizers);
//			featStyle.addRule(myRule);
//			style.addFeatureTypeStyle(featStyle);					
//			NamedLayer namedLayer = new NamedLayer();
//			namedLayer.setName("comunidades");
//			namedLayer.addStyle(style);			
//			sld.addStyledLayer(namedLayer);			
//			SLDTransformer st = new SLDTransformer();
//			String result =  st.transform(style);
//			return result;	        		
//	    }catch(Exception e) {
//	    	e.printStackTrace(); 
//	    	return null;
//	    }
		try{
			XmlBuilder xmlBuilder = new XmlBuilder();
			xmlBuilder.writeHeader();
			xmlBuilder.openTag(SLDTags.SLD_ROOT, SLDTags.VERSION_ATTR, SLDTags.VERSION_1_0_0);
			xmlBuilder.openTag(SLDTags.NAMEDLAYER);
			xmlBuilder.writeTag(SLDTags.NAME,layerName);
			xmlBuilder.openTag(SLDTags.USERSTYLE);
			xmlBuilder.openTag(SLDTags.FEATURETYPESTYLE);
			xmlBuilder.writeTag(SLDTags.FEATURETYPENAME,"FeatureTypeName");
			xmlBuilder.openTag(SLDTags.RULE);
			if (this.defaultSymbol instanceof ISLDCompatible)
			{
				ISLDCompatible symSLD = (ISLDCompatible) this.defaultSymbol;
				xmlBuilder.writeRaw(symSLD.toSLD());
			}
			else
				throw new RuntimeException("Cannot convert default Symbol " + this.defaultSymbol.getDescription() + " to SLD");

			xmlBuilder.closeTag();
			xmlBuilder.closeTag();
			xmlBuilder.closeTag();
			xmlBuilder.closeTag();
			xmlBuilder.closeTag();
			return xmlBuilder.getXML(); 
		}
		catch(Exception e)
		{
			e.printStackTrace();
			return null;
		}		
	}	
	
	/**
	 * @see com.iver.cit.gvsig.fmap.rendering.Legend#getXMLEntity()
	 */
	public XMLEntity getXMLEntity() {
		XMLEntity xml = new XMLEntity();
		xml.putProperty("className",this.getClass().getName());
		xml.putProperty("labelFieldName", labelFieldName);
        xml.putProperty("labelHeightFieldName", labelFieldHeight);
        xml.putProperty("labelRotationFieldName", labelFieldRotation);
		xml.addChild(defaultSymbol.getXMLEntity());

		return xml;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.rendering.VectorialLegend#setXMLEntity()
	 */
	public void setXMLEntity03(XMLEntity xml) {
		setLabelField(xml.getStringProperty("labelFieldName"));

		FSymbol auxSym = FSymbol.createFromXML03(xml.getChild(0));
		setDefaultSymbol(auxSym);
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.rendering.VectorialLegend#setXMLEntity()
	 */
	public void setXMLEntity(XMLEntity xml) {
		setLabelField(xml.getStringProperty("labelFieldName"));
        if (xml.contains("labelHeightFieldName"))
            setLabelHeightField(xml.getStringProperty("labelHeightFieldName"));
        if (xml.contains("labelRotationFieldName"))
            setLabelRotationField(xml.getStringProperty("labelRotationFieldName"));

		FSymbol auxSym = FSymbol.createFromXML(xml.getChild(0));
		setDefaultSymbol(auxSym);
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.rendering.Legend#cloneLegend()
	 */
	public Legend cloneLegend() throws XMLException {
		return (Legend) LegendFactory.createFromXML(getXMLEntity());
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.rendering.VectorialLegend#setDataSource(com.hardcode.gdbms.engine.data.DataSource)
	 */
	public void setDataSource(DataSource ds) {
		// No hacemos nada, no lo vamos a usar
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.rendering.VectorialLegend#getShapeType()
	 */
	public int getShapeType() {
		return shapeType;
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.rendering.VectorialLegend#setShapeType(int)
	 */
	public void setShapeType(int shapeType) {
		if (this.shapeType != shapeType) {
			switch (shapeType) {
				case FShape.POINT:
					defaultSymbol = new FSymbol(FConstant.SYMBOL_TYPE_POINT);

					break;

				case FShape.LINE:
					defaultSymbol = new FSymbol(FConstant.SYMBOL_TYPE_LINE);

					break;

				case FShape.POLYGON:
					defaultSymbol = new FSymbol(FConstant.SYMBOL_TYPE_FILL);

					break;
			}

			this.shapeType = shapeType;
		}
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.rendering.VectorialLegend#getLabelHeightField()
	 */
	public String getLabelHeightField() {
		return labelFieldHeight;
	}

	/**
	 * Inserta el texto que refleja el alto del campo.
	 *
	 * @param str String.
	 */
	public void setLabelHeightField(String str) {
		labelFieldHeight = str;
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.rendering.VectorialLegend#getLabelRotationField()
	 */
	public String getLabelRotationField() {
		return labelFieldRotation;
	}

	/**
	 * Inserta el texto que refleja la rotacin del campo.
	 *
	 * @param str String.
	 */
	public void setLabelRotationField(String str) {
		labelFieldRotation = str;
	}

    public ISymbol getSymbolByFeature(IFeature feat) {
        return defaultSymbol;
    }

	public void useDefaultSymbol(boolean b) {
		// TODO Auto-generated method stub
	}

    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.rendering.VectorialLegend#getUsedFields()
     */
    public String[] getUsedFields() {
        ArrayList usedFields = new ArrayList();
        if (getLabelField() != null)
            usedFields.add(getLabelField());
        if (getLabelHeightField() != null)
            usedFields.add(getLabelHeightField());
        if (getLabelRotationField() != null)
            usedFields.add(getLabelRotationField());
                
        return (String[]) usedFields.toArray(new String[0]);
    }

	/* public boolean isBWithHeightText() {
		return bWithHeightText;
	}

	public void setBWithHeightText(boolean withHeightText) {
		bWithHeightText = withHeightText;
	} */
}
