/*
 * Created on 15-may-2006
 *
 * gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
*
* $Id: RTreeJsi.java 5676 2006-06-05 16:59:08Z azabala $
* $Log$
* Revision 1.2  2006-06-05 16:59:08  azabala
* implementada busqueda de vecino mas proximo a partir de rectangulos
*
* Revision 1.1  2006/05/24 21:58:04  azabala
* *** empty log message ***
*
*
*/
package com.iver.cit.gvsig.fmap.spatialindex;

import java.awt.Point;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import org.geotools.index.quadtree.QuadTree;
import org.geotools.index.quadtree.StoreException;

import com.infomatiq.jsi.*;
import com.infomatiq.jsi.rtree.*;
import com.vividsolutions.jts.geom.Envelope;

/**
 * RTree spatial index implementation based in library
 * JSI (java spatial index).
 * 
 * http://jsi.sourceforge.net/
 * 
 * This RTree has better performance that Spatial Index Library
 * RTree, and that JTS'RTree, because
 * it uses the GNU's Trove Collections API.
 * 
 * We are doing some probes with it, because it offers
 * a Nearest Neighbour algorithm implementation 
 * (useful for Spatial Join geoprocess, for example).
 * 
 * It isnt persistent, and We've found some problems
 * with delete operations.
 * 
 * 
 * 
 * 
 * @author azabala
 *
 */
public class RTreeJsi implements ISpatialIndex, INearestNeighbourFinder {
	private RTree rtree;
	
	public RTreeJsi(){
		rtree = new RTree();
	}
	
	public void create(){
		Properties props = new Properties();
//		props.setProperty("MaxNodeEntries", "500");
//		props.setProperty("MinNodeEntries", "200");
		rtree.init(props);
	}
	
	class ListIntProcedure implements IntProcedure{
		ArrayList solution = new ArrayList();
		
		public boolean execute(int arg0) {
			solution.add(new Integer(arg0));
			return true;
		}
		
		public List getSolution(){
			return solution;
		}
	}
	
	
	public List query(Rectangle2D rect) {
		ListIntProcedure solution = new ListIntProcedure();
		rtree.intersects(toJsiRect(rect), solution);
		return solution.getSolution();
	}
	
	private Rectangle toJsiRect(Rectangle2D rect){
		Rectangle jsiRect = new Rectangle((float)rect.getMinX(),
				(float)rect.getMinY(),
				(float)rect.getMaxX(),
				(float)rect.getMaxY());
		return jsiRect;
	}

	public void insert(Rectangle2D rect, int index) {
		rtree.add(toJsiRect(rect), index);
	}

	public void delete(Rectangle2D rect, int index) {
		rtree.delete(toJsiRect(rect), index);
	}

	public List findNNearest(int numberOfNearest, Point point) {
		ListIntProcedure solution = new ListIntProcedure();
		com.infomatiq.jsi.Point jsiPoint =
			new com.infomatiq.jsi.Point(point.x, point.y);
		//FIXME REVISAR
		rtree.nearest(jsiPoint, solution, Float.POSITIVE_INFINITY);
		return solution.getSolution();
	}

	public List findNNearest(int numberOfNearest, Rectangle2D rect) {
		ListIntProcedure solution = new ListIntProcedure();
		com.infomatiq.jsi.Rectangle jsiRect =
			toJsiRect(rect);
		
		rtree.nearest(jsiRect, solution, Float.POSITIVE_INFINITY);
		return solution.getSolution();
	}
	
	public static void main(String[] args){
		RTreeJsi q = new RTreeJsi();
		q.create();
		
			for(int i = 1; i <= 99; i++){
				q.insert(new Rectangle2D.Double(100+i, 200+i, 200+i, 500+i),i);
			}
		
			for(int i = 1; i <= 99; i++){
				q.delete(new Rectangle2D.Double(100+i, 200+i, 200+i, 500+i),i);
				System.out.println("Rect="+i);
			}
	}

}

