/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.tools.Behavior;

import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.tools.BehaviorException;
import com.iver.cit.gvsig.fmap.tools.Events.MeasureEvent;
import com.iver.cit.gvsig.fmap.tools.Listeners.CircleListener;
import com.iver.cit.gvsig.fmap.tools.Listeners.ToolListener;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D; 
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.geom.Arc2D; 

import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;


/**
 * Behaviour que espera un listener de tipo CircleListener.
 *
 * @author Laura
 */
public class CircleBehavior extends Behavior {
	private Point2D m_FirstPoint;
	private Point2D m_LastPoint;
	private Point2D m_PointAnt;
	private CircleListener listener;
	protected boolean isClicked = false;

	/**
	 * Crea un nuevo CircleBehavior.
	 *
	 * @param zili listener.
	 */
	public CircleBehavior(CircleListener zili) {
		listener = zili;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.tools.Behavior.Behavior#paintComponent(java.awt.Graphics)
	 */
	public void paintComponent(Graphics g) {
		
		double radio;
		BufferedImage img = getMapControl().getImage();
		g.drawImage(img, 0, 0, null);
		g.setColor(Color.black);
		g.setXORMode(Color.white);
		if ((m_FirstPoint != null) && (m_LastPoint != null)) {			
			radio = m_LastPoint.distance(m_FirstPoint);
			Arc2D.Double arc = new Arc2D.Double(m_FirstPoint.getX()-radio,
												m_FirstPoint.getY()-radio,
												2*radio,
												2*radio, 0, 360, Arc2D.OPEN);
			((Graphics2D) g).draw(arc);			
		}
	}

	/**
	 * Reimplementacin del mtodo mousePressed de Behavior.
	 *
	 * @param e MouseEvent
	 */
	public void mousePressed(MouseEvent e) {
		
		Point pScreen = e.getPoint();
		m_PointAnt = pScreen;
		m_FirstPoint = m_PointAnt;

		if (e.getButton() == MouseEvent.BUTTON1) {
			m_PointAnt = pScreen;
			m_FirstPoint = m_PointAnt;
			isClicked = true;
		}

		if (listener.cancelDrawing()) {
			getMapControl().cancelDrawing();
			isClicked = false;
		}
		getMapControl().repaint();
	}

	/**
	 * Reimplementacin del mtodo mouseReleased de Behavior.
	 *
	 * @param e MouseEvent
	 *
	 * @throws BehaviorException Excepcin lanzada cuando el Behavior.
	 */
	public void mouseReleased(MouseEvent e) throws BehaviorException {
		m_FirstPoint = null;
		m_LastPoint = null;
		isClicked = false;
	}

	
	/**
	 * Reimplementacin del mtodo mouseDragged de Behavior.
	 *
	 * @param e MouseEvent
	 *
	 * @throws BehaviorException Excepcin lanzada cuando el Behavior.
	 */
	public void mouseDragged(MouseEvent e) throws BehaviorException {
		mouseMoved(e);
	}	
	/**
	 * Reimplementacin del mtodo mouseDragged de Behavior.
	 *
	 * @param e MouseEvent
	 */
	public void mouseMoved(MouseEvent e)  throws BehaviorException {
		m_LastPoint = e.getPoint();
		
	    if (m_FirstPoint == null) return;
		Point2D p1;
		Point2D p2;
		Point pScreen = e.getPoint(); 

		ViewPort vp = getMapControl().getMapContext().getViewPort();

		p1 = vp.toMapPoint(m_FirstPoint);
		p2 = vp.toMapPoint(pScreen);

		//	Fijamos el nuevo extent
		Rectangle2D.Double r = new Rectangle2D.Double();
		r.setFrameFromDiagonal(p1, p2);

		Rectangle2D rectPixel = new Rectangle();
		rectPixel.setFrameFromDiagonal(m_FirstPoint, pScreen);

		Double[] x = new Double[2];
		Double[] y = new Double[2];
		x[0] = new Double(p1.getX());
		x[1] = new Double(p2.getX());
		y[0] = new Double(p1.getY());
		y[1] = new Double(p2.getY());			
		MeasureEvent event = new MeasureEvent(x, y, e);
		listener.circle(event);						
		getMapControl().repaint();
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.tools.Behavior.Behavior#setListener(com.iver.cit.gvsig.fmap.tools.ToolListener)
	 */
	public void setListener(ToolListener listener) {
		this.listener = (CircleListener)listener;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.tools.Behavior.Behavior#getListener()
	 */
	public ToolListener getListener() {
		return listener;
	}
}
