/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.tools.Behavior;

import com.iver.cit.gvsig.fmap.tools.BehaviorException;
import com.iver.cit.gvsig.fmap.tools.Events.MoveEvent;
import com.iver.cit.gvsig.fmap.tools.Listeners.PanListener;
import com.iver.cit.gvsig.fmap.tools.Listeners.ToolListener;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;


/**
 * Behaviour que espera un listener de tipo PanListener.
 *
 * @author Vicente Caballero Navarro
 */
public class DraggerBehavior extends Behavior {
	private Point2D m_FirstPoint;
	private Point2D m_LastPoint;
	private Point2D m_PointAnt;
	private PanListener listener;

	/**
	 * Crea un nuevo MoveBehavior.
	 *
	 * @param pli listener.
	 */
	public DraggerBehavior(PanListener pli) {
		listener = pli;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.tools.Behavior.Behavior#paintComponent(java.awt.Graphics)
	 */
	public void paintComponent(Graphics g) {
	}


	/**
	 * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
	 */
	public void mousePressed(MouseEvent e) {
		Point pScreen = e.getPoint();

		m_PointAnt = pScreen;
		m_FirstPoint = m_PointAnt;

		if (e.getButton() == MouseEvent.BUTTON1) {
			m_PointAnt = pScreen;
			m_FirstPoint = m_PointAnt;
		}

		if (listener.cancelDrawing()) {
			getMapControl().cancelDrawing();
		}
	}

	/**
	 * Reimplementacin del mtodo mouseReleased de Behavior.
	 *
	 * @param e MouseEvent
	 *
	 * @throws BehaviorException Excepcin lanzada cuando el Behavior.
	 */
	public void mouseReleased(MouseEvent e) throws BehaviorException {
		m_FirstPoint = null;
	}

	/**
	 * Reimplementacin del mtodo mouseDragged de Behavior.
	 *
	 * @param e MouseEvent
	 * @throws BehaviorException
	 */
	public void mouseDragged(MouseEvent e) throws BehaviorException {
		m_LastPoint = e.getPoint();
		MoveEvent event = new MoveEvent(m_FirstPoint, e.getPoint(), e);
		listener.move(event);
		getMapControl().repaint();
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.tools.Behavior.Behavior#setListener(com.iver.cit.gvsig.fmap.tools.ToolListener)
	 */
	public void setListener(ToolListener listener) {
		this.listener = (PanListener) listener;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.tools.Behavior.Behavior#getListener()
	 */
	public ToolListener getListener() {
		return listener;
	}
}
