package org.gvsig.gpe.gml;
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: GMLTags.java 397 2008-01-24 09:10:48Z jpiera $
 * $Log$
 * Revision 1.13  2007/05/24 07:29:47  csanchez
 * Añadidos Alias GML2
 *
 * Revision 1.12  2007/05/15 10:39:39  jorpiell
 * Add the number of decimals property
 *
 * Revision 1.11  2007/05/15 09:35:09  jorpiell
 * the tag names cant have blanc spaces
 *
 * Revision 1.10  2007/05/15 07:31:06  jorpiell
 * Add the geometryProperties tags
 *
 * Revision 1.9  2007/05/14 11:20:25  jorpiell
 * ProjectionFactory updated
 *
 * Revision 1.8  2007/05/14 09:52:26  jorpiell
 * Tupes separator tag updated
 *
 * Revision 1.7  2007/05/14 09:30:08  jorpiell
 * Add the fid tag
 *
 * Revision 1.6  2007/05/07 12:58:42  jorpiell
 * Add some methods to manage the multigeometries
 *
 * Revision 1.5  2007/04/13 13:16:00  jorpiell
 * Add the multiple geometries
 *
 * Revision 1.4  2007/04/12 17:06:44  jorpiell
 * First GML writing tests
 *
 * Revision 1.3  2007/04/12 10:35:04  jorpiell
 * Add the innerboundaryIs
 *
 * Revision 1.2  2007/04/12 10:23:41  jorpiell
 * Add some writers and the GPEXml parser
 *
 * Revision 1.1  2007/02/28 11:48:31  csanchez
 * *** empty log message ***
 *
 * Revision 1.1  2007/02/20 10:53:20  jorpiell
 * Añadidos los proyectos de kml y gml antiguos
 *
 * Revision 1.4  2007/01/14 13:12:11  jaume
 * objects to strings
 *
 * Revision 1.3  2006/12/22 11:25:44  csanchez
 * Nuevo parser GML 2.x para gml's sin esquema
 *
 * Revision 1.2  2006/08/10 12:37:05  jorpiell
 * Aadido el tag description al GML parser
 *
 * Revision 1.1  2006/08/10 12:00:49  jorpiell
 * Primer commit del driver de Gml
 *
 *
 */
/**
 * Class containing a description for all the TAGS defined in 
 * the GML File
 * 
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 * @author Carlos Snchez Perin (sanchez_carper@gva.es)
 * 
 */
public class GMLTags {
	public static final String GML_NAMESPACE_URI = "http://www.opengis.net/gml";	
	public static final String GML_DEAFULT_BLANC_SPACE = "_";	
	public static final int GML_DEAFULT_DECIMAL_DIGITS = 3;
	public static final String VERSION = "version";
	public static final String XML_TARGET_NAMESPACE = "targetNamespace";
	public static final String XML_ELEMENT_FORM_DEFAULT = "elementFormDefault";
	public static final String XML_ATTRIBUTE_FORM_DEFAULT = "attributeFormDefault";
	public static final String XML_NAMESPACE = "xmlns";
	public static final String XML_SCHEMA_LOCATION = "schemaLocation";
	public static final String XML_XSI = "xsi";
	public static final String GML_NAME = "name";
	public static final String GML_DESCRIPTION = "description";
	public static final String GML_BOUNDEDBY = "boundedBy";
	public static final String GML_BOX = "Box";
	public static final String GML_COORDINATES = "coordinates";
	public static final String GML_COORD = "coord";
	public static final String GML_X = "X";
	public static final String GML_Y = "Y";
	public static final String GML_Z = "Z";
	public static final String GML_COORDINATES_DECIMAL = "decimal";
	public static final String GML_COORDINATES_CS = "cs";
	public static final String GML_COORDINATES_TS = "ts";
	public static final String GML_SRS_NAME = "srsName";
	public static final String GML_GID = "gid";
	public static final String GML_FID = "fid";
	public static final String GML_ID = "id";
	public static final String GML_NAMESPACE = "gml";
	public static final String GML_FEATUREMEMBER = "featureMember";
	public static final String GML_FEATURCOLLECTION = "FeatureCollection";
	public static final String GML_ABSTRACT_FEATURECOLLECTION = "_FeatureCollection";
			
	//Namespaces
	public static final String NAMESPACE_GML = "xmlns:gml=\"http://www.opengis.net/gml\"";
	public static final String NAMESPACE_XML = "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"";
	
	
	//Geometries
	public static final String GML_POINT = "Point";
	public static final String GML_LINESTRING = "LineString";
	public static final String GML_LINEARRING = "LinearRing";
	public static final String GML_POLYGON = "Polygon";
	public static final String GML_MULTIPOINT = "MultiPoint";
	public static final String GML_MULTILINESTRING = "MultiLineString";
	public static final String GML_MULTIPOLYGON = "MultiPolygon";
	public static final String GML_MULTIGEOMETRY = "MultiGeometry";
	public static final String GML_POINTPROPERTY = "pointProperty";
	public static final String GML_LINESTRINGPROPERTY = "lineStringProperty";
	public static final String GML_POLYGONPROPERTY = "polygonProperty";
	public static final String GML_GEOMETRYPROPERTY = "geometryProperty";
	public static final String GML_MULTIPOINTPROPERTY = "multiPointProperty";
	public static final String GML_MULTILINESTRINGPROPERTY = "multiLineStringProperty";
	public static final String GML_MULTIPOLYGONPROPERTY = "multiPolygonProperty";
	public static final String GML_MULTIGEOMETRYPROPERTY = "multiGeometryProperty";
	
	public static final String GML_DEFAULT_TUPLES_SEPARATOR  = "[\\n\\s]+";
	public static final String GML_DEFAULT_COORDINATES_SEPARATOR = ",";
	public static final String GML_DEFAULT_COORDINATES_DECIMAL = ".";	
	
	public static final String GML_OUTERBOUNDARYIS = "outerBoundaryIs";
	public static final String GML_INNERBOUNDARYIS = "innerBoundaryIs";
	public static final String GML_POINTMEMBER = "pointMember";
	public static final String GML_LINESTRINGMEMBER = "lineStringMember";
	public static final String GML_POLYGONMEMBER = "polygonMember";
	public static final String GML_GEOMETRYMEMBER = "geometryMember";
	
	/*************************************************************
	 * New GML 3 definitions									 *
	 *************************************************************/
	//Atributos de Interpolacin
	public static final String GML_INTERPOLATION = "interpolation";
	public static final String GML_DEFAULT_INTERPOLATION = "linear";
	
	//Atributos de Dimensin (x,y,z) 
	public static final int GML_DEFAULT_DIMENSION = 2;
	public static final String GML_DIMENSION = "dimension";
	public static final String GML_SRSDIMENSION = "srsDimension";
	
	//Curve
	public static final String GML_MULTICURVE = "MultiCurve";
	public static final String GML_CURVEMEMBER = "curveMember";
	public static final String GML_CURVE = "Curve";
	public static final String GML_SEGMENTS = "segments";
	public static final String GML_MULTICURVEPROPERTY = "multiCurveProperty";
	public static final String GML_CURVEPROPERTY = "curveProperty";
	public static final String GML_LINESTRINGSEGMENT = "LineStringSegment";
	
	//Surface
	public static final String GML_MULTISURFACE = "MultiSurface";
	public static final String GML_SURFACEMEMBER = "surfaceMember";
	public static final String GML_PATCHES = "patches";
	public static final String GML_POLYGONPATCH = "PolygonPatch";
	public static final String GML_EXTERIOR = "exterior";
	public static final String GML_INTERIOR = "interior";
	public static final String GML_SURFACE = "Surface";
	
	//Coords
	//Sustituye al coordinates
	public static final String GML_POSLIST = "posList";
	public static final String GML_POS = "pos";
	
	//Envelope (contenido en el BoundedBy)
	public static final String GML_ENVELOPE = "Envelope";
	public static final String GML_LOWERCORNER = "lowerCorner";
	public static final String GML_UPPERCORNER = "upperCorner";
	
	
	/***************************************************************/
	
	//Point Alias
	public static final String GML_CENTEROF = "centerOf";
	public static final String GML_LOCATION = "location";
	public static final String GML_POSITION = "position";
	
	//Polygon Alias
	public static final String GML_EXTENTOF = "extentOf";
	public static final String GML_COVERAGE = "coverage";
	
	//LineString Alias
	public static final String GML_EDGEOF = "edgeOf";
	public static final String GML_CENTERLINEOF = "centerLineOf";
	
	//MultiPoint
	public static final String GML_MULTICENTEROF = "multiCenterOf";
	public static final String GML_MULTILOCATION = "multiLocation";
	public static final String GML_MULTIPOSITION = "multiPosition";
	
	//MultiLine
	public static final String GML_MULTIEDGEOF = "multiEdgeOf";
	public static final String GML_MULTICENTERLINEOF = "multiCenterLineOf";
	
	//MultiPolygon
	public static final String GML_MULTIEXTENTOF = "multiExtentOf";
	public static final String GML_MULTICOVERAGE = "multiCoverage";
	
	//srs
	public static final String SRS = "http://www.opengis.net/gml/srs/";
	public static final String SRS_EPSG_NAME = "EPSG";
	public static final String SRS_EPSG = "epsg.xml";
	public static final String SRS_UNKNOWN = "unknown";
	
}
