package org.gvsig.gpe.gml;

import java.io.IOException;

import org.gvsig.gpe.gml.parser.profiles.Gml2BindingProfile;
import org.gvsig.gpe.xml.exceptions.GPEXmlEmptyFileException;
import org.xmlpull.v1.XmlPullParserException;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: GPEGmlParser.java 162 2007-06-29 12:19:48Z jorpiell $
 * $Log$
 * Revision 1.15  2007/06/29 12:19:34  jorpiell
 * The schema validation is made independently of the concrete writer
 *
 * Revision 1.14  2007/06/07 14:53:30  jorpiell
 * Add the schema support
 *
 * Revision 1.13  2007/05/24 07:29:47  csanchez
 * Añadidos Alias GML2
 *
 * Revision 1.12  2007/05/17 11:20:51  jorpiell
 * Add the layer methods
 *
 * Revision 1.11  2007/05/16 12:34:55  csanchez
 * GPEParser Prototipo final de lectura
 *
 * Revision 1.10  2007/05/14 11:23:12  jorpiell
 * ProjectionFactory updated
 *
 * Revision 1.9  2007/05/14 09:52:26  jorpiell
 * Tupes separator tag updated
 *
 * Revision 1.8  2007/05/09 06:54:25  jorpiell
 * Change the File by URI
 *
 * Revision 1.7  2007/05/07 07:06:46  jorpiell
 * Add a constructor with the name and the description fields
 *
 * Revision 1.6  2007/04/25 11:08:38  csanchez
 * Parseo correcto con XSOM de esquemas, EntityResolver para los imports
 *
 * Revision 1.5  2007/04/20 12:04:10  csanchez
 * Actualizacion protoripo libGPE, Añadidos test para el parser, parseo con XSOM
 *
 * Revision 1.4  2007/04/19 11:51:43  csanchez
 * Actualizacion protoripo libGPE
 *
 * Revision 1.1  2007/04/18 12:54:45  csanchez
 * Actualizacion protoripo libGPE
 *
 *
 */
/**
 * @author Carlos Snchez Perin (sanchez_carper@gva.es)
 **/

public class GPEGml2Parser extends GPEDefaultGmlParser{
	
	public GPEGml2Parser(String name, String description) {
		super(name, description);
		setProfile(new Gml2BindingProfile());
	}

	/**********************************************
	 * <getVersions>
	 * Returns the gml versions that are supported
	 * @return String[] Accepted Versions
	 **********************************************/
	public String[] getVersions() {
		String[] versions = new String[3];
		versions[0] = "2.0.0";
		versions[0] = "2.1.0";
		versions[0] = "2.1.2";
		return versions;
	}
	
	/****************************
	 * <initParse>
	 * Starts to parse the file.
	 * @throws XmlPullParserException 
	 * @throws GPEXmlEmptyFileException 
	 ****************************/
	protected void initParse() throws GPEXmlEmptyFileException, XmlPullParserException {
		super.initParse();
		try{
		getProfile().getFeatureCollectionBinding().
			parse(getParser(), this);		
		} catch (IOException e) {
			getErrorHandler().addError(e);
			e.printStackTrace();
		}				
	}
}
