package org.gvsig.gpe.gml.parser.sfp0.geometries;

import java.io.IOException;

import org.gvsig.gpe.gml.GMLTags;
import org.gvsig.gpe.gml.GPEDefaultGmlParser;
import org.gvsig.gpe.gml.utils.CompareUtils;
import org.kxml2.io.KXmlParser;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id$
 * $Log$
 *
 */
/**
 * @author Jorge Piera LLodr (jorge.piera@iver.es)
 */
public class InteriorTypeBinding {
	/**
	 * It parses the gml:interior tag
	 * @param parser
	 * The XML parser
	 * @param handler
	 * The GPE parser that contains the content handler and
	 * the error handler
	 * @return
	 * An array of coordinates
	 * @throws XmlPullParserException
	 * @throws IOException
	 */
	public double[][] parse(XmlPullParser parser,GPEDefaultGmlParser handler) throws XmlPullParserException, IOException  {
		boolean endFeature = false;
		int currentTag;

		double[][] coordinates = null;


		String tag = parser.getName();
		currentTag = parser.getEventType();

		while (!endFeature){
			switch(currentTag){
			case KXmlParser.START_TAG:
				if (CompareUtils.compareWithOutNamespace(tag,GMLTags.GML_LINEARRING)){
					coordinates = handler.getProfile().getLinearRingTypeBinding().
					parseCoordinates(parser, handler);
				}
				break;
			case KXmlParser.END_TAG:
				if (CompareUtils.compareWithOutNamespace(tag,GMLTags.GML_INTERIOR)){						
					endFeature = true;
				}
				break;
			case KXmlParser.TEXT:					

				break;
			}
			if (!endFeature){					
				currentTag = parser.next();
				tag = parser.getName();
			}
		}			
		return coordinates;
	}
}