package org.gvsig.gpe.gml.parser.v2.features;

import java.io.IOException;
import java.util.Hashtable;

import org.gvsig.gpe.gml.GMLTags;
import org.gvsig.gpe.gml.GPEDefaultGmlParser;
import org.gvsig.gpe.gml.utils.CompareUtils;
import org.gvsig.gpe.gml.utils.GMLObject;
import org.gvsig.gpe.gml.utils.GMLUtilsParser;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: FeatureCollectionBinding.java 350 2008-01-09 12:53:07Z jpiera $
 * $Log$
 * Revision 1.2  2007/06/29 12:19:34  jorpiell
 * The schema validation is made independently of the concrete writer
 *
 * Revision 1.1  2007/06/07 14:53:30  jorpiell
 * Add the schema support
 *
 *
 */
/**
 * This class parses the gml objects that has a 
 * gml:FeatureCollection type. Example:
 * <p>
 * <pre>
 * <code>
 * &lt;gml:featureCollection&gt;
 * &lt;gml:featureMember&gt; 
 * &lt;/gml:featureMember&gt;
 * &lt;gml:featureMember&gt; 
 * &lt;/gml:featureMember&gt;
 * &lt;/gml:featureCollection&gt;
 * </code>
 * </pre>
 * </p>
 * @author Jorge Piera LLodr (jorge.piera@iver.es)
 */
public class FeatureCollectionBinding {
	/**
	 * It parses a feature collection
	 * @param parser
	 * The XML parser
	 * @param handler
	 * The GPE parser that contains the content handler and
	 * the error handler
	 * @return
	 * A feature
	 * @throws XmlPullParserException
	 * @throws IOException
	 */
	public Object parse(XmlPullParser parser,GPEDefaultGmlParser handler) throws XmlPullParserException, IOException {
		boolean endCollection = false;
		int currentTag;		
		Object layer = null;	
		//Used to finish to parse the current feature collection
		String layerRootType = parser.getName();

		Hashtable attributes = GMLUtilsParser.getAttributes(parser);		
		String fid = FeatureTypeBinding.getID(attributes);

		layer = handler.getContentHandler().startLayer(fid, null, null, null, null, null, null);

		String tag = parser.getName();
		currentTag = parser.getEventType();

		while (!endCollection){
			switch(currentTag){
			case XmlPullParser.START_TAG:
				if (CompareUtils.compareWithOutNamespace(tag,GMLTags.GML_NAME)){
					parser.next();
					handler.getContentHandler().addNameToLayer(parser.getText(), layer);
				}else if (CompareUtils.compareWithOutNamespace(tag,GMLTags.GML_DESCRIPTION)){
					parser.next();
					handler.getContentHandler().addDescriptionToLayer(parser.getText(), layer);
				}else if (CompareUtils.compareWithOutNamespace(tag,GMLTags.GML_BOUNDEDBY)){
					Object bbox = handler.getProfile().getBoundedByTypeBinding().
							parse(parser, handler);
					handler.getContentHandler().addBboxToLayer(bbox, layer);
				}else if (CompareUtils.compareWithOutNamespace(tag,GMLTags.GML_FEATUREMEMBER)){
					GMLObject object = (GMLObject)handler.getProfile().getFeatureMemberTypeBinding().
							parse(parser, handler);
					if (object.getType() == GMLObject.FEATURE){
						handler.getContentHandler().addFeatureToLayer(object.getObject(), layer);
					}else{
						handler.getContentHandler().addParentLayerToLayer(layer, object.getObject());
					}
				}else{
					if (!(layerRootType.compareTo(tag) == 0)){						
						//Feature members
						GMLObject object =(GMLObject)handler.getProfile().getFeatureTypeBinding().parse(parser, handler);
						handler.getContentHandler().addFeatureToLayer(object.getObject(), layer);
					}
				}
				break;
			case XmlPullParser.END_TAG:
				if (tag.compareTo(layerRootType) == 0){						
					endCollection = true;
					handler.getContentHandler().endLayer(layer);
				}
				break;
			case XmlPullParser.TEXT:					

				break;
			case XmlPullParser.END_DOCUMENT:
				endCollection = true;
				break;
			}
			if (!endCollection){					
				currentTag = parser.next();
				tag = parser.getName();
			}
		}			
		return layer;	
	}
}
