package org.gvsig.gpe.gml.parser.v2.geometries;

import java.io.IOException;

import org.gvsig.gpe.gml.GMLTags;
import org.gvsig.gpe.gml.GPEDefaultGmlParser;
import org.gvsig.gpe.gml.utils.CompareUtils;
import org.kxml2.io.KXmlParser;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
*
* $Id:CoordinatesTypeBinding.java 371 2008-01-10 09:30:19Z jpiera $
* $Log$
* Revision 1.3  2007/05/14 09:52:53  jorpiell
* Nullpointer exception fixed
*
* Revision 1.2  2007/05/14 09:31:06  jorpiell
* Add the a new class to compare tags
*
* Revision 1.1  2007/05/07 12:58:42  jorpiell
* Add some methods to manage the multigeometries
*
* Revision 1.1  2007/05/07 07:06:46  jorpiell
* Add a constructor with the name and the description fields
*
*
*/
/**
 * It parses a gml:CoordinatesType object. Example:
 * <p>
 * <pre>
 * <code>
 * &lt;coordinates&gt;60.0,60.0 60.0,90.0 90.0,90.0&lt;/coordinates&gt;
 * </code>
 * </pre>
 * </p> 
 * @author Jorge Piera LLodr (jorge.piera@iver.es)
 */
public class CoordinatesTypeBinding {
	
	/**
	 * It parses the gml:coordinates tag
	 * @param parser
	 * The XML parser
	 * @param handler
	 * The GPE parser that contains the content handler and
	 * the error handler
	 * @return
	 * It retuns a matrix of doubles with 3 columns (x,y,z) and
	 * one row for each coordinate.
	 * @throws XmlPullParserException
	 * @throws IOException
	 */
	public double[][] parse(XmlPullParser parser,GPEDefaultGmlParser handler) throws XmlPullParserException, IOException  {
		boolean endFeature = false;
		int currentTag;
		
		String TUPLES_SEPARATOR  = GMLTags.GML_DEFAULT_TUPLES_SEPARATOR;
		String COORDINATES_SEPARATOR = GMLTags.GML_DEFAULT_COORDINATES_SEPARATOR;
		String COORDINATES_DECIMAL = GMLTags.GML_DEFAULT_COORDINATES_DECIMAL;		
		
		double[][] aCoordinates = null;
		
		for (int i=0 ; i<parser.getAttributeCount() ; i++){
			if (parser.getAttributeName(i).compareTo(GMLTags.GML_COORDINATES_DECIMAL) == 0){
				COORDINATES_DECIMAL = parser.getAttributeValue(i);
			}else if (parser.getAttributeName(i).compareTo(GMLTags.GML_COORDINATES_CS) == 0){
				COORDINATES_SEPARATOR = parser.getAttributeValue(i);
			}else if (parser.getAttributeName(i).compareTo(GMLTags.GML_COORDINATES_TS) == 0){
				TUPLES_SEPARATOR = parser.getAttributeValue(i);
			}
		}		

		String tag = parser.getName();
		currentTag = parser.getEventType();

		while (!endFeature){
			switch(currentTag){
			case KXmlParser.START_TAG:
				if (CompareUtils.compareWithOutNamespace(tag,GMLTags.GML_COORDINATES)){
					parser.next();						
					String[] coordinates = parser.getText().trim().split(TUPLES_SEPARATOR);
					aCoordinates = new double[3][coordinates.length];
					for (int i=0 ; i<coordinates.length ; i++){					
						String[] coordinate = coordinates[i].trim().split(COORDINATES_SEPARATOR);
						aCoordinates[0][i] = DoubleTypeBinding.parse(coordinate[0], COORDINATES_DECIMAL);
						aCoordinates[1][i] = DoubleTypeBinding.parse(coordinate[1], COORDINATES_DECIMAL);
						if (coordinate.length == 3){
							aCoordinates[2][i] = DoubleTypeBinding.parse(coordinate[2], COORDINATES_DECIMAL);
						}else{
							aCoordinates[2][i] = 0.0;
						}
					}					
				}
				break;
			case KXmlParser.END_TAG:
				if (CompareUtils.compareWithOutNamespace(tag,GMLTags.GML_COORDINATES)){						
					endFeature = true;
				}
				break;
			case KXmlParser.TEXT:					

				break;
			}
			if (!endFeature){					
				currentTag = parser.next();
				tag = parser.getName();
			}
		}			

		return aCoordinates;	
	}

}
