package org.gvsig.gpe.gml.parser.v2.geometries;

import java.io.IOException;

import org.gvsig.gpe.gml.GMLTags;
import org.gvsig.gpe.gml.GPEDefaultGmlParser;
import org.gvsig.gpe.gml.utils.CompareUtils;
import org.kxml2.io.KXmlParser;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id:PolygonTypeBinding.java 195 2007-11-26 09:02:22Z jpiera $
 * $Log$
 * Revision 1.4  2007/05/14 11:18:51  jorpiell
 * ProjectionFactory updated
 *
 * Revision 1.3  2007/05/14 09:31:06  jorpiell
 * Add the a new class to compare tags
 *
 * Revision 1.2  2007/05/08 10:24:16  jorpiell
 * Add comments to create javadocs
 *
 * Revision 1.1  2007/05/07 12:58:42  jorpiell
 * Add some methods to manage the multigeometries
 *
 * Revision 1.1  2007/05/07 07:06:46  jorpiell
 * Add a constructor with the name and the description fields
 *
 *
 */
/**
 * It parses a gml:PolygonType object. Example:
 * <p>
 * <pre>
 * <code>
 * &lt;Polygon gid="_877789"&gt;
 * &lt;outerBoundaryIs&gt;
 * &lt;LinearRing&gt;
 * &lt;coordinates&gt;0.0,0.0 100.0,0.0 50.0,100.0 0.0,0.0&lt;/coordinates&gt;
 * &lt;/LinearRing&gt;
 * &lt;/outerBoundaryIs&gt;
 * &lt;/Polygon&gt;
 * </code>
 * </pre>
 * </p> 
 * @author Jorge Piera LLodr (jorge.piera@iver.es)
 */
public class PolygonTypeBinding extends GeometryBinding{
	protected Object polygon = null;
	
	/**
	 * It parses the gml:Polygon tag
	 * @param parser
	 * The XML parser
	 * @param handler
	 * The GPE parser that contains the content handler and
	 * the error handler
	 * @return
	 * A polygon
	 * @throws XmlPullParserException
	 * @throws IOException
	 */
	public Object parse(XmlPullParser parser,GPEDefaultGmlParser handler) throws XmlPullParserException, IOException {
		boolean endFeature = false;
		int currentTag;			
		
		super.setAtributtes(parser, handler.getErrorHandler());

		String tag = parser.getName();
		currentTag = parser.getEventType();

		while (!endFeature){
			switch(currentTag){
			case KXmlParser.START_TAG:
				polygon = parseTag(parser, handler, tag, id, srsName);
				break;
			case KXmlParser.END_TAG:
				endFeature = parseLastTag(parser, handler, tag);
				if (endFeature){
					handler.getContentHandler().endPolygon(polygon);
				}
				break;
			case KXmlParser.TEXT:					

				break;
			}
			if (!endFeature){					
				currentTag = parser.next();
				tag = parser.getName();
			}
		}			

		return polygon;
	}
	
	/**
	 * It parses the XML tag
	 * @param parser
	 * @param handler
	 * @param tag
	 * @param id
	 * @param srsName
	 * @return
	 * @throws XmlPullParserException
	 * @throws IOException
	 */
	protected Object parseTag(XmlPullParser parser,GPEDefaultGmlParser handler, String tag, String id, String srsName) throws XmlPullParserException, IOException{
		if (CompareUtils.compareWithOutNamespace(tag,GMLTags.GML_OUTERBOUNDARYIS)){
			double[][] coordinates = handler.getProfile().getOuterBoundaryIsTypeBinding().
			parse(parser, handler);
			this.polygon = handler.getContentHandler().startPolygon(id,
					coordinates[0],
					coordinates[1],
					coordinates[2],
					srsName);
		}else if (CompareUtils.compareWithOutNamespace(tag,GMLTags.GML_INNERBOUNDARYIS)){
			double[][] coordinates = handler.getProfile().getInnerBoundaryIsTypeBinding().
			parse(parser, handler);
			Object innerPolygon = handler.getContentHandler().startInnerPolygon(null,
					coordinates[0],
					coordinates[1],
					coordinates[2],
					srsName);
			handler.getContentHandler().endInnerPolygon(innerPolygon);
			handler.getContentHandler().addInnerPolygonToPolygon(innerPolygon,this.polygon);
		}
		return polygon;
	}
	
	/**
	 * Parses the last tag
	 * @param parser
	 * @param handler
	 * @param tag
	 * @return
	 */
	protected boolean parseLastTag(XmlPullParser parser,GPEDefaultGmlParser handler, String tag){
		if (CompareUtils.compareWithOutNamespace(tag,GMLTags.GML_POLYGON)){						
			return true;
		}
		return false;
	}
}
