package org.gvsig.gpe.gml.writer.v2.geometries;

import java.io.IOException;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;

import org.gvsig.gpe.GPEDefaults;
import org.gvsig.gpe.gml.GMLTags;
import org.gvsig.gpe.gml.writer.GPEGmlWriterHandlerImplementor;
import org.gvsig.gpe.xml.writer.Writer;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: DoubleWriter.java 350 2008-01-09 12:53:07Z jpiera $
 * $Log$
 * Revision 1.6  2007/05/15 10:39:39  jorpiell
 * Add the number of decimals property
 *
 * Revision 1.5  2007/05/14 11:18:12  jorpiell
 * Add the ErrorHandler to all the methods
 *
 * Revision 1.4  2007/05/10 12:21:40  jorpiell
 * Add a decimal formatter for the Double values
 *
 * Revision 1.3  2007/05/08 10:24:16  jorpiell
 * Add comments to create javadocs
 *
 * Revision 1.2  2007/04/14 16:07:30  jorpiell
 * The writer has been created
 *
 * Revision 1.1  2007/04/12 17:06:44  jorpiell
 * First GML writing tests
 *
 *
 */
/**
 * It writes a double. 
 * @author Jorge Piera LLodr (jorge.piera@iver.es)
 */
public class DoubleWriter {
	private DecimalFormat decimalFormat = null;
	
	/**
	 * Writes a double in GML
	 * @param writer
	 * Writer to write the labels
	 * @param handler
	 * The writer handler implementor
	 * @param value
	 * Docble value
	 * @throws IOException
	 */
	public void write(Writer writer, GPEGmlWriterHandlerImplementor handler, double value)throws IOException{
		writer.write(getDecimalFormat().format(value));
	}
	
	/**
	 * Format the double
	 * @return
	 */
	private DecimalFormat getDecimalFormat(){
		if (decimalFormat == null){
			DecimalFormatSymbols dfs = new DecimalFormatSymbols();
			dfs.setDecimalSeparator(GPEDefaults.getStringProperty(GPEDefaults.DECIMAL).toCharArray()[0]);
			decimalFormat = new DecimalFormat("#.#",dfs);
			int decimalDigits = GPEDefaults.getIntPropertyProperty(GPEDefaults.DECIMAL_DIGITS);
			if (decimalDigits == -1){
				decimalDigits = GMLTags.GML_DEAFULT_DECIMAL_DIGITS;
			}
			decimalFormat.setMaximumFractionDigits(decimalDigits);
		}		
		return decimalFormat;
	}
}
