package org.gvsig.gpe.xml;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.InvocationTargetException;

import org.gvsig.gpe.GPEParser;
import org.gvsig.gpe.xml.exceptions.GPEXmlEmptyFileException;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: GPEXmlParser.java 193 2007-11-26 08:58:56Z jpiera $
 * $Log$
 * Revision 1.11  2007/06/22 12:22:40  jorpiell
 * The typeNotFoundException has been deleted. It never was thrown
 *
 * Revision 1.10  2007/06/14 13:50:05  jorpiell
 * The schema jar name has been changed
 *
 * Revision 1.9  2007/06/07 14:53:30  jorpiell
 * Add the schema support
 *
 * Revision 1.8  2007/05/09 06:54:25  jorpiell
 * Change the File by URI
 *
 * Revision 1.7  2007/05/07 07:07:04  jorpiell
 * Add a constructor with the name and the description fields
 *
 * Revision 1.6  2007/04/19 12:01:55  jorpiell
 * Updated the getEncoding method
 *
 * Revision 1.5  2007/04/19 11:56:03  csanchez
 * Actualizacion protoripo libGPE
 *
 * Revision 1.4  2007/04/19 07:30:40  jorpiell
 * Add the add methods to teh contenhandler and change the register mode
 *
 * Revision 1.3  2007/04/18 12:54:45  csanchez
 * Actualizacion protoripo libGPE
 *
 * Revision 1.2  2007/04/12 11:47:15  jorpiell
 * Add a getParser method
 *
 * Revision 1.1  2007/04/12 10:23:41  jorpiell
 * Add some writers and the GPEXml parser
 *
 *
 */
/**
 * This class can be implemented by all the classes that 
 * implements a GPE driver based on the XML format.
 * @author Jorge Piera LLodr (jorge.piera@iver.es)
 */
public abstract class GPEXmlParser extends GPEParser {
	private XmlPullParser parser = null;
	private String encoding = "UTF-8";

	public GPEXmlParser(String name, String description) {
		super(name, description);
	}

	/*
	 * (non-Javadoc)
	 * @see org.gvsig.gpe.GPEParser#parseStream()
	 */	
	protected void parseStream(){
		try{
			parser = GPEXmlParserFactory.getParser();
			parser.setInput(getInputStream(), encoding);			
			GPEXmlSchemaDocument schemaDocuemnt = new GPEXmlSchemaDocument(parser,
					getMainFile(),
					getErrorHandler());
			schemaDocuemnt.parse();
			getContentHandler().setSchemaDocument(schemaDocuemnt);
			initParse();
		} // EL PARSER LANZAR UN EVENTO AL ERRORHANDLER SEGN EL ERROR RECOGIDO 
		catch (FileNotFoundException e) {
			System.out.println("ERROR: Fichero "+ getMainFile().getPath() +" no encontrado");
			getErrorHandler().addError(e);
		} catch (IllegalArgumentException e) {
			System.out.println("ERROR: Argumento Incorrecto para KXML: "+ e.getMessage());
			getErrorHandler().addError(e);
		} catch (SecurityException e) {
			System.out.println("ERROR: Fallo de seguridad en la clase KXML: "+ e.getMessage());
			getErrorHandler().addError(e);
		} catch (InstantiationException e) {
			System.out.println("ERROR: Instancia de KXML incorrecta: "+ e.getMessage());
			getErrorHandler().addError(e);
		} catch (IllegalAccessException e) {
			System.out.println("ERROR: Acceso Ilegal a memoria de KXML: "+ e.getMessage());
			getErrorHandler().addError(e);
		} catch (InvocationTargetException e) {
			System.out.println("ERROR: Invocacin de KXML Incorrecta: "+ e.getMessage());
			getErrorHandler().addError(e);
		} catch (NoSuchMethodException e) {
			System.out.println("ERROR: Mtodo no implementado del parser KXML: "+ e.getMessage());
			getErrorHandler().addError(e);
		} catch (ClassNotFoundException e) {
			System.out.println("ERROR: Clase KXML no encontrada: "+ e.getMessage());
			getErrorHandler().addError(e);	
		} catch (XmlPullParserException e) {
			System.out.println("ERROR: Excepcin en el parser KXML: "+ e.getMessage());
			getErrorHandler().addError(e);
		} catch (IOException e) {
			System.out.println("ERROR: No se puede leer/escribir la codificacin del fichero: "+ e.getMessage());
			getErrorHandler().addError(e);
		} catch (GPEXmlEmptyFileException e) {
			getErrorHandler().addError(e);
		}		
	}

	/*
	 * (non-Javadoc)
	 * @see org.gvsig.gpe.GPEParser#parseURI()
	 */
	protected void parseURI() {
		try {
			//Creating an input stream to gets the encoding
			setInputStream(createInputStream(new File(getMainFile())));
			encoding = getEncoding();
			//Creates an input stream to read the file
			setInputStream(createInputStream(new File(getMainFile())));
			parseStream();
		} catch (FileNotFoundException e) {
			System.out.println("ERROR: Fichero "+ getMainFile().getPath() +" no encontrado");
			getErrorHandler().addError(e);
		} catch (IOException e) {
			System.out.println("ERROR: No se puede leer/escribir la codificacin del fichero: "+ e.getMessage());
			getErrorHandler().addError(e);
		}
	}

	/**
	 * Creates an input stream from a file. 
	 * @param file
	 * @return InputStream
	 * @throws FileNotFoundException 
	 */
	protected abstract InputStream createInputStream(File file) throws FileNotFoundException;

	/**
	 * This method start the parse process. It is called
	 * after the XML parser is initialized
	 */
	protected abstract void initParse() throws GPEXmlEmptyFileException, XmlPullParserException;

	/**
	 * @return Returns the encoding or UTF-8 encoding by default.
	 * @throws IOException 
	 * @throws KmlException 
	 */
	private String getEncoding() throws IOException {
		String encoding = "UTF-8";
		int srcCount = 0;
		char[] srcBuf = new char[128];
		InputStream _is = getInputStream(); 

		// read four bytes 
		int chk = 0;
		try {
			while (srcCount < 4) {
				int i = _is.read();
				if (i == -1)
					break;
				chk = (chk << 8) | i;
				srcBuf[srcCount++] = (char) i;
			}

			if (srcCount == 4) {
				switch (chk) {
				case 0x00000FEFF :
					encoding = "UTF-32BE";
					srcCount = 0;
					break;

				case 0x0FFFE0000 :
					encoding = "UTF-32LE";
					srcCount = 0;
					break;

				case 0x03c :
					encoding = "UTF-32BE";
					srcBuf[0] = '<';
					srcCount = 1;
					break;

				case 0x03c000000 :
					encoding = "UTF-32LE";
					srcBuf[0] = '<';
					srcCount = 1;
					break;

				case 0x0003c003f :
					encoding = "UTF-16BE";
					srcBuf[0] = '<';
					srcBuf[1] = '?';
					srcCount = 2;
					break;

				case 0x03c003f00 :
					encoding = "UTF-16LE";
					srcBuf[0] = '<';
					srcBuf[1] = '?';
					srcCount = 2;
					break;

				case 0x03c3f786d :
					while (true) {
						int i = _is.read();
						if (i == -1)
							break;
						srcBuf[srcCount++] = (char) i;
						if (i == '>') {
							String s = new String(srcBuf, 0, srcCount);
							int i0 = s.indexOf("encoding");
							if (i0 != -1) {
								while (s.charAt(i0) != '"'
									&& s.charAt(i0) != '\'')
									i0++;
								char deli = s.charAt(i0++);
								int i1 = s.indexOf(deli, i0);
								encoding = s.substring(i0, i1);
							}
							break;
						}
					}

				default :
					if ((chk & 0x0ffff0000) == 0x0FEFF0000) {
						encoding = "UTF-16BE";
						srcBuf[0] =
							(char) ((srcBuf[2] << 8) | srcBuf[3]);
						srcCount = 1;
					}
					else if ((chk & 0x0ffff0000) == 0x0fffe0000) {
						encoding = "UTF-16LE";
						srcBuf[0] =
							(char) ((srcBuf[3] << 8) | srcBuf[2]);
						srcCount = 1;
					}
					else if ((chk & 0x0ffffff00) == 0x0EFBBBF00) {
						encoding = "UTF-8";
						srcBuf[0] = srcBuf[3];
						srcCount = 1;
					}
				}
			}
		}
		catch (IOException ex) {
			return null;
		}
		return encoding;
	}

	/**
	 * @return the parser
	 */
	protected XmlPullParser getParser() {
		return parser;
	}	

}
