package org.gvsig.gpe.xml.writer;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URI;
import java.util.Enumeration;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import org.gvsig.gpe.GPEDefaults;
import org.gvsig.gpe.gml.GMLTags;
import org.gvsig.gpe.writers.GPEWriterHandlerImplementor;
import org.gvsig.gpe.xml.exceptions.GPEXmlToZipFileException;
import org.gvsig.xmlschema.som.IXSSchema;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: GPEXmlWriterHandlerImplementor.java 350 2008-01-09 12:53:07Z jpiera $
 * $Log$
 * Revision 1.12  2007/06/28 13:05:09  jorpiell
 * The Qname has been updated to the 1.5 JVM machine. The schema validation is made in the GPEWriterHandlerImplementor class
 *
 * Revision 1.11  2007/06/22 12:22:40  jorpiell
 * The typeNotFoundException has been deleted. It never was thrown
 *
 * Revision 1.10  2007/06/14 13:50:05  jorpiell
 * The schema jar name has been changed
 *
 * Revision 1.9  2007/06/07 14:53:30  jorpiell
 * Add the schema support
 *
 * Revision 1.8  2007/05/07 07:07:04  jorpiell
 * Add a constructor with the name and the description fields
 *
 * Revision 1.7  2007/04/26 14:40:03  jorpiell
 * Some writer handler methods updated
 *
 * Revision 1.6  2007/04/19 07:25:49  jorpiell
 * Add the add methods to teh contenhandler and change the register mode
 *
 * Revision 1.5  2007/04/17 10:30:11  jorpiell
 * Add a method to compress a file
 *
 * Revision 1.4  2007/04/14 16:07:30  jorpiell
 * The writer has been created
 *
 * Revision 1.3  2007/04/13 07:17:57  jorpiell
 * Add the writting tests for the simple geometries
 *
 * Revision 1.2  2007/04/12 17:06:44  jorpiell
 * First GML writing tests
 *
 * Revision 1.1  2007/04/12 10:23:41  jorpiell
 * Add some writers and the GPEXml parser
 *
 *
 */
/**
 * @author Jorge Piera LLodr (jorge.piera@iver.es)
 */
public abstract class GPEXmlWriterHandlerImplementor extends GPEWriterHandlerImplementor {
	protected Writer writer = null;
	private String targetNamespace = null;
	private String targetNamespacePrefix = null;
	
	public GPEXmlWriterHandlerImplementor(String name, String description) {
		super(name, description);		
	}
	
	/**
	 * Writes the XML header
	 * @throws IOException
	 */
	protected void initXmlHeader() throws IOException{
		if (getOutputStream() == null){
			writer = createWriter();
		}else{
			writer = new Writer(getOutputStream());
		}
		writer.write("<?xml version=\"");
		writer.write(GPEDefaults.getStringProperty(GPEDefaults.XML_VERSION).toString());
		writer.write("\" encoding=\"");
		writer.write(GPEDefaults.getStringProperty(GPEDefaults.XML_ENCODING).toString());
		writer.write("\"?>");
	}
	
	/**
	 * Creates an input stream from a file. 
	 * @param file
	 * @return
	 * @throws FileNotFoundException 
	 * @throws FileNotFoundException 
	 */
	protected abstract Writer createWriter() throws IOException;
	
	/**
	 * @return the schemaLocation tag
	 */
	protected String getSchemaLocations(){
		StringBuffer schemaLocation = new StringBuffer();
		Enumeration uris = getSchemaDocument().getURIs();
		int i = 0;
		while (uris.hasMoreElements()){
			if (i > 0){
				schemaLocation.append("\b");
			}
			URI uri = (URI)uris.nextElement();
			IXSSchema schema = getSchemaDocument().getSchema(uri);
			schemaLocation.append(schema.getTargetNamespace());
			schemaLocation.append(" ");
			schemaLocation.append(getSchemaDocument().getSchemaLocation(uri));
			i++;
		}
		return schemaLocation.toString();
	}	
	
	/**
	 * @return the XML header prefixes
	 */
	protected String getDocumentPrefixes(){
		StringBuffer buffer = new StringBuffer();
		if (getSchemaDocument().hasPrefixes() ||
				(getSchemaDocument().getTargetNamespace() != null)){
			//The TargetNamespace for the unqualified names
			if (getSchemaDocument().getTargetNamespace() != null){
				String prefix = getSchemaDocument().getTargetNamespace();
				buffer.append(GMLTags.XML_NAMESPACE);
				buffer.append("=\"");	
				buffer.append(prefix);
				buffer.append("\" ");
			}
			//The other schemas
			Enumeration prefixes = getSchemaDocument().getPrefixes();
			int i = 0;
			while (prefixes.hasMoreElements()){
				if (i > 0){
					buffer.append("\b");
				}
				String prefix = (String)prefixes.nextElement();
				String namespaceURI = getSchemaDocument().getNamespaceURI(prefix);
				buffer.append(GMLTags.XML_NAMESPACE + ":" + prefix);
				buffer.append("=\"");			
				buffer.append(namespaceURI);
				buffer.append("\" ");
				i++;
			}
		}else{
			String prefix = GPEDefaults.getStringProperty(GPEDefaults.NAMESPACE_PREFIX);
			String uri = GPEDefaults.getStringProperty(GPEDefaults.NAMESPACE_URI);
			if ((prefix != null) && (uri != null)){
				buffer.append(" " + GMLTags.XML_NAMESPACE + ":" + prefix + "=\"");
				buffer.append(uri + "\" ");				
			}
		}		
		return buffer.toString();
	}
	
	/**
	 * Returns the selected target namespace prefix. If
	 * the namespace doesn't exists it returns the 
	 * default Namespace prefix; 
	 * @return the namespace prefix
	 */
	protected String getTargetNamespacePrefix() {
		if (targetNamespacePrefix == null){
			String prefix = null;
			if (getSchemaDocument().getTargetNamespace() != null){
				prefix = getSchemaDocument().getNamespacePrefix(
						getSchemaDocument().getTargetNamespace());
			}else{
				prefix = GPEDefaults.getStringProperty(GPEDefaults.NAMESPACE_PREFIX);
			}
			if (prefix != null){
				targetNamespacePrefix = prefix + ":";
			}else{
				targetNamespacePrefix = "";
			}
		}
		return targetNamespacePrefix;
	}
	
	/**
	 * Returns the selected target namespace. If
	 * the namespace doesn't exists it returns the 
	 * default Namespace; 
	 * @return the namespace
	 */
	protected String getTargetNamespace() {
		if (targetNamespace == null){
			if (getSchemaDocument().getTargetNamespace() != null){
				targetNamespace = getSchemaDocument().getTargetNamespace();
			}else{
				targetNamespace = GPEDefaults.getStringProperty(GPEDefaults.NAMESPACE_URI);
			}
		}
		return targetNamespace;
	}
	
	/**
	 * This method compress a file using the ZIP
	 * compression
	 * @throws IOException 
	 */
	protected void compressFile() throws GPEXmlToZipFileException{
		int BUFFER_SIZE = 1024;
		int i = 0;
		boolean fileExists = true;
		File zipFile = null;
		String fileNameAux = null;
		while (fileExists){
			fileNameAux = getOutputFile().getAbsolutePath() + i;
			zipFile = new File(fileNameAux);
			if (!zipFile.exists()){
				fileExists = false;
			}
			i++;
		}
		try{
			ZipOutputStream out = new ZipOutputStream(
					new BufferedOutputStream(
							new FileOutputStream(zipFile)));
			byte[] data = new byte[BUFFER_SIZE];
			FileInputStream fi = new FileInputStream(getOutputFile());
			InputStream origin = new BufferedInputStream( fi, BUFFER_SIZE );
			ZipEntry entry = new ZipEntry(getOutputFile().getName());
			out.putNextEntry( entry );
			int count;
			while(( count = origin.read(data, 0, BUFFER_SIZE ) ) != -1 ){
				out.write(data, 0, count);
			}
			//Close the file
			origin.close();
			//Close the zip file
			out.close();	
			//Change the file name and delete the first
			zipFile.renameTo(getOutputFile());
			new File(fileNameAux).delete();
		} catch (FileNotFoundException e) {
			throw new GPEXmlToZipFileException(getFileName(),e);
		} catch (IOException e) {
			throw new GPEXmlToZipFileException(getFileName(),e);
		} 
	}
	
}
	

