package org.gvsig.gpe.kml.parser;

import org.gvsig.gpe.kml.GPEDeafultKmlParser;
import org.gvsig.gpe.kml.exceptions.KmlBodyParseException;
import org.xmlpull.v1.XmlPullParser;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: AbstractKmlParser.java 366 2008-01-10 09:09:38Z jpiera $
 * $Log$
 * Revision 1.1  2007/05/11 07:06:29  jorpiell
 * Refactoring of some package names
 *
 * Revision 1.2  2007/05/08 09:28:17  jorpiell
 * Add comments to create javadocs
 *
 * Revision 1.1  2007/04/13 13:16:21  jorpiell
 * Add KML reading support
 *
 *
 */
/**
 * This class must be implemented by all the classes that
 * implementa a KML version parser
 * @author Jorge Piera LLodr (jorge.piera@iver.es)
 */
public abstract class AbstractKmlParser {
	private XmlPullParser parser = null;
	private GPEDeafultKmlParser handler = null;
	
	public AbstractKmlParser(XmlPullParser parser, GPEDeafultKmlParser handler) {
		super();
		this.parser = parser;
		this.handler = handler;
	}

	/**
	 * @return 
	 * the supported version
	 */
	public abstract String getVersion();

	/**
	 * Initialize the parsing process
	 */
	public abstract void initParse() throws KmlBodyParseException;
	
	/**
	 * @return the handler
	 */
	public GPEDeafultKmlParser getHandler() {
		return handler;
	}

	/**
	 * @return the parser
	 */
	public XmlPullParser getParser() {
		return parser;
	}
}
