package org.gvsig.gpe.kml.parser;

import java.io.IOException;

import org.gvsig.gpe.kml.GPEDeafultKmlParser;
import org.gvsig.gpe.kml.KmlTags;
import org.gvsig.gpe.kml.exceptions.KmlBodyParseException;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;


/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: KmlParserV2_1.java 366 2008-01-10 09:09:38Z jpiera $
 * $Log$
 * Revision 1.1  2007/05/11 07:06:29  jorpiell
 * Refactoring of some package names
 *
 * Revision 1.3  2007/05/08 09:28:17  jorpiell
 * Add comments to create javadocs
 *
 * Revision 1.2  2007/05/02 11:46:50  jorpiell
 * Writing tests updated
 *
 * Revision 1.2  2007/04/14 16:08:07  jorpiell
 * Kml writing support added
 *
 * Revision 1.1  2007/04/13 13:16:21  jorpiell
 * Add KML reading support
 *
 * Revision 1.1  2007/03/07 08:19:10  jorpiell
 * Pasadas las clases de KML de libGPE-GML a libGPE-KML
 *
 * Revision 1.1  2007/02/28 11:48:31  csanchez
 * *** empty log message ***
 *
 * Revision 1.1  2007/02/20 10:53:20  jorpiell
 * Añadidos los proyectos de kml y gml antiguos
 *
 * Revision 1.1  2007/02/12 13:49:18  jorpiell
 * Aadido el driver de KML
 *
 *
 */
/**
 * This is a parser for KML 2.1
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class KmlParserV2_1 extends AbstractKmlParser{
	private Object layer = null;	
	
	public KmlParserV2_1(XmlPullParser parser, GPEDeafultKmlParser handler) {
		super(parser, handler);		
	}

	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.remoteClient.gmlEngine.IFeaturesParser#getVersion()
	 */
	public String getVersion() {
		return "2.1";
	}

	/*
	 * (non-Javadoc)
	 * @see org.gvsig.gpe.kml.versions.AbstractKmlParser#initParse()
	 */
	public void initParse() throws KmlBodyParseException {
		boolean endFeature = false;
		int currentTag;		

		try {
			String tag = getParser().getName();
			currentTag = getParser().getEventType();

			while (!endFeature){
				switch(currentTag){
				case XmlPullParser.START_TAG:
					if (tag.compareTo(KmlTags.DOCUMENT) == 0){
						 getHandler().getProfile().getDocumentBinding().parse(getParser(), getHandler());
					}else if (tag.compareTo(KmlTags.FOLDER) == 0){
						 getHandler().getProfile().getFolderBinding().parse(getParser(), getHandler(), null);
					}else if (tag.compareTo(KmlTags.PLACEMARK) == 0){
						if (layer == null){
							layer = getHandler().getContentHandler().startLayer(null, null, null, KmlTags.DEFAULT_SRS, null, null, null);
						}
						Object feature = getHandler().getProfile().getPlaceMarketBinding().parse(getParser(), getHandler());
						getHandler().getContentHandler().addFeatureToLayer(feature, layer);			
					}
					break;
				case XmlPullParser.END_DOCUMENT:
					endFeature = true;

					break;					
				}
				if (!endFeature){					
					currentTag = getParser().next();
					tag = getParser().getName();
				}
			}
			if (layer != null){
				getHandler().getContentHandler().endLayer(layer);
			}
		} catch (XmlPullParserException e) {
			throw new KmlBodyParseException (e);
		} catch (IOException e) {
			throw new KmlBodyParseException (e);
		}		
	}

}
