package org.gvsig.gpe.kml.parser;

import java.util.Hashtable;

import org.gvsig.exceptions.BaseException;
import org.gvsig.gpe.kml.GPEDeafultKmlParser;
import org.gvsig.gpe.kml.KmlTags;
import org.gvsig.gpe.kml.exceptions.KmlImpossibleCreateParser;
import org.xmlpull.v1.XmlPullParser;


/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: KmlParsersFactory.java 366 2008-01-10 09:09:38Z jpiera $
 * $Log$
 * Revision 1.1  2007/05/11 07:06:29  jorpiell
 * Refactoring of some package names
 *
 * Revision 1.3  2007/05/08 09:28:17  jorpiell
 * Add comments to create javadocs
 *
 * Revision 1.2  2007/05/02 11:46:50  jorpiell
 * Writing tests updated
 *
 * Revision 1.1  2007/04/13 13:16:21  jorpiell
 * Add KML reading support
 *
 * Revision 1.2  2007/04/12 10:21:52  jorpiell
 * Add the writers
 *
 * Revision 1.1  2007/03/07 08:19:10  jorpiell
 * Pasadas las clases de KML de libGPE-GML a libGPE-KML
 *
 * Revision 1.1  2007/02/28 11:48:31  csanchez
 * *** empty log message ***
 *
 * Revision 1.1  2007/02/20 10:53:20  jorpiell
 * Añadidos los proyectos de kml y gml antiguos
 *
 * Revision 1.1  2007/02/12 13:49:18  jorpiell
 * Aadido el driver de KML
 *
 *
 */
/**
 * This class is used to register the differnet kml 
 * parsers
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class KmlParsersFactory {	
	private static Hashtable supportedVersions = new Hashtable();	
	
	static {
		supportedVersions.put("http://earth.google.com/kml/2.1",KmlParserV2_1.class);	
		supportedVersions.put(KmlTags.UNKNOWN_VERSION,KmlParserV2_1.class);		
	}
	
	/**
	 * To create a new KML parser
	 * @param nameSpace
	 * KML namespace used to determine the parser to use
	 * @param parser
	 * XML parser used to parse the XML
	 * @param handler
	 * GPE parser tha contains the handlers
	 * @return
	 * A KML parser
	 * @throws BaseException
	 */
	public AbstractKmlParser createParser(String nameSpace,XmlPullParser parser, GPEDeafultKmlParser handler) throws BaseException{
		Class clazz = (Class) supportedVersions.get(nameSpace);
		if (clazz == null){
			clazz = (Class) supportedVersions.get(KmlTags.UNKNOWN_VERSION);
		}
		if (clazz != null){
			Class [] args = {XmlPullParser.class,GPEDeafultKmlParser.class};
			Object [] params = {parser,handler};
			try {
				return (AbstractKmlParser)clazz.getConstructor(args).newInstance(params);
			} catch (Exception e) {
				throw new KmlImpossibleCreateParser(nameSpace,e);
			}
		}
		return null ;		
	}
	
	
}
