package org.gvsig.gpe.kml.parser.v21.geometries;

import java.io.IOException;

import org.gvsig.gpe.kml.GPEDeafultKmlParser;
import org.gvsig.gpe.kml.KmlTags;
import org.gvsig.gpe.warnings.PolygonNotClosedWarning;
import org.kxml2.io.KXmlParser;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id:LinearRingBinding.java 357 2008-01-09 17:50:08Z jpiera $
 * $Log$
 * Revision 1.3  2007/06/07 14:53:59  jorpiell
 * Add the schema support
 *
 * Revision 1.2  2007/05/16 09:30:09  jorpiell
 * the writting methods has to have the errorHandler argument
 *
 * Revision 1.1  2007/05/11 07:06:29  jorpiell
 * Refactoring of some package names
 *
 * Revision 1.3  2007/05/08 08:22:37  jorpiell
 * Add comments to create javadocs
 *
 * Revision 1.2  2007/04/14 16:08:07  jorpiell
 * Kml writing support added
 *
 * Revision 1.1  2007/04/13 13:16:21  jorpiell
 * Add KML reading support
 *
 *
 */
/**
 * It parses the linearRing KML tag. Example:
 * <p>
 * <pre>
 * <code>
 * &lt;LinearRing&gt;
 * &lt;coordinates&gt;0.0,0.0 100.0,0.0 50.0,100.0 0.0,0.0&lt;/coordinates&gt;
 * &lt;/LinearRing&gt;
 * </code>
 * </pre>
 * </p> 
 * @author Jorge Piera LLodr (jorge.piera@iver.es)
 * @see http://code.google.com/apis/kml/documentation/kml_tags_21.html#linearring
 */
public class LinearRingBinding {

	/**
	 * It parses the linearRing tag
	 * @param parser
	 * The XML parser
	 * @param handler
	 * The GPE parser that contains the content handler and
	 * the error handler
	 * @return
	 * It retuns a matrix of doubles with 3 columns (x,y,z) and
	 * one row for each coordinate.
	 * @throws IOException 
	 * @throws XmlPullParserException 
	 * @throws XmlPullParserException
	 * @throws IOException
	 */
	public double[][] parse(XmlPullParser parser,GPEDeafultKmlParser handler) throws XmlPullParserException, IOException {
		boolean endFeature = false;
		int currentTag;
		double[][] coordinates = null;

		String tag = parser.getName();
		currentTag = parser.getEventType();

		while (!endFeature){
			switch(currentTag){
			case KXmlParser.START_TAG:
				if (tag.compareTo(KmlTags.COORDINATES) == 0){
					coordinates =  handler.getProfile().getCoordinatesTypeBinding().parse(parser, handler);
				}
				break;
			case KXmlParser.END_TAG:
				if (tag.compareTo(KmlTags.LINEARRING) == 0){						
					endFeature = true;
				}
				break;
			case KXmlParser.TEXT:					

				break;
			}
			if (!endFeature){					
				currentTag = parser.next();
				tag = parser.getName();
			}
		}	
		//The first and the last coordinates has to be the same
		if ((coordinates[0][0] != coordinates[0][coordinates[0].length - 1]) ||
				(coordinates[1][0] != coordinates[1][coordinates[1].length - 1]) ||
				(coordinates[2][0] != coordinates[2][coordinates[2].length - 1])){
			handler.getErrorHandler().addWarning(new PolygonNotClosedWarning(coordinates));
		}
		return coordinates;
	}
}
