package org.gvsig.gpe.kml.parser.v21.geometries;

import java.io.IOException;

import org.gvsig.gpe.kml.GPEDeafultKmlParser;
import org.gvsig.gpe.kml.KmlTags;
import org.kxml2.io.KXmlParser;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id:MultiGeometryBinding.java 357 2008-01-09 17:50:08Z jpiera $
 * $Log$
 * Revision 1.2  2007/06/07 14:53:59  jorpiell
 * Add the schema support
 *
 * Revision 1.1  2007/05/15 12:37:45  jorpiell
 * Add multyGeometries
 *
 *
 */
/**
 * It parses a MultyGeometry tag. Example:
 * <p>
 * <pre>
 * <code>
 * &lt;MultiGeometry&gt;
 * &lt;Point gid="P6776"&gt;
 * &lt;coord&gt;&lt;X&gt;50.0&lt;/X&gt;&lt;Y&gt;50.0&lt;/Y&gt;&lt;/coord&gt;
 * &lt;/Point&gt;
 * &lt;Polygon gid="_877789"&gt;
 * &lt;outerBoundaryIs&gt;
 * &lt;LinearRing&gt;
 * &lt;coordinates&gt;0.0,0.0 100.0,0.0 50.0,100.0 0.0,0.0&lt;/coordinates&gt;
 * &lt;/LinearRing&gt;
 * &lt;/outerBoundaryIs&gt;
 * &lt;/Polygon&gt;
 * &lt;MultiGeometry&gt;
 * </code>
 * </pre>
 * </p> 
 * @author Jorge Piera LLodr (jorge.piera@iver.es)
 * @see http://code.google.com/apis/kml/documentation/kml_tags_21.html#multigeometry
 */
public class MultiGeometryBinding {
	/**
	 * It parses the MultiGeometry tag
	 * @param parser
	 * The XML parser
	 * @param handler
	 * The GPE parser that contains the content handler and
	 * the error handler
	 * @return
	 * A multigeometry
	 * @throws XmlPullParserException
	 * @throws IOException
	 */
	public Object parse(XmlPullParser parser,GPEDeafultKmlParser handler) throws XmlPullParserException, IOException {
		boolean endFeature = false;
		int currentTag;
		Object multiGeometry = null;		
		
		String id = handler.getProfile().getGeometryBinding().getID(parser, handler);
		
		multiGeometry = handler.getContentHandler().startMultiGeometry(id, KmlTags.DEFAULT_SRS);
		
		String tag = parser.getName();
		currentTag = parser.getEventType();

		while (!endFeature){
			switch(currentTag){
			case KXmlParser.START_TAG:
					if (tag.compareTo(KmlTags.POINT) == 0){
						Object point = handler.getProfile().getPointTypeBinding().parse(parser, handler);
						handler.getContentHandler().addGeometryToMultiGeometry(point, multiGeometry);
					}else if (tag.compareTo(KmlTags.LINESTRING) == 0){
						Object lineString = handler.getProfile().getLineStringTypeBinding().parse(parser, handler);
						handler.getContentHandler().addGeometryToMultiGeometry(lineString, multiGeometry);
					}if (tag.compareTo(KmlTags.POLYGON) == 0){
						Object polygon = handler.getProfile().getPolygonTypeBinding().parse(parser, handler);
						handler.getContentHandler().addGeometryToMultiGeometry(polygon, multiGeometry);
					}
					break;
				case KXmlParser.END_TAG:
					if (tag.compareTo(KmlTags.MULTIGEOMETRY) == 0){						
						endFeature = true;	
						handler.getContentHandler().endMultiGeometry(multiGeometry);
					}
					break;
				case KXmlParser.TEXT:					
					
					break;
				}
				if (!endFeature){					
					currentTag = parser.next();
					tag = parser.getName();
				}
			}			
		return multiGeometry;	
	}
}
