package org.gvsig.gpe.kml.parser.v21.geometries;

import java.io.IOException;

import org.gvsig.gpe.kml.GPEDeafultKmlParser;
import org.gvsig.gpe.kml.KmlTags;
import org.kxml2.io.KXmlParser;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;


/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id:PointTypeBinding.java 357 2008-01-09 17:50:08Z jpiera $
 * $Log$
 * Revision 1.2  2007/06/07 14:53:59  jorpiell
 * Add the schema support
 *
 * Revision 1.1  2007/05/11 07:06:29  jorpiell
 * Refactoring of some package names
 *
 * Revision 1.5  2007/05/08 08:22:37  jorpiell
 * Add comments to create javadocs
 *
 * Revision 1.4  2007/05/02 11:46:50  jorpiell
 * Writing tests updated
 *
 * Revision 1.3  2007/04/20 08:38:59  jorpiell
 * Tests updating
 *
 * Revision 1.2  2007/04/14 16:08:07  jorpiell
 * Kml writing support added
 *
 * Revision 1.1  2007/04/13 13:16:21  jorpiell
 * Add KML reading support
 *
 * Revision 1.1  2007/03/07 08:19:10  jorpiell
 * Pasadas las clases de KML de libGPE-GML a libGPE-KML
 *
 * Revision 1.1  2007/02/28 11:48:31  csanchez
 * *** empty log message ***
 *
 * Revision 1.1  2007/02/20 10:53:20  jorpiell
 * Añadidos los proyectos de kml y gml antiguos
 *
 * Revision 1.1  2007/02/12 13:49:18  jorpiell
 * Aadido el driver de KML
 *
 *
 */
/**
 * It parses a Point tag. Example:
 * <p>
 * <pre>
 * <code>
 * &lt;Point gid="P6776"&gt;
 * &lt;coord&gt;&lt;X&gt;50.0&lt;/X&gt;&lt;Y&gt;50.0&lt;/Y&gt;&lt;/coord&gt;
 * &lt;/Point&gt;
 * </code>
 * </pre>
 * </p> 
 * @author Jorge Piera LLodr (jorge.piera@iver.es)
 * @see http://code.google.com/apis/kml/documentation/kml_tags_21.html#point
 */
public class PointTypeBinding{

	/**
	 * It parses the Point tag
	 * @param parser
	 * The XML parser
	 * @param handler
	 * The GPE parser that contains the content handler and
	 * the error handler
	 * @return
	 * A point
	 * @throws IOException 
	 * @throws XmlPullParserException 
	 * @throws XmlPullParserException 
	 * @throws IOException 
	 */
	public Object parse(XmlPullParser parser,GPEDeafultKmlParser handler) throws XmlPullParserException, IOException {
		boolean endFeature = false;
		int currentTag;
		Object point = null;			
		
		String id = handler.getProfile().getGeometryBinding().getID(parser, handler);
		
		
			String tag = parser.getName();
			currentTag = parser.getEventType();
			
			while (!endFeature){
				switch(currentTag){
				case KXmlParser.START_TAG:
					if (tag.compareTo(KmlTags.COORDINATES) == 0){
						double[][] coordinates = handler.getProfile().getCoordinatesTypeBinding().parse(parser, handler);
						point = handler.getContentHandler().startPoint(id,
									coordinates[0][0],
									coordinates[1][0],
									coordinates[2][0],
									KmlTags.DEFAULT_SRS);
					}
					break;
				case KXmlParser.END_TAG:
					if (tag.compareTo(KmlTags.POINT) == 0){						
						endFeature = true;
						handler.getContentHandler().endPoint(point);
					}
					break;
				case KXmlParser.TEXT:					
					
					break;
				}
				if (!endFeature){					
					currentTag = parser.next();
					tag = parser.getName();
				}
			}			
		
		return point;	
	}



}
