package org.gvsig.gpe.kml.writer.v21.geometries;

import java.io.IOException;

import org.gvsig.gpe.kml.KmlTags;
import org.gvsig.gpe.kml.writer.GPEKmlWriterHandlerImplementor;
import org.gvsig.gpe.xml.writer.Writer;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: LineStringWriter.java 361 2008-01-10 08:41:21Z jpiera $
 * $Log$
 * Revision 1.4  2007/05/16 09:30:09  jorpiell
 * the writting methods has to have the errorHandler argument
 *
 * Revision 1.3  2007/05/08 07:53:08  jorpiell
 * Add comments to the writers
 *
 * Revision 1.2  2007/05/02 11:46:50  jorpiell
 * Writing tests updated
 *
 * Revision 1.1  2007/04/14 16:08:07  jorpiell
 * Kml writing support added
 *
 *
 */
/**
 * It writes the LineString tag. Example:
 * <p>
 * <pre>
 * <code>
 * &lt;LineString&gt;
 * &lt;coord&gt;&lt;X&gt;56.1&lt;/X&gt;&lt;Y&gt;0.45&lt;/Y&gt;&lt;/coord&gt;
 * &lt;coord&gt;&lt;X&gt;67.23&lt;/X&gt;&lt;Y&gt;0.98&lt;/Y&gt;&lt;/coord&gt;
 * &lt;/LineString&gt;
 * </code>
 * </pre>
 * </p> 
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 * @see http://code.google.com/apis/kml/documentation/kml_tags_21.html#linestring
 */
public class LineStringWriter {
		
	/**
	 * It writes the lineString kml init tag
	 * @param writer
	 * Writer to write the labels
	 * @param handler
	 * The writer handler implementor
	 * @param id
	 * LineString id
	 * @param longitudes
	 * Array of longitudes (>= -180 and <= 180)
	 * @param latitudes
	 * Array of latitudes (>= -90 and <= 90)
	 * @param altitudes
	 * Array of altitudes (Meters above sea level)
	 * @throws IOException
	 */
	public void start(Writer writer, GPEKmlWriterHandlerImplementor handler,String id, double[] longitudes, double[] latitudes,
			double[] altitudes) throws IOException{
		handler.getProfile().getGeometryWriter().startGeometry(writer, handler, KmlTags.LINESTRING, id);
		handler.getProfile().getCoordinatesWriter().write(writer, handler, longitudes, latitudes, altitudes);
	}
	
	/**
	 * It writes the lineString kml end tag
	 * @param writer
	 * Writer to write the labels
	 * @param handler
	 * The writer handler implementor
	 * @throws IOException
	 */
	public void end(Writer writer, GPEKmlWriterHandlerImplementor handler) throws IOException{
		handler.getProfile().getGeometryWriter().endGeometry(writer, handler, KmlTags.LINESTRING);
	}
}
