package org.gvsig.gpe;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedHashMap;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: GPEDefaults.java 373 2008-01-10 09:33:05Z jpiera $
 * $Log$
 * Revision 1.10  2007/06/20 09:35:37  jorpiell
 * Add the javadoc comments
 *
 * Revision 1.9  2007/06/07 14:52:28  jorpiell
 * Add the schema support
 *
 * Revision 1.8  2007/05/15 10:39:14  jorpiell
 * Add the number of decimals property
 *
 * Revision 1.7  2007/05/15 09:34:39  jorpiell
 * the tag names cant have blanc spaces
 *
 * Revision 1.6  2007/04/26 14:23:16  jorpiell
 * Add a getStringProperty method
 *
 * Revision 1.5  2007/04/19 11:50:20  csanchez
 * Actualizacion protoripo libGPE
 *
 * Revision 1.4  2007/04/18 11:03:36  jorpiell
 * Add the default schema property
 *
 * Revision 1.3  2007/04/14 16:06:13  jorpiell
 * The writer handler has been updated
 *
 * Revision 1.2  2007/04/12 17:06:42  jorpiell
 * First GML writing tests
 *
 * Revision 1.1  2007/04/12 11:39:20  jorpiell
 * Add the GPEDefaults class
 *
 *
 */
/**
 * This class is used to add the properties that are used
 * by the GPE parsers.
 * @author Jorge Piera LLodr (jorge.piera@iver.es)
 * @author Carlos Snchez Perin (sanchez_carper@gva.es)
 */
public class GPEDefaults {
	private static LinkedHashMap properties = new LinkedHashMap();
	//Decimal sepatarator
	public static final String DECIMAL = "decimal";
	private static final String DECIMAL_VALUE = ".";
	//Coordinates separator. Ex: xSEPARATORy 
	public static final String COORDINATES_SEPARATOR = "coordinatesSeparator";
	private static final String COORDINATES_SEPARATOR_VALUE = ",";
	//Set of tuples separator Ex: x1,y1SEPARATORx2,y2
	public static final String TUPLES_SEPARATOR = "tuplesSeparator";
	private static final String TUPLES_SEPARATOR_VALUE = " ";
	//Number of decimal digits
	public static final String DECIMAL_DIGITS = "decimalDigits";
	private static final Integer DECIMAL_DIGITS_VALUE = new Integer(20);
	//Namespace prefix to create the XML files
	public static final String NAMESPACE_PREFIX = "namespacePrefix";
	private static final String NAMESPACE_PREFIX_VALUE = "cit";
	//Namespace URI to create the XML files
	public static final String NAMESPACE_URI= "namespaceURI";
	private static final String NAMESPACE_URI_VALUE=  "http://www.gvsig.org/cit";
	//Default schema name
	public static final String XSD_SCHEMA_FILE = "schemaName";
	private static final String XSD_SCHEMA_FILE_VALUE = "cit.xsd";
	//XML version = 1.0
	public static final String XML_VERSION = "xmlVersion";
	private static final String XML_VERSION_VALUE =  "1.0";
	//XML encoding (UTF-8) by default
	public static final String XML_ENCODING = "xmlEncoding";
	private static final String XML_ENCODING_VALUE = "UTF-8";
	//Default output file
	public static final String DEFAULT_FILE_NAME = "defaultFileName";
	private static final String DEFAULT_FILE_NAME_VALUE = "output";
	//Default black space symbol
	public static final String DEFAULT_BLANC_SPACE = "defaultBlancSpace";
	private static final String DEFAULT_BLANC_SPACE_VALUE = "BBLLAANNCC";
	//Default black space symbol
	public static final String SRS_BASED_ON_XML  = "srsBasedOnXml";
	private static final Boolean SRS_BASED_ON_XML_VALUE = new Boolean(true);
		
	static{
		properties.put(DECIMAL, DECIMAL_VALUE);
		properties.put(COORDINATES_SEPARATOR, COORDINATES_SEPARATOR_VALUE);
		properties.put(TUPLES_SEPARATOR, TUPLES_SEPARATOR_VALUE);
		properties.put(NAMESPACE_PREFIX, NAMESPACE_PREFIX_VALUE);
		properties.put(NAMESPACE_URI, NAMESPACE_URI_VALUE);
		properties.put(XML_VERSION, XML_VERSION_VALUE);
		properties.put(XML_ENCODING, XML_ENCODING_VALUE);
		properties.put(DEFAULT_FILE_NAME, DEFAULT_FILE_NAME_VALUE);
		properties.put(XSD_SCHEMA_FILE, XSD_SCHEMA_FILE_VALUE);
		properties.put(DEFAULT_BLANC_SPACE, DEFAULT_BLANC_SPACE_VALUE);	
		properties.put(DECIMAL_DIGITS, DECIMAL_DIGITS_VALUE);
		properties.put(SRS_BASED_ON_XML, SRS_BASED_ON_XML_VALUE);
	}
	
	/**
	 * @return the keys used to retrieve all the properties
	 */
	public static Iterator getKeys(){
		return properties.keySet().iterator();
	}
	
	/**
	 * Gets a String property
	 * @param key
	 * Property name
	 * @return
	 */
	public static String getStringProperty(String key){
		Object obj = getProperty(key);
		if (obj == null){
			return null;
		}
		return (String)obj;
	}
	
	/**
	 * Gets a int property
	 * @param key
	 * Property name
	 * @return
	 * The int property or -1
	 */
	public static int getIntPropertyProperty(String key){
		Object obj = getProperty(key);
		if (obj == null){
			return -1;
		}
		if (obj instanceof Integer){
			return ((Integer)obj).intValue();
		}return -1;
	}
	
	/**
	 * Gets a boolean property. If the property doesn't exist
	 * it returns false.
	 * @param key
	 * Property name
	 * @return
	 * The boolean property or false
	 */
	public static boolean getBooleanProperty(String key){
		Object obj = getProperty(key);
		if (obj == null){
			return false;
		}
		if (obj instanceof Boolean){
			return ((Boolean)obj).booleanValue();
		}return false;
	}
	
	/**
	 * Gets a property
	 * @param key
	 * Property name
	 * @return
	 */
	public static Object getProperty(String key){
		return properties.get(key);
	}
	
	/**
	 * Sets a property
	 * @param key
	 * @param value
	 */
	public static void setProperty(String key, Object value){
		properties.put(key, value);
	}
	
	
}
