/*
 * Created on 10-nov-2005
 *
 * gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 * 
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *  
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 * 
 *    or
 * 
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 * 
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.utiles;

/**
 * <p>May be used by objects that want to be serialized with
 * <code>XMLEntity</code>, and to be instantiated after de-serialized.</p>
 * 
 * <p>For example, write a className and the <code>XMLEntity</code> that represents it.
 * Then, when you read the file, read the class, instantiate it
 * and read the <code>XMLEntity</code> after it. Then call <code>setXMLEntity()</code>.
 * Use it for drivers, symbols and so on.</p>
 * 
 * @author fjp
 */
public interface IPersistance {
	/**
	 * Gets the class name that allows create objects via introspection.
	 * 
	 * @return the name of the class that persists
	 */
    String getClassName();
    
    /**
     * <p>Returns an XML entity with all needed information about the class to de-serialize
     *  and create instances of it.</p>
     * 
     * @return the XML entity of the class that persists
     */
    XMLEntity getXMLEntity();
    
    /**
     * <p>Sets an XML entity with all needed information about the class to de-serialize
     *  and create instances of it.</p>
     * 
     * @param xml the XML entity of the class that persists
     */
    void setXMLEntity(XMLEntity xml);
}
