/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Instituto de Desarrollo Regional and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   Instituto de Desarrollo Regional (Universidad de Castilla La-Mancha)
 *   Campus Universitario s/n
 *   02071 Alabacete
 *   Spain
 *
 *   +34 967 599 200
 */

package org.gvsig.crs;

import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import org.cresques.cts.ICoordTrans;
import org.cresques.cts.IProjection;
import org.gvsig.crs.ogr.GetTransepsg;
import org.gvsig.crs.proj.CrsProj;
import org.gvsig.crs.proj.JNIBaseCrs;
import org.gvsig.crs.proj.OperationCrsException;

import com.iver.andami.messages.NotificationManager;

/**
 * Clase que implementa las operaciones de conversin y transformacin 
 * de coordenadas entre CRSs.
 * 
 * @author Miguel Garca Jimnez (garciajimenez.miguel@gmail.com)
 * @author Diego Guerrero Sevilla (diego.guerrero@uclm.es)
 *
 */

public class COperation implements ICOperation {
	private Crs sourceCrs;
	private Crs targetCrs;
	private CrsProj transCrsProj = null;
	private boolean transInTarget;
	
	public COperation(ICrs from, ICrs to) throws CrsException {
		sourceCrs = (Crs) from;
		targetCrs = (Crs) to;
	}

	public ICrs getSource() {
		return sourceCrs;
	}

	public ICrs getTarget() {
		return targetCrs;
	}

	public Point2D operate(Point2D pt) throws CrsException {
		
		if (sourceCrs.getAbrev().equals(targetCrs.getAbrev()) && transCrsProj == null)
			return pt;
		
		double x[] = {pt.getX()};
		double y[] = {pt.getY()};
		double z[] = {0D};
		int errno = 0;
		try {
			if (transCrsProj != null){
				if (transInTarget)
					errno = JNIBaseCrs.operate( x , y, z,sourceCrs.getCrsProj(), transCrsProj);
				else
					errno = JNIBaseCrs.operate( x , y, z,transCrsProj,targetCrs.getCrsProj());
				if (errno != -38)
					return new Point2D.Double(x[0],y[0]);
			}
			
			// Si el punto estaba fuera del mbito del nadgrid operamos sin nadgrid (convertimos)
			x[0] = pt.getX();
			y[0] = pt.getY();
			z[0] = 0D;
			errno = JNIBaseCrs.operate( x , y, z,sourceCrs.getCrsProj(), targetCrs.getCrsProj());
			return new Point2D.Double(x[0],y[0]);
			
		} catch (OperationCrsException e) {
			throw new CrsException(e);
		}
	//	System.out.println("x="+x[0]+"y="+y[0]);
	//	if(!targetCrs.isProjected())
	//		return new Point2D.Double(x[0]*((180)/Math.PI),y[0]*((180)/Math.PI));
	//	else
	}

	public double [] operate(double []ptOrig) throws CrsException {
		
		if (sourceCrs.getAbrev().equals(targetCrs.getAbrev()) && transCrsProj == null)
				return ptOrig;
		
		double x[] = {ptOrig[0]};
		double y[] = {ptOrig[1]};
		double z[] = {ptOrig[2]};
		int errno = 0;
		try {
			if (transCrsProj != null){
				if (transInTarget)
					errno = JNIBaseCrs.operate( x , y, z,sourceCrs.getCrsProj(), transCrsProj);
				else
					errno = JNIBaseCrs.operate( x , y, z,transCrsProj,targetCrs.getCrsProj());
				if (errno != -38){
					double ptDest[] = {x[0], y[0], z[0]};
					return ptDest;
				}
			}
			
			// Si el punto estaba fuera del mbito del nadgrid operamos sin nadgrid (convertimos)
			 x[0] = ptOrig[0];
			 y[0] = ptOrig[1];
			 z[0] = ptOrig[2];
			 JNIBaseCrs.operate(x, y, z,sourceCrs.getCrsProj(), targetCrs.getCrsProj());
			 double ptDest[] = {x[0], y[0], z[0]};
			 return ptDest;
		} catch (OperationCrsException e) {
			throw new CrsException(e);
		}
	}

	public IProjection getPOrig() {
		return null;
	}

	public IProjection getPDest() {
		return null;
	}

	public Point2D convert(Point2D ptOrig, Point2D ptDest) {
		try {
			ptDest = operate(ptOrig);
		} catch (CrsException e) {
			// TODO LWS implementar una gestin completa de los errores.
			NotificationManager.addError(e);
		}
		return ptDest;
	}

	public Rectangle2D convert(Rectangle2D rect) {
		Point2D pt1 = new Point2D.Double(rect.getMinX(), rect.getMinY());
		Point2D pt2 = new Point2D.Double(rect.getMaxX(), rect.getMaxY());
		try {
			pt1 = operate(pt1);
			pt2 = operate(pt2);
		} catch (CrsException e) {
			// TODO LWS implementar una gestin completa de los errores.
			e.printStackTrace();
		}
		rect = new Rectangle2D.Double();
		rect.setFrameFromDiagonal(pt1, pt2);
		return rect;
	}

	public ICoordTrans getInverted() {
		try {
			return new COperation(targetCrs, sourceCrs);
		} catch (CrsException e) {
			e.printStackTrace();
		}
		return null;
	}
	
	public void setNadCrsProj(CrsProj nadCrs, boolean inTarget){
		this.transInTarget = inTarget;
		this.transCrsProj = nadCrs;
	}
}
