/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Instituto de Desarrollo Regional and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   Instituto de Desarrollo Regional (Universidad de Castilla La-Mancha)
 *   Campus Universitario s/n
 *   02071 Alabacete
 *   Spain
 *
 *   +34 967 599 200
 */

package org.gvsig.crs;

import java.util.TreeMap;

import org.cresques.cts.ICRSFactory;
import org.cresques.cts.IProjection;
import org.gvsig.crs.repository.EpsgRepository;
import org.gvsig.crs.repository.EsriRepository;
import org.gvsig.crs.repository.ICrsRepository;
import org.gvsig.crs.repository.Iau2000Repository;

/**
 * Clase que consigue el CRS a travs del cdigo de la EPSG o de
 * la cadena WKT
 * 
 * @author Diego Guerrero Sevilla (diego.guerrero@uclm.es) 
 *
 */

public class CrsFactory implements ICRSFactory {
	static TreeMap data = new TreeMap();
	
	public CrsFactory() {	
	}
	/**
	 * Obtiene un CRS a partir de su cdigo (p.e. EPSG:23030).
	 * @param code
	 * @return
	 * @throws CrsException
	 */
	public ICrs getCRS(String code) throws CrsException {
		
		/*if (data.containsKey(code))	     // EL CRS no esta identificado univocamente por su cdigo,
			return (Crs) data.get(code);*/  // tambien lo caracteriza la informacion sobre transformaciones.
		
		Crs crs = null;
		String repoId = "";
		String crsCode = "";
		String params = "";
		
		if(code.indexOf(":", code.indexOf(":")+1)<0){
			repoId = code.substring(0, code.indexOf(":"));
			crsCode = code.substring(code.indexOf(":")+1);
		
			ICrsRepository repo = null;
			crs = null;
			
			if(repoId.equals("EPSG")){
				repo = new EpsgRepository();
				crs = (Crs)repo.getCrs(crsCode);
			}else if (repoId.equals("IAU2000")){
				repo = new Iau2000Repository();
				crs = (Crs)repo.getCrs(crsCode);
			}else if (repoId.equals("ESRI")){
				repo = new EsriRepository();
				crs = (Crs)repo.getCrs(crsCode);
			}
		}
		
		/*
		 * En el caso de que code incluya tambin parmetros de transformacin.
		 */
		else{ 
			crsCode = code.substring(0,code.indexOf(":",code.indexOf(":")+1));
			params = code.substring(code.indexOf("@")+1,code.lastIndexOf("@"));
			int inTarget = Integer.parseInt(code.substring(code.lastIndexOf("@")+1));
			
			crs = (Crs)getCRS(crsCode);
			crs.setTransParam(params);
			if(inTarget == 1)
				crs.setTransInTarget(true);
			else
				crs.setTransInTarget(false);
				
			//crs = new Crs(Integer.parseInt(crsCode.substring(crsCode.indexOf(":")+1)), crs.getWKT(), params);
		}
	
		// LWS Esta lnea sobra, cuando el cuadro de dilogo est
		// mejor hecho.
		/*code = crs.getAbrev();
		
		data.put(code, crs);*/

		return crs;
	}
	
	/**
	 * 
	 * @param epsg_code
	 * @param code
	 * @return
	 * @throws CrsException
	 */
	public ICrs getCRS(int epsg_code, String code) throws CrsException {
		
		/*if (data.containsKey(code))		 // EL CRS no esta identificado univocamente por su cdigo,
			return (Crs) data.get(code);*/	// tambien lo caracteriza la informacion sobre transformaciones.

		Crs crs = new Crs(epsg_code, code);
		
		// LWS Esta lnea sobra, cuando el cuadro de dilogo est
		// mejor hecho.
		/*code = crs.getAbrev();
		
		data.put(code, crs);*/

		return crs;
	}
	
	/**
	 * 
	 */
	public IProjection get(String name) {
		// TODO Auto-generated method stub
		try {
			return getCRS(name);
		} catch (CrsException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}
	
	/**
	 * 
	 */
	public boolean doesRigurousTransformations() {
		return true;
	}
}
