/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Instituto de Desarrollo Regional and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   Instituto de Desarrollo Regional (Universidad de Castilla La-Mancha)
 *   Campus Universitario s/n
 *   02071 Alabacete
 *   Spain
 *
 *   +34 967 599 200
 */

package org.gvsig.crs;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;

/**
 * Clase para la conexin con la base de datos de hsqldb.
 * Establece el driver necesario, as como la cadena de
 * conexin a la base de datos de la EPSG y la IAU2000
 * 
 * @author Jos Luis Gmez Martnez (jolugomar@gmail.com)
 *
 */

public class EpsgConnection {
	
	Connection connect;

	public EpsgConnection() {	
		try {
			Class.forName("org.hsqldb.jdbcDriver");
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Establece la conexin con la base de datos de la EPSG
	 *
	 */
	public void setConnectionEPSG() {
		try {			
			connect =  DriverManager.getConnection("jdbc:hsqldb:file:gvSIG/extensiones/org.gvsig.crs/db_epsg", "sa", "");
		} catch (SQLException e1) {
			e1.printStackTrace();
		}
	}
	
	/**
	 * Establece la conexin con la base de datos de la IAU2000
	 *
	 */
	public void setConnectionIAU2000() {
		try {			
			connect =  DriverManager.getConnection("jdbc:hsqldb:file:gvSIG/extensiones/org.gvsig.crs/db_iau2000", "sa", "");			
		} catch (SQLException e1) {
			e1.printStackTrace();
		}
	}
	
	/**
	 * Establece la conexin con la base de datos de ESRI
	 *
	 */
	public void setConnectionEsri() {
		try {			
			connect =  DriverManager.getConnection("jdbc:hsqldb:file:gvSIG/extensiones/org.gvsig.crs/db_esri", "sa", "");			
		} catch (SQLException e1) {
			e1.printStackTrace();
		}
	}
	
	public Connection getConnection(){
		return connect;
	}
	
	public void shutdown() throws SQLException {

        Statement st = connect.createStatement();

        // db writes out to files and performs clean shuts down
        // otherwise there will be an unclean shutdown
        // when program ends
        st.execute("SHUTDOWN");
        connect.close();    // if there are no other open connection
    }
}
