/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Instituto de Desarrollo Regional and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   Instituto de Desarrollo Regional (Universidad de Castilla La-Mancha)
 *   Campus Universitario s/n
 *   02071 Alabacete
 *   Spain
 *
 *   +34 967 599 200
 */

package org.gvsig.crs;

import org.cresques.cts.IProjection;


/**
 * Interface para un Sistema de Referencuia Coordenado:
 *  CRS (Coordinate Reference Sysytem).
 * @author Jos Luis Gmez Martnez (jolugomar@gmail.com)
 * @author Diego Guerrero Sevilla (diego.guerrero@uclm.es)
 *
 */
public interface ICrs extends IProjection {
	/**
	 * Abreviatura del CRS
	 * @return
	 */
	String getAbrev();
	
	/**
	 * Cdigo del CRS
	 * @return
	 */
	int getCode();
	//public boolean eq(ICrs other);
	
	/**
	 * Cadena WKT del CRS utilizado
	 * @return
	 */
	String getWKT();
	
	/**
	 * Mtodo para la aplicacin de la propiedad de nadgrids
	 * sobre el CRS destino o fuente 
	 * @param param
	 */
	void setTransParam(String param);
	
	/**
	 * Cadena nadgrids a utilizar en el crs fuente o destino en una transformacion
	 * @return
	 */
	String getTransParam();
	
	/**
	 * true si el nadgrid se asocia al crs destino
	 * @return
	 */
	boolean isTransInTarget();

	/**
	 * Le indicamos al crs si el nadgrid est asociado al crs destion al fuente
	 * en una transformacin.
	 * @param targetNad true si el nadgrid se asocia al crs destino
	 * @return
	 */
	void setTransInTarget(boolean targetNad);
	
	/**
	 * Nombre de la transformacion a realizar con el CRS
	 * @return
	 */	
	void setTrans(String code);
	
	/**
	 * Campos de la cadena wkt
	 * @return
	 */
	CrsWkt getCrsWkt();
	
}
