/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Instituto de Desarrollo Regional and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   Instituto de Desarrollo Regional (Universidad de Castilla La-Mancha)
 *   Campus Universitario s/n
 *   02071 Alabacete
 *   Spain
 *
 *   +34 967 599 200
 */


package org.gvsig.crs.gui.listeners;

import java.awt.CardLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.sql.ResultSet;
import java.sql.SQLException;

import javax.swing.JComboBox;
import javax.swing.JOptionPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.gvsig.crs.CrsWkt;
import org.gvsig.crs.ICrs;
import org.gvsig.crs.Query;
import org.gvsig.crs.gui.CRSMainTrPanel;
import org.gvsig.crs.gui.panels.InfoTransformationsRecentsPanel;
import org.gvsig.crs.ogr.GetTransepsg;
import org.gvsig.crs.persistence.CrsData;
import org.gvsig.crs.persistence.RecentCRSsPersistence;
import org.gvsig.crs.persistence.RecentTrsPersistence;
import org.gvsig.crs.persistence.TrData;

import com.iver.andami.PluginServices;
import com.iver.andami.ui.mdiManager.IWindow;
import com.iver.cit.gvsig.project.documents.view.gui.BaseView;

/**
 * Esta clase contiene todos los listeners necesarios para el manejo 
 * de los eventos del panel de eleccin de CRS de la capa y de su
 * tranformacin. 
 * 
 * @author Diego Guerrero Sevilla (diego.guerrero@uclm.es) 
 * @author Jos Luis Gmez Martnez (jolugomar@gmail.com)
 *
 */

public class CRSMainTrPanelListener implements ActionListener, 
			ListSelectionListener, ItemListener, MouseListener, KeyListener{

	private CRSMainTrPanel panel = null;	
	
	public CRSMainTrPanelListener(CRSMainTrPanel p) {
		panel=p;	
	}
	
	/**
	 * Manejador de los eventos relacionados con los botones del panel
	 * de transformacin y al combobox de las transformaciones
	 */
	public void actionPerformed(ActionEvent e) {
		if(	e.getSource() == panel.getJButtonAccept()){
			if (panel.isEpsg_tr()) {			
				GetTransepsg epsgParams = new GetTransepsg(panel.getEpsgTrPanel().getTransformation_code(),panel.getEpsgTrPanel().connect, panel.getEpsgTrPanel().inverseTranformation);
				panel.getEpsgTrPanel().setValues(epsgParams.getParamValue());	
				panel.setProjection(panel.getEpsgTrPanel().getProjection());
				panel.setEpsg_tr(false);
				
				/*
				 * Actualizar Transformaciones recientes
				 */
				String authority = "EPSG";
	    		String name = (String)panel.getEpsgTrPanel().sorter.getValueAt(panel.getEpsgTrPanel().selectedRowTable,1);
	    		int code = panel.getEpsgTrPanel().getTransformation_code();
	    		String crsSource = "EPSG:"+String.valueOf(panel.getEpsgTrPanel().getSource()); 	    			
	    		String crsTarget = "EPSG:"+String.valueOf(panel.getEpsgTrPanel().getTarget()); 	    			
	    		String details = (String)panel.getEpsgTrPanel().sorter.getValueAt(panel.getEpsgTrPanel().selectedRowTable,5);
	    		TrData trData = new TrData(authority,code,name,crsSource,crsTarget,details);
	    		RecentTrsPersistence trPersistence = new RecentTrsPersistence(RecentCRSsPersistence.pluginClassInstance);
	    		trPersistence.addTrData(trData);
				
	    		
			}
			else if (panel.isManual_tr()) {
				boolean domain = panel.getManualTrPanel().correctJTextField();
				if (domain && panel.getManualTrPanel().correctDomain()){
					panel.setProjection(panel.getManualTrPanel().getProjection());
					panel.setManual_tr(false);
				}
				else if (!domain) {
					JOptionPane.showMessageDialog(panel, 
							PluginServices.getText(this,"numeric_format"), 
							"Warning...", JOptionPane.WARNING_MESSAGE);					
					return;
				}				
				else {
					JOptionPane.showMessageDialog(panel, 
							PluginServices.getText(this,"incorrect_domain"), 
							"Warning...", JOptionPane.WARNING_MESSAGE);					
					return;
				}
				
				/*
				 * Actualizar Transformaciones recientes
				 */
				String authority = "USR";
	    		String name = PluginServices.getText(this,"manual");
	    		int code = 0;
	    		String crsSource = panel.getManualTrPanel().getSourceAbrev();
	    		String crsTarget = panel.getManualTrPanel().getTargetAbrev();
	    		String details = panel.getManualTrPanel().getValues();
	    		 
	    		TrData trData = new TrData(authority,code,name,crsSource,crsTarget,details);
	    		RecentTrsPersistence trPersistence = new RecentTrsPersistence(RecentCRSsPersistence.pluginClassInstance);
	    		trPersistence.addTrData(trData);
				
			}
			else if (panel.isNads_tr()) {
				panel.setProjection(panel.getNadsTrPanel().getProjection());
				panel.setTargetNad(panel.getNadsTrPanel().getNad());
				panel.setNads_tr(false);
				
				/*
				 * Actualizar Transformaciones recientes
				 */
				String authority = "NADGR";
	    		String name = "----";
	    		int code = 0;
	    		String crsSource = panel.getNadsTrPanel().getSourceAbrev();
	    		String crsTarget = panel.getNadsTrPanel().getTargetAbrev();
	    		String details = "";
	    		if (panel.getNadsTrPanel().getNad())
	    			details = panel.getNadsTrPanel().getNadFile()+ " ("+panel.getNadsTrPanel().getTargetAbrev() +")";
	    		else
	    			details = panel.getNadsTrPanel().getNadFile()+ "("+panel.getNadsTrPanel().getSourceAbrev() +")"; 
	    		TrData trData = new TrData(authority,code,name,crsSource,crsTarget,details);
	    		RecentTrsPersistence trPersistence = new RecentTrsPersistence(RecentCRSsPersistence.pluginClassInstance);
	    		trPersistence.addTrData(trData);
			}
			/*
			 * Ver que es necesario cuando aceptas en el panel de transformaciones
			 * recientes.
			 */
			else if (panel.isRecents_tr()) {
				String[] transformation = ((String)panel.getRecentsTrPanel().sorter.getValueAt(panel.getRecentsTrPanel().selectedRowTable,0)).split(":");
				if (transformation[0].equals("USR")){
	        		panel.getRecentsTrPanel().setParamsManual((String)panel.getRecentsTrPanel().sorter.getValueAt(panel.getRecentsTrPanel().selectedRowTable,4));
	        		panel.setProjection(panel.getRecentsTrPanel().getProjection());
	        	}
				if (transformation[0].equals("EPSG")){
					GetTransepsg epsgParams = new GetTransepsg(panel.getRecentsTrPanel().getTrCode(),panel.getEpsgTrPanel().connect, panel.getRecentsTrPanel().getInverseTransformation());
					panel.getRecentsTrPanel().setParamsEPGS(epsgParams.getParamValue());
					panel.setProjection(panel.getRecentsTrPanel().getProjection());
	        	}
				if (transformation[0].equals("NADGR")){
					panel.setProjection(panel.getRecentsTrPanel().getProjectionNad((String)panel.getRecentsTrPanel().sorter.getValueAt(panel.getRecentsTrPanel().selectedRowTable,4)));
				}
				//panel.setProjection(panel.getRecentsTrPanel().getProjection());
				panel.setRecents_tr(false);
				
				/*
				 * Actualizar Transformaciones recientes
				 */
				String authCode = (String)panel.getRecentsTrPanel().sorter.getValueAt(panel.getRecentsTrPanel().selectedRowTable,0);
				String authority = authCode.split(":")[0];
	    		String name = (String)panel.getRecentsTrPanel().sorter.getValueAt(panel.getRecentsTrPanel().selectedRowTable,1);
	    		int code = Integer.parseInt(authCode.split(":")[1]);
	    		String crsSource = (String)panel.getRecentsTrPanel().sorter.getValueAt(panel.getRecentsTrPanel().selectedRowTable,2);
	    		String crsTarget = (String)panel.getRecentsTrPanel().sorter.getValueAt(panel.getRecentsTrPanel().selectedRowTable,3);
	    		String details = (String)panel.getRecentsTrPanel().sorter.getValueAt(panel.getRecentsTrPanel().selectedRowTable,4);	    		
	    		TrData trData = new TrData(authority,code,name,crsSource,crsTarget,details);
	    		RecentTrsPersistence trPersistence = new RecentTrsPersistence(RecentCRSsPersistence.pluginClassInstance);
	    		trPersistence.addTrData(trData);
			}
			else{
				panel.setSin_tr(true);
				panel.setProjection(panel.getProjection());							
			}
			
			/*
    		 * Actualizar recientes...
    		 */
    		String authority = ((ICrs)panel.getProjection()).getCrsWkt().getAuthority()[0];
    		String name = ((ICrs)panel.getProjection()).getCrsWkt().getName();
    		int code = ((ICrs)panel.getProjection()).getCode();
    		CrsData crsData = new CrsData(authority,code,name);
    		RecentCRSsPersistence persistence = new RecentCRSsPersistence(RecentCRSsPersistence.pluginClassInstance);
    		persistence.addCrsData(crsData);
			
			PluginServices.getMDIManager().closeWindow(panel);
		}
		
		if(	e.getSource() == panel.getJButtonBefore()){
			CardLayout cl = (CardLayout)(panel.getJPanelMain().getLayout());
		    cl.show(panel.getJPanelMain(), "primero");
		    panel.setEpsg_tr(false);
        	panel.setNads_tr(false);
        	panel.setManual_tr(false);
        	panel.setRecents_tr(false);
        	panel.getJButtonNext().setVisible(true);
		    panel.getJButtonAccept().setVisible(false);
		    panel.getJButtonAccept().setEnabled(true);
		    panel.getJButtonBefore().setEnabled(false);
		}
		
		if(	e.getSource() == panel.getJButtonCancel()){
			panel.setCancelYes(true);
			panel.setProjection(panel.getCrsfirst());
			PluginServices.getMDIManager().closeWindow(panel);
		}

		if(	e.getSource() == panel.getJButtonNext()){
			panel.getJButtonNext().setVisible(false);
			panel.getJButtonAccept().setVisible(true);
			panel.getJButtonBefore().setEnabled(true);
			if (panel.getNewSelection().equals(PluginServices.getText(this,"transformacion_manual"))) {
				CardLayout cl = (CardLayout)(panel.getJPanelMain().getLayout());
				if (panel.getDataSource().equals(PluginServices.getText(this,"EPSG"))){
					panel.getManualTrPanel().setWKT(panel.getCrsMainPanel().getEpsgPanel().getWKT());
					panel.getManualTrPanel().setCode(panel.getCrsMainPanel().getEpsgPanel().epsg_code);
					panel.getManualTrPanel().setTargetAuthority(panel.getCrsWkt_target().getAuthority());
				}
				else if (panel.getDataSource().equals(PluginServices.getText(this,"recientes"))) {
					panel.getManualTrPanel().setWKT(panel.getCrsMainPanel().getRecentsPanel().getCrs().getWKT());
					panel.getManualTrPanel().setCode(panel.getCrsMainPanel().getRecentsPanel().getCodeCRS());
					panel.getManualTrPanel().setTargetAuthority(panel.getCrsWkt_target().getAuthority());
				}
				if(!panel.getManualTrPanel().getStatus()) {
					panel.getJButtonAccept().setEnabled(false);
				}
				panel.setManual_tr(true);
			    cl.show(panel.getJPanelMain(), "manual");		    
			}		
			else if(panel.getNewSelection().equals(PluginServices.getText(this,"nadgrids"))){
				CardLayout cl = (CardLayout)(panel.getJPanelMain().getLayout());
				if (panel.getDataSource().equals(PluginServices.getText(this,"EPSG"))){
					panel.getNadsTrPanel().setWKT(panel.getCrsMainPanel().getEpsgPanel().getWKT());
					panel.getNadsTrPanel().setCode(panel.getCrsMainPanel().getEpsgPanel().epsg_code);
					panel.getNadsTrPanel().setTargetAuthority(panel.getCrsWkt_target().getAuthority());
				}
				else if (panel.getDataSource().equals(PluginServices.getText(this,"recientes"))) {
					panel.getNadsTrPanel().setWKT(panel.getCrsMainPanel().getRecentsPanel().getCrs().getWKT());
					panel.getNadsTrPanel().setCode(panel.getCrsMainPanel().getRecentsPanel().getCodeCRS());
					panel.getNadsTrPanel().setTargetAuthority(panel.getCrsWkt_target().getAuthority());
				}
				if (panel.getNadsTrPanel().getJComboNadFile().getSelectedIndex() == 0)
					panel.getJButtonAccept().setEnabled(false);
				else panel.getJButtonAccept().setEnabled(true);
				panel.setNads_tr(true);
			    cl.show(panel.getJPanelMain(), "nad");		    
			}
			else if(panel.getNewSelection().equals(PluginServices.getText(this,"transformacion_epsg"))){
				CardLayout cl = (CardLayout)(panel.getJPanelMain().getLayout());
				if (panel.getDataSource().equals(PluginServices.getText(this,"EPSG"))){
					panel.getEpsgTrPanel().setWKT(panel.getCrsMainPanel().getEpsgPanel().getWKT());
					panel.getEpsgTrPanel().setSource(panel.getCrsMainPanel().getEpsgPanel().epsg_code);
				}
				else if (panel.getDataSource().equals(PluginServices.getText(this,"recientes"))) {
					panel.getEpsgTrPanel().setWKT(panel.getCrsMainPanel().getRecentsPanel().getCrs().getWKT());
					panel.getEpsgTrPanel().setSource(panel.getCrsMainPanel().getRecentsPanel().getCodeCRS());
				}
				int numr = panel.getEpsgTrPanel().dtm.getRowCount();
				if (numr == 0 )
					panel.getJButtonAccept().setEnabled(false);
				panel.setEpsg_tr(true);				
			    cl.show(panel.getJPanelMain(), "epsg");		    			
			}
			/*
			 * Parte necesaria para la transformacion reciente, cuando se vaya a cargar
			 * el panel, tendremos que ver que hay que pasarle.
			 */
			else if(panel.getNewSelection().equals(PluginServices.getText(this, "recents_transformation"))) {
				CardLayout cl = (CardLayout)(panel.getJPanelMain().getLayout());
				if (panel.getDataSource().equals(PluginServices.getText(this,"EPSG"))){
					panel.getRecentsTrPanel().setWKT(panel.getCrsMainPanel().getEpsgPanel().getWKT());
					panel.getRecentsTrPanel().loadRecents(panel.getCrsMainPanel().getEpsgPanel().getCodeCRS(),Integer.parseInt(panel.getCrsWkt_target().getAuthority()[1]));
					//panel.getRecentsTrPanel().setTargetAuthority(panel.getCrsWkt_target().getAuthority());
				}
				else if (panel.getDataSource().equals(PluginServices.getText(this,"recientes"))) {
					panel.getRecentsTrPanel().setWKT(panel.getCrsMainPanel().getRecentsPanel().getCrs().getWKT());
					panel.getRecentsTrPanel().loadRecents(panel.getCrsMainPanel().getRecentsPanel().getCodeCRS(),Integer.parseInt(panel.getCrsWkt_target().getAuthority()[1]));
					//panel.getRecentsTrPanel().setTargetAuthority(panel.getCrsWkt_target().getAuthority());
				}
				
				int numr = panel.getRecentsTrPanel().dtm.getRowCount();
				if (numr == 0 )
					panel.getJButtonAccept().setEnabled(false);
				panel.setRecents_tr(true);				
			    cl.show(panel.getJPanelMain(), "recents");
			}
		}	
		if(	e.getSource() == panel.getJComboOptions()){
			JComboBox cb = (JComboBox)e.getSource();
			panel.setNewSelection((String)cb.getSelectedItem());
		    if (panel.getNewSelection().equals(PluginServices.getText(this,"sin_transformacion"))){
				panel.getJButtonAccept().setVisible(true);
				panel.getJButtonNext().setVisible(false);					
			}	
		    else {
		    	panel.getJButtonAccept().setVisible(false);
		    	panel.getJButtonNext().setVisible(true);
		    }
		}
		if(	e.getSource() == panel.getRecentsTrPanel().getJButtonInfo()){
			String[] data = new String[5];
			data[0] = (String)panel.getRecentsTrPanel().sorter.getValueAt(panel.getRecentsTrPanel().selectedRowTable,0);
			data[1] = (String)panel.getRecentsTrPanel().sorter.getValueAt(panel.getRecentsTrPanel().selectedRowTable,1);
			data[2] = (String)panel.getRecentsTrPanel().sorter.getValueAt(panel.getRecentsTrPanel().selectedRowTable,2);
			data[3] = (String)panel.getRecentsTrPanel().sorter.getValueAt(panel.getRecentsTrPanel().selectedRowTable,3);
			data[4] = (String)panel.getRecentsTrPanel().sorter.getValueAt(panel.getRecentsTrPanel().selectedRowTable,4);
			
			InfoTransformationsRecentsPanel info = new InfoTransformationsRecentsPanel(data);
			info.setSize(new Dimension(450,200));
			info.setLayout(new GridLayout(0,1));
			info.setLayout(new FlowLayout(FlowLayout.LEFT,10,5));
			PluginServices.getMDIManager().addWindow(info);
		}
	}

	/**
	 * Manejador de los eventos relacionados al cambio de valor
	 * dentro de las tablas del panel de tranformaciones.
	 */
	public void valueChanged(ListSelectionEvent e) {
		if (e.getSource() == panel.getCrsMainPanel().getRecentsPanel().getJTable().getSelectionModel()){
			ListSelectionModel lsm = (ListSelectionModel)e.getSource();
	        if (lsm.isSelectionEmpty()) {
	        	panel.getCrsMainPanel().getRecentsPanel().selectedRowTable = -1;
	        	panel.getCrsMainPanel().getRecentsPanel().setCodeCRS(-1);
	        	panel.getJButtonAccept().setEnabled(false);		        	
	        	panel.getJComboOptions().setEnabled(false); 
	        	panel.getJButtonNext().setVisible(false);
	        	panel.getJComboOptions().setSelectedIndex(0);	        	
	        } 
	        else {        	
	        	panel.getCrsMainPanel().getRecentsPanel().initCrs();		        			        	
	        	String sour = (String)panel.getCrsMainPanel().getRecentsPanel().sorter.getValueAt(panel.getCrsMainPanel().getRecentsPanel().selectedRowTable,0);
	        	IWindow activeWindow = PluginServices.getMDIManager().getActiveWindow();
		    	BaseView activeView = (BaseView) activeWindow;
		    	String authority = ((ICrs) activeView.getMapControl().getProjection()).getCrsWkt().getAuthority()[0];
	        	if (sour.equals("EPSG") && authority.equals("EPSG")){
	        		//panel.compareDatum(panel.getCrsMainPanel().getRecentsPanel().getCrs().getWKT());
	        		panel.getJComboOptions().setEnabled(true);
	        		panel.getJButtonAccept().setEnabled(true);
	        	}
	        	else {
	        		panel.getJComboOptions().setSelectedIndex(0);	        		
	        		panel.getJComboOptions().setEnabled(false);
	        		panel.getJButtonAccept().setEnabled(true);
	        		panel.getJButtonAccept().setVisible(true);	        			        		
	        		panel.getJButtonNext().setVisible(false);
	        	}
	        }
		}
		
		if (e.getSource() == panel.getCrsMainPanel().getEpsgPanel().getJTable().getSelectionModel()){
			String[] not_soported = {"engineering", "vertical", "compound", "geocentric", "geographic 3D"};
			int[] valid_method_code = {9800, 9801, 9802, 9803, 9804, 9805, 9806, 9807, 9808, 9809, 9810, 
					9811, 9812, 9813, 9814, 9815, 9816, 9817, 9602, 9659, 9818, 9819, 9820, 9822, 
					9827};
			boolean soported = true;
			boolean soported2 = false;
			ListSelectionModel lsm = (ListSelectionModel)e.getSource();
			if (lsm.isSelectionEmpty()) {
				panel.getCrsMainPanel().getEpsgPanel().selectedRowTable = -1;
				panel.getCrsMainPanel().getEpsgPanel().setCodeCRS(-1);
	        	panel.getJButtonAccept().setEnabled(false);
	        	panel.getCrsMainPanel().getEpsgPanel().getInfoCrs().setEnabled(false);
	        	panel.getJComboOptions().setEnabled(false);
	        	panel.getJButtonNext().setVisible(false);
	        	panel.getJComboOptions().setSelectedIndex(0);	        
			}
			else {
				panel.getCrsMainPanel().getEpsgPanel().selectedRowTable = lsm.getMinSelectionIndex();	        	
	        	String crs_kind = (String)panel.getCrsMainPanel().getEpsgPanel().sorter.getValueAt(panel.getCrsMainPanel().getEpsgPanel().selectedRowTable,2);
	        	for (int i = 0; i < not_soported.length; i++) {
	        		if (crs_kind.equals(not_soported[i])) 
	        			soported = false;	        		
	        	}
	        	for (int i=0; i< valid_method_code.length;i++) {
	        		if (panel.getCrsMainPanel().getEpsgPanel().getProjectionCode((String)panel.getCrsMainPanel().getEpsgPanel().sorter.getValueAt(panel.getCrsMainPanel().getEpsgPanel().selectedRowTable,0)) == valid_method_code[i]) {
	        			soported2 = true;
	        		}
	        	}
	        	if (crs_kind.equals("geographic 2D")) {
	        		soported2 = true;
	        	}
	        	if (soported && soported2){
	        		panel.getCrsMainPanel().getEpsgPanel().setCodeCRS(Integer.parseInt((String)panel.getCrsMainPanel().getEpsgPanel().sorter.getValueAt(panel.getCrsMainPanel().getEpsgPanel().selectedRowTable,0)));
	        		panel.getCrsMainPanel().getEpsgPanel().setWKT();        		
	        		panel.getJButtonAccept().setEnabled(true);
	            	int base = panel.getCrsMainPanel().getEpsgPanel().getCodeCRS();						
					//panel.compareDatum(panel.getCrsMainPanel().getEpsgPanel().getWKT());
	            	IWindow activeWindow = PluginServices.getMDIManager().getActiveWindow();
			    	BaseView activeView = (BaseView) activeWindow;
			    	String authority = ((ICrs) activeView.getMapControl().getProjection()).getCrsWkt().getAuthority()[0];
	            	if (authority.equals("EPSG")){
	            		panel.getJComboOptions().setEnabled(true);
	            		panel.getJButtonAccept().setEnabled(true);
	            	}
	            	else {
		        		panel.getJComboOptions().setSelectedIndex(0);	        		
		        		panel.getJComboOptions().setEnabled(false);
		        		panel.getJButtonAccept().setEnabled(true);
		        		panel.getJButtonAccept().setVisible(true);	        			        		
		        		panel.getJButtonNext().setVisible(false);
		        	}	        		
	        		panel.getCrsMainPanel().getEpsgPanel().getInfoCrs().setEnabled(true);
	        	}
	        	else {
	        		JOptionPane.showMessageDialog(panel, PluginServices.getText(this,"crs_not_soported"), "Warning...",
							JOptionPane.WARNING_MESSAGE);
	        		panel.getCrsMainPanel().getEpsgPanel().setCodeCRS(-1);
	        		panel.getCrsMainPanel().getEpsgPanel().setCodeCRS(0);
	        		panel.getJButtonAccept().setEnabled(false);
	        		panel.getCrsMainPanel().getEpsgPanel().getInfoCrs().setEnabled(false);
	        	}	        				        	  
		    } 
		}
		
		if (e.getSource() == panel.getCrsMainPanel().getIauPanel().getJTable().getSelectionModel()){
			ListSelectionModel lsm = (ListSelectionModel)e.getSource();				
			String[] not_soported = {"Oblique_Cylindrical_Equal_Area"};
			boolean soported = true;
	        if (lsm.isSelectionEmpty()) {
	        	panel.getCrsMainPanel().getIauPanel().selectedRowTable = -1;
	        	panel.getCrsMainPanel().getIauPanel().setCodeCRS(-1);
	        	panel.getJButtonAccept().setEnabled(false);		        	
	        	panel.getJComboOptions().setEnabled(false);     
	        	panel.getJButtonNext().setVisible(false);
	        	panel.getJComboOptions().setSelectedIndex(0);	        	
	        } 
	        else {        	
	        	panel.getCrsMainPanel().getIauPanel().selectedRowTable = lsm.getMinSelectionIndex();
	        	panel.getCrsMainPanel().getIauPanel().setCodeCRS(Integer.parseInt((String)panel.getCrsMainPanel().getIauPanel().sorter.getValueAt(panel.getCrsMainPanel().getIauPanel().selectedRowTable,0)));
	        	panel.getCrsMainPanel().getIauPanel().setWKT();
	        	CrsWkt crs = new CrsWkt(panel.getCrsMainPanel().getIauPanel().getWKT());
	        	String crs_kind = crs.getProjection();
	        	for (int i = 0; i < not_soported.length; i++) {
	        		if (crs_kind.equals(not_soported[i]))
	        			soported = false;	        		
	        	}
	        	if (soported){
	        		panel.getJComboOptions().setEnabled(false);
	        		panel.getJButtonAccept().setEnabled(true);	
	        	}
	        	else {
	        		JOptionPane.showMessageDialog(panel, PluginServices.getText(this,"crs_not_soported"), "Warning...",
							JOptionPane.WARNING_MESSAGE);
	        		panel.getCrsMainPanel().getIauPanel().setCodeCRS(-1);
	        		panel.getCrsMainPanel().getIauPanel().setCodeCRS(0);
	        		panel.getJButtonAccept().setEnabled(false);	        		
	        	}	        		            	
	        }
		}
		
		if (e.getSource() == panel.getCrsMainPanel().getEsriPanel().getJTable().getSelectionModel()){
			ListSelectionModel lsm = (ListSelectionModel)e.getSource();				
			String[] not_soported = {"Oblique_Cylindrical_Equal_Area"};
			boolean soported = true;
	        if (lsm.isSelectionEmpty()) {
	        	panel.getCrsMainPanel().getEsriPanel().selectedRowTable = -1;
	        	panel.getCrsMainPanel().getEsriPanel().setCodeCRS(-1);
	        	panel.getJButtonAccept().setEnabled(false);		        	
	        	panel.getJComboOptions().setEnabled(false);     
	        	panel.getJButtonNext().setVisible(false);
	        	panel.getJComboOptions().setSelectedIndex(0);	        	
	        } 
	        else {        	
	        	panel.getCrsMainPanel().getEsriPanel().selectedRowTable = lsm.getMinSelectionIndex();
	        	panel.getCrsMainPanel().getEsriPanel().setCodeCRS(Integer.parseInt((String)panel.getCrsMainPanel().getEsriPanel().sorter.getValueAt(panel.getCrsMainPanel().getEsriPanel().selectedRowTable,0)));
	        	panel.getCrsMainPanel().getEsriPanel().setWKT();
	        	CrsWkt crs = new CrsWkt(panel.getCrsMainPanel().getEsriPanel().getWKT());
	        	String crs_kind = crs.getProjection();
	        	for (int i = 0; i < not_soported.length; i++) {
	        		if (crs_kind.equals(not_soported[i]))
	        			soported = false;	        		
	        	}
	        	if (soported){
	        		panel.getJComboOptions().setEnabled(false);
	        		panel.getJButtonAccept().setEnabled(true);	
	        	}
	        	else {
	        		JOptionPane.showMessageDialog(panel, PluginServices.getText(this,"crs_not_soported"), "Warning...",
							JOptionPane.WARNING_MESSAGE);
	        		panel.getCrsMainPanel().getEsriPanel().setCodeCRS(-1);
	        		panel.getCrsMainPanel().getEsriPanel().setCodeCRS(0);
	        		panel.getJButtonAccept().setEnabled(false);	        		
	        	}	        		            	
	        }
		}
		
		
		if (e.getSource() == panel.getEpsgTrPanel().getJTable().getSelectionModel()){
			ListSelectionModel lsm = (ListSelectionModel)e.getSource();
	        if (lsm.isSelectionEmpty()) {
	        	panel.getEpsgTrPanel().selectedRowTable = -1;	        	
	        	panel.getJButtonAccept().setEnabled(false);
	        } 
	        else {
	        	panel.getEpsgTrPanel().selectedRowTable = lsm.getMinSelectionIndex();
	        	panel.getEpsgTrPanel().setTrasformation_code(Integer.parseInt((String)panel.getEpsgTrPanel().sorter.getValueAt(panel.getEpsgTrPanel().selectedRowTable,0)));
	            String sentence = "SELECT target_crs_code " +
								"FROM epsg_coordoperation " +                        
								"WHERE coord_op_code = " + panel.getEpsgTrPanel().getTransformation_code() ;
	            ResultSet result = Query.select(sentence,panel.getEpsgTrPanel().connect.getConnection());
	            try {
					result.next();
					int tar = result.getInt("target_crs_code");
					if (tar == panel.getEpsgTrPanel().crs_target) 
						panel.getEpsgTrPanel().inverseTranformation = false;					
					else panel.getEpsgTrPanel().inverseTranformation = true;
				} catch (SQLException e1) {							
					e1.printStackTrace();
				}
				panel.getJButtonAccept().setEnabled(true);
	        }
		}
		
		/*
		 * una vez cambiamos la transformacion a elegir, habra que ver que parametros
		 * tenemos que definir para que cuando aceptemos hayan sido definidos
		 * anteriormente
		 */
		if (e.getSource() == panel.getRecentsTrPanel().getJTable().getSelectionModel()){
			ListSelectionModel lsm = (ListSelectionModel)e.getSource();
	        if (lsm.isSelectionEmpty()) {
	        	panel.getRecentsTrPanel().selectedRowTable = -1;
	        	panel.getRecentsTrPanel().setCode(0);
	        	panel.getJButtonAccept().setEnabled(false);
	        	panel.getRecentsTrPanel().getJButtonInfo().setEnabled(false);
	        } 
	        else {
	        	panel.getRecentsTrPanel().selectedRowTable = lsm.getMinSelectionIndex();
	        	String[] cad = ((String)panel.getRecentsTrPanel().sorter.getValueAt(panel.getRecentsTrPanel().selectedRowTable,2)).split(":");	        	
	        	panel.getRecentsTrPanel().setCode(Integer.parseInt(cad[1]));
	        	String[] transformation = ((String)panel.getRecentsTrPanel().sorter.getValueAt(panel.getRecentsTrPanel().selectedRowTable,0)).split(":");
	        	if (transformation[0].equals("EPSG")){	        		
					panel.getRecentsTrPanel().setTrCode(Integer.parseInt(transformation[1]));
					String sentence = "SELECT target_crs_code " +
									"FROM epsg_coordoperation " +                        
									"WHERE coord_op_code = " + panel.getRecentsTrPanel().getTrCode();
				    ResultSet result = Query.select(sentence,panel.getRecentsTrPanel().connect.getConnection());
				    
					try {
						result.next();						
						int tar = result.getInt("target_crs_code");
						int crs_target = Integer.parseInt(((String)panel.getRecentsTrPanel().sorter.getValueAt(panel.getRecentsTrPanel().selectedRowTable,3)).split(":")[1]);
						int crs_base = -1;
						sentence = "SELECT source_geogcrs_code " +
								  "FROM epsg_coordinatereferencesystem " +	                              
	                              "WHERE coord_ref_sys_code = " + crs_target;
						result = Query.select(sentence,panel.getRecentsTrPanel().connect.getConnection());
						try {
							result.next();
							crs_base = result.getInt("source_geogcrs_code");
						} catch (SQLException e1) {
							crs_base = -1;
						}						
						
						if (tar == crs_target || tar == crs_base) 
							panel.getRecentsTrPanel().setInverseTransformation(false);					
						else panel.getRecentsTrPanel().setInverseTransformation(true);
						} catch (SQLException e1) {
							// TODO Auto-generated catch block
							e1.printStackTrace();
						}
				    
				}
	        	panel.getRecentsTrPanel().getJButtonInfo().setEnabled(true);	        	
	        	panel.getJButtonAccept().setEnabled(true);
	        }
		}
	}

	/**
	 * Manejador de los eventos relacionados con el cambio de opcin
	 * dentro del combobox de eleccin de repositorio, y del combobox
	 * de eleccin del archivo .gsb para el panel de transformacin
	 * de nadgrids.
	 */
	public void itemStateChanged(ItemEvent e) {
		if (e.getSource() == panel.getCrsMainPanel().getJComboOptions()) {
			CardLayout cl = (CardLayout)(panel.getCrsMainPanel().getJPanelMain().getLayout());
			String op = (String)e.getItem();		
			if (op.equals("EPSG")){
				String[] not_soported = {"engineering", "vertical", "compound", "geocentric", "geographic 3D"};
				int[] valid_method_code = {9800, 9801, 9802, 9803, 9804, 9805, 9806, 9807, 9808, 9809, 9810, 
						9811, 9812, 9813, 9814, 9815, 9816, 9817, 9602, 9659, 9818, 9819, 9820, 9822, 
						9827};
				boolean soported = true;
				boolean soported2 = false;
				panel.getCrsMainPanel().getEpsgPanel().connection();
				if(panel.getCrsMainPanel().getEpsgPanel().getJTable().getSelectedRowCount()>0) {
					panel.getCrsMainPanel().getEpsgPanel().selectedRowTable = panel.getCrsMainPanel().getEpsgPanel().getJTable().getSelectedRow();	        	
		        	String crs_kind = (String)panel.getCrsMainPanel().getEpsgPanel().sorter.getValueAt(panel.getCrsMainPanel().getEpsgPanel().selectedRowTable,2);
		        	for (int i = 0; i < not_soported.length; i++) {
		        		if (crs_kind.equals(not_soported[i]))
		        			soported = false;		        		
		        	}
		        	for (int i=0; i< valid_method_code.length;i++) {
		        		if (panel.getCrsMainPanel().getEpsgPanel().getProjectionCode((String)panel.getCrsMainPanel().getEpsgPanel().sorter.getValueAt(panel.getCrsMainPanel().getEpsgPanel().selectedRowTable,0)) == valid_method_code[i]) {
		        			soported2 = true;
		        		}
		        	}
		        	if (crs_kind.equals("geographic 2D")) {
		        		soported2 = true;
		        	}
		        	if (soported && soported2){
		        		panel.getCrsMainPanel().getEpsgPanel().setCodeCRS(Integer.parseInt((String)panel.getCrsMainPanel().getEpsgPanel().sorter.getValueAt(panel.getCrsMainPanel().getEpsgPanel().selectedRowTable,0)));
		        		panel.getCrsMainPanel().getEpsgPanel().setWKT();        		
		        		panel.getJButtonAccept().setEnabled(true);
		            	int base = panel.getCrsMainPanel().getEpsgPanel().getCodeCRS();						
						//panel.compareDatum(panel.getCrsMainPanel().getEpsgPanel().getWKT());
		            	panel.getJComboOptions().setEnabled(true);
		        		panel.getJButtonAccept().setEnabled(true);
		        		panel.getCrsMainPanel().getEpsgPanel().getInfoCrs().setEnabled(true);
		        	}
		        	else {
		        		JOptionPane.showMessageDialog(panel, PluginServices.getText(this,"crs_not_soported"), "Warning...",
								JOptionPane.WARNING_MESSAGE);
		        		panel.getCrsMainPanel().getEpsgPanel().setCodeCRS(-1);
		        		panel.getCrsMainPanel().getEpsgPanel().setCodeCRS(0);
		        		panel.getJButtonAccept().setEnabled(false);
		        		panel.getCrsMainPanel().getEpsgPanel().getInfoCrs().setEnabled(false);
		        	}		        	
				}
				else{
					panel.getJButtonNext().setVisible(false);
					panel.getJComboOptions().setEnabled(false);
					panel.getJComboOptions().setSelectedIndex(0);
					panel.getJButtonAccept().setEnabled(false);
				}
			} else if (op.equals("IAU2000")) {
				panel.getCrsMainPanel().getIauPanel().connection();
				if(panel.getCrsMainPanel().getIauPanel().getJTable().getSelectedRowCount()>0) {
					panel.getCrsMainPanel().getIauPanel().selectedRowTable = panel.getCrsMainPanel().getIauPanel().getJTable().getSelectedRow();;
		        	panel.getCrsMainPanel().getIauPanel().setCodeCRS(Integer.parseInt((String)panel.getCrsMainPanel().getIauPanel().sorter.getValueAt(panel.getCrsMainPanel().getIauPanel().selectedRowTable,0)));
		        	panel.getCrsMainPanel().getIauPanel().setWKT();  
		    		panel.getJComboOptions().setEnabled(false);
		    		panel.getJButtonAccept().setEnabled(true);			
					panel.getJButtonNext().setVisible(false);
				}
				else {
					panel.getJButtonNext().setVisible(false);
					panel.getJComboOptions().setEnabled(false);
					panel.getJComboOptions().setSelectedIndex(0);
					panel.getJButtonAccept().setEnabled(false);
				}
			}
			else if (op.equals("Recientes")) {
				if(panel.getCrsMainPanel().getRecentsPanel().getJTable().getSelectedRowCount()>0) {
					panel.getCrsMainPanel().getRecentsPanel().initCrs();		        	
		        	String sour =  panel.getCrsMainPanel().getRecentsPanel().getCrs().getCrsWkt().getAuthority()[0]; //(String)panel.getCrsMainPanel().getRecentsPanel().sorter.getValueAt(panel.getCrsMainPanel().getRecentsPanel().selectedRowTable,0);
		        	IWindow activeWindow = PluginServices.getMDIManager().getActiveWindow();
			    	BaseView activeView = (BaseView) activeWindow;
			    	String authority = ((ICrs) activeView.getMapControl().getProjection()).getCrsWkt().getAuthority()[0];
		        	if (sour.equals("EPSG") && authority.equals("EPSG")){
		        		//panel.compareDatum(panel.getCrsMainPanel().getRecentsPanel().getCrs().getWKT());
		        		panel.getJComboOptions().setEnabled(true);
		        		panel.getJButtonAccept().setEnabled(true);
		        	}
		        	else {
		        		panel.getJComboOptions().setSelectedIndex(0);	        		
		        		panel.getJComboOptions().setEnabled(false);
		        		panel.getJButtonAccept().setEnabled(true);
		        		panel.getJButtonAccept().setVisible(true);	        			        		
		        		panel.getJButtonNext().setVisible(false);
		        	}				
				}			
				else {
					panel.getJButtonNext().setVisible(false);
					panel.getJComboOptions().setEnabled(false);
					panel.getJComboOptions().setSelectedIndex(0);
					panel.getJButtonAccept().setEnabled(false);
				}
			}
			else if (op.equals("ESRI")) {
				panel.getCrsMainPanel().getEsriPanel().connection();
				if(panel.getCrsMainPanel().getEsriPanel().getJTable().getSelectedRowCount()>0) {
					panel.getCrsMainPanel().getEsriPanel().selectedRowTable = panel.getCrsMainPanel().getEsriPanel().getJTable().getSelectedRow();;
		        	panel.getCrsMainPanel().getEsriPanel().setCodeCRS(Integer.parseInt((String)panel.getCrsMainPanel().getEsriPanel().sorter.getValueAt(panel.getCrsMainPanel().getEsriPanel().selectedRowTable,0)));
		        	panel.getCrsMainPanel().getEsriPanel().setWKT();  
		    		panel.getJComboOptions().setEnabled(false);
		    		panel.getJButtonAccept().setEnabled(true);			
					panel.getJButtonNext().setVisible(false);
				}
				else {
					panel.getJButtonNext().setVisible(false);
					panel.getJComboOptions().setEnabled(false);
					panel.getJComboOptions().setSelectedIndex(0);
					panel.getJButtonAccept().setEnabled(false);
				}
			}
		    cl.show(panel.getCrsMainPanel().getJPanelMain(), (String)e.getItem());
		    panel.getCrsMainPanel().setDataSource((String)e.getItem());
		}
	    	    
	    if (e.getSource() == panel.getNadsTrPanel().getJComboNadFile()){
			if (panel.getNadsTrPanel().getJComboNadFile().getSelectedIndex()!=0){
				panel.getNadsTrPanel().setNadFile(panel.getNadsTrPanel().getJComboNadFile().getSelectedItem().toString());//nadFile = getJComboNadFile().getSelectedItem().toString();
				panel.getNadsTrPanel().getTreePanel().setRoot(PluginServices.getText(this,"grids_en")+": "+panel.getNadsTrPanel().getNadFile());//nadFile);
				panel.getNadsTrPanel().initializeTree();
				// Guardar en persistencia nadFile:
				panel.getNadsTrPanel().saveNadFileName(panel.getNadsTrPanel().getNadFile());//nadFile);
				panel.getJButtonAccept().setEnabled(true);
			}
			else{
				panel.getNadsTrPanel().setNadFile(null);//nadFile = null;
				panel.getNadsTrPanel().getTreePanel().setRoot("");
				panel.getJButtonAccept().setEnabled(false);
			}
			
		}
	}
		
	/**
	 * Mtodo para manejar el doble click sobre las tablas existentes en
	 * el panel de transformacin. Aceptar el CRS seleccionado, as como
	 * la transformacin en el caso de que elijamos transformacin EPSG.
	 */
	public void mouseClicked(MouseEvent e) {
		// TODO Auto-generated method stub
		if (panel.getJButtonAccept().isVisible()) {
			if (e.getSource() == panel.getCrsMainPanel().getRecentsPanel().getJTable()){
				if (e.getClickCount() == 2){
					if (panel.isEpsg_tr()) {			
						GetTransepsg epsgParams = new GetTransepsg(panel.getEpsgTrPanel().getTransformation_code(),panel.getEpsgTrPanel().connect, panel.getEpsgTrPanel().inverseTranformation);
						panel.getEpsgTrPanel().setValues(epsgParams.getParamValue());	
						panel.setProjection(panel.getEpsgTrPanel().getProjection());
						panel.setEpsg_tr(false);
			    		
					}
					else if (panel.isManual_tr()) {
						panel.setProjection(panel.getManualTrPanel().getProjection());
						panel.setManual_tr(false);
					}
					else if (panel.isNads_tr()) {
						panel.setProjection(panel.getNadsTrPanel().getProjection());
						panel.setTargetNad(panel.getNadsTrPanel().getNad());
						panel.setNads_tr(false);
					}
					else{
						panel.setSin_tr(true);
						panel.setProjection(panel.getProjection());
					}
										
		    		// * Actualizar recientes...
		    		 
		    		String authority = ((ICrs)panel.getProjection()).getCrsWkt().getAuthority()[0];
		    		String name = ((ICrs)panel.getProjection()).getCrsWkt().getName();
		    		int code = ((ICrs)panel.getProjection()).getCode();
		    		CrsData crsData = new CrsData(authority,code,name);
		    		RecentCRSsPersistence persistence = new RecentCRSsPersistence(RecentCRSsPersistence.pluginClassInstance);
		    		persistence.addCrsData(crsData);
					
					PluginServices.getMDIManager().closeWindow(panel);
				}
			}
			if (e.getSource() == panel.getCrsMainPanel().getEpsgPanel().getJTable()){
				if (e.getClickCount() == 2){
					if (panel.isEpsg_tr()) {			
						GetTransepsg epsgParams = new GetTransepsg(panel.getEpsgTrPanel().getTransformation_code(),panel.getEpsgTrPanel().connect, panel.getEpsgTrPanel().inverseTranformation);
						panel.getEpsgTrPanel().setValues(epsgParams.getParamValue());	
						panel.setProjection(panel.getEpsgTrPanel().getProjection());
						panel.setEpsg_tr(false);
						
						/*
						 * Actualizar Transformaciones recientes
						 */
						String authority = "EPSG";
			    		String name = (String)panel.getEpsgTrPanel().sorter.getValueAt(panel.getEpsgTrPanel().selectedRowTable,1);
			    		int code = panel.getEpsgTrPanel().getTransformation_code();
			    		String crsSource = "EPSG:"+String.valueOf(panel.getEpsgTrPanel().getSource()); 	    			
			    		String crsTarget = "EPSG:"+String.valueOf(panel.getEpsgTrPanel().getTarget());
			    		String details = (String)panel.getEpsgTrPanel().sorter.getValueAt(panel.getEpsgTrPanel().selectedRowTable,5);;
			    		TrData trData = new TrData(authority,code,name,crsSource,crsTarget,details);
			    		RecentTrsPersistence trPersistence = new RecentTrsPersistence(RecentCRSsPersistence.pluginClassInstance);
			    		trPersistence.addTrData(trData);
					}
					else if (panel.isManual_tr()) {
						panel.setProjection(panel.getManualTrPanel().getProjection());
						panel.setManual_tr(false);
					}
					else if (panel.isNads_tr()) {
						panel.setProjection(panel.getNadsTrPanel().getProjection());
						panel.setTargetNad(panel.getNadsTrPanel().getNad());
						panel.setNads_tr(false);
					}
					else{
						panel.setSin_tr(true);
						panel.setProjection(panel.getProjection());
					}
					
		    		 //* Actualizar recientes...
		    		 
		    		String authority = ((ICrs)panel.getProjection()).getCrsWkt().getAuthority()[0];
		    		String name = ((ICrs)panel.getProjection()).getCrsWkt().getName();
		    		int code = ((ICrs)panel.getProjection()).getCode();
		    		CrsData crsData = new CrsData(authority,code,name);
		    		RecentCRSsPersistence persistence = new RecentCRSsPersistence(RecentCRSsPersistence.pluginClassInstance);
		    		persistence.addCrsData(crsData);
					
					PluginServices.getMDIManager().closeWindow(panel);
				}
			}
			if (e.getSource() == panel.getCrsMainPanel().getIauPanel().getJTable()){
				if (e.getClickCount() == 2){
					if (panel.isEpsg_tr()) {			
						GetTransepsg epsgParams = new GetTransepsg(panel.getEpsgTrPanel().getTransformation_code(),panel.getEpsgTrPanel().connect, panel.getEpsgTrPanel().inverseTranformation);
						panel.getEpsgTrPanel().setValues(epsgParams.getParamValue());	
						panel.setProjection(panel.getEpsgTrPanel().getProjection());
						panel.setEpsg_tr(false);
					}
					else if (panel.isManual_tr()) {
						panel.setProjection(panel.getManualTrPanel().getProjection());
						panel.setManual_tr(false);
					}
					else if (panel.isNads_tr()) {
						panel.setProjection(panel.getNadsTrPanel().getProjection());
						panel.setTargetNad(panel.getNadsTrPanel().getNad());
						panel.setNads_tr(false);
					}
					else{
						panel.setSin_tr(true);
						panel.setProjection(panel.getProjection());
					}
					
		    		 //* Actualizar recientes...
		    		 
		    		String authority = ((ICrs)panel.getProjection()).getCrsWkt().getAuthority()[0];
		    		String name = ((ICrs)panel.getProjection()).getCrsWkt().getName();
		    		int code = ((ICrs)panel.getProjection()).getCode();
		    		CrsData crsData = new CrsData(authority,code,name);
		    		RecentCRSsPersistence persistence = new RecentCRSsPersistence(RecentCRSsPersistence.pluginClassInstance);
		    		persistence.addCrsData(crsData);
					
					PluginServices.getMDIManager().closeWindow(panel);
				}
			}
			if (e.getSource() == panel.getCrsMainPanel().getEsriPanel().getJTable()){
				if (e.getClickCount() == 2){
					if (panel.isEpsg_tr()) {			
						GetTransepsg epsgParams = new GetTransepsg(panel.getEpsgTrPanel().getTransformation_code(),panel.getEpsgTrPanel().connect, panel.getEpsgTrPanel().inverseTranformation);
						panel.getEpsgTrPanel().setValues(epsgParams.getParamValue());	
						panel.setProjection(panel.getEpsgTrPanel().getProjection());
						panel.setEpsg_tr(false);
					}
					else if (panel.isManual_tr()) {
						panel.setProjection(panel.getManualTrPanel().getProjection());
						panel.setManual_tr(false);
					}
					else if (panel.isNads_tr()) {
						panel.setProjection(panel.getNadsTrPanel().getProjection());
						panel.setTargetNad(panel.getNadsTrPanel().getNad());
						panel.setNads_tr(false);
					}
					else{
						panel.setSin_tr(true);
						panel.setProjection(panel.getProjection());
					}
					
		    		 //* Actualizar recientes...
		    		 
		    		String authority = ((ICrs)panel.getProjection()).getCrsWkt().getAuthority()[0];
		    		String name = ((ICrs)panel.getProjection()).getCrsWkt().getName();
		    		int code = ((ICrs)panel.getProjection()).getCode();
		    		CrsData crsData = new CrsData(authority,code,name);
		    		RecentCRSsPersistence persistence = new RecentCRSsPersistence(RecentCRSsPersistence.pluginClassInstance);
		    		persistence.addCrsData(crsData);
					
					PluginServices.getMDIManager().closeWindow(panel);
				}
			}
		}
		if (e.getSource() == panel.getEpsgTrPanel().getJTable()){
			if (e.getClickCount() == 2){
				GetTransepsg epsgParams = new GetTransepsg(panel.getEpsgTrPanel().getTransformation_code(),panel.getEpsgTrPanel().connect, panel.getEpsgTrPanel().inverseTranformation);
				panel.getEpsgTrPanel().setValues(epsgParams.getParamValue());	
				panel.setProjection(panel.getEpsgTrPanel().getProjection());
				panel.setEpsg_tr(false);
				
				/*
				 * Actualizar Transformaciones recientes
				 */
				String authority = "EPSG";
	    		String name = (String)panel.getEpsgTrPanel().sorter.getValueAt(panel.getEpsgTrPanel().selectedRowTable,1);
	    		int code = panel.getEpsgTrPanel().getTransformation_code();
	    		String crsSource = "EPSG:"+String.valueOf(panel.getEpsgTrPanel().getSource()); 	    			
	    		String crsTarget = "EPSG:"+String.valueOf(panel.getEpsgTrPanel().getTarget());
	    		String details = (String)panel.getEpsgTrPanel().sorter.getValueAt(panel.getEpsgTrPanel().selectedRowTable,5);;
	    		TrData trData = new TrData(authority,code,name,crsSource,crsTarget,details);
	    		RecentTrsPersistence trPersistence = new RecentTrsPersistence(RecentCRSsPersistence.pluginClassInstance);
	    		trPersistence.addTrData(trData);
				
				/*
	    		 * Actualizar Crss recientes...
	    		 */
	    		authority = ((ICrs)panel.getProjection()).getCrsWkt().getAuthority()[0];
	    		name = ((ICrs)panel.getProjection()).getCrsWkt().getName();
	    		code = ((ICrs)panel.getProjection()).getCode();
	    		CrsData crsData = new CrsData(authority,code,name);
	    		RecentCRSsPersistence persistence = new RecentCRSsPersistence(RecentCRSsPersistence.pluginClassInstance);
	    		persistence.addCrsData(crsData);
				
				PluginServices.getMDIManager().closeWindow(panel);
			}
		}
		if (e.getSource() == panel.getRecentsTrPanel().getJTable()){
			if (e.getClickCount() == 2){
				String[] transf = ((String)panel.getRecentsTrPanel().sorter.getValueAt(panel.getRecentsTrPanel().selectedRowTable,0)).split(":");
				if (transf[0].equals("USR")){
	        		panel.getRecentsTrPanel().setParamsManual((String)panel.getRecentsTrPanel().sorter.getValueAt(panel.getRecentsTrPanel().selectedRowTable,4));
	        		panel.setProjection(panel.getRecentsTrPanel().getProjection());
	        	}
				if (transf[0].equals("EPSG")){
					GetTransepsg epsgParams = new GetTransepsg(panel.getRecentsTrPanel().getTrCode(),panel.getEpsgTrPanel().connect, panel.getRecentsTrPanel().getInverseTransformation());
					panel.getRecentsTrPanel().setParamsEPGS(epsgParams.getParamValue());
					panel.setProjection(panel.getRecentsTrPanel().getProjection());
	        	}
				if (transf[0].equals("NADGR")){
					panel.setProjection(panel.getRecentsTrPanel().getProjectionNad((String)panel.getRecentsTrPanel().sorter.getValueAt(panel.getRecentsTrPanel().selectedRowTable,4)));
				}
				//panel.setProjection(panel.getRecentsTrPanel().getProjection());
				panel.setRecents_tr(false);				
				
				/*
				 * Actualizar Transformaciones recientes
				 */
				String authCode = (String)panel.getRecentsTrPanel().sorter.getValueAt(panel.getRecentsTrPanel().selectedRowTable,0);
				String authority = authCode.split(":")[0];
	    		String name = (String)panel.getRecentsTrPanel().sorter.getValueAt(panel.getRecentsTrPanel().selectedRowTable,1);
	    		int code = Integer.parseInt(authCode.split(":")[1]);
	    		String crsSource = (String)panel.getRecentsTrPanel().sorter.getValueAt(panel.getRecentsTrPanel().selectedRowTable,2);
	    		String crsTarget = (String)panel.getRecentsTrPanel().sorter.getValueAt(panel.getRecentsTrPanel().selectedRowTable,3);
	    		String details = (String)panel.getRecentsTrPanel().sorter.getValueAt(panel.getRecentsTrPanel().selectedRowTable,4);	    		
	    		TrData trData = new TrData(authority,code,name,crsSource,crsTarget,details);
	    		RecentTrsPersistence trPersistence = new RecentTrsPersistence(RecentCRSsPersistence.pluginClassInstance);
	    		trPersistence.addTrData(trData);
								
				/*
	    		 * Actualizar recientes...
	    		 */
	    		authority = ((ICrs)panel.getProjection()).getCrsWkt().getAuthority()[0];
	    		name = ((ICrs)panel.getProjection()).getCrsWkt().getName();
	    		code = ((ICrs)panel.getProjection()).getCode();
	    		CrsData crsData = new CrsData(authority,code,name);
	    		RecentCRSsPersistence persistence = new RecentCRSsPersistence(RecentCRSsPersistence.pluginClassInstance);
	    		persistence.addCrsData(crsData);
				
				PluginServices.getMDIManager().closeWindow(panel);
			}
		}
	}

	public void mouseEntered(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void mouseExited(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void mousePressed(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void mouseReleased(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void keyPressed(KeyEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	/**
	 * Mtodo para cuando en la tranformacin manual, no se hayan insertado
	 * valores de transformacin, no se habilite el botn de aceptar.
	 */
	public void keyReleased(KeyEvent e) {
		// TODO Auto-generated method stub
		if(!panel.getManualTrPanel().getStatus()) 
			panel.getJButtonAccept().setEnabled(false);		
		else panel.getJButtonAccept().setEnabled(true);		
	}

	public void keyTyped(KeyEvent arg0) {
		// TODO Auto-generated method stub
		
	}

}
