/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Instituto de Desarrollo Regional and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   Instituto de Desarrollo Regional (Universidad de Castilla La-Mancha)
 *   Campus Universitario s/n
 *   02071 Alabacete
 *   Spain
 *
 *   +34 967 599 200
 */

package org.gvsig.crs.ogr;

/**
 * 
 * @author Miguel Garca Jimnez (garciajimenez.miguel@gmail.com)
 *
 */
public class CrsOgr extends JNIBaseCRS
{
	public static String exportToWkt(OGRSpatialReference ORGSpace)throws CrsOgrException, crsgdalException
	{
		String result = exportToWktNat(ORGSpace.getPtr());
		if(result.length() == 0) throw new CrsOgrException(ORGSpace);
		return result;
	}
	
	public static int importFromWkt(OGRSpatialReference ORGSpace, String cadenas) throws crsgdalException, CrsOgrException, OGRException
	{
		int result = importFromWktNat(ORGSpace.getPtr(), cadenas);
		if(result != 0) throw new OGRException(result,"Error en Wkt().La creacion del objeto no tuvo exito. ",ORGSpace);
		return result;
	}
	
	public static int setUTM(OGRSpatialReference ORGSpace ,int zona, boolean norte_sur)throws OGRException, crsgdalException, CrsOgrException{
		
		if(ORGSpace.getPtr() <= 0)
			throw new OGRException("Error en setUTM(). La llamada de creacin de objeto no tuvo exito.");
		
		int ns=-1;
		if(norte_sur)ns=1;
		else ns=0;
		
		int res = setUTMNat(ORGSpace.getPtr(), zona, ns);
		
		if(res != 0)
			throw new OGRException(res,"Error en setUTM(). No se ha podido asignar la zona especificada. ",ORGSpace);
		return res;
	}
	
	public static int setWellKnownGeogCS(OGRSpatialReference ORGSpace, String cs)throws OGRException, crsgdalException, CrsOgrException{
		
		if(ORGSpace.getPtr() <= 0)
			throw new OGRException("Error en setWellKnownGeogCS(). La llamada de creacin de objeto no tuvo exito.");
		
		int res = setWellKnownGeogCSNat(ORGSpace.getPtr(), cs);
		
		if(res != 0)
			throw new OGRException(res,"Error en setWellKnownGeoCS(). No se ha podido asignar el sistema de coordenadas especificado. ",ORGSpace);
		return res;
	}
	
	public static int importFromEPSG(OGRSpatialReference ORGSpace, int cod) throws OGRException, crsgdalException, CrsOgrException
	{
		int result = importFromEPSGNat(ORGSpace.getPtr(), cod);
		if(result != 0) throw new OGRException(result,"Error en EPSG().La creacion del objeto no tuvo exito. ",ORGSpace);
		return result;
	}
	
	public static int importFromProj4(OGRSpatialReference ORGSpace, String cs)throws OGRException, crsgdalException, CrsOgrException{
		
		if(ORGSpace.getPtr() <= 0)
			throw new OGRException("Error en Proj4(). La llamada de creacin de objeto no tuvo exito.");
		
		int res = importFromProj4Nat(ORGSpace.getPtr(), cs);
		
		if(res != 0)
			throw new OGRException(res, "Error en Proj4(). No se ha podido asignar el sistema de coordenadas especificado. ", ORGSpace);
		
		return res;
		
	}
	
	public static String exportToProj4(OGRSpatialReference ORGSpace)throws CrsOgrException, crsgdalException
	{
		String result = exportToProj4Nat(ORGSpace.getPtr());
		if(result.length() == 0) throw new CrsOgrException(ORGSpace);
		return result;
	}
	
	public static int importFromPCI(OGRSpatialReference ORGSpace,String cod, String cs, double[] coord)throws CrsOgrException, crsgdalException, OGRException
	{
		int result = importFromPCINat(ORGSpace.getPtr(),cod, cs, coord);
		if(result != 0) throw new OGRException(result,"Error en PCI().La creacion del objeto no tuvo exito. ",ORGSpace);
		return result;
	}
	
	public static int importFromUSGS(OGRSpatialReference ORGSpace, long code, long zone, double[] params, long datum) throws crsgdalException, OGRException, CrsOgrException
	{
		int result = importFromUSGSNat(ORGSpace.getPtr(),code, zone, params, datum);
		if(result != 0) throw new OGRException(result,"Error en USGS().La creacion del objeto no tuvo exito. ",ORGSpace);
		return result;	
		
	}
	
	public static int importFromESRI(OGRSpatialReference ORGSpace, String cadenas) throws crsgdalException, CrsOgrException, OGRException
	{
		int result = importFromESRINat(ORGSpace.getPtr(), cadenas);
		if(result != 0) throw new OGRException(result,"Error en ESRI().La creacion del objeto no tuvo exito. ",ORGSpace);
		return result;
	}
}

