package org.gvsig.remoteClient.gml;

import org.gvsig.remoteClient.gml.exceptions.GMLException;
import org.gvsig.remoteClient.gml.factories.IGeometriesFactory;
import org.gvsig.remoteClient.gml.schemas.XMLSchemaParser;


/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: IGMLFeaturesIterator.java 18271 2008-01-24 09:06:43Z jpiera $
 * $Log$
 * Revision 1.3  2007-01-15 13:11:00  csanchez
 * Sistema de Warnings y Excepciones adaptado a BasicException
 *
 * Revision 1.2  2006/12/22 11:25:44  csanchez
 * Nuevo parser GML 2.x para gml's sin esquema
 *
 * Revision 1.1  2006/08/10 12:00:49  jorpiell
 * Primer commit del driver de Gml
 *
 *
 */
/**
 * Iterator to retrieve the GML objects parsed. It must to
 * be implemented by the iterators for GML 2.x and 3.x
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 * @author Carlos Snchez Perin (sanchez_carper@gva.es)
 * 
 */
public abstract class IGMLFeaturesIterator {
	private XMLSchemaParser parser = null;
	private IGeometriesFactory factory = null;
	private String featureRoot = null; 

	
	public IGMLFeaturesIterator(XMLSchemaParser parser, IGeometriesFactory factory) {
		this.parser = parser;
		this.factory = factory;
		//It takes the <featureMember> like featureRoot
		featureRoot = parser.getName();
	}
	/**
	 * It Returns true if there are more features to parse
	 * @return
	 * @throws BaseException 
	 */
	public abstract boolean hasNext()throws GMLException;
    
	/**
	 * It returns the next feature
	 * @return
	 */
	public abstract Object next()throws GMLException;	
	
	/**
	 * @return Returns the featureRoot.
	 */
	public String getFeatureRoot() {
		return featureRoot;
	}
	/**
	 * @return Returns the factory.
	 */
	public IGeometriesFactory getFactory() {
		return factory;
	}
	/**
	 * @return Returns the parser.
	 */
	public XMLSchemaParser getParser() {
		return parser;
	}
}
