package org.gvsig.remoteClient.gml.engine;

import java.io.IOException;
import java.io.InputStream;

import org.gvsig.remoteClient.gml.engine.readers.IReaderFile;
import org.gvsig.remoteClient.gml.exceptions.GMLException;
import org.gvsig.remoteClient.gml.factories.IGeometriesFactory;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: AbstractDriver.java 18271 2008-01-24 09:06:43Z jpiera $
 * $Log$
 * Revision 1.1  2007-02-12 13:49:18  jorpiell
 * Aadido el driver de KML
 *
 *
 */
/**
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public abstract class AbstractDriver implements IDriver{
	private InputStream inputStream = null;
	private IReaderFile reader = null;	
	protected IFeaturesParser featuresParser = null;
	
	/**
	 * Constructor
	 * @param file
	 * File to read	
	 * @param factory
	 * Geometries factory that is used to create the parsed geometries
	 * When the file is not found
	 * @throws BaseException 
	 */
	public AbstractDriver(InputStream file, IGeometriesFactory factory) throws GMLException{
		//file to parse...
		this.inputStream = file;			
		initializeReader();
		initialize(factory);
	}
	
	/**
	 * Initializes the file reader
	 *
	 */
	protected abstract void initializeReader() throws GMLException;
		

	/**
	 * Initializes the IReader, tries to parse  the file header 
	 * to obtain the version and parses all the needed schemas
	 * @param factory
	 * Geometries factory that is used to create the parsed 
	 * geometries
	 * @throws BaseException 
	 */
	protected abstract void initialize(IGeometriesFactory factory) throws GMLException;

	/**
	 * @return Returns the reader.
	 */
	public IReaderFile getReader() {
		return reader;
	}

	/**
	 * @param reader The reader to set.
	 */
	protected void setReader(IReaderFile reader) {
		this.reader = reader;
	}

	/**
	 * @return Returns the inputStream.
	 */
	public InputStream getInputStream() {
		return inputStream;
	}

	/**
	 * @return Returns the encoding.
	 * @throws KmlException 
	 */
	protected String getEncoding() throws Exception {
		String encoding = "UTF-8";
		InputStream is = getInputStream();		
		int index = -1;
		int endIndex = -1;
		String header = "";
		boolean endHeader = false;
		try {
			int i = 1;
			while((!endHeader) && (i < 100)){
				byte[] buffer = new byte[1];
				is.read(buffer);
				header = header + new String(buffer);
				//This if is to locate the encoding string
				if (index == -1) { 
					String searchText = "encoding=\"";
					index = header.indexOf(searchText);
					if (index > -1){
						header = "";
					}
				}
				//This if is to locate the encoding value
				if (index > -1){
					String searchText = "\"";
					endIndex = header.indexOf(searchText);
					if (endIndex > -1){
						encoding = header.substring(0, endIndex);
					}
				}
				//To finish to parse the header
				if (endIndex > -1){
					String searchText = ">";
					int headerEndIndex = header.indexOf(searchText);
					if (headerEndIndex > -1){
						endHeader = true;
					}
				}
				i++;
			}		
		} catch (IOException e) {
			throw new GMLException(e);
		}		
		return encoding;
	}	
}
