package org.gvsig.remoteClient.gml.factories;

import java.lang.reflect.InvocationTargetException;
import java.util.Hashtable;

import org.gvsig.remoteClient.gml.GMLFeaturesParser;
import org.gvsig.remoteClient.gml.exceptions.GMLParserException;
import org.gvsig.remoteClient.gml.schemas.XMLSchemaParser;
import org.gvsig.remoteClient.gml.v2.GMLFeaturesParser_v2;
import org.gvsig.remoteClient.gml.warnings.GMLWarningInfo;
import org.gvsig.remoteClient.gml.warnings.GMLWarningVersionNotSupported;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: FeaturesParserFactory.java 13912 2007-09-20 09:33:50Z jaume $
 * $Log$
 * Revision 1.4  2007-09-20 09:30:12  jaume
 * removed unnecessary imports
 *
 * Revision 1.3  2007/01/15 13:11:00  csanchez
 * Sistema de Warnings y Excepciones adaptado a BasicException
 *
 * Revision 1.2  2006/12/22 11:25:44  csanchez
 * Nuevo parser GML 2.x para gml's sin esquema
 *
 * Revision 1.1  2006/08/10 12:00:49  jorpiell
 * Primer commit del driver de Gml
 *
 *
 */
/**
 * This class has all the classes to parse different types 
 * of GML files. 
 * 
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 * @author Carlos Snchez Perin (sanchez_carper@gva.es)
 * 
 */
public class FeaturesParserFactory {
	private static Hashtable supportedVersions = new Hashtable();
	static {
		supportedVersions.put("1.0",GMLFeaturesParser_v2.class);
		supportedVersions.put("2.0",GMLFeaturesParser_v2.class);
		supportedVersions.put("1.0.0",GMLFeaturesParser_v2.class);
		supportedVersions.put("2.0.0",GMLFeaturesParser_v2.class);
		supportedVersions.put("2.1.1",GMLFeaturesParser_v2.class);
		supportedVersions.put("2.1.2",GMLFeaturesParser_v2.class);
		supportedVersions.put("2.x",GMLFeaturesParser_v2.class);
		//Default GML version if it's not especified in the schema or it hasn't schema
		supportedVersions.put("99.99.99",GMLFeaturesParser_v2.class);
	}
	public GMLWarningInfo warnings = null;
	private GMLFeaturesParser gmlParser = null;
	
	/**
	 * Gets the features iterator
	 * @param version
	 * Depending of this value, a different parser will be
	 * created and returned
	 * @param factory
	 * Factory to create the geometries
	 * @param parser
	 * Used to parse the GML file
	 * @return
	 * @throws IllegalArgumentException
	 * @throws SecurityException
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 * @throws InvocationTargetException
	 * @throws NoSuchMethodException
	 */
	public FeaturesParserFactory(){
		warnings = new GMLWarningInfo(); 
	}
	public GMLFeaturesParser createParser(String version, IGeometriesFactory factory, XMLSchemaParser parser) throws GMLParserException{
		Class clazz = (Class) supportedVersions.get(version);
		if (clazz == null){
			clazz = (Class) supportedVersions.get("99.99.99");
			//WARNING VERSION NOT SUPPORTED OR NOT SPECIFIED
			warnings.setElement(new GMLWarningVersionNotSupported(version));
		}
		if (clazz != null){
			//Utilizamos un interfaz con los tipos y las geometrias bsicas de GML
			//Otra opcion seria parsear el schemas de opengis, para obtener los tipos bsicos?
			Class [] args = {IGeometriesFactory.class,XMLSchemaParser.class};
			Object [] params = {factory,parser};
			//pasamos el parser con el que estamos leyendo el gml y el interfaz de geometrias
			//para continuar con la comprobacin del fichero GML
			try {
				gmlParser = (GMLFeaturesParser) clazz.getConstructor(args).newInstance(params);
			} catch (Exception e) {
				throw new GMLParserException(e);
			}
		}
		return gmlParser ;		
	}
}
