/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2007 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.gvsig.gui.beans.buttonspanel;

import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Iterator;

import javax.swing.JButton;

import org.gvsig.gui.beans.BaseComponent;
import org.gvsig.gui.beans.messages.Messages;
/**
 * <code>ButtonsPanel</code> ofrece un widget con un conjunto de botones
 * preestablecidos, aunque tambin se pueden aadir botones con el mtodo
 * {@link #addButton(String, int)}
 * 
 * @version 15/03/2007
 * @author Borja Sanchez Zamorano (borja.sanchez@iver.es)
 */
public class ButtonsPanel extends BaseComponent {
	private static final long serialVersionUID = -7264041442217894577L;
	private ArrayList buttonsList = new ArrayList();
	static private int eventId = Integer.MIN_VALUE;
	private ArrayList actionCommandListeners = new ArrayList();

	public static final int BUTTON_ACCEPT = 1;
	public static final int BUTTON_CANCEL = 2;
	public static final int BUTTON_APPLY = 3;
	public static final int BUTTON_YES = 4;
	public static final int BUTTON_NO = 5;
	public static final int BUTTON_CLOSE = 6;
	public static final int BUTTON_EXIT = 7;
	public static final int BUTTON_SEEDETAILS = 8;
	public static final int BUTTON_HIDEDETAILS = 9;
	public static final int BUTTON_PAUSE = 10;
	public static final int BUTTON_RESTART = 11;
	public static final int BUTTON_SAVE = 12;
	/**
	 * Sirve para cuando se crean botones nuevos, saber el ltimo nmero usado
	 * internamente, as '<code>new_id = BUTTON_LAST + 1;</code>' podra ser
	 * el ndice del nuevo botn.
	 */
	public static final int BUTTON_LAST = 12;
	
	public static final int BUTTONS_ACCEPT = 1;
	public static final int BUTTONS_ACCEPTCANCEL = 2;
	public static final int BUTTONS_ACCEPTCANCELAPPLY = 3;
	public static final int BUTTONS_CANCEL = 4;
	public static final int BUTTONS_YESNO = 5;
	public static final int BUTTONS_CLOSE = 6;
	public static final int BUTTONS_EXIT = 7;
	public static final int BUTTONS_NONE = 8;

	/**
	 * Crea un ButtonsPanel con un Layout por defecto.
	 */
	public ButtonsPanel() {
		setLayout(new java.awt.FlowLayout(FlowLayout.RIGHT));
	}

	/**
	 * Crea un ButtonsPanel con un Layout por defecto.
	 * @param items Que botones vamos a usar en la creacin.
	 */
	public ButtonsPanel(int items) {
		setLayout(new java.awt.FlowLayout(FlowLayout.RIGHT));
		switch (items) {
			case BUTTONS_ACCEPT:
				addAccept();
				break;
			case BUTTONS_ACCEPTCANCEL:
				addAccept();
				addCancel();
				break;
			case BUTTONS_ACCEPTCANCELAPPLY:
				addAccept();
				addCancel();
				addApply();
				break;
			case BUTTONS_CANCEL:
				addCancel();
				break;
			case BUTTONS_YESNO:
				addYes();
				addNo();
				break;
			case BUTTONS_CLOSE:
				addClose();
				break;
			case BUTTONS_EXIT:
				addExit();
				break;
			case BUTTONS_NONE:
				break;
		}
	}

	/**
	 * Aadir el disparador de cuando se pulsa un botn.
	 * @param listener
	 */
	public void addButtonPressedListener(ButtonsPanelListener listener) {
		if (!actionCommandListeners.contains(listener))
			actionCommandListeners.add(listener);
	}

	/**
	 * Borrar el disparador de eventos de los botones.
	 * @param listener
	 */
	public void removeButtonPressedListener(ButtonsPanelListener listener) {
	  actionCommandListeners.remove(listener);
	}

	private void callActionCommandListeners(int buttonID) {
		Iterator acIterator = actionCommandListeners.iterator();
		while (acIterator.hasNext()) {
			ButtonsPanelListener listener = (ButtonsPanelListener) acIterator.next();
			listener.actionButtonPressed(new ButtonsPanelEvent(this, buttonID));
		}
		eventId++;
	}

	/**
	 * Aadir el boton Aceptar.
	 */
	public void addAccept() {
		addButton(Messages.getText("aceptar"), BUTTON_ACCEPT);
	}
	
	/**
	 * Aadir el boton Guardar.
	 */
	public void addSave() {
		addButton(Messages.getText("guardar"), BUTTON_SAVE);
	}
	
	/**
	 * Aadir el boton Cancelar.
	 */
	public void addCancel() {
		addButton(Messages.getText("cancelar"), BUTTON_CANCEL);
	}
	
	/**
	 * Aadir el boton S.
	 */
	public void addYes() {
		addButton(Messages.getText("si"), BUTTON_YES);
	}

	/**
	 * Aadir el boton No.
	 */
	public void addNo() {
		addButton(Messages.getText("no"), BUTTON_NO);
	}

	/**
	 * Aadir el boton Aplicar.
	 */
	public void addApply() {
		addButton(Messages.getText("aplicar"), BUTTON_APPLY);
	}

	/**
	 * Aadir el boton Cerrar.
	 */
	public void addClose() {
		addButton(Messages.getText("cerrar"), BUTTON_CLOSE);
	}

	/**
	 * Aadir el boton Salir.
	 */
	public void addExit() {
		addButton(Messages.getText("salir"), BUTTON_EXIT);
	}

	/**
	 * Aadir el boton Ver detalles.
	 */
	public void addSeeDetails() {
		addButton(Messages.getText("verdetalles"), BUTTON_SEEDETAILS);
	}
	
	/**
	 * Aadir el boton Ocultar detalles.
	 */
	public void addHideDetails() {
		addButton(Messages.getText("ocultardetalles"), BUTTON_HIDEDETAILS);
	}
	
	/**
	 * Aadir el boton Pausar.
	 */
	public void addPause() {
		addButton(Messages.getText("pausar"), BUTTON_PAUSE);
	}

	/**
	 * Aadir el boton Reanudar.
	 */
	public void addRestart() {
		addButton(Messages.getText("reanudar"), BUTTON_RESTART);
	}

	/**
	 * Aadimos un botn definido por el usuario.
	 * 
	 * @param text Texto que contendr el botn
	 * @param id Entero para identificar los eventos del botn
	 */
	public void addButton(String text, int id) {
		JButton button = new JButton();
		buttonsList.add(button);
		button.setText(text);
		button.setActionCommand(id + "");
		button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				callActionCommandListeners(Integer.parseInt(e.getActionCommand()));
			}
		});

		add(button);
	}

	/**
	 * Obtener un botn por su Entero
	 * @param id Nmero del disparador del botn
	 * @return El botn especificado o <code>null</code> si no se encontr el botn.
	 */
	public JButton getButton(int id) {
		Iterator acIterator = buttonsList.iterator();
		while (acIterator.hasNext()) {
			JButton button = (JButton) acIterator.next();
			if (Integer.parseInt(button.getActionCommand()) == id)
				return button;
		}
		return null;
	}
}
