/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2005 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.gvsig.gui.beans.graphic;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.FlowLayout;

import javax.swing.JPanel;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.Plot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

/**
 * 
 * Nacho Brodin (brodin_ign@gva.es)
 */

public class GraphicChartPanel extends JPanel {
	
	private static final int	HEIGHT_MARGIN = 5;
	private static final int	WIDTH_MARGIN = 4;
	private JFreeChart 			chart;
	private ChartPanel 			jPanelChart = null;
	private XYSeries 			series[] = null;
	private XYSeriesCollection 	dataset = null;
	private int 				width = 300, height = 150;

	
	public GraphicChartPanel(int width, int height) {
		this.width = width;
		this.height = height;

		int nSeries = 3;
		series = new XYSeries[nSeries];
		dataset = new XYSeriesCollection();
		
		for(int iSerie = 0; iSerie < nSeries; iSerie++){
			series[iSerie] = new XYSeries("");
			for (int i = 0; i < 256; i++) 
		         series[iSerie].add(new Integer(i),  new Integer(i * (iSerie + 1)));
			dataset.addSeries(series[iSerie]);
		}
			    
	    createChart();
	    	    
		initialize();
		
		Plot plot = this.jPanelChart.getChart().getPlot();
        plot.setOutlineStroke(new BasicStroke(1));
        plot.setOutlinePaint(Color.magenta);
	}
	
	/**
	 * This method initializes this			
	 * 
	 */
	private void initialize() {
        FlowLayout flowLayout = new FlowLayout();
        flowLayout.setVgap(0);
        flowLayout.setHgap(0);
        this.setLayout(flowLayout);
        this.setSize(new java.awt.Dimension(width, height));
		this.setPreferredSize(new java.awt.Dimension(width, height));
		this.add(getChart(), null);	
	}

	/**
	 * 
	 * @return
	 */
	public ChartPanel getChart(){
		if(jPanelChart == null){
			jPanelChart = new ChartPanel(chart);
			jPanelChart.setSize(new java.awt.Dimension(width - WIDTH_MARGIN, height - HEIGHT_MARGIN));
			jPanelChart.setPreferredSize(new java.awt.Dimension(width - WIDTH_MARGIN, height - HEIGHT_MARGIN));
			jPanelChart.setBorder(javax.swing.BorderFactory.createLineBorder(java.awt.Color.gray,1));
		}
		return 	jPanelChart;
	}

	   /**
     * Creates a chart.
     * 
     * @param dataset  the dataset.
     * 
     * @return A chart.
     */
    private void createChart() {
            
    	chart = 
		     ChartFactory.createXYLineChart(null, null, null, 
		    	dataset, 
		    	PlotOrientation.VERTICAL,
		        false,
		        true, 
		        true);
        
        // color
        XYPlot plot = chart.getXYPlot();
        plot.getRenderer().setSeriesPaint(0, Color.red);
        plot.getRenderer().setSeriesPaint(1, Color.green); 
        plot.getRenderer().setSeriesPaint(2, Color.blue);
        plot.getRenderer().setSeriesPaint(3, Color.cyan);
        plot.getRenderer().setSeriesPaint(4, Color.black);
        plot.getRenderer().setSeriesPaint(5, Color.darkGray);
        plot.getRenderer().setSeriesPaint(6, Color.gray);
        plot.getRenderer().setSeriesPaint(7, Color.magenta);
        plot.getRenderer().setSeriesPaint(8, Color.yellow);
        plot.getRenderer().setSeriesPaint(9, Color.orange);
        
    }
    
    /**
     * Asigna nuevos valores para la grfica
     * @param values Matriz de [nmero de grficas][Valores por grfica]
     */
    public void setNewChart(int[][] values, String names[]){
    	dataset.removeAllSeries();
    	series = new XYSeries[values.length];
    	for (int iGraf = 0; iGraf < values.length; iGraf++) {
    		series[iGraf] = new XYSeries(names[iGraf]);
    		for (int i = 0; i < values[iGraf].length; i++) {
    			series[iGraf].add(new Integer(i),  new Integer(values[iGraf][i]));
    		}
    		dataset.addSeries(series[iGraf]);
    	}
    	jPanelChart.repaint();
    }
    
    /**
     * Reemplaza los valores de la grfica
     * @param values Matriz de [nmero de grficas][Valores por grfica]
     */
    public void replaceValuesChart(int[][] values){
    	for (int iGraf = 0; iGraf < values.length; iGraf++) {
    		for (int i = 0; i < values[iGraf].length; i++) {
    			series[iGraf].getDataItem(i).setY(values[iGraf][i]);
    		}
    	}
    	jPanelChart.repaint();
    }
    
    /**
     * Limpia la grfica
     */
    public void cleanChart(){
    	dataset.removeAllSeries();
    	jPanelChart.repaint();
    }
    
	/**
	 * Asigna el tamao del componente
	 * @param w Nuevo ancho del componente padre
	 * @param h Nuevo alto del componente padre
	 */
	public void setComponentSize(int w, int h){
		width = w;
		height = h;
		this.setSize(new java.awt.Dimension(w, h));
		this.setPreferredSize(new java.awt.Dimension(w, h));
		jPanelChart.setSize(new java.awt.Dimension(w - WIDTH_MARGIN, h - HEIGHT_MARGIN));
		jPanelChart.setPreferredSize(new java.awt.Dimension(w - WIDTH_MARGIN, h - HEIGHT_MARGIN));
	}
}
