/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2007 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.gvsig.gui.beans.messages;

import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
/**
 * <code>Messages</code> es una clase que accede a los recursos para la i18n
 */
public class Messages {
    /** DOCUMENT ME! */
    private static final String BUNDLE_NAME = "org.gvsig.gui.beans.translations.text";

    /** DOCUMENT ME! */
    private static ResourceBundle RESOURCE_BUNDLE = null;

    /**
     * Inicializa la clase con el locale adecuado
     *
     * @param loc Locale de la aplicacin
     */
    private static void init() {
        Locale loc = Locale.getDefault();
        RESOURCE_BUNDLE = ResourceBundle.getBundle(BUNDLE_NAME, loc);
    }

    
    /**
     * Obtiene el valor del recurso con clave 'key'
     *
     * @param key clave del recurso que se quiere obtener
     *
     * @return recurso que se quiere obtener o !key! en caso de no encontrarlo.
     *         En dicho caso no se notifica al framework ya que  estos son los
     *         mensajes propios de la aplicacin y deben de estar todos
     */
    public static String getText(String key) {
        try {
            // La primera vez, cargamos las cadenas.
            if (RESOURCE_BUNDLE == null)
                init();
            return RESOURCE_BUNDLE.getString(key);
        } catch (MissingResourceException e) {
            return '!' + key + '!';
        }
    }
}
