/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.gvsig.gui.beans.table;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;

import org.gvsig.gui.beans.BaseComponent;
import org.gvsig.gui.beans.table.exceptions.NotInitializeException;
import org.gvsig.gui.beans.table.listeners.TableListener;
import org.gvsig.gui.beans.table.models.ModelLoader;

/**
 * Componente tabla
 * 
 * @author Nacho Brodin (brodin_ign@gva.es)
 *
 */
public class Table extends BaseComponent {
	private static final int		INTERNAL_MARGIN = 4;
	
    private JTable 					table = null;
    private JScrollPane 			scrollPanel = null;
    private DefaultTableModel		tableModel = null;
    private int						width, height;
    private String[]				columnNames = null;
    private int[]					columnWidths = null;
    private TableListener			tableListener = null;
    private TableContainer			tableContainer = null;
    public String					tableModelClass = null;
    
      
    /**
     * 
     * @param width Ancho de la tabla en pixeles
     * @param height Alto de la tabla en pixeles
     * @param columnNames Vector de nombres de columna
     * @param columnsWidth	Vector de anchos para cada columna. Ha de tener el mismo nmero de elementos que columnNames. 
     * Si vale null las columnas se pondrn equidistantes. 
     * @param tableListener Listener de la tabla
     */
    public Table(int width, int height, String[] columnNames, int[] columnWidths, TableListener tableListener, String tableModelClass){
    	this.width = width;
    	this.height = height;
    	this.columnNames = columnNames;
		this.columnWidths = columnWidths;
    	this.tableListener = tableListener;
    	this.tableModelClass = tableModelClass;
    	initialize();
    }
		    
    /**
	 * This method initializes this
	 * 
	 */
	private void initialize() {
		 this.setSize(new java.awt.Dimension(width, height));
		 FlowLayout flowLayout = new FlowLayout();
		 flowLayout.setHgap(0);
		 flowLayout.setVgap(0);
		 this.setLayout(flowLayout);
		 this.add(getJScrollPane(), null);
	}

	/**
	 * Asigna el tamao del componente
	 * @param w Nuevo ancho del componente padre
	 * @param h Nuevo alto del componente padre
	 */
	public void setComponentSize(int w, int h){
		this.setSize(new java.awt.Dimension(w, h));
		scrollPanel.setSize(w, h);
		//this.setPreferredSize(new java.awt.Dimension(w, h));
		scrollPanel.setPreferredSize(new Dimension(w, h));
	}
		
	
	/**
     * Obtiene la Tabla
     * @return Tabla de bandas de la imagen
     */
    public JTable getTable() {
        if (table == null) {
            //tableModel = new ListModel(columnNames);
        	ModelLoader loader = new ModelLoader();
        	table = loader.load(tableModelClass, columnNames);
        	tableModel = loader.getTableModel();
            tableModel.addTableModelListener(tableListener);
            table.setPreferredScrollableViewportSize(new Dimension(width, height));
            table.addMouseListener(new MouseAdapter() {
 	           public void mouseClicked(MouseEvent e) {
 	        	   try{
 	        		   if(tableContainer != null)
 	        			   tableContainer.setSelectedIndex(table.getSelectedRow());
 	        	   }catch(NotInitializeException ex){}
 	           }
 	        });
            
            TableColumn column = null;
            
            int widthPerColumn = (int)(width / columnNames.length);
            for (int i = 0; i < columnNames.length; i++) {
            	column = table.getColumnModel().getColumn(i);
            	column.setResizable(true);
            	if(columnWidths == null)
            		column.setPreferredWidth(widthPerColumn);
            	else
            		column.setPreferredWidth(columnWidths[i]);
            }
        }

        return table;
    }
    
    /**
	 * This method initializes jScrollPane	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JScrollPane getJScrollPane() {
		if (scrollPanel == null) {
			scrollPanel = new JScrollPane(getTable());
			scrollPanel.setSize(width, height);
			scrollPanel.setPreferredSize(new Dimension(width , height));           
		}
		return scrollPanel;
	}

	/**
	 * Obtiene el contenedor padre de la tabla
	 * @return
	 */
	public TableContainer getTableContainer() {
		return tableContainer;
	}

	/**
	 * Asigna el contenedor padre de la tabla
	 * @param tableContainer
	 */
	public void setTableContainer(TableContainer tableContainer) {
		this.tableContainer = tableContainer;
	}
	
	/**
	 * Obtiene el modelo de la tabla
	 * @return 
	 */
	public DefaultTableModel getTableModel() {
		return tableModel;
	}
	
	//********************Mtodos de Tabla*****************************//
	
	/**
	 * Aade una fila a la tabla.
	 * @param list Lista de cadenas
	 */
	public void addRow(Object[] list){
		((DefaultTableModel)this.getTable().getModel()).addRow(list);
		/*try{
			for(int i = 0; i < list.length; i++)	
				((DefaultTableModel)this.getTable().getModel()).setValueAt(list[i], 
								((DefaultTableModel)this.getTable().getModel()).getRowCount() - 1, 
								i);
		}catch(ArrayIndexOutOfBoundsException exc){
			//Se intenta acceder a una columna que no existe. Terminamos el bucle.
		}*/
	}
	
	/**
	 * Elimina una fila de la tabla.
	 * @param i Fila a eliminar
	 */
	public void delRow(int i){
		((DefaultTableModel)this.getTable().getModel()).removeRow(i);
	}
	
	/**
	 * Elimina todas las filas de la tabla.
	 */
	public void removeAllRows(){
		int nRows = ((DefaultTableModel)this.getTable().getModel()).getRowCount();
		for(int i = 0; i < nRows; i++)
			((DefaultTableModel)this.getTable().getModel()).removeRow(0);
	}


}
