/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.gvsig.gui.beans.table;

import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.JPanel;
import javax.swing.table.DefaultTableModel;

import org.gvsig.gui.beans.BaseComponent;
import org.gvsig.gui.beans.table.exceptions.NotInitializeException;
import org.gvsig.gui.beans.table.listeners.TableListener;

/**
 * Contenedor para los componentes de la tabla. Incluye la tabla  y el panel
 * de control.
 *
 * @author Nacho Brodin (brodin_ign@gva.es)
 *
 */
public class TableContainer extends BaseComponent{

	private static final int		INTERNAL_MARGIN = 10;
	private  int					HEIGHT_CONTROL = 24;

	private Table 					pTable = null;
	private JPanel	 				pControl = null;
	private int 					width = 500, height = 120;
	private TableControlerPanel 	pTableControl = null;
	private JPanel 					pGeneral = null;
    private String[]				columnNames = null;
    private int[]					columnWidths = null;
    private TableListener			tableListener = null;
    //Variable que indica si la tabla ha sido inicializada
    private boolean					initTable = false;
    private String					tableModelClass = "ListModel";
    private int						lastAssignW, lastAssignH;

    /**
     *
     * @param width Ancho de la tabla en pixeles
     * @param height Alto de la tabla en pixeles
     * @param columnNames Vector de nombres de columna
     * @param columnsWidth	Vector de anchos para cada columna. Ha de tener el mismo nmero de elementos que columnNames.
     * Si vale null las columnas se pondrn equidistantes.
     */
	public TableContainer(int width, int height, String[] columnNames, int[] columnWidths) {
		this.width = width;
		this.height = height;
		this.columnNames = columnNames;
		this.columnWidths = columnWidths;
	}

	/**
	 * This method initializes this
	 *
	 */
	public void initialize() {
        initTable = true;
		this.tableListener = new TableListener(this);
        this.add(getPGeneral(), new GridBagLayout());
	}

	/**
	 * Asigna el tamao del componente
	 * @param w Nuevo ancho del componente padre
	 * @param h Nuevo alto del componente padre
	 */
	public void setComponentSize(int w, int h){
		lastAssignW = w;
		lastAssignH = h;
		//getPControl().setSize(w - INTERNAL_MARGIN, HEIGHT_CONTROL);
		getPControl().setPreferredSize(new java.awt.Dimension(w - INTERNAL_MARGIN, HEIGHT_CONTROL));
		getPGeneral().setPreferredSize(new java.awt.Dimension(w, h));
		getPTable().setComponentSize(w - INTERNAL_MARGIN, h - INTERNAL_MARGIN - HEIGHT_CONTROL - (INTERNAL_MARGIN / 2));
	}

	/**
	 * This method initializes jPanel
	 *
	 * @return javax.swing.JPanel
	 */
	public Table getPTable() {
		if (pTable == null) {
			pTable = new Table(	width - INTERNAL_MARGIN ,
								height - INTERNAL_MARGIN - HEIGHT_CONTROL - (INTERNAL_MARGIN / 2),
								columnNames,
								columnWidths,
								tableListener,
								tableModelClass);
			pTable.setTableContainer(this);
			FlowLayout flowLayout = new FlowLayout();
			flowLayout.setHgap(0);
			flowLayout.setVgap(0);
			pTable.setLayout(flowLayout);
		}
		return pTable;
	}

	/**
	 * This method initializes jPanel1
	 *
	 * @return javax.swing.JPanel
	 */
	private JPanel getPControl() {
		if (pControl == null) {
			FlowLayout flowLayout = new FlowLayout();
			flowLayout.setAlignment(java.awt.FlowLayout.LEFT);
			flowLayout.setVgap(0);
			flowLayout.setHgap(0);
			pControl = new JPanel();
			pControl.setBorder(javax.swing.BorderFactory.createLineBorder(java.awt.Color.gray,1));
			pControl.setLayout(flowLayout);
			pControl.setSize(width - INTERNAL_MARGIN, HEIGHT_CONTROL);
			pControl.setPreferredSize(new java.awt.Dimension(width - INTERNAL_MARGIN, HEIGHT_CONTROL));
			pControl.add(getPTableControl(), null);
		}
		return pControl;
	}

	/**
	 * This method initializes jPanel
	 *
	 * @return javax.swing.JPanel
	 */
	public TableControlerPanel getPTableControl() {
		if (pTableControl == null) {
			pTableControl = new TableControlerPanel(tableListener);
		}
		return pTableControl;
	}

	/**
	 * This method initializes jPanel
	 *
	 * @return javax.swing.JPanel
	 */
	private JPanel getPGeneral() {
		if (pGeneral == null) {
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.gridx = 0;
			gridBagConstraints.gridy = 0;
	        gridBagConstraints.insets = new java.awt.Insets(0,0,(INTERNAL_MARGIN / 2),0);
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.gridx = 0;
			gridBagConstraints1.gridy = 1;
	        gridBagConstraints1.insets = new java.awt.Insets(0,0,0,0);
			pGeneral = new JPanel();
			pGeneral.setPreferredSize(new java.awt.Dimension(width, height));
			pGeneral.setLayout(new GridBagLayout());
			pGeneral.setBorder(javax.swing.BorderFactory.createLineBorder(java.awt.Color.gray,1));
			pGeneral.add(getPTable(), gridBagConstraints);
			pGeneral.add(getPControl(), gridBagConstraints1);
		}
		return pGeneral;
	}

	//********************Mtodos de Tabla*****************************//

	/**
	 * Aade una fila a la tabla.
	 * @param list Lista de cadenas
	 */
	public void addRow(Object[] list)throws NotInitializeException{
		if(!initTable)
			throw new NotInitializeException();

		TableListener.comboEventEnable = false;
		((Table)pTable).addRow(list);
		pTableControl.addPointToTable(((Table)pTable).getTable().getRowCount());
		setSelectedIndex(getRowCount() - 1);
		TableListener.comboEventEnable = true;
	}

	/**
	 * Elimina una fila de la tabla.
	 * @param i Fila a eliminar
	 */
	public void delRow(int i)throws NotInitializeException{
		if(!initTable)
			throw new NotInitializeException();

		TableListener.comboEventEnable = false;
		try{
			setSelectedIndex(getSelectedIndex() - 1);
		}catch(ArrayIndexOutOfBoundsException ex){
			try{
				setSelectedIndex(getSelectedIndex() + 1);
			}catch(ArrayIndexOutOfBoundsException exc){}
		}
		((Table)pTable).delRow(i);
		pTableControl.setNItems(getRowCount());
		TableListener.comboEventEnable = true;
	}

	/**
	 * Elimina todas las filas de la tabla.
	 */
	public void removeAllRows()throws NotInitializeException{
		if(!initTable)
			throw new NotInitializeException();

		TableListener.comboEventEnable = false;
		((Table)pTable).removeAllRows();
		pTableControl.setNItems(0);
		TableListener.comboEventEnable = true;
	}

	/**
	 * Obtiene el nmero de filas en la tabla
	 * @return Nmero de filas de la tabla
	 */
	public int getRowCount()throws NotInitializeException{
		if(!initTable)
			throw new NotInitializeException();

		return ((Table)pTable).getTable().getRowCount();
	}

	/**
	 * Selecciona un punto de la lista
	 * @param i punto a seleccionar
	 */
	public void setSelectedIndex(int i)throws NotInitializeException{
		if(!initTable)
			throw new NotInitializeException();

		TableListener.comboEventEnable = false;
		try{
			pTableControl.setSelectedIndex(i);
			((Table)pTable).getTable().setRowSelectionInterval(i, i);
		}catch(IllegalArgumentException ex){

		}
		TableListener.comboEventEnable = true;
	}

	/**
	 * Obtiene el punto seleccionado de la lista
	 * @return Posicin del punto seleccionado de la tabla
	 */
	public int getSelectedIndex()throws NotInitializeException{
		if(!initTable)
			throw new NotInitializeException();

		return ((Table)pTable).getTable().getSelectedRow();
	}

	/**
	 * Asigna un valor a una posicin de la tabla
	 * @param value Valor
	 * @param row Fila
	 * @param col Columna
	 */
	public void setValueAt(Object value, int row, int col)throws NotInitializeException{
		if(!initTable)
			throw new NotInitializeException();

		((Table)pTable).getTable().setValueAt(value, row, col);
	}

	/**
	 * Dice si una tabla es editable o no.
	 * Este flag hay que asignarlo antes de la inicializacin de tabla.
	 * @param editable
	 */
	public void setEditable(boolean editable)throws NotInitializeException{
		if(!initTable)
			throw new NotInitializeException();

		if(!editable)
			((Table)pTable).getTable().setBackground(getPGeneral().getBackground());
		else
			((Table)pTable).getTable().setBackground(Color.white);

		((Table)pTable).getTable().setEnabled(editable);
	}

	/**
	 * Asigna el modelo de la tabla
	 * @param model cadena con el nombre del modelo
	 */
	public void setModel(String model){
		tableModelClass = model;
	}

	/**
	 * Obtiene el model de la tabla
	 * @return
	 */
	public DefaultTableModel getModel(){
		return (DefaultTableModel)((Table)pTable).getTable().getModel();
	}

	/**
	 * Asigna al panel de control de tabla la propiedad de visible/invisible a true o false
	 * @param visible
	 */
	public void setControlVisible(boolean visible){
		getPTableControl().setVisible(visible);
		if(!visible)
			HEIGHT_CONTROL = 0;
		else
			HEIGHT_CONTROL = 24;
	}

	/**
	 * Obtiene el control de tabla
	 * @return TableControlerPanel
	 */
	public TableControlerPanel getControl(){
		return this.pTableControl;
	}

	/**
	 * Desactiva o activa el evento de nueva linea. Si se desactiva tendr que ser
	 * gestionado por el cliente
	 * @param enabled true para activar y false para desactivar
	 */
	public void setEnableControlsListener(boolean enabled){
		tableListener.enableNewLineListener = enabled;
	}

	/**
	 * Activar o desactivar los componentes del panel
	 */
	public void setEnabled(boolean enabled){
		getPTable().getTable().setEnabled(enabled);
		if(!enabled)
			getPTableControl().disableAllControls();
		else
			getPTableControl().restoreControlsValue();
	}
 }
