/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.gvsig.gui.beans.treelist;

import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.util.ArrayList;
import java.util.Hashtable;

import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.ListSelectionModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;

import org.gvsig.gui.beans.BaseComponent;
import org.gvsig.gui.beans.treelist.listeners.TreeListComponentListener;
import org.gvsig.gui.beans.treelist.listeners.TreeListListener;
import org.gvsig.i18n.Messages;

/**
 * Componente consistente en un men de arbol al que se le pueden aadir entradas
 * y una lista de elementos debajo de este. 
 * Haciendo doble click o arrastrando un elemento del men a la lista este queda aadido
 * en esta. Haciendo doble click en un elemento de la lista se elimina de esta y arrastrando
 * elementos dentro de la lista se varia su posicin en ella.
 * Nacho Brodin (brodin_ign@gva.es)
 */

public class TreeListContainer extends BaseComponent {

	private int 		wComp = 190;
	private int 		hComp = 360;
	private int 		hTree = (int)Math.floor(hComp * 0.68);
	private int 		hList = hComp - hTree;
	private Hashtable	map;
	
	private JPanel pMain = null;
	private JScrollPane pTree = null;
	private JScrollPane pList = null;
	private JTree tree = null;
	private JList list = null;
	private DefaultMutableTreeNode raiz = null;
	private TreeListListener listener = null;
	private ArrayList		listListeners = new ArrayList();

	/**
	 * This method initializes 
	 * 
	 */
	public TreeListContainer() {
		super();
		listener = new TreeListListener();
		initialize();
		listener.setList(getList());
		listener.setTree(getTree());
	}

	/**
	 * This method initializes this
	 * 
	 */
	private void initialize() {
		map = new Hashtable();
		raiz =  new DefaultMutableTreeNode(Messages.getText("filtros"));
        this.setPreferredSize(new java.awt.Dimension(wComp, hComp));
        FlowLayout flowLayout = new FlowLayout();
        flowLayout.setHgap(0);
        flowLayout.setVgap(0);
        this.setLayout(flowLayout);
        this.add(getPMain(), null);

	}

	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPMain() {
		if (pMain == null) {
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.gridx = 0;
			gridBagConstraints1.gridy = 1;
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.gridx = 0;
			gridBagConstraints.gridy = 0;
			pMain = new JPanel();
			pMain.setLayout(new GridBagLayout());
			pMain.setPreferredSize(new java.awt.Dimension(wComp,hComp));
			pMain.add(getPTree(), gridBagConstraints);
			pMain.add(getPList(), gridBagConstraints1);
		}
		return pMain;
	}

	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JScrollPane getPTree() {
		if (pTree == null) {
			pTree = new JScrollPane();
			pTree.setPreferredSize(new java.awt.Dimension(wComp, hTree));
			pTree.setViewportBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.LOWERED));
			pTree.setVerticalScrollBarPolicy(javax.swing.JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
			pTree.setViewportView(getTree());
		}
		return pTree;
	}

	/**
	 * This method initializes jPanel1	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JScrollPane getPList() {
		if (pList == null) {
			pList = new JScrollPane();
			pList.setPreferredSize(new java.awt.Dimension(wComp, hList));
			pList.setBorder(javax.swing.BorderFactory.createEtchedBorder(javax.swing.border.EtchedBorder.RAISED));
			pList.setBackground(java.awt.Color.white);
			pList.setVerticalScrollBarPolicy(javax.swing.JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
			pList.setViewportView(getList());
		}
		return pList;
	}

	/**
	 * This method initializes jTree	
	 * 	
	 * @return javax.swing.JTree	
	 */
	public JTree getTree() {
		if (tree == null) {
			tree = new JTree(raiz);
			tree.addMouseListener(listener);
			tree.addMouseMotionListener(listener);
		}
		return tree;
	}

	/**
	 * This method initializes jList	
	 * 	
	 * @return javax.swing.JList	
	 */
	public JList getList() {
		if (list == null) {
			DefaultListModel m = new DefaultListModel(); 
			list = new JList(m);
			list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			list.addMouseListener(listener);
			list.addMouseMotionListener(listener);
		}
		return list;
	}

	
	public void setComponentSize(int w, int h){
		wComp = w;
		hComp = h;
		hTree = (int)Math.floor(hComp * 0.68);
		hList = hComp - hTree;
        this.setPreferredSize(new java.awt.Dimension(wComp, hComp));
		pMain.setPreferredSize(new java.awt.Dimension(wComp, hComp));
		pTree.setPreferredSize(new java.awt.Dimension(wComp, hTree));
		pList.setPreferredSize(new java.awt.Dimension(wComp, hList));
	}
	
	//-------------------------------------------
	//MTODOS DE ARBOL
	
	/**
	 * Aade una nueva categoria al arbol
	 * @param name	Etiqueta que aparece en el arbol. 
	 * @param pos	Posicin en el arbol de la nueva categoria
	 */
	public void addClass(String name, int pos){
		DefaultTreeModel model =(DefaultTreeModel)tree.getModel();
		DefaultMutableTreeNode r = new DefaultMutableTreeNode( name );
		model.insertNodeInto(r, raiz, pos);
	}
	
	/**
	 * Aade una entrada a una categoria
	 * @param name Nombre de la entrada a aadir
	 * @param parentName Categoria a la que aadimos
	 * @param value Valor asociado a la entrada
	 */
	public void addEntry(String name, String parentName, String value){
		DefaultTreeModel model =(DefaultTreeModel)tree.getModel();
		for(int i = 0; i < model.getChildCount(raiz); i++){
			if(model.getChild(raiz, i).toString().equals(parentName)){
				DefaultMutableTreeNode node = (DefaultMutableTreeNode)model.getChild(raiz, i);
				node.add(new DefaultMutableTreeNode(name));
				if (value!=null)
					map.put(name,value);
			}
		}
	}

	/**
	 * Mtodo que comprueba si una entrada existe en la lista.
	 * @param value Valor que se quiere comprobar si est en la lista
	 * @return true si el valor est en la lista y false si no lo est
	 */
	public boolean isInList(String value){
		DefaultListModel model = (DefaultListModel)getList().getModel();
		for(int i = 0; i < model.getSize(); i++)
			if(((String)model.get(i)).equals(value))
				return true;
		return false;
	}
	
	/**
	 * Aade un listener TreeListComponent
	 * @param e
	 */
	public void addTreeListListener(TreeListComponentListener e){
		listListeners.add(e);
		listener.setListeners(listListeners);
	}
	
	/**
	 * Aade un elemento a la lista 
	 * @param element Elemento a aadir
	 */
	public void addElementInList(String element){
		DefaultListModel model = (DefaultListModel)getList().getModel();	
		model.addElement(element);
	}
	
	/**
	 * Elimina un elemento a la lista 
	 * @param element Elemento a eliminar
	 */
	public void removeElementInList(String element){
		DefaultListModel model = (DefaultListModel)getList().getModel();	
		model.removeElement(element);
	}
	
	/**
	 * Elimina un elemento a la lista por indice 
	 * @param element Indice del elemento a eliminar
	 */
	public void removeElementInList(int element){
		DefaultListModel model = (DefaultListModel)getList().getModel();	
		model.remove(element);
	}

	public Hashtable getMap() {
		return map;
	}

	public void setMap(Hashtable map) {
		this.map = map;
	}
}
