package org.agil.core.spatialindex;

import java.io.IOException;

import java.util.*;

/**
 *  Implements a memory page file. Stores all pages into a Hashtable with keys
 *  equal to each node page number. <p>
 *
 *  Created: Wed May 19 09:33:26 1999 <p>
 *
 *
 *
 *@author     Marios Hadjieleftheriou
 *@version    2
 */
public class MemoryPageFile extends PageFile {
	private Hashtable file = new Hashtable();


	/**
	 *  Creates an exact copy of this object.
	 *
	 *@return    Description of the Returned Value
	 */
	public Object clone() {
		return new MemoryPageFile();
		//return this;
	}


	/**
	 *  Description of the Method
	 */
	public void dumpMemory() {
		for (Enumeration e = file.elements(); e.hasMoreElements(); ) {
			System.out.println(e.nextElement());
		}
	}


	/**
	 *  Description of the Method
	 *
	 *@exception  IOException  Description of Exception
	 */
	public void close() throws IOException {
		file.clear();
		file = null;
	}


	/**
	 *  Description of the Method
	 *
	 *@param  tree        Description of Parameter
	 *@param  dimension   Description of Parameter
	 *@param  fillFactor  Description of Parameter
	 *@param  capacity    Description of Parameter
	 */
	protected void initialize(RTree tree, int dimension, float fillFactor, int capacity) {
		super.initialize(tree, dimension, fillFactor, capacity);
		file.clear();
	}


	/**
	 *  Description of the Method
	 *
	 *@param  page                Description of Parameter
	 *@return                     Description of the Returned Value
	 *@exception  PageFaultError  Description of Exception
	 */
	protected AbstractNode readNode(long page) throws PageFaultError {
		if (page < 0) {
			throw new IllegalArgumentException("Page number cannot be negative.");
		}
		AbstractNode ret = (AbstractNode) file.get(new Long(page));
		if (ret == null) {
			throw new PageFaultError("Invalid page number request.");
		}
		return ret;
	}


	/**
	 *  Description of the Method
	 *
	 *@param  n                   Description of Parameter
	 *@return                     Description of the Returned Value
	 *@exception  PageFaultError  Description of Exception
	 */
	protected long writeNode(AbstractNode n) throws PageFaultError {
		if (n == null) {
			throw new IllegalArgumentException("Node cannot be null.");
		}
		long i = 0;
		if (n.pageNumber < 0) {
			while (true) {
				if (!file.containsKey(new Long(i))) {
					break;
				}
				i++;
			}
			n.pageNumber = i;
		}
		else {
			i = n.pageNumber;
		}
		file.put(new Long(i), n);
		return i;
	}


	/**
	 *  Description of the Method
	 *
	 *@param  page                Description of Parameter
	 *@return                     Description of the Returned Value
	 *@exception  PageFaultError  Description of Exception
	 */
	protected AbstractNode deletePage(long page) throws PageFaultError {
		return (AbstractNode) file.remove(new Long(page));
	}
}
// MemoryPageFile
